package de.associativememory.control;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;

import de.associativememory.system.Globals;
import n4e.core.controls.AWTextField;
import n4e.core.tool.Tool4Throwable;

/**
 * The <CODE>Value</CODE> class for values of the layers and the weights.
 * description: AssociativeMemory
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne , germany.
 * @version 08.02.2017 14:31:59
 * @since 0.1
 */
public class Value {

	private AWTextField tfField;
	private int value;
	private boolean set_color;
	
	/**
	 * The constructor <CODE>Value</CODE> creates a new instance(object)
	 * of the class.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param editable true -> the control is editable.
	 * @param set_color true -> set a background color.
	 * @since 0.1
	 */
	public Value(Container c, int x, int y, int width, int height, boolean editable, boolean set_color) {
		
		this.set_color = set_color;
		
		tfField = new AWTextField();
		tfField.setBounds(x, y, width, height);
		tfField.setFont(Globals.fontStandard11);
		tfField.setEditable(editable);
		
		tfField.addMouseListener(new java.awt.event.MouseListener() {
			public void mouseClicked(MouseEvent e) {
				try {
					if (e.getButton() == 1) { // left button.
						if (!editable) {
							return;
						}
						value = Integer.parseInt(tfField.getText());
						if (value == 0) {
							value = 1;
						} else if (value < 0) {
							value = 1;
						} else if (value > 0) {
							value = -1;
						}
						setValue(value);
					}
				} catch (Exception ex) {
					Globals.t4d.showException(Tool4Throwable.getCallStackOut(ex));
				}
			}
			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}
			public void mousePressed(MouseEvent e) {}
			public void mouseReleased(MouseEvent e) {}
    	});

		c.add(tfField);
	}
	
	/**
	 * The method <CODE>setValue</CODE> sets the value an the value of the text field.
	 * @param val the value.
	 */
	public void setValue(int val) {
		value = val;
		String sval = Integer.toString(value);
		Color color = Color.WHITE;
		if (value > 0) {
			sval = "+" + sval;
		}
		if (set_color) {
			if (value > 0) {
				color = Color.GREEN;
			} if (value < 0) {
				color = Color.YELLOW;
			}
		}
		tfField.setText(sval);
		tfField.setBackground(color);
	}

	/**
	 * The method <CODE>getValue</CODE> returns the value.
	 */
	public int getValue() {
		return value;
	}

	/**
	 * The method <CODE>getTextField</CODE> returns the text field.
	 */
	public AWTextField getTextField() {
		return tfField;
	}

}
