package de.associativememory.system;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JLabel;

import de.associativememory.control.Value;
import n4e.core.controls.AWTextField;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.MainDialog;
import n4e.core.tool.Tool4Throwable;

/**
 * The <CODE>AssociativeMemory</CODE> class for associative memory processing.
 * description: associative memory processing.
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne , germany.
 * @version 08.02.2017 12:19:59
 * @since 0.1
 */

public class AssociativeMemory extends MainDialog {

	private Value[] layerX;
	private Value[] layerY;
	private Value[][] weights;
	private AWTextField tfNumberXElements;
	private AWTextField tfNumberYElements;
	private JLabel lblEnergy;

	private JButton btnDeleteX;
	private JButton btnExecute;
	private JButton btnTraining;
	private JButton btnReset;
	private JButton btnClose;

	private int no_y_elements;
	private int no_x_elements;
	
	private int x_vector_line;
	private int y_vector_column;
	private int weights_array_line;
	private int weights_array_column;
	
	/**
	 * The constructor <CODE>AssociativeMemory</CODE> creates a new instance(object)
	 * of the class.
	 * @since 0.1
	 */
	public AssociativeMemory() {
		super("associative memory", "0.1", "2017", 480, 640);

		try {

			setResizable(false);
			Globals.am = this;
			Globals.doAfterInitials();
			Image icon = Globals.t4md.getImagePure("app.jpg");
			setIconImage(icon);
			windowInit();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @exception Exception if error.
	 * @since 0.1
	 */
	private void windowInit() throws Exception {

		try {

			no_y_elements = 4;
			no_x_elements = 6;
			x_vector_line = 20;
			y_vector_column = 20;
			weights_array_line = 50;
			weights_array_column = 50;
			int controls_line = 350;

			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 49, screen.width, 20);
			Container con = getContentPane();
			con.setLayout(null);
			con.setSize(new Dimension(screen.width, screen.height));
			con.setBackground(Color.LIGHT_GRAY);
			
			createLabel(con, 10, controls_line, 120, 20, Globals.message.getMessage("msg0002"));
			tfNumberXElements = createTextField(con, 140, controls_line, 40, 20, no_x_elements);
			createLabel(con, 10, controls_line + 25, 120, 20, Globals.message.getMessage("msg0003"));
			tfNumberYElements = createTextField(con, 140, controls_line + 25, 40, 20, no_y_elements);
			createLabel(con, 10, controls_line + 50, 120, 20, Globals.message.getMessage("msg0010"));
			lblEnergy = createLabel(con, 140, controls_line + 50, 40, 20, "0");

			btnDeleteX = createButton(con, y_vector_column - 15, x_vector_line, 23, 23, Globals.message.getMessage("msg0009"), false);
			btnExecute = createButton(con, 190, controls_line + 50, 100, 23, Globals.message.getMessage("msg0008"), true);
			btnTraining = createButton(con, 300, controls_line + 50, 100, 23, Globals.message.getMessage("msg0006"), true);
			btnReset = createButton(con, 410, controls_line + 50, 100, 23, Globals.message.getMessage("msg0004"), true);
			btnClose = createButton(con, 520, controls_line + 50, 100, 23, Globals.message.getMessage("msg0001"), true);

			createLabel(con, y_vector_column + 10, x_vector_line, 20, 20, "Y:");
			createLabel(con, 7, x_vector_line + 25, 20, 20, "X:");

	        createControls();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}

		setVisible(true);

	}

	/**
	 * The method <CODE>createLabel</CODE> creates a label.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @return a label.
	 * @since 0.1
	 */
	private JLabel createLabel(Container c, int x, int y, int width, int height, String text) {
        JLabel label = new JLabel(text);
		label.setBounds(x, y, width, height);
        c.add(label);
        return label;
	}
	
	/**
	 * The method <CODE>createTextField</CODE> creates a text field.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param value the value of the control.
	 * @return a text field.
	 * @since 0.1
	 */
	private AWTextField createTextField(Container c, int x, int y, int width, int height, int value) {
		AWTextField tf = new AWTextField();
        tf.setBounds(x, y, width, height);
        tf.setFont(Globals.fontStandard11);
        tf.setText(Integer.toString(value));
		c.add(tf);
		return tf;
	}
	
	/**
	 * The method <CODE>createButton</CODE> creates a button.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @param show true: show the text.
	 * @return a button.
	 * @since 0.1
	 */
	private JButton createButton(Container c, int x, int y, int width, int height, String text, boolean show) {
		String curr_text = "";
		if (show) {
			curr_text = text;
		}
		JButton btn = new JButton(curr_text);
	    btn.setBounds(x, y, width, height);
	    btn.setFont(GlobalsDialog.fontStandard12);
	    btn.setToolTipText(text);
	    btn.addActionListener(this);
		c.add(btn);
		return btn;
	}
	
	/**
	 * The method <CODE>createControls</CODE> creates layers and the weights of the neural network.
	 * @since 0.1
	 */
	private void createControls() {
		
		no_x_elements = Integer.parseInt(tfNumberXElements.getText());
		no_y_elements = Integer.parseInt(tfNumberYElements.getText());
		
		layerX = new Value[no_x_elements];
        for (int i=0;i<no_x_elements;i++) {
        	layerX[i] = new Value(c, y_vector_column, weights_array_line + (25 * i), 20, 20, true, true);
        	layerX[i].setValue(0);
        }

        layerY = new Value[no_y_elements];
        for (int j=0;j<no_y_elements;j++) {
        	layerY[j] = new Value(c, weights_array_column + (30 * j), x_vector_line, 20, 20, true, true);
        	layerY[j].setValue(0);
        }

		weights = new Value[no_x_elements][no_y_elements];
        for (int i=0;i<no_x_elements;i++) {
            for (int j=0;j<no_y_elements;j++) {
            	weights[i][j] = new Value(c, weights_array_column + (30 * j), weights_array_line + (25 * i), 25, 20, false, false);
            	weights[i][j].setValue(0);
            }
        }
	        
	}
	
	/**
	 * The method <CODE>clearControls</CODE> clears the controls of the neural network.
	 * @since 0.1
	 */
	private void clearControls() {
		
        for (int i=0;i<no_x_elements;i++) {
        	layerX[i].getTextField().setVisible(false);
			c.remove(layerX[i].getTextField());
        }

        for (int j=0;j<no_y_elements;j++) {
        	layerY[j].getTextField().setVisible(false);
			c.remove(layerY[j].getTextField());
        }

        for (int i=0;i<no_x_elements;i++) {
            for (int j=0;j<no_y_elements;j++) {
            	weights[i][j].getTextField().setVisible(false);
    			c.remove(weights[i][j].getTextField());
            }
        }
	       
        lblEnergy.setText("0");
	}

	/**
	 * The method <CODE>checkPlaisibilityWeights</CODE> check the plausibility of the weights.
	 * @return true if plausibility is ok.
	 * @since 0.1
	 */
	private boolean checkPlaisibilityWeights() {
		
		int x_val = Integer.parseInt(tfNumberXElements.getText());
		int y_val = Integer.parseInt(tfNumberYElements.getText());

		if (x_val < 1 || x_val > 12 || y_val < 1 || y_val > 19) {
			Globals.t4d.showErrorDialog(Globals.message.getMessage("msg0005"));
			return false;
		}

        return true;
	}
	
	/**
	 * The method <CODE>checkPlaisibilityLayer</CODE> check the plausibility of the layers.
	 * @return true if plausibility is ok.
	 * @since 0.1
	 */
	private boolean checkPlaisibilityLayer() {
		
        for (int i=0;i<no_x_elements;i++) {
        	if (layerX[i].getValue() != 1 && layerX[i].getValue() != -1) {
    			Globals.t4d.showErrorDialog(Globals.message.getMessage("msg0007"));
    			return false;
        	}
        }

        for (int j=0;j<no_y_elements;j++) {
        	if (layerY[j].getValue() != 1 && layerY[j].getValue() != -1) {
    			Globals.t4d.showErrorDialog(Globals.message.getMessage("msg0007"));
    			return false;
        	}
        }

        return true;
	}

	/**
	 * The method <CODE>training</CODE> training of the neural network.
	 * @since 0.1
	 */
	private void training() {
        for (int i=0;i<no_x_elements;i++) {
            for (int j=0;j<no_y_elements;j++) {
            	int weight = weights[i][j].getValue();
            	weight += layerX[i].getValue() * layerY[j].getValue();
            	weights[i][j].setValue(weight);
            }
        }
        calcEnergy();
	}
	
	/**
	 * The method <CODE>deleteX</CODE> delete the x layer.
	 * @since 0.1
	 */
	private void deleteX() {
        for (int j=0;j<no_y_elements;j++) {
        	layerY[j].setValue(0);
        }
	}
	
	/**
	 * The method <CODE>execute</CODE> execute of the neural network.
	 * @since 0.1
	 */
	private void execute() {
		
        for (int j=0;j<no_y_elements;j++) {
        	layerY[j].setValue(0);
        }

        boolean changed = false;
        do {
        	
        	changed = false;
        	
	        for (int j=0;j<no_y_elements;j++) {
	        	int a = 0;
	            for (int i=0;i<no_x_elements;i++) {
	            	a += weights[i][j].getValue() * layerX[i].getValue();
	            }
	            layerY[j].setValue(T(a));
	        }
	
	        for (int i=0;i<no_x_elements;i++) {
	        	int a = 0;
	            for (int j=0;j<no_y_elements;j++) {
	            	a += weights[i][j].getValue() * layerY[j].getValue();
	            }
	            if (layerX[i].getValue() != T(a)) {
	            	changed = true;
	            }
	            layerX[i].setValue(T(a));
	        }
	        
        } while (changed);

        calcEnergy();
	}
	
	/**
	 * The method <CODE>T</CODE> the transfer function.
	 * @param a the activation value.
	 * @since 0.1
	 */
	private int T(int a) {
		if (a > 0) {
			return 1;
		} else {
			return -1;
		}
	}

	/**
	 * The method <CODE>calcEnergy</CODE> calculates the energy.
	 * @since 0.1
	 */
	private void calcEnergy() {
    	int e = 0;
        for (int i=0;i<no_x_elements;i++) {
            for (int j=0;j<no_y_elements;j++) {
            	e += weights[i][j].getValue() * layerX[i].getValue() * layerY[j].getValue();
            }
        }
        lblEnergy.setText(Integer.toString(-e));
	}
	
	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 * @since 0.1
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			if (e.getSource() == btnDeleteX) {
				deleteX();
			} else if (e.getSource() == btnExecute) {
				execute();
			} else if (e.getSource() == btnTraining) {
				if (checkPlaisibilityLayer()) {
					training();
				}
			} else if (e.getSource() == btnReset) {
				if (checkPlaisibilityWeights()) {
					clearControls();
					createControls();
				}
			} else if (e.getSource() == btnClose) {
				System.exit(0);
			}
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}
	
	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 0.1
	 */
	public static void main(String[] args) {
		try {
			new Globals();
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new AssociativeMemory();
	}

}
