package de.associativememorypicture.system;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;

import de.associativememorypicture.control.Value;
import n4e.core.controls.AWTextField;
import n4e.core.controls.ExampleFileFilter;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.MainDialog;
import n4e.core.tool.Tool4Throwable;

/**
 * The <CODE>AssociativeMemoryPicture</CODE> class for associative memory picture processing.
 * description: associative memory processing.
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne , germany.
 * @version 09.02.2017 09:55:59
 * @since 0.1
 */

public class AssociativeMemoryPicture extends MainDialog {

	private int[] layerX;
	private int[] layerY;
	private int[][] weights;
	private Value[][] pictureX;
	private Value[][] pictureY;
	private JLabel lblEnergy;
	private JLabel lblPatternSize;
	private JLabel lblCurrPattern;
	private AWTextField tfNoise;

	private JButton btnGreenX;
	private JButton btnYellowX;
	private JButton btnGreenY;
	private JButton btnYellowY;
	private JButton btnDeleteY;
	private JButton btnNoise;
	private JButton btnStepLeft;
	private JButton btnStepRight;
	
	private JButton btnStep;
	private JButton btnExecute;
	private JButton btnTraining;
	private JButton btnReset;
	private JButton btnOpen;
	private JButton btnSave;
	private JButton btnClose;
	
	private int picture_line;
	private int picture_X_column;
	private int picture_Y_column;
	private int curr_pic;
	
	private ArrayList<int[][]> pattern;
	
	/**
	 * The constructor <CODE>AssociativeMemoryPicture</CODE> creates a new instance(object)
	 * of the class.
	 * @since 0.1
	 */
	public AssociativeMemoryPicture() {
		super("associative memory picture", "0.1", "2017", 480, 740);

		try {

			setResizable(false);
			Globals.amp = this;
			Globals.doAfterInitials();
			Image icon = Globals.t4md.getImagePure("app.jpg");
			setIconImage(icon);
			windowInit();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @exception Exception.
	 * @since 0.1
	 */
	private void windowInit() throws Exception {

		try {
			
			pattern = new ArrayList<int[][]>();
			picture_line = 50;
			picture_X_column = 50;
			picture_Y_column = 400;
			curr_pic = 0;
			int controls_line = 370;

			layerX = new int[100];
			layerY = new int[100];
			weights = new int[100][100];
			
			pictureX = new Value[10][10];
			pictureY = new Value[10][10];

			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 49, screen.width, 20);
			Container con = getContentPane();
			con.setLayout(null);
			con.setSize(new Dimension(screen.width, screen.height));
			con.setBackground(Color.LIGHT_GRAY);
			
			createLabel(con, 10, controls_line, 120, 20, Globals.message.getMessage("msg0012"));
			lblPatternSize = createLabel(con, 140, controls_line, 40, 20, "0");

			createLabel(con, 10, controls_line + 30, 120, 20, Globals.message.getMessage("msg0003"));
			lblEnergy = createLabel(con, 140, controls_line + 30, 40, 20, "0");

			btnGreenX = createButton(con, picture_X_column, 20, 23, 23, Globals.message.getMessage("msg0008") + " X", false, Color.GREEN);
			btnYellowX = createButton(con, picture_X_column + 25, 20, 23, 23, Globals.message.getMessage("msg0009") + " X", false, Color.YELLOW);
			btnNoise = createButton(con, picture_X_column + 50, 20, 23, 23, Globals.message.getMessage("msg0015"), false, null);
			tfNoise = createTextField(con, picture_X_column + 75, 20, 40, 20, 0);
			btnGreenY = createButton(con, picture_Y_column, 20, 23, 23, Globals.message.getMessage("msg0008") + " Y", false, Color.GREEN);
			btnYellowY = createButton(con, picture_Y_column + 25, 20, 23, 23, Globals.message.getMessage("msg0009") + " Y", false, Color.YELLOW);
			btnDeleteY = createButton(con, picture_Y_column + 50, 20, 23, 23, Globals.message.getMessage("msg0007"), false, null);

			btnStepLeft = createButton(con, picture_X_column + 50, 320, 50, 23, "<<", true, null);
			lblCurrPattern = createLabel(con, picture_X_column + 110, 320, 20, 20, "0");
			btnStepRight = createButton(con, picture_X_column + 130, 320, 50, 23, ">>", true, null);
			
			btnStep = createButton(con, 290, controls_line, 100, 23, Globals.message.getMessage("msg0002"), true, null);
			btnExecute = createButton(con, 400, controls_line, 100, 23, Globals.message.getMessage("msg0006"), true, null);
			btnTraining = createButton(con, 510, controls_line, 100, 23, Globals.message.getMessage("msg0005"), true, null);
			btnReset = createButton(con, 620, controls_line, 100, 23, Globals.message.getMessage("msg0004"), true, null);
			
			btnOpen = createButton(con, 400, controls_line + 30, 100, 23, Globals.message.getMessage("msg0010"), true, null);
			btnSave = createButton(con, 510, controls_line + 30, 100, 23, Globals.message.getMessage("msg0011"), true, null);
			btnClose = createButton(con, 620, controls_line + 30, 100, 23, Globals.message.getMessage("msg0001"), true, null);

	        createControls();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}

		setVisible(true);

	}

	/**
	 * The method <CODE>createLabel</CODE> creates a label.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @return a label.
	 * @since 0.1
	 */
	private JLabel createLabel(Container c, int x, int y, int width, int height, String text) {
        JLabel label = new JLabel(text);
		label.setBounds(x, y, width, height);
        c.add(label);
        return label;
	}
	
	/**
	 * The method <CODE>createTextField</CODE> creates a text field.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param value the value of the control.
	 * @return a text field.
	 * @since 0.1
	 */
	private AWTextField createTextField(Container c, int x, int y, int width, int height, int value) {
		AWTextField tf = new AWTextField();
        tf.setBounds(x, y, width, height);
        tf.setFont(Globals.fontStandard11);
        tf.setText(Integer.toString(value));
		c.add(tf);
		return tf;
	}

	/**
	 * The method <CODE>createButton</CODE> creates a button.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @param show true: show the text.
	 * @return a button.
	 * @since 0.1
	 */
	private JButton createButton(Container c, int x, int y, int width, int height, String text, boolean show, Color color) {
		String curr_text = "";
		if (show) {
			curr_text = text;
		}
		JButton btn = new JButton(curr_text);
	    btn.setBounds(x, y, width, height);
	    btn.setFont(GlobalsDialog.fontStandard12);
	    btn.setToolTipText(text);
	    if (color != null) {
	    	btn.setBackground(color);
	    }
	    btn.addActionListener(this);
		c.add(btn);
		return btn;
	}
	
	/**
	 * The method <CODE>createControls</CODE> creates layers of the neural network.
	 * @since 0.1
	 */
	private void createControls() {
		
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pictureX[i][j] = new Value(c, picture_X_column + (25 * j), picture_line + (25 * i), 20, 20);
            	pictureX[i][j].setValue(0);
            }
        }
	        
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pictureY[i][j] = new Value(c, picture_Y_column + (25 * j), picture_line + (25 * i), 20, 20);
            	pictureY[i][j].setValue(0);
            }
        }

	}
	
	/**
	 * The method <CODE>resetControls</CODE> resets layers of the neural network.
	 * @since 0.1
	 */
	private void resetControls() {
		
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pictureX[i][j].setValue(0);
            }
        }
	        
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pictureY[i][j].setValue(0);
            }
        }
        
        for (int ndx=0;ndx<100;ndx++) {
        	layerX[ndx] = 0;
        	layerY[ndx] = 0;
        }

        for (int i=0;i<100;i++) {
            for (int j=0;j<100;j++) {
            	weights[i][j] = 0;
            }
        }
        
	}

	/**
	 * The method <CODE>training</CODE> training of the neural network.
	 * @since 0.1
	 */
	private void training() {
		int ndx = 0;
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	layerX[ndx] = pictureX[i][j].getValue();
            	layerY[ndx] = pictureY[i][j].getValue();
            	ndx++;
            }
        }

        for (int i=0;i<100;i++) {
            for (int j=0;j<100;j++) {
            	weights[i][j] += layerX[i] * layerY[j];
            }
        }
        calcEnergy();
	}
	
	/**
	 * The method <CODE>deleteY</CODE> delete the y layer.
	 * @since 0.1
	 */
	private void deleteY() {
        for (int i=0;i<10;i++) {
	        for (int j=0;j<10;j++) {
	        	pictureY[i][j].setValue(0);
	        }
        }
        for (int i=0;i<100;i++) {
        	layerY[i] = 0;
        }
	}
	
	/**
	 * The method <CODE>execute</CODE> execution of the neural network (step).
	 * @since 0.1
	 */
	private boolean execute() {
		
		int ndx = 0;
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	layerX[ndx] = pictureX[i][j].getValue();
            	ndx++;
            }
        }

        for (int j=0;j<100;j++) {
        	layerY[j] = 0;
        }

        for (int j=0;j<100;j++) {
        	int a = 0;
            for (int i=0;i<100;i++) {
            	a += weights[i][j] * layerX[i];
            }
            layerY[j] = T(a);
        }

        boolean changed = false;
        for (int i=0;i<100;i++) {
        	int a = 0;
            for (int j=0;j<100;j++) {
            	a += weights[i][j] * layerY[j];
            }
            if (layerX[i] != T(a)) {
            	changed = true;
            }
            layerX[i] = T(a);
        }

        ndx = 0;
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pictureX[i][j].setValue(layerX[ndx]);
            	pictureY[i][j].setValue(layerY[ndx]);
            	ndx++;
            }
        }

        calcEnergy();
        
        return changed;
	}
	
	/**
	 * The method <CODE>executeAll</CODE> execute all steps of the neural network.
	 * @since 0.1
	 */
	private void executeAll() {
		while (execute());
	}
	
	/**
	 * The method <CODE>T</CODE> the transfer function.
	 * @param a the activation value.
	 * @since 0.1
	 */
	private int T(int a) {
		if (a > 0) {
			return 1;
		} else {
			return -1;
		}
	}

	/**
	 * The method <CODE>calcEnergy</CODE> calculates the energy.
	 * @since 0.1
	 */
	private void calcEnergy() {
    	int e = 0;
        for (int i=0;i<100;i++) {
            for (int j=0;j<100;j++) {
            	e += weights[i][j] * layerX[i] * layerY[j];
            }
        }
        lblEnergy.setText(Integer.toString(-e));
	}
	
	/**
	 * The method <CODE>colorPicture</CODE> colors the picture.
	 * @param pic the picture array.
	 * @param value the color value.
	 * @since 0.1
	 */
	private void colorPicture(Value[][] pic, int value) {
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pic[i][j].setValue(value);
            }
        }
	}

	/**
	 * The method <CODE>savePicturesAsPatters</CODE> save both pictures as patterns.
	 * @since 0.1
	 */
	private void savePicturesAsPatters() {
		int[][] pic_val = new int[10][10];
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				pic_val[i][j] = pictureX[i][j].getValue();
			}
		}
		pattern.add(pic_val);

		pic_val = new int[10][10];
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				pic_val[i][j] = pictureY[i][j].getValue();
			}
		}
		pattern.add(pic_val);
		
		int len = pattern.size() / 2;
		lblPatternSize.setText(Integer.toString(len));
		curr_pic = len;
		lblCurrPattern.setText(Integer.toString(curr_pic));
	}	
	
	/**
	 * The method <CODE>getPicturesFromPatters</CODE> get both pictures from pattern list.
	 * @since 0.1
	 */
	private void getPicturesFromPatters() {
		int[][] pic_val = pattern.get(curr_pic * 2 - 2);
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				pictureX[i][j].setValue(pic_val[i][j]);
			}
		}

		pic_val = pattern.get(curr_pic * 2 - 1);
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				pictureY[i][j].setValue(pic_val[i][j]);
			}
		}
		
		lblCurrPattern.setText(Integer.toString(curr_pic));
	}
	
	/**
	 * The method <CODE>checkPlausies</CODE> check plausibilities.
	 * @since 0.1
	 */
	private boolean checkPlausies() {
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				if (pictureX[i][j].getValue() == 0 || pictureY[i][j].getValue() == 0) {
	    			Globals.t4d.showErrorDialog(Globals.message.getMessage("msg0013"));
	    			return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 * @param e the action event.
	 * @since 0.1
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			if (e.getSource() == btnDeleteY) {
				deleteY();
			} else if (e.getSource() == btnNoise) {
				ArrayList<String> perclist = new ArrayList<String>();
				double percent = Integer.parseInt(tfNoise.getText());
				if (percent < 0 || percent > 100) {
	    			Globals.t4d.showErrorDialog(Globals.message.getMessage("msg0016"));
	    			return;
				}
				int pixel = (int) (100 * (percent/ 100));
				for (int ndx=0;ndx<pixel;ndx++) {
					int x = 0;
					int y = 0;
					String pair = null;
					do {
						x = (int) (Math.random() * 10);
						y = (int) (Math.random() * 10);
						pair = Integer.toString(x) + Integer.toString(y); 
					} while (perclist.contains(pair));
					perclist.add(pair);
					int val = pictureX[x][y].getValue();
					val *= -1;
					pictureX[x][y].setValue(val);
				}
			} else if (e.getSource() == btnGreenX) {
				colorPicture(pictureX, 1);
			} else if (e.getSource() == btnYellowX) {
				colorPicture(pictureX, -1);
			} else if (e.getSource() == btnGreenY) {
				colorPicture(pictureY, 1);
			} else if (e.getSource() == btnYellowY) {
				colorPicture(pictureY, -1);
			} else if (e.getSource() == btnStepLeft) {
				if (pattern.size() == 0) {
					return;
				}
				curr_pic -= 1;
				if (curr_pic < 1) {
					curr_pic = 1;
				}
				getPicturesFromPatters();
			} else if (e.getSource() == btnStepRight) {
				if (pattern.size() == 0) {
					return;
				}
				curr_pic += 1;
				if (curr_pic > pattern.size() / 2) {
					curr_pic = pattern.size() / 2;
				}
				getPicturesFromPatters();
			} else if (e.getSource() == btnStep) {
				execute();
			} else if (e.getSource() == btnExecute) {
				executeAll();
			} else if (e.getSource() == btnTraining) {
				if (checkPlausies()) {
					savePicturesAsPatters();
					training();
				}
			} else if (e.getSource() == btnReset) {
				pattern = new ArrayList<int[][]>();
				lblPatternSize.setText(Integer.toString(pattern.size()));
				lblCurrPattern.setText(Integer.toString(pattern.size()));
		        lblEnergy.setText("0");
				resetControls();
			} else if (e.getSource() == btnOpen) {
				
		        JFileChooser chooser = null;
		        File curr_dir = new File(Globals.t4f.getFullPathToSystem("data"));
		        if (curr_dir.exists()) {
		            chooser = new JFileChooser(curr_dir);
		        } else {
		            chooser = new JFileChooser(".");
		        }

				String[] extension = new String[1];
				String message = Globals.message.getMessage("msg0014");
				extension[0] = "dat";

		        chooser.setFileFilter(new ExampleFileFilter(extension, message));
		        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				
		        int state = chooser.showOpenDialog(this);
		        if (state == JFileChooser.APPROVE_OPTION) {
		        	
		        	resetControls();
					pattern = new ArrayList<int[][]>();
		        	String path = chooser.getSelectedFile().getPath();
		        	String content = Globals.t4f.getContent(path);
		        	StringTokenizer st = new StringTokenizer(content, "\r\n");
		        	int count = Integer.parseInt(st.nextToken());
		        	while(count > 0) {
		        		int[][] pic = new int[10][10];
		        		for (int i=0;i<10;i++) {
		        			String line = st.nextToken();
		        			for (int j=0;j<10;j++) {
		        				String token = line.substring(j, j+1);
		        				if (token.equals("#")) {
		        					pic[i][j] = 1;
		        				} else if (token.equals("-")) {
		        					pic[i][j] = -1;
		        				}
		        			}
		        		}
		        		pattern.add(pic);
		        		count--;
		        	}
		        	
					int len = pattern.size() / 2;
					lblPatternSize.setText(Integer.toString(len));
					
					// training for all the patterns.
					for (int ndx=0;ndx<len;ndx++) {
						curr_pic = ndx + 1;
						getPicturesFromPatters();
						training();						
					}
					
					curr_pic = 1;
					getPicturesFromPatters();
		        }
			} else if (e.getSource() == btnSave) {
				
		        JFileChooser chooser = null;
		        File curr_dir = new File(Globals.t4f.getFullPathToSystem("data"));
		        if (curr_dir.exists()) {
		            chooser = new JFileChooser(curr_dir);
		        } else {
		            chooser = new JFileChooser(".");
		        }

				String[] extension = new String[1];
				String message = Globals.message.getMessage("msg0014");
				extension[0] = "dat";

		        chooser.setFileFilter(new ExampleFileFilter(extension, message));
				chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				
		        int state = chooser.showSaveDialog(this);
		        if (state == JFileChooser.APPROVE_OPTION) {
		        	String path = chooser.getSelectedFile().getPath();
		        	if (path.indexOf(".") == -1) {
		        		path += "." + extension[0];
		        	}
		        	int len = pattern.size();
		        	String content = Integer.toString(len) + "\r\n";
		        	for (int ndx=0;ndx<len;ndx++) {
		        		int[][] pic = pattern.get(ndx);
		        		for (int i=0;i<10;i++) {
		        			String line = "";
		        			for (int j=0;j<10;j++) {
		        				if (pic[i][j]== 1) {
		        					line += "#";
		        				} else if (pic[i][j]== -1) {
		        					line += "-";
		        				}
		        			}
		        			content += line + "\r\n";
		        		}
	        			content += "\r\n";
		        	}

		        	Globals.t4f.putTheFile(path, "", content, false);
			        
		        }
		        
			} else if (e.getSource() == btnClose) {
				System.exit(0);
			}
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}
	
	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 0.1
	 */
	public static void main(String[] args) {
		try {
			new Globals();
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new AssociativeMemoryPicture();
	}

}
