package n4e.expertanddecision.dbaccess;

/**
* <p>title: n4e application</p>
* <p>description: expertanddecision</p>
* <p>copyright: copyright (c) 2006-2011 by thomas kaffka; mba; all rights reserved.</p>
* <p>company: n.n.</p>
* @author thomas kaffka, mba.
* @version Fri May 01 16:26:19 CEST 2015
*/

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;


/**
 * The <CODE>DbAccessCheckExpAttribute</CODE> class checks fields of ExpAttribute table in database.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version Fri May 01 16:26:19 CEST 2015
 * @since 1.0
*/
public class DbAccessCheckExpAttribute extends DbCheck {
  // technical fields
  public int errCode = 0;
  public String errMess = "";
  private boolean hasdata = false;
  private Tool4Number t4n = new Tool4Number();
  private Tool4DbMessages t4m = null;

  // database standard fields
  private String uniqueid = ""; // unique key
  private long version = 0; // version counter for database store without locking
  private java.util.Date created = null; // date of creation
  private String creator = ""; // user created
  private java.util.Date changed = null; // date of last change data
  private String changedfrom = ""; // user changed
  private int inuse = 0; // read but not updated
  private String client = ""; // foreign key to client table

  // individual database fields
  private String name = ""; // Name
  private String codomain = ""; // Wertebereich
  private String valuetype = ""; // Wertetyp
  private int initialattribute = 0; // Initialattribut
  private String question = ""; // Frage
  private String adddata = ""; // Ergaenzung
  private String description = ""; // Beschreibung
  private ArrayList myDbAccessExpCodomain = new ArrayList() ; // ExpCodomain
  private ArrayList myDbAccessExpCondition = new ArrayList() ; // ExpCondition
  private ArrayList myDbAccessResultAtt = new ArrayList() ; // ResultAtt
  private DbAccessExpSystem myDbAccessExpSystem = new DbAccessExpSystem() ; // ExpSystem


  // sets and gets of standard table fields
  public void setUniqueid(String input){uniqueid = input;}
  public String getUniqueid(){return uniqueid;}
  public void setVersion(long input){version = input;}
  public long getVersion(){return version;}
  public void setCreated(java.util.Date input){created = input;}
  public java.util.Date getCreated(){return created;}
  public void setCreator(String input){creator = input;}
  public String getCreator(){return creator;}
  public void setChanged(java.util.Date input){changed = input;}
  public java.util.Date getChanged(){return changed;}
  public void setChangedfrom(String input){changedfrom = input;}
  public String getChangedfrom(){return changedfrom;}
  public void setInuse(int input){inuse = input;}
  public int getInuse(){return inuse;}
  public void setClient(String input){client = input;}
  public String getClient(){return client;}

  // sets and gets of individual table fields
  public void setName(String input){name = input;}
  public String getName(){return name;}
  public void setCodomain(String input){codomain = input;}
  public String getCodomain(){return codomain;}
  public void setValuetype(String input){valuetype = input;}
  public String getValuetype(){return valuetype;}
  public void setInitialattribute(int input){initialattribute = input;}
  public int getInitialattribute(){return initialattribute;}
  public void setQuestion(String input){question = input;}
  public String getQuestion(){return question;}
  public void setAdddata(String input){adddata = input;}
  public String getAdddata(){return adddata;}
  public void setDescription(String input){description = input;}
  public String getDescription(){return description;}
  public void setMyDbAccessExpCodomain(ArrayList input){myDbAccessExpCodomain = input;}
  public ArrayList getMyDbAccessExpCodomain(){return myDbAccessExpCodomain;}
  public void setMyDbAccessExpCondition(ArrayList input){myDbAccessExpCondition = input;}
  public ArrayList getMyDbAccessExpCondition(){return myDbAccessExpCondition;}
  public void setMyDbAccessResultAtt(ArrayList input){myDbAccessResultAtt = input;}
  public ArrayList getMyDbAccessResultAtt(){return myDbAccessResultAtt;}
  public void setMyDbAccessExpSystem(DbAccessExpSystem input){myDbAccessExpSystem = input;}
  public DbAccessExpSystem getMyDbAccessExpSystem(){return myDbAccessExpSystem;}


  /**
   * The constructor <CODE>DbAccessCheckExpAttribute</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessCheckExpAttribute(Tool4DbMessages pt4m){
    t4m = pt4m;
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens of dataset
   * @param dbaccessclass Class DbAccessCustomer.
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessExpAttribute dbaccessclass , String field_list){
    return checkData(dbaccessclass , false , field_list);
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens
   * @param dbaccessclass Class DbAccess...
   * @param first = true => check standard fields
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessExpAttribute dbaccessclass , boolean first , String field_list){
    // fill standard fields
    setUniqueid(dbaccessclass.getUniqueid());
    setVersion(dbaccessclass.getVersion());
    setCreated(dbaccessclass.getCreated());
    setCreator(dbaccessclass.getCreator());
    setChanged(dbaccessclass.getChanged());
    setChangedfrom(dbaccessclass.getChangedfrom());
    setInuse(dbaccessclass.getInuse());
    setClient(dbaccessclass.getClient());
    hasdata = true;

    // fill individual fields
    setName(dbaccessclass.getName());
    setCodomain(dbaccessclass.getCodomain());
    setValuetype(dbaccessclass.getValuetype());
    setInitialattribute(dbaccessclass.getInitialattribute());
    setQuestion(dbaccessclass.getQuestion());
    setAdddata(dbaccessclass.getAdddata());
    setDescription(dbaccessclass.getDescription());
    setMyDbAccessExpCodomain(dbaccessclass.getMyDbAccessExpCodomain());
    setMyDbAccessExpCondition(dbaccessclass.getMyDbAccessExpCondition());
    setMyDbAccessResultAtt(dbaccessclass.getMyDbAccessResultAtt());
    setMyDbAccessExpSystem(dbaccessclass.getMyDbAccessExpSystem());


    try{
      // check standard fields
      if (!first) {
        if (getUniqueid() == null || (getUniqueid() != null && getUniqueid().length() == 0)) {
           errCode = ERROR_0009;
           errMess = t4m.getMessage("msg0009", null);
           return false;
        }
        if (getUniqueid() != null && getUniqueid().length() > 35) {
           errCode = ERROR_0010;
           errMess = t4m.getMessage("msg0010", null);
           return false;
        }
        if (getClient() == null || (getClient() != null && getClient().length() == 0)) {
           errCode = ERROR_0011;
           errMess = t4m.getMessage("msg0011", null);;
           return false;
        }
        if (getClient() != null && getClient().length() > 35) {
           errCode = ERROR_0012;
           errMess = t4m.getMessage("msg0012", null);
           return false;
        }
      }
      if (getCreator() != null && getCreator().length() > 35) {
        errCode = ERROR_0013;
        errMess = t4m.getMessage("msg0013", null);
        return false;
      }
      if (getChangedfrom() != null && getChangedfrom().length() > 35) {
        errCode = ERROR_0014;
        errMess = t4m.getMessage("msg0014", null);
        return false;
      }

      // check individual fields
      if ((first) && (getName() == null || (getName() != null && getName().length() == 0))) {
        errCode = ERROR_7_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("name" , "label"));
        return false;
      } else if ((field_list.indexOf(",name,")!=-1) && (getName() == null || (getName() != null && getName().length() == 0))) {
        errCode = ERROR_8_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("name" , "label"));
        return false;
      }
      if (getName() != null && (getName().length() > 30)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name" , "label") + "|30");
        return false;
      }
      if ((first) && (getCodomain() == null || (getCodomain() != null && getCodomain().length() == 0))) {
        errCode = ERROR_12_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("codomain" , "label"));
        return false;
      } else if ((field_list.indexOf(",codomain,")!=-1) && (getCodomain() == null || (getCodomain() != null && getCodomain().length() == 0) || (getCodomain() != null && getCodomain().equals("n4eempty")))) {
        errCode = ERROR_13_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("codomain" , "label"));
        return false;
      }
      if (getCodomain() != null && (getCodomain().length() > 35)) {
        errCode = ERROR_14_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("codomain" , "label") + "|35");
        return false;
      }
      if ((first) && (getValuetype() == null || (getValuetype() != null && getValuetype().length() == 0))) {
        errCode = ERROR_12_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("valuetype" , "label"));
        return false;
      } else if ((field_list.indexOf(",valuetype,")!=-1) && (getValuetype() == null || (getValuetype() != null && getValuetype().length() == 0) || (getValuetype() != null && getValuetype().equals("n4eempty")))) {
        errCode = ERROR_13_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("valuetype" , "label"));
        return false;
      }
      if (getValuetype() != null && (getValuetype().length() > 35)) {
        errCode = ERROR_14_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("valuetype" , "label") + "|35");
        return false;
      }
      if (getQuestion() != null && (getQuestion().length() > 300)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("question" , "label") + "|300");
        return false;
      }
      if (getAdddata() != null && (getAdddata().length() > 300)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("adddata" , "label") + "|300");
        return false;
      }
      if (getDescription() != null && (getDescription().length() > 500)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description" , "label") + "|500");
        return false;
      }

    } catch (Exception e) {
      System.err.println(e.getMessage());
      errCode = ERROR_Ex;
      errMess = e.getMessage();
    }
    return true;
  }
}
