package n4e.expertanddecision.dbaccess;

/**
* <p>title: n4e application</p>
* <p>description: expertanddecision</p>
* <p>copyright: copyright (c) 2006-2011 by thomas kaffka; mba; all rights reserved.</p>
* <p>company: n.n.</p>
* @author thomas kaffka, mba.
* @version Fri May 01 16:26:19 CEST 2015
*/

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;


/**
 * The <CODE>DbAccessCheckKeySet</CODE> class checks fields of KeySet table in database.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version Fri May 01 16:26:19 CEST 2015
 * @since 1.0
*/
public class DbAccessCheckKeySet extends DbCheck {
  // technical fields
  public int errCode = 0;
  public String errMess = "";
  private boolean hasdata = false;
  private Tool4Number t4n = new Tool4Number();
  private Tool4DbMessages t4m = null;

  // database standard fields
  private String uniqueid = ""; // unique key
  private long version = 0; // version counter for database store without locking
  private java.util.Date created = null; // date of creation
  private String creator = ""; // user created
  private java.util.Date changed = null; // date of last change data
  private String changedfrom = ""; // user changed
  private int inuse = 0; // read but not updated
  private String client = ""; // foreign key to client table

  // individual database fields
  private String name = ""; // Schluesselname
  private String description = ""; // Beschreibung
  private String description2 = ""; // Beschreibung
  private String description3 = ""; // Beschreibung
  private String description4 = ""; // Beschreibung
  private String description5 = ""; // Beschreibung
  private int showatclients = 0; // Keyset fuer uebrige Clients
  private String label1 = ""; // Bezeichnung Parameter 1
  private String label2 = ""; // Bezeichnung Parameter 2
  private String label3 = ""; // Bezeichnung Parameter 3
  private String label4 = ""; // Bezeichnung Parameter 4
  private String label5 = ""; // Bezeichnung Parameter 5
  private String explanation1 = ""; // Erklaerung Parameter 1
  private String explanation2 = ""; // Erklaerung Parameter 2
  private String explanation3 = ""; // Erklaerung Parameter 3
  private String explanation4 = ""; // Erklaerung Parameter 4
  private String explanation5 = ""; // Erklaerung Parameter 5
  private ArrayList myDbAccessKeyWord = new ArrayList() ; // KeyWord


  // sets and gets of standard table fields
  public void setUniqueid(String input){uniqueid = input;}
  public String getUniqueid(){return uniqueid;}
  public void setVersion(long input){version = input;}
  public long getVersion(){return version;}
  public void setCreated(java.util.Date input){created = input;}
  public java.util.Date getCreated(){return created;}
  public void setCreator(String input){creator = input;}
  public String getCreator(){return creator;}
  public void setChanged(java.util.Date input){changed = input;}
  public java.util.Date getChanged(){return changed;}
  public void setChangedfrom(String input){changedfrom = input;}
  public String getChangedfrom(){return changedfrom;}
  public void setInuse(int input){inuse = input;}
  public int getInuse(){return inuse;}
  public void setClient(String input){client = input;}
  public String getClient(){return client;}

  // sets and gets of individual table fields
  public void setName(String input){name = input;}
  public String getName(){return name;}
  public void setDescription(String input){description = input;}
  public String getDescription(){return description;}
  public void setDescription2(String input){description2 = input;}
  public String getDescription2(){return description2;}
  public void setDescription3(String input){description3 = input;}
  public String getDescription3(){return description3;}
  public void setDescription4(String input){description4 = input;}
  public String getDescription4(){return description4;}
  public void setDescription5(String input){description5 = input;}
  public String getDescription5(){return description5;}
  public void setShowatclients(int input){showatclients = input;}
  public int getShowatclients(){return showatclients;}
  public void setLabel1(String input){label1 = input;}
  public String getLabel1(){return label1;}
  public void setLabel2(String input){label2 = input;}
  public String getLabel2(){return label2;}
  public void setLabel3(String input){label3 = input;}
  public String getLabel3(){return label3;}
  public void setLabel4(String input){label4 = input;}
  public String getLabel4(){return label4;}
  public void setLabel5(String input){label5 = input;}
  public String getLabel5(){return label5;}
  public void setExplanation1(String input){explanation1 = input;}
  public String getExplanation1(){return explanation1;}
  public void setExplanation2(String input){explanation2 = input;}
  public String getExplanation2(){return explanation2;}
  public void setExplanation3(String input){explanation3 = input;}
  public String getExplanation3(){return explanation3;}
  public void setExplanation4(String input){explanation4 = input;}
  public String getExplanation4(){return explanation4;}
  public void setExplanation5(String input){explanation5 = input;}
  public String getExplanation5(){return explanation5;}
  public void setMyDbAccessKeyWord(ArrayList input){myDbAccessKeyWord = input;}
  public ArrayList getMyDbAccessKeyWord(){return myDbAccessKeyWord;}


  /**
   * The constructor <CODE>DbAccessCheckKeySet</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessCheckKeySet(Tool4DbMessages pt4m){
    t4m = pt4m;
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens of dataset
   * @param dbaccessclass Class DbAccessCustomer.
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessKeySet dbaccessclass , String field_list){
    return checkData(dbaccessclass , false , field_list);
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens
   * @param dbaccessclass Class DbAccess...
   * @param first = true => check standard fields
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessKeySet dbaccessclass , boolean first , String field_list){
    // fill standard fields
    setUniqueid(dbaccessclass.getUniqueid());
    setVersion(dbaccessclass.getVersion());
    setCreated(dbaccessclass.getCreated());
    setCreator(dbaccessclass.getCreator());
    setChanged(dbaccessclass.getChanged());
    setChangedfrom(dbaccessclass.getChangedfrom());
    setInuse(dbaccessclass.getInuse());
    setClient(dbaccessclass.getClient());
    hasdata = true;

    // fill individual fields
    setName(dbaccessclass.getName());
    setDescription(dbaccessclass.getDescription());
    setDescription2(dbaccessclass.getDescription2());
    setDescription3(dbaccessclass.getDescription3());
    setDescription4(dbaccessclass.getDescription4());
    setDescription5(dbaccessclass.getDescription5());
    setShowatclients(dbaccessclass.getShowatclients());
    setLabel1(dbaccessclass.getLabel1());
    setLabel2(dbaccessclass.getLabel2());
    setLabel3(dbaccessclass.getLabel3());
    setLabel4(dbaccessclass.getLabel4());
    setLabel5(dbaccessclass.getLabel5());
    setExplanation1(dbaccessclass.getExplanation1());
    setExplanation2(dbaccessclass.getExplanation2());
    setExplanation3(dbaccessclass.getExplanation3());
    setExplanation4(dbaccessclass.getExplanation4());
    setExplanation5(dbaccessclass.getExplanation5());
    setMyDbAccessKeyWord(dbaccessclass.getMyDbAccessKeyWord());


    try{
      // check standard fields
      if (!first) {
        if (getUniqueid() == null || (getUniqueid() != null && getUniqueid().length() == 0)) {
           errCode = ERROR_0009;
           errMess = t4m.getMessage("msg0009", null);
           return false;
        }
        if (getUniqueid() != null && getUniqueid().length() > 35) {
           errCode = ERROR_0010;
           errMess = t4m.getMessage("msg0010", null);
           return false;
        }
        if (getClient() == null || (getClient() != null && getClient().length() == 0)) {
           errCode = ERROR_0011;
           errMess = t4m.getMessage("msg0011", null);;
           return false;
        }
        if (getClient() != null && getClient().length() > 35) {
           errCode = ERROR_0012;
           errMess = t4m.getMessage("msg0012", null);
           return false;
        }
      }
      if (getCreator() != null && getCreator().length() > 35) {
        errCode = ERROR_0013;
        errMess = t4m.getMessage("msg0013", null);
        return false;
      }
      if (getChangedfrom() != null && getChangedfrom().length() > 35) {
        errCode = ERROR_0014;
        errMess = t4m.getMessage("msg0014", null);
        return false;
      }

      // check individual fields
      if ((first) && (getName() == null || (getName() != null && getName().length() == 0))) {
        errCode = ERROR_7_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("name" , "label"));
        return false;
      } else if ((field_list.indexOf(",name,")!=-1) && (getName() == null || (getName() != null && getName().length() == 0))) {
        errCode = ERROR_8_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("name" , "label"));
        return false;
      }
      if (getName() != null && (getName().length() > 32)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name" , "label") + "|32");
        return false;
      }
      if (getDescription() != null && (getDescription().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description" , "label") + "|100");
        return false;
      }
      if (getDescription2() != null && (getDescription2().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description2" , "label") + "|100");
        return false;
      }
      if (getDescription3() != null && (getDescription3().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description3" , "label") + "|100");
        return false;
      }
      if (getDescription4() != null && (getDescription4().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description4" , "label") + "|100");
        return false;
      }
      if (getDescription5() != null && (getDescription5().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description5" , "label") + "|100");
        return false;
      }
      if (getLabel1() != null && (getLabel1().length() > 30)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("label1" , "label") + "|30");
        return false;
      }
      if (getLabel2() != null && (getLabel2().length() > 30)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("label2" , "label") + "|30");
        return false;
      }
      if (getLabel3() != null && (getLabel3().length() > 30)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("label3" , "label") + "|30");
        return false;
      }
      if (getLabel4() != null && (getLabel4().length() > 30)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("label4" , "label") + "|30");
        return false;
      }
      if (getLabel5() != null && (getLabel5().length() > 30)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("label5" , "label") + "|30");
        return false;
      }
      if (getExplanation1() != null && (getExplanation1().length() > 255)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("explanation1" , "label") + "|255");
        return false;
      }
      if (getExplanation2() != null && (getExplanation2().length() > 255)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("explanation2" , "label") + "|255");
        return false;
      }
      if (getExplanation3() != null && (getExplanation3().length() > 255)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("explanation3" , "label") + "|255");
        return false;
      }
      if (getExplanation4() != null && (getExplanation4().length() > 255)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("explanation4" , "label") + "|255");
        return false;
      }
      if (getExplanation5() != null && (getExplanation5().length() > 255)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("explanation5" , "label") + "|255");
        return false;
      }

    } catch (Exception e) {
      System.err.println(e.getMessage());
      errCode = ERROR_Ex;
      errMess = e.getMessage();
    }
    return true;
  }
}
