package n4e.expertanddecision.dbaccess;

/**
* <p>title: n4e application</p>
* <p>description: expertanddecision</p>
* <p>copyright: copyright (c) 2006-2011 by thomas kaffka; mba; all rights reserved.</p>
* <p>company: n.n.</p>
* @author thomas kaffka, mba.
* @version Fri May 01 16:26:19 CEST 2015
*/

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;
import java.rmi.server.UID;
import java.util.zip.*;
import java.io.*;
import java.nio.*;
import java.security.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;
import n4e.core.list.*;



/**
 * The <CODE>DbAccessProject</CODE> class reads fields and put to data sink of Project table in database.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version Fri May 01 16:26:19 CEST 2015
 * @since 1.0
*/
public class DbAccessProject extends DbAccess {



    // individual database fields
  private String name = ""; // Name
  private String old_name = ""; // (old value of) Name
  private java.util.Date projdate = null; // Datum
  private java.util.Date old_projdate = null; // (old value of) Datum
  private String remarks = ""; // Bemerkungen
  private String old_remarks = ""; // (old value of) Bemerkungen
  private DbAccessExpSystem myDbAccessExpSystem = null ; // ExpSystem
  private DbAccessExpSystem old_myDbAccessExpSystem = null ; // (old value of) ExpSystem
  private ArrayList myDbAccessResultAtt = null ; // ResultAtt
  private ArrayList old_myDbAccessResultAtt = null ; // (old value of) ResultAtt


  // sets and gets of individual table fields
  public void setName(String input){name = input;}
  public String getName(){return name;}
  public void setProjdate(java.util.Date input){projdate = input;}
  public java.util.Date getProjdate(){return projdate;}
  public void setRemarks(String input){remarks = input;}
  public String getRemarks(){return remarks;}
  public void setMyDbAccessExpSystem(DbAccessExpSystem input){myDbAccessExpSystem = input;}
  public DbAccessExpSystem getMyDbAccessExpSystem(){return myDbAccessExpSystem;}
  public void setMyDbAccessResultAtt(ArrayList input){myDbAccessResultAtt = input;}
  public ArrayList getMyDbAccessResultAtt(){return myDbAccessResultAtt;}


  /**
   * The constructor <CODE>DbAccessProject</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessProject(){
      setWorkFields();
  }

  /**
   * The constructor <CODE>DbAccessProject</CODE> initialized the object.
   * @param sysUser the system user.
   * @since 1.0
  */
  public DbAccessProject(List4SysUser sysUser){
    setSystemUser(sysUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }
  }

  /**
   * The constructor <CODE>DbAccessProject</CODE> initialized the object.
   * @param pUser the user object.
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception
   * @since 1.0
  */
  public DbAccessProject(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    initiate(pUser, uniqueid , fieldlist , privfkey , publfkey , null, true);

  }

  /**
   * The constructor <CODE>DbAccessProject</CODE> initialized the object.
   * @param conn the database connection.
   * @param pUser the user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception
   * @since 1.0
  */
  public DbAccessProject(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , null, true);

  }

  /**
   * The constructor <CODE>DbAccessProject</CODE> initialized the object with additional sqlwhere.
   * @param pUser the user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception
   * @since 1.0
  */
  public DbAccessProject(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd) throws Exception {

    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd, true);

  }

  /**
   * The constructor <CODE>DbAccessProject</CODE> initialized the object with additional sqlwhere.
   * @param conn the database connection.
   * @param pUser the user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception
   * @since 1.0
  */
  public DbAccessProject(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd, true);

  }

  /**
   * The method <CODE>initiate</CODE> initializes the object with additional sqlwhere.
   * @param pUser the user object.
   * @param uniqueid the unique key.
   * @param fieldlist List of fields to load.
   * @param privfkey List of privfkeys to load.
   * @param publfkey List of publfkeys to load.
   * @param sqlAdd additional where clause.
   * @param read_with_client read with use of the clientid.
   * @exception Exception.
   * @since 1.0
  */
  public void initiate(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd , boolean read_with_client) throws Exception {

    this.read_with_client = read_with_client;
    setSystemUser(pUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    publicforeignkeytable.put("idresultatt","ResultAtt|idproject");

    privateforeignkeytable.put("idexpsystem","ExpSystem|null");



    success = loadFromTable(uniqueid , fieldlist , privfkey , publfkey , sqlAdd);
  }

  /**
   * The method <CODE>setWorkFields</CODE> sets the work fiels to current values.
   * @since 1.0
  */
  public void setWorkFields() {
      dbtablename = "Project"; // name of own database table
      if (sysUser != null) {
          databasename = sysUser.getClientDbsystemforeclient(); // name of own database
          myDatabaseConnectionSet = DatabaseManager.getDatabaseConnectionSet(databasename);
          setProtocol(dbtablename);
      }
      tblnameofprivk = ",ExpSystem,"; // list of table names of private foreign keys
      tblnameofpublvk = ",ResultAtt,"; // list of table names of public foreign keys
      keynameofpublvk = ",idproject,"; // list of key names of table names of
      tblnameofownerfield = ","; // list of names of table names of public foreign keys owner key
      indivTableFields = ",name,projdate,remarks,"; // list of inidividual table fields
      privateforeignkey = ";idexpsystem;"; // list of private foreign key fields
      publicforeignkey = ";idresultatt;"; // list of public foreign key fields
  }

  /**
   * The method <CODE>loadFromTable</CODE> gets tabledata from database with fieldcheck.
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean loadFromTable(String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    errCode = 0;
    errMess = "";

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    hasdata = false;
    // create connection to defined database system
    if (this.myconn) {
        this.conn = myDatabaseConnectionSet.conn();
    }
    DatabaseSQLResultSet resSet = new DatabaseSQLResultSet(databasename , conn);



    // handle fields to load
    String sqlFieldlist = "";
    String sqlPrivfkey = "";

    // build sql field list
    if (fieldlist != null && fieldlist.equals("all")) {
      sqlFieldlist = indivTableFields.substring(1);
    } else { // make individual fieldlist
      if (fieldlist != null) {
        sqlFieldlist = fieldlist;
      } else {
        sqlFieldlist = "";
      }
    }

    // build sql field list of private foreign keys
    if (privfkey != null && privfkey.equals("all")) {
      sqlPrivfkey = privateforeignkey.substring(1);
    } else { // make individual fieldlist
      if (privfkey != null) {
          int pos = privfkey.indexOf("(");
          if (pos >= 0) {
            StringTokenizer stok = new StringTokenizer(privfkey , ";"); // public foreign keys separated by ;
            while (stok.hasMoreTokens()) { // for each public foreign key
                String xfield = stok.nextToken();
                pos = xfield.indexOf("(");
                if (pos >= 0) {
                    sqlPrivfkey += "," + xfield.substring(0, pos);
                } else {
                    sqlPrivfkey += "," + xfield;
                }
            }
            sqlPrivfkey = sqlPrivfkey.substring(1);
          } else {
            sqlPrivfkey = t4st.replaceToken(privfkey , ";" , ",");
          }
      } else {
        sqlPrivfkey = "";
      }
    }

    StringTokenizer stok = null;

    String workingSqlFieldlist = stdTableFields + sqlFieldlist;
    if (workingSqlFieldlist.endsWith(",")) {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?sqlPrivfkey:"")+",";
    } else {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?","+sqlPrivfkey:"")+",";
    }


    int len = workingSqlFieldlist.length();
    workingSqlFieldlist = workingSqlFieldlist.substring(1,len-1);
    if (workingSqlFieldlist.endsWith(",")) {
      len = workingSqlFieldlist.length();
      workingSqlFieldlist = workingSqlFieldlist.substring(0,len-1);
    }

    resetFields();

    // grip into data sink
    if (uniqueid == null && sqlAdd == null) {
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
        errCode = ERROR_0001;
        errMess = t4m.getMessage("msg0001", null);
        return false;
    }

    String client_temp = client;
    if (!read_with_client) {
        client = null;
    }

    String where1 = ((client!=null&&client.length()>0)?("client='"+client+"'"):"");
    String where2 = ((uniqueid!=null&&uniqueid.length()>0)?("uniqueid='"+ uniqueid +"'"):"");
    String where3 = ((sqlAdd!=null&&sqlAdd.length()>0)?sqlAdd:"");
    String where = "";
    if (where1.length()==0 && where2.length()==0 && where3.length()==0) { // 000
      where = "";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()==0) { // 100
      where = " WHERE (" + where1 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()==0) { // 010
      where = " WHERE (" + where2 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()==0) { // 110
      where = " WHERE (" + where1 + ") AND (" + where2 + ")";
    } else if (where1.length()==0 && where2.length()==0 && where3.length()>0) { // 001
      where = " WHERE (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()>0) { // 101
      where = " WHERE (" + where1 + ") AND (" + where3 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()>0) { // 011
      where = " WHERE (" + where2 + ") AND (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()>0) { // 111
      where = " WHERE (" + where1 + ") AND (" + where2 + ") AND (" + where3 + ")";
    }

    String sqlStatement = "SELECT " + workingSqlFieldlist + " FROM Project " + where;
    conn.connectionlog("DbAccessProject","sql statement", sqlStatement);
    ResultSet result = resSet.getResultSet(sqlStatement);
    if (result.next() && (result.isFirst() && result.isLast())) {
      hasdata = true;
      // get standard fields
      setUniqueid(result.getString(1));
      setVersion(result.getLong(2));
      long temp_timestamp = result.getLong(3);
      java.util.Date temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setCreated(temp_date);
      setCreator(result.getString(4));
      temp_timestamp = result.getLong(5);
      temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setChanged(temp_date);
      setChangedfrom(result.getString(6));
      setInuse(result.getInt(7));
      setClient(result.getString(8));

      //get individual fields
      int i = 9;
      workingSqlFieldlist = sqlFieldlist + (sqlPrivfkey.length()>0?","+sqlPrivfkey:""); // only individuals.
      stok = new StringTokenizer(workingSqlFieldlist ,",");
      while (stok.hasMoreTokens()) {
        String xfield = stok.nextToken();
        if (false) {}
          // dummy
          else if (xfield.equals("name")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setName(value);old_name = value;}
          else if (xfield.equals("projdate")) {java.util.Date value = result.getDate(i);setProjdate(value);old_projdate = value;}
          else if (xfield.equals("remarks")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setRemarks(value);old_remarks = value;}
          else if (xfield.equals("idexpsystem")) {}

        i++;
      }

      //get individual privateforeignkeys
      if (privfkey != null && privfkey.length()>0) {
        String pffieldlist = "";
        if (privfkey.equals("all")) { // load all private foreign key objects with field list 'all'
          privfkey = privateforeignkey.substring(1);
        }
        stok = new StringTokenizer(privfkey , ";"); // private foreign keys separated by ;
        while (stok.hasMoreTokens()) { // for each private foreign key
          String xfield = stok.nextToken();
          if (xfield == null || xfield.length() == 0) continue;
          if (xfield.indexOf("(") > -1) { // private foreign key with field list
            StringTokenizer stfield = new StringTokenizer(xfield ,"(");
            if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (pffieldlist.endsWith(")")) {
              pffieldlist = pffieldlist.substring(0, pffieldlist.length()-1);
            }
          } else {
            pffieldlist = "all";
          }
          if (false) {}
            else if (xfield.equals("idexpsystem")) {
              DbAccessExpSystem tempIdexpsystem = new DbAccessExpSystem(conn , sysUser , result.getString("idexpsystem") , pffieldlist , null , null);
              if (tempIdexpsystem.errCode == -17) {
                setMyDbAccessExpSystem(null);
                old_myDbAccessExpSystem = null;
              } else if (tempIdexpsystem.errCode == 0) {
                setMyDbAccessExpSystem(tempIdexpsystem);
                old_myDbAccessExpSystem = tempIdexpsystem;
              } else  if (tempIdexpsystem.errCode != ERROR_0001) {
                errCode = tempIdexpsystem.errCode;
                errMess = tempIdexpsystem.errMess;
              }
            }

          }
        }

     } else { // no data found
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
       errCode = ERROR_0002;
       errMess = t4m.getMessage("msg0002", null);
       return false;
     }

    //get individual publicforeignkeys
    if (publfkey != null && publfkey.length() > 0) {
      String pffieldlist = "";
      if (publfkey.equals("all")) { // load all public foreign key objects with field list 'all'
        publfkey = publicforeignkey.substring(1);
      }
      DatabaseDatasetLoader dsl = new DatabaseDatasetLoader(this.conn);
      stok = new StringTokenizer(publfkey , ";"); // public foreign keys separated by ;
       while (stok.hasMoreTokens()) { // for each public foreign key
        String xfield = stok.nextToken();
        if (xfield == null || xfield.length() == 0) continue;
        StringTokenizer stfield = null;
        if (xfield.indexOf("(") > -1) { // public foreign key with field list
          stfield = new StringTokenizer(xfield , "(");
          if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
          if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
          if (pffieldlist.endsWith(")")) {
            pffieldlist = pffieldlist.substring(0, pffieldlist.length() - 1);
          }
        } else {
          pffieldlist = "all";
        }
        String orderby = null;
        if (pffieldlist.indexOf("|") > -1) { // order by feilds available.
            stfield = new StringTokenizer(pffieldlist ,"|");
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (stfield.hasMoreTokens()) orderby = stfield.nextToken();
        }
        String pfTableName = (String) publicforeignkeytable.get(xfield);
        String pfkeyfield = "";
        StringTokenizer staccess = new StringTokenizer(pfTableName , "|");
        if (staccess.hasMoreTokens()) pfTableName = staccess.nextToken();
        if (staccess.hasMoreTokens()) pfkeyfield = staccess.nextToken();
        String sql = pfkeyfield + " = '" + result.getString("uniqueid") + "'";
        ArrayList keyList = dsl.getDataSet(sysUser , pfTableName , pffieldlist , null , null , sql , orderby , "expertanddecision");
        if (false) {}
          else if (xfield.equals("idresultatt")) {
            setMyDbAccessResultAtt(keyList);
          }

      }
    }

    if (!read_with_client) {
        client = client_temp;
    }

    resSet.closeResultSet();
    resSet = null;
    if (this.myconn) {
        myDatabaseConnectionSet.connClose(conn);
    }



    return true;
  }

  /**
   * The method <CODE>putToSink</CODE> stores the data to database (data sink).
   * @param conn Connection to Database, if null => make own connection
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean putToSink(DatabaseConnection conn) throws Exception {
    boolean ok = true;

    // get connection
    boolean ownconn = false;
    if (conn == null) {
      conn = myDatabaseConnectionSet.conn();
      conn.setAutoCommit(false);
      ownconn = true;
    }

    initializeStandardFields();

    try {
        if (hasdata) {
          ok = putCurrentDataToSink(conn);
        } else {
          ok = putNewDataToSink(conn);
        }
    } catch (Exception ex) {
        String message = ex.getMessage();
        if (message != null) {
            // check if duplicate data in an unique key.
            // 'duplicate' unique index of mysql.
            // 'unique constraint or index violation' unique index of hsqldb.
            if (message.toLowerCase().indexOf("duplicate") >= 0 ||
                 message.toLowerCase().indexOf("unique constraint or index violation") >= 0) {
              errCode = ERROR_0104;
              errMess = t4m.getMessage("msg0104", null);
              ok = false;
            } else {
                conn.getDatabaseSystem().writeLine2Log(Tool4Throwable.getCallStackOut(ex));
                conn.getDatabaseSystem().flushLog();
                conn.rollback();
                System.out.println(Tool4Throwable.getMessage4Log("see log of database system: " + conn.getDatabaseSystem().getSystemName()));
                throw new Exception(ex);
            }
        } else {
            conn.getDatabaseSystem().writeLine2Log(Tool4Throwable.getCallStackOut(ex));
            conn.getDatabaseSystem().flushLog();
            conn.rollback();
            System.out.println(Tool4Throwable.getMessage4Log("see log of database system: " + conn.getDatabaseSystem().getSystemName()));
            throw new Exception(ex);
        }
    }

    if (ownconn) {
      if (ok) {
        conn.commit();
      } else {
        conn.rollback();
      }
      myDatabaseConnectionSet.connClose(conn);
    }

    return ok;
  }

  /**
   * The method <CODE>putNewDataToSink</CODE> executes an sql insert.
   * @param conn the database connection.
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean putNewDataToSink(DatabaseConnection conn) throws Exception {
    return putNewDataToSink(conn , new Boolean(true));
  }

  /**
   * The method <CODE>putNewDataToSink</CODE> executes an sql insert.
   * @param conn the database connection.
   * @param dotrigger, do the trigger.
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean putNewDataToSink(DatabaseConnection conn , Boolean dotrigger) throws Exception {
    boolean ok = true;



    // identify fields to insert
    String insertFieldlist = "";
    String insertValuelist = "";
    if (name != null && name.length()>0) {
        insertFieldlist += ",name";
        insertValuelist += ",'"+t4st.replaceCharsWithSQL(name)+"'";
    }
    if (projdate != null) {
        insertFieldlist += ",projdate";
        insertValuelist += ",?";
    }
    if (remarks != null && remarks.length()>0) {
        insertFieldlist += ",remarks";
        insertValuelist += ",'"+t4st.replaceCharsWithSQL(remarks)+"'";
    }
    if (myDbAccessExpSystem != null) {
        insertFieldlist += ",idexpsystem";
        insertValuelist += ",'"+myDbAccessExpSystem.getUniqueid()+"'";
    }
    if (myDbAccessResultAtt != null) {
      int len = myDbAccessResultAtt.size();
      for (int i=0;i<len;i++) {
        DbAccessResultAtt updateClass = (DbAccessResultAtt) myDbAccessResultAtt.get(i);
        updateClass.setMyDbAccessProject(this);
        updateClass.putToSink(conn);
      }
    }


    // test before store
    DbAccessCheckProject test = new DbAccessCheckProject(t4m);
    if (!test.checkData(this , true , insertFieldlist + ",")) {
      errCode = test.errCode;
      errMess = test.errMess;
      return false;
    }

    String sqlTest = insertFieldlist + ",";
    String temp_changed = "null";
    java.util.Date temp_changed_date = getChanged();
    if (temp_changed_date != null) {
        temp_changed = Long.toString(temp_changed_date.getTime());
    }
    String temp_changedfrom = getChangedfrom();
    if (temp_changedfrom == null) {
        temp_changedfrom = "null";
    } else {
        temp_changedfrom = "'" + temp_changedfrom + "'";
    }

    if (insertFieldlist.length() > 0) {
      String insertFields = "uniqueid,version,created,creator,changed,changedfrom,inuse,client" + (insertFieldlist.length()>0?insertFieldlist:"");
      String valueFields = "'"+ uniqueid +"',"+Long.toString(version)+","+Long.toString(created.getTime())+",'"+ creator +"',"+temp_changed+","+temp_changedfrom+","+Integer.toString(inuse)+",'"+getClient()+"'"+ (insertValuelist.length()>0?insertValuelist:"");
      String sqlStatement = "INSERT INTO Project ("+insertFields+") VALUES ("+valueFields+")";
      conn.connectionlog("DbAccessProject","sql statement", sqlStatement);
      setKeyWordsAsUsed(conn);
      PreparedStatement ps = conn.prepareStatement(sqlStatement);
      int psndx = 0;
      if (sqlTest.indexOf(",projdate,") > -1) {
        psndx++;
        String sdate = t4d.getDate2SqlString(projdate);
        java.sql.Date ddate = null;
        if (sdate != null && sdate.length() > 0) {
          ddate = java.sql.Date.valueOf(sdate);
        }
        ps.setDate(psndx  , ddate);
      }

      ps.execute();
      ps.close();
    }



    if (ok) {
        errCode = MSG_0006;
        errMess = t4m.getMessage("msg0006", null);
    }
    return ok;
  }

  /**
   * The method <CODE>putCurrentDataToSink</CODE> executes an sql update.
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean putCurrentDataToSink(DatabaseConnection conn) throws Exception {
    boolean ok = true;



    // identify fields to update
    String sqlStatement = "UPDATE Project SET version="+Long.toString(version)+",changed="+Long.toString(changed.getTime())+",changedfrom='"+changedfrom +"',inuse="+Integer.toString(inuse)+"";
    String updateFieldlist = "";
    if (hasChanged("name")) {updateFieldlist += ",name"; sqlStatement += (name==null?",name=null":",name='"+t4st.replaceCharsWithSQL(name)+"'");}
    if (hasChanged("projdate")) {updateFieldlist += ",projdate"; sqlStatement +=",projdate=?";}
    if (hasChanged("remarks")) {updateFieldlist += ",remarks"; sqlStatement += (remarks==null?",remarks=null":",remarks='"+t4st.replaceCharsWithSQL(remarks)+"'");}
    if (hasChanged("idexpsystem")) {updateFieldlist += ",idexpsystem";sqlStatement+=",idexpsystem="+(myDbAccessExpSystem==null?"null":"'"+myDbAccessExpSystem.getUniqueid()+"'");}
    if (myDbAccessResultAtt != null) {
      int len = myDbAccessResultAtt.size();
      for (int i=0;i<len;i++) {
        DbAccessResultAtt updateClass = (DbAccessResultAtt) myDbAccessResultAtt.get(i);
        updateClass.putToSink(conn);
      }
    }


    // test before store
    DbAccessCheckProject test = new DbAccessCheckProject(t4m);
    if (!test.checkData(this , false , updateFieldlist + ",")) {
      errCode = test.errCode;
      errMess = test.errMess;
      return false;
    }

    if (updateFieldlist.length() > 0) {
      sqlStatement+=" WHERE uniqueid='"+ uniqueid +"'";
      conn.connectionlog("DbAccessProject","sql statement", sqlStatement );
      setKeyWordsAsUsed(conn);
      PreparedStatement ps = conn.prepareStatement(sqlStatement);
      int psndx = 0;
      if (sqlStatement.indexOf(",projdate=?") > -1) {
        psndx++;
        String sdate = t4d.getDate2SqlString(projdate);
        java.sql.Date ddate = null;
        if (sdate!=null && sdate.length() > 0) {
          ddate = java.sql.Date.valueOf(sdate);
        }
        ps.setDate(psndx , ddate);
      }

      int count = ps.executeUpdate();
      ps.close();
      if (count == 0) {
        ok = false;
        errCode = ERROR_0003;
        errMess = t4m.getMessage("msg0003", null);
      }
    }



    if (ok) {
        errCode = MSG_0007;
        errMess = t4m.getMessage("msg0007", null);
    }
    return ok;
  }

  /**
   * The method <CODE>deleteInSink</CODE> deletes the data in database (data sink).
   * @param conn Connection to Database, if null => make own connection
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean deleteInSink(DatabaseConnection conn) throws Exception {
    boolean ok = true;

    // test before delete
    if (!hasdata) {
      errCode = ERROR_0004;
      errMess = t4m.getMessage("msg0004", null);
      return false;
    }
    if (uniqueid == null || (uniqueid != null && uniqueid.length() == 0)) {
      errCode = ERROR_0005;
      errMess = t4m.getMessage("msg0005", null);
      return false;
    }

    // get connection
    boolean ownconn = false;
    if (conn == null) {
      conn = myDatabaseConnectionSet.conn();
      conn.setAutoCommit(false);
      ownconn = true;
    }



    // delete publicforeignkeys
    ArrayList report_delete = new ArrayList();
    if (publfkey4load != null && publfkey4load.length()>1) {
      String pffieldlist = "";
      String publfkey = publfkey4load;
      DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
      StringTokenizer stok = new StringTokenizer(publfkey ,";"); // public foreign keys separated by ;
      while (stok.hasMoreTokens()) { // for each public foreign key
        String xfield = stok.nextToken();
        String pfTableName = (String) publicforeignkeytable.get(xfield);
        String pfkeyfield = "";
        StringTokenizer staccess = new StringTokenizer(pfTableName ,"|");
        if (staccess.hasMoreTokens()) pfTableName = staccess.nextToken();
        if (staccess.hasMoreTokens()) pfkeyfield = staccess.nextToken();
        String sql = pfkeyfield + " = '"+uniqueid+"'";
        ArrayList keyList = dsl.getDataSet(sysUser , pfTableName , null , null , null , sql , null , "expertanddecision");
        int daclen = keyList.size();
        for (int dacndx=0;dacndx<daclen;dacndx++) {
          if (false) {}
         else if (xfield.equals("idresultatt")) {
            DbAccessResultAtt pfDbAccessResultAtt = (DbAccessResultAtt) keyList.get(dacndx);
            report_delete.add("ResultAtt" + "|" + pfDbAccessResultAtt.getUniqueid());
            pfDbAccessResultAtt.deleteInSink(conn);
         }

        }
      }
    }

    String sqlStatement = "DELETE FROM Project WHERE uniqueid='" + uniqueid + "'";
    conn.connectionlog("DbAccessProject","sql statement", sqlStatement );

    Statement  s = conn.createStatement();
    s.execute(sqlStatement);
    s.close();

    if (ownconn) {
      if (ok) {
        reportDelete(report_delete);
        conn.commit();
      } else {
        conn.rollback();
      }
      myDatabaseConnectionSet.connClose(conn);
    }



    errCode = MSG_0008;
    errMess = t4m.getMessage("msg0008", null);
    return ok;
  }

  /**
   * The method <CODE>reportDelete</CODE> report the deleted data.
   * @param report_delete the list of the associated sentences.
   * @exception Exception.
   * @since 1.0
   */
  private void reportDelete(ArrayList report_delete) throws Exception {
      if (!protocol) {
          return;
      }
  
      // report the deletion of all associated sentences.
      if (report_delete != null && report_delete.size() > 0) {
          int len = report_delete.size();
          for (int ndx=0;ndx<len;ndx++) {
              String sentence = (String) report_delete.get(ndx);
              String tablename = "";
              String tableuid = "";
              StringTokenizer st = new StringTokenizer(sentence, "|");
              if (st.hasMoreTokens()) {
                  tablename = st.nextToken();
              }
              if (st.hasMoreTokens()) {
                  tableuid = st.nextToken();
              }
              putProtocolToSink(tablename, tableuid);
          }
      }
  
      // report the deletion of the main sentence.
      putProtocolToSink(dbtablename, uniqueid);
  
  }

  /**
   * The method <CODE>putProtocolToSink</CODE> saves the report protocol.
   * @param tablename the name of the table.
   * @param tableuid the uniqueid of the sentence.
   * @exception Exception.
   * @since 1.0
   */
  private void putProtocolToSink(String tablename, String tableuid) throws Exception {
  
      DbAccessProtocol xprotocol = new DbAccessProtocol(new List4SysUser(null, sysUser.getUserUniqueid(),
                        sysUser.getUserUserid(), sysUser.getUserName(), sysUser.getUserLanguageCode(), 1, null, null, null,
                        sysUser.getClientUniqueid(), null, null, null, sysUser.getClientDbsystemforen4e(), null), null, "all", null, null);
  
      if (tablename.length() > 30) {
          tablename = tablename.substring(0, 30);
      }
      xprotocol.setName(tablename);
      xprotocol.setProtocoltype("dec20d6:1244d75eb11:-8000"); // delete sentence.
      xprotocol.setActiondate(new java.util.Date());
      xprotocol.setTableuid(tableuid);
      String the_user = sysUser.getUserUserid();
      if (the_user.length() > 35) {
          the_user = the_user.substring(0, 35);
      }
      xprotocol.setShorttext(the_user);
  
      // create a fingerprint of the deletion data.
      String pattern4fingerprint = xprotocol.getName() + xprotocol.getProtocoltype()
              + Long.toString(xprotocol.getActiondate().getTime()) + xprotocol.getShorttext()
              + xprotocol.getTableuid();
  
      MessageDigest md = MessageDigest.getInstance("SHA");
      byte digest[] = md.digest(pattern4fingerprint.getBytes());
      String key = "";
      int len = digest.length;
      for (int i=0;i<len;i++) {
          key += Integer.toHexString(digest[i]&0xff);
      }
      xprotocol.setText(key);
  
      xprotocol.putToSink(null);
  
  }

  /**
   * The method <CODE>initializeStandardFields</CODE> sets standard fiels to initial value.
   * @since 1.0
  */
  private void initializeStandardFields() {
    if (hasdata) { // update existing data set
      version++;
      changed = new java.util.Date();
      changedfrom = sysUser.getUserUniqueid();
      inuse = 0;
    } else { // insert new data set
      uniqueid = uniqueid==null||uniqueid.trim().length()==0?new UID().toString():uniqueid;
      version = 0;
      created = new java.util.Date();
      creator = sysUser.getUserUniqueid();
      changed = null;
      changedfrom = null;
      inuse = 0;
    }
  }

  /**
   * The method <CODE>resetFields</CODE> sets fields to initvalue.
   * @exception Exception
   * @since 1.0
  */
  private void resetFields() throws Exception {
    uniqueid = null;
    version = 0;
    created = null;
    creator = null;
    changed = null;
    changedfrom = null;
    inuse = 0;
    name = null;
    old_name = null;
    projdate = null;
    old_projdate = null;
    remarks = null;
    old_remarks = null;
    myDbAccessExpSystem = null;
    old_myDbAccessExpSystem = null;
    myDbAccessResultAtt = new ArrayList();
    old_myDbAccessResultAtt = null;

  }

  /**
   * The method <CODE>getDefData</CODE> returns data concerning a table field.
   * @param fieldname the name of table field
   * @param datatype type of data to return
   * @return def data
   * @exception Exception
   * @since 1.0
  */
  public String getDefData(String fieldname , String datatype) throws Exception {
    if (datatype.equals("label")) {
      if (false) {}
      else if (fieldname.equals("name")) {return getTranslationText(1);}
      else if (fieldname.equals("projdate")) {return getTranslationText(37);}
      else if (fieldname.equals("remarks")) {return getTranslationText(38);}

//       return dbtablename + fieldname;
    } else if (datatype.equals("type")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "char";}
      else if (fieldname.equals("projdate")) {return "date";}
      else if (fieldname.equals("remarks")) {return "char";}
      else if (fieldname.equals("idexpsystem")) {return "privateforeignkey";}
      else if (fieldname.equals("idresultatt")) {return "publicforeignkey";}

    } else if (datatype.equals("length")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "50";}
      else if (fieldname.equals("projdate")) {return "15";}
      else if (fieldname.equals("remarks")) {return "500";}
      else if (fieldname.equals("idexpsystem")) {return "35";}
      else if (fieldname.equals("idresultatt")) {return "35";}

    } else if (datatype.equals("needed")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "no";}
      else if (fieldname.equals("projdate")) {return "no";}
      else if (fieldname.equals("remarks")) {return "no";}
      else if (fieldname.equals("idexpsystem")) {return "no";}
      else if (fieldname.equals("idresultatt")) {return "no";}

    }
    return "";
  }

  /**
   * The method <CODE>setFieldAsString</CODE> returns the field contens as a string.
   * @param fieldname the name of table field
   * @param valuexyz the value of table field
   * @exception Exception
   * @since 1.0
  */
  public void setFieldAsString(String fieldname , String valuexyz) throws Exception {
//    if (valuexyz == null) valuexyz = "0";
//    if (valuexyz.equals("null")) valuexyz = "0";
    if (valuexyz != null)  {
        valuexyz = valuexyz.trim();
        if (valuexyz.length() == 0) {
            valuexyz = null;
        }
    }
    try  {
    if (false) {}
    else if (fieldname.equals("uniqueid")) {uniqueid = valuexyz;}
    else if (fieldname.equals("version")) {version = Long.parseLong(valuexyz == null?"0":valuexyz);}
    else if (fieldname.equals("created")) {created = new java.util.Date(t4n.getString2Long(valuexyz == null?"0":valuexyz, SystemService.getLocaleDef()));}
    else if (fieldname.equals("creator")) {creator = valuexyz;}
    else if (fieldname.equals("changed")) {changed = new java.util.Date(Long.parseLong(valuexyz == null?"0":valuexyz));}
    else if (fieldname.equals("changedfrom")) {changedfrom = valuexyz;}
    else if (fieldname.equals("inuse")) {inuse = (valuexyz == null?0:Integer.parseInt(valuexyz));}
    else if (fieldname.equals("client")) {client = valuexyz;}
    else if (fieldname.equals("name")) {name = valuexyz;}
    else if (fieldname.equals("projdate")) {projdate = t4d.getString2Date(valuexyz, SystemService.getDateDef());}
    else if (fieldname.equals("remarks")) {remarks = valuexyz;}
    else if (fieldname.equals("idexpsystem")) {
      DbAccessExpSystem temp = new DbAccessExpSystem(sysUser , valuexyz , "all" , null , null);
      if (temp.errCode == 0) {
        setMyDbAccessExpSystem(temp);
      } else {
        errCode = temp.errCode;
        errMess = temp.errMess;
      }
    }

    } catch (ParseException ex) {
       errCode = ERROR_0105;
       errMess = t4m.getMessage("msg0121", getDefData(fieldname , "label"));
    } catch (Exception ex) {
       throw new Exception(ex);
    }
  }

  /**
   * The method <CODE>getBlobField</CODE> returns the field contens of blob fields.
   * @param fieldname the name of table field.
   * @return field contens.
   * @exception Exception
   * @since 1.0
  */
  public byte[] getBlobField(String fieldname) throws Exception {
    if (false) {}

    return null;
  }

  /**
   * The method <CODE>getFieldAsStringKey</CODE> returns the field contens as a string.
   * @param fieldname the name of table field.
   * @param getkey, get the key value.
   * @return field contens.
   * @exception Exception
   * @since 1.0
  */
  public String getFieldAsStringKey(String fieldname , Boolean getkey) throws Exception {
    if (false) {}
    else if (fieldname.equals("uniqueid")) {return uniqueid == null?"":uniqueid ;}
    else if (fieldname.equals("version")) {return Long.toString(version);}
    else if (fieldname.equals("created")) {return created == null?"":created.toString();}
    else if (fieldname.equals("creator")) {return creator == null?"":creator ;}
    else if (fieldname.equals("changed")) {return changed == null?"":changed.toString();}
    else if (fieldname.equals("changedfrom")) {return changedfrom == null?"":changedfrom ;}
    else if (fieldname.equals("inuse")) {return Integer.toString(inuse);}
    else if (fieldname.equals("client")) {return client == null?"":client ;}
    else if (fieldname.equals("name")) {return name == null?"":name ;}
    else if (fieldname.equals("projdate")) {return projdate == null?"":t4d.getDate2String(projdate, SystemService.getDateDef());}
    else if (fieldname.equals("remarks")) {return remarks == null?"":remarks ;}
    else if (fieldname.equals("idexpsystem")) {return getMyDbAccessExpSystem()==null?"":getMyDbAccessExpSystem().getUniqueid();}

    return "";
  }

  /**
   * The method <CODE>hasChanged</CODE> returns the status of field change.
   * @param fieldname the name of table field
   * @return field length
   * @exception Exception
   * @since 1.0
  */
  public boolean hasChanged(String fieldname) throws Exception {
    if (false) {}
    else if (fieldname.equals("name")) {
      if (name != null || old_name != null) {
        if ((name == null && old_name != null) || (name != null && old_name == null)) {
          return true;
        } else {
          if (!name.equals(old_name)) {return true;}
        }
      }
    }
    else if (fieldname.equals("projdate")) {
      if (projdate != null || old_projdate != null) {
        if ((projdate == null && old_projdate != null) || (projdate != null && old_projdate == null)) {
          return true;
        } else {
          if (!t4d.getDate2String(projdate, SystemService.getDateDef()).equals(t4d.getDate2String(old_projdate, SystemService.getDateDef()))) {return true;}
        }
      }
    }
    else if (fieldname.equals("remarks")) {
      if (remarks != null || old_remarks != null) {
        if ((remarks == null && old_remarks != null) || (remarks != null && old_remarks == null)) {
          return true;
        } else {
          if (!remarks.equals(old_remarks)) {return true;}
        }
      }
    }
    else if (fieldname.equals("idexpsystem")) {
      if (myDbAccessExpSystem != old_myDbAccessExpSystem) {
          return true;
      }
    }

    return false;
  }

  /**
   * The method <CODE>getDataSet</CODE> returns a data set from table.
   * @param addsqlorder order of sql result
   * @return dataSet
   * @exception Exception
   * @since 1.0
  */
  public ArrayList getDataSet(String addsqlorder) throws Exception {
    DatabaseDatasetLoader dbl = new DatabaseDatasetLoader();
    ArrayList dataSet = dbl.getDataSet(sysUser , dbtablename , fields4load , privfkey4load , publfkey4load , sqlAdd , addsqlorder , "expertanddecision");
    return dataSet;
  }

  /**
   * The method <CODE>getTranslationText</CODE> returns the translated text.
   * @param ndx the text index.
   * @return the translation.
   * @exception Exception
   * @since 1.0
  */
  public String getTranslationText(int ndx) throws Exception {
    if (false) {
    } else if (sysUser.getUserLanguageCode().equals("de")) {
      if (false) {}
      else if (ndx == 38) {return "Bemerkungen";}
      else if (ndx == 37) {return "Datum";}
      else if (ndx == 1) {return "Name";}

    } else if (sysUser.getUserLanguageCode().equals("en")) {
      if (false) {}
      else if (ndx == 38) {return "Remarks";}
      else if (ndx == 37) {return "Date";}
      else if (ndx == 1) {return "Name";}


    }
    return "no text available";
  }


}
