package n4e.expertanddecision.dbaccess;

/**
* <p>title: n4e application</p>
* <p>description: expertanddecision</p>
* <p>copyright: copyright (c) 2006-2011 by thomas kaffka; mba; all rights reserved.</p>
* <p>company: n.n.</p>
* @author thomas kaffka, mba.
* @version Fri May 01 16:26:19 CEST 2015
*/

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;
import java.util.zip.*;
import java.io.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;
import n4e.core.list.*;



/**
 * The <CODE>DbAccessReadExpAttribute</CODE> class reads fields of ExpAttribute table in database.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version Fri May 01 16:26:19 CEST 2015
*/
public class DbAccessReadExpAttribute extends DbAccessRead {

  // individual database fields
  private String name = ""; // Name
  private String codomain = ""; // Wertebereich
  private String valuetype = ""; // Wertetyp
  private int initialattribute = 0; // Initialattribut
  private String question = ""; // Frage
  private String adddata = ""; // Ergaenzung
  private String description = ""; // Beschreibung
  private ArrayList myDbAccessExpCodomain = null ; // ExpCodomain
  private ArrayList myDbAccessExpCondition = null ; // ExpCondition
  private ArrayList myDbAccessResultAtt = null ; // ResultAtt
  private DbAccessReadExpSystem myDbAccessReadExpSystem = null ; // ExpSystem


  // sets and gets of individual table fields
  public void setName(String input){name = input;}
  public String getName(){return name;}
  public void setCodomain(String input){codomain = input;}
  public String getCodomain(){return codomain;}
  public void setValuetype(String input){valuetype = input;}
  public String getValuetype(){return valuetype;}
  public void setInitialattribute(int input){initialattribute = input;}
  public int getInitialattribute(){return initialattribute;}
  public void setQuestion(String input){question = input;}
  public String getQuestion(){return question;}
  public void setAdddata(String input){adddata = input;}
  public String getAdddata(){return adddata;}
  public void setDescription(String input){description = input;}
  public String getDescription(){return description;}
  public void setMyDbAccessExpCodomain(ArrayList input){myDbAccessExpCodomain = input;}
  public ArrayList getMyDbAccessExpCodomain(){return myDbAccessExpCodomain;}
  public void setMyDbAccessExpCondition(ArrayList input){myDbAccessExpCondition = input;}
  public ArrayList getMyDbAccessExpCondition(){return myDbAccessExpCondition;}
  public void setMyDbAccessResultAtt(ArrayList input){myDbAccessResultAtt = input;}
  public ArrayList getMyDbAccessResultAtt(){return myDbAccessResultAtt;}
  public void setMyDbAccessReadExpSystem(DbAccessReadExpSystem input){myDbAccessReadExpSystem = input;}
  public DbAccessReadExpSystem getMyDbAccessReadExpSystem(){return myDbAccessReadExpSystem;}


  /**
   * The constructor <CODE>DbAccessReadExpAttribute</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessReadExpAttribute(){
      setWorkFields();
  }

  /**
   * The constructor <CODE>DbAccessReadExpAttribute</CODE> initialized the object.
   * @param sysUser the system user.
   * @since 1.0
  */
  public DbAccessReadExpAttribute(List4SysUser sysUser){
    setSystemUser(sysUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }
  }

  /**
   * The constructor <CODE>DbAccessReadExpAttribute</CODE> initialized the object
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadExpAttribute(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , null , true);

  }

  /**
   * The constructor <CODE>DbAccessReadExpAttribute</CODE> initialized the object
   * @param conn database connection
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadExpAttribute(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , null , true);

  }

  /**
   * The constructor <CODE>DbAccessReadExpAttribute</CODE> initialized the object with additional sqlwhere
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadExpAttribute(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd , true);

  }

  /**
   * The constructor <CODE>DbAccessReadExpAttribute</CODE> initialized the object with additional sqlwhere
    this.conn 0 conn;
   * @param conn database connection
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadExpAttribute(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd , true);

  }

  /**
   * The method <CODE>initiate</CODE> initializes the object with additional sqlwhere.
   * @param pUser the system user object.
   * @param uniqueid the unique key.
   * @param fieldlist List of fields to load.
   * @param privfkey List of privfkeys to load.
   * @param publfkey List of publfkeys to load.
   * @param sqlAdd additional where clause.
   * @param read_with_client read with use of the client id.
   * @exception Exception
   * @since 1.0
  */
  public void initiate(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd , boolean read_with_client) throws Exception {

    this.read_with_client = read_with_client;
    setSystemUser(pUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    publicforeignkeytable.put("idexpcodomain","ExpCodomain|idexpattribute");
    publicforeignkeytable.put("idexpcondition","ExpCondition|idexpattribute");
    publicforeignkeytable.put("idresultatt","ResultAtt|idexpattribute");

    privateforeignkeytable.put("idexpsystem","ExpSystem|null");


    success = loadFromTable(uniqueid , fieldlist , privfkey , publfkey , sqlAdd );
  }

  /**
   * The method <CODE>setWorkFields</CODE> sets the work fiels to current values.
   * @since 1.0
  */
  public void setWorkFields() {
      dbtablename = "ExpAttribute"; // name of own database table
      if (sysUser != null) {
          databasename = sysUser.getClientDbsystemforeclient(); // name of own database
      }
      tblnameofprivk = ",ExpSystem,"; // list of table names of private foreign keys
      tblnameofpublvk = ",ExpCodomain,ExpCondition,ResultAtt,"; // list of table names of public foreign keys
      keynameofpublvk = ",idexpattribute,idexpattribute,idexpattribute,"; // list of key names of table names of public foreign keys
      tblnameofownerfield = ","; // list of names of table names of owner key
      indivTableFields = ",name,codomain,valuetype,initialattribute,question,adddata,description,"; // list of inidividual table fields
      privateforeignkey = ";idexpsystem;"; // list of private foreign key fields
      publicforeignkey = ";idexpcodomain;idexpcondition;idresultatt;"; // list of public foreign key fields
  }

  /**
   * The method <CODE>loadFromTable</CODE> gets tabledata from database.
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean loadFromTable(String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    errCode = 0;
    errMess = "";

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    hasdata = false;
    // create connection to defined database system
    String dbsystem = databasename;
    DatabaseConnectionSet myDatabaseConnectionSet = DatabaseManager.getDatabaseConnectionSet(dbsystem);
    if (this.myconn) {
        this.conn = myDatabaseConnectionSet.conn();
    }
    DatabaseSQLResultSet resSet = new DatabaseSQLResultSet(dbsystem , conn);

    // handle fields to load
    String sqlFieldlist = "";
    String sqlPrivfkey = "";

    // build sql field list
    if (fieldlist != null && fieldlist.equals("all")) {
      sqlFieldlist = indivTableFields.substring(1);
    } else { // make individual fieldlist
      if (fieldlist != null) {
        sqlFieldlist = fieldlist;
      } else {
        sqlFieldlist = "";
      }
    }

    // build sql field list of private foreign keys
    if (privfkey != null && privfkey.equals("all")) {
      sqlPrivfkey = privateforeignkey.substring(1);
    } else { // make individual fieldlist
      if (privfkey != null) {
          int pos = privfkey.indexOf("(");
          if (pos >= 0) {
            StringTokenizer stok = new StringTokenizer(privfkey , ";"); // public foreign keys separated by ;
            while (stok.hasMoreTokens()) { // for each public foreign key
                String xfield = stok.nextToken();
                pos = xfield.indexOf("(");
                if (pos >= 0) {
                    sqlPrivfkey += "," + xfield.substring(0, pos);
                } else {
                    sqlPrivfkey += "," + xfield;
                }
            }
            sqlPrivfkey = sqlPrivfkey.substring(1);
          } else {
            sqlPrivfkey = t4st.replaceToken(privfkey , ";" , ",");
          }
      } else {
        sqlPrivfkey = "";
      }
    }

    StringTokenizer stok = null;

    String workingSqlFieldlist = stdTableFields + sqlFieldlist;
    if (workingSqlFieldlist.endsWith(",")) {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?sqlPrivfkey:"")+",";
    } else {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?","+sqlPrivfkey:"")+",";
    }


    int len = workingSqlFieldlist.length();
    workingSqlFieldlist = workingSqlFieldlist.substring(1,len-1);
    if (workingSqlFieldlist.endsWith(",")) {
      len = workingSqlFieldlist.length();
      workingSqlFieldlist = workingSqlFieldlist.substring(0,len-1);
    }

    resetFields();

    // grip into data sink
    if (uniqueid == null && sqlAdd == null) {
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
        errCode = ERROR_0001;
        errMess = t4m.getMessage("msg0001", null);
        return false;
    }

    String client_temp = client;
    if (!read_with_client) {
        client = null;
    }

    String where1 = ((client!=null&&client.length()>0)?("client='"+client+"'"):"");
    String where2 = ((uniqueid!=null&&uniqueid.length()>0)?("uniqueid='"+uniqueid+"'"):"");
    String where3 = ((sqlAdd!=null&&sqlAdd.length()>0)?sqlAdd:"");
    String where = "";
    if (where1.length()==0 && where2.length()==0 && where3.length()==0) { // 000
      where = "";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()==0) { // 100
      where = " WHERE (" + where1 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()==0) { // 010
      where = " WHERE (" + where2 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()==0) { // 110
      where = " WHERE (" + where1 + ") AND (" + where2 + ")";
    } else if (where1.length()==0 && where2.length()==0 && where3.length()>0) { // 001
      where = " WHERE (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()>0) { // 101
      where = " WHERE (" + where1 + ") AND (" + where3 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()>0) { // 011
      where = " WHERE (" + where2 + ") AND (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()>0) { // 111
      where = " WHERE (" + where1 + ") AND (" + where2 + ") AND (" + where3 + ")";
    }

    String sqlStatement = "SELECT "+workingSqlFieldlist+" FROM ExpAttribute " + where;
    conn.connectionlog("DbAccessReadExpAttribute","sql statement", sqlStatement);
    ResultSet result = resSet.getResultSet(sqlStatement);
    if (result.next() && (result.isFirst() && result.isLast())) {
      hasdata = true;
      // get standard fields
      setUniqueid(result.getString(1));
      setVersion(result.getLong(2));
      long temp_timestamp = result.getLong(3);
      java.util.Date temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setCreated(temp_date);
      setCreator(result.getString(4));
      temp_timestamp = result.getLong(5);
      temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setChanged(temp_date);
      setChangedfrom(result.getString(6));
      setInuse(result.getInt(7));
      setClient(result.getString(8));

      //get individual fields
      int i = 9;
      workingSqlFieldlist = sqlFieldlist + (sqlPrivfkey.length()>0?","+sqlPrivfkey:"");
      stok = new StringTokenizer(workingSqlFieldlist ,",");
      while (stok.hasMoreTokens()) {
        String xfield = stok.nextToken();
        if (false) {}
          // dummy
          else if (xfield.equals("name")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setName(value);}
          else if (xfield.equals("codomain")) {String value = result.getString(i);setCodomain(value);}
          else if (xfield.equals("valuetype")) {String value = result.getString(i);setValuetype(value);}
          else if (xfield.equals("initialattribute")) {int value = result.getInt(i);setInitialattribute(value);}
          else if (xfield.equals("question")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setQuestion(value);}
          else if (xfield.equals("adddata")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setAdddata(value);}
          else if (xfield.equals("description")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setDescription(value);}
          else if (xfield.equals("idexpsystem")) {}

        i++;
      }

      //get individual privateforeignkeys
      if (privfkey != null && privfkey.length()>0) {
        String pffieldlist = "";
        if (privfkey.equals("all")) { // load all private foreign key objects with field list 'all'
          privfkey = privateforeignkey.substring(1);
        }
        stok = new StringTokenizer(privfkey , ";"); // private foreign keys separated by ;
        while (stok.hasMoreTokens()) { // for each private foreign key
          String xfield = stok.nextToken();
          if (xfield == null || xfield.length() == 0) continue;
          if (xfield.indexOf("(") > -1) { // private foreign key with field list
            StringTokenizer stfield = new StringTokenizer(xfield ,"(");
            if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (pffieldlist.endsWith(")")) {
              pffieldlist = pffieldlist.substring(0, pffieldlist.length()-1);
            }
          } else {
            pffieldlist = "all";
          }
          if (false) {}
          else if (xfield.equals("idexpsystem")) {
            DbAccessReadExpSystem tempIdexpsystem = new DbAccessReadExpSystem(conn , sysUser , result.getString("idexpsystem") , pffieldlist , null , null);
            if (tempIdexpsystem.errCode == -17) {
              setMyDbAccessReadExpSystem(null);
            } else if (tempIdexpsystem.errCode == 0) {
              setMyDbAccessReadExpSystem(tempIdexpsystem);
            } else if (tempIdexpsystem.errCode != ERROR_0001) {
              errCode = tempIdexpsystem.errCode;
              errMess = tempIdexpsystem.errMess;
            }
          }

        }
      }

    } else { // no data found
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
       errCode = ERROR_0002;
       errMess = t4m.getMessage("msg0002", null);
       return false;
    }

    //get individual publicforeignkeys
    if (publfkey != null && publfkey.length()>0) {
      String pffieldlist = "";
      if (publfkey.equals("all")) { // load all public foreign key objects with field list 'all'
        publfkey = publicforeignkey.substring(1);
      }
      DatabaseDatasetLoader dsl = new DatabaseDatasetLoader(this.conn);
      stok = new StringTokenizer(publfkey ,";"); // public foreign keys separated by ;
      while (stok.hasMoreTokens()) { // for each public foreign key
        String xfield = stok.nextToken();
        if (xfield == null || xfield.length() == 0) continue;
        StringTokenizer stfield = null;
        if (xfield.indexOf("(") > -1) { // public foreign key with field list
          stfield = new StringTokenizer(xfield ,"(");
          if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
          if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
          if (pffieldlist.endsWith(")")) {
            pffieldlist = pffieldlist.substring(0 , pffieldlist.length()-1);
          }
        } else {
          pffieldlist = "all";
        }
        String orderby = null;
        if (pffieldlist.indexOf("|") > -1) { // order by feilds available.
            stfield = new StringTokenizer(pffieldlist ,"|");
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (stfield.hasMoreTokens()) orderby = stfield.nextToken();
        }
        String pfTableName = (String) publicforeignkeytable.get(xfield);
        String pfkeyfield = "";
        StringTokenizer staccess = new StringTokenizer(pfTableName ,"|");
        if (staccess.hasMoreTokens()) pfTableName = staccess.nextToken();
        if (staccess.hasMoreTokens()) pfkeyfield = staccess.nextToken();
        String sql = pfkeyfield + " = '" + result.getString("uniqueid")+"'";
        ArrayList keyList = dsl.getDataSetRead(sysUser , pfTableName , pffieldlist , null , null , sql , orderby , "expertanddecision");
        if (false) {}
        else if (xfield.equals("idexpcodomain")) {
          setMyDbAccessExpCodomain(keyList);
        }
        else if (xfield.equals("idexpcondition")) {
          setMyDbAccessExpCondition(keyList);
        }
        else if (xfield.equals("idresultatt")) {
          setMyDbAccessResultAtt(keyList);
        }

      }
    }

    if (!read_with_client) {
        client = client_temp;
    }

    resSet.closeResultSet();
    resSet = null;
    if (this.myconn) {
        myDatabaseConnectionSet.connClose(conn);
    }

    return true;
  }

  /**
   * The method <CODE>resetFields</CODE> sets fields to initvalue.
   * @exception Exception
   * @since 1.0
  */
  private void resetFields() throws Exception {
    uniqueid = null;
    version = 0;
    created = null;
    creator = null;
    changed = null;
    changedfrom = null;
    inuse = 0;
    name = null;
    codomain = null;
    valuetype = null;
    initialattribute = 0;
    question = null;
    adddata = null;
    description = null;
    myDbAccessExpCodomain = new ArrayList();
    myDbAccessExpCondition = new ArrayList();
    myDbAccessResultAtt = new ArrayList();
    myDbAccessReadExpSystem = null;

  }

  /**
   * The method <CODE>getDefData</CODE> returns data concerning a table field.
   * @param fieldname the name of table field
   * @param datatype type of data to return
   * @return def data
   * @exception Exception
   * @since 1.0
  */
  public String getDefData(String fieldname , String datatype) throws Exception {
    if (datatype.equals("label")) {
      if (false) {}
      else if (fieldname.equals("name")) {return getTranslationText(1);}
      else if (fieldname.equals("codomain")) {return getTranslationText(27);}
      else if (fieldname.equals("valuetype")) {return getTranslationText(28);}
      else if (fieldname.equals("initialattribute")) {return getTranslationText(29);}
      else if (fieldname.equals("question")) {return getTranslationText(30);}
      else if (fieldname.equals("adddata")) {return getTranslationText(31);}
      else if (fieldname.equals("description")) {return getTranslationText(2);}

//       return dbtablename + fieldname;
    } else if (datatype.equals("type")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "char";}
      else if (fieldname.equals("codomain")) {return "key";}
      else if (fieldname.equals("valuetype")) {return "key";}
      else if (fieldname.equals("initialattribute")) {return "flag";}
      else if (fieldname.equals("question")) {return "char";}
      else if (fieldname.equals("adddata")) {return "char";}
      else if (fieldname.equals("description")) {return "char";}
      else if (fieldname.equals("idexpcodomain")) {return "publicforeignkey";}
      else if (fieldname.equals("idexpcondition")) {return "publicforeignkey";}
      else if (fieldname.equals("idresultatt")) {return "publicforeignkey";}
      else if (fieldname.equals("idexpsystem")) {return "privateforeignkey";}

    } else if (datatype.equals("length")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "30";}
      else if (fieldname.equals("codomain")) {return "35";}
      else if (fieldname.equals("valuetype")) {return "35";}
      else if (fieldname.equals("initialattribute")) {return "1";}
      else if (fieldname.equals("question")) {return "300";}
      else if (fieldname.equals("adddata")) {return "300";}
      else if (fieldname.equals("description")) {return "500";}
      else if (fieldname.equals("idexpcodomain")) {return "35";}
      else if (fieldname.equals("idexpcondition")) {return "35";}
      else if (fieldname.equals("idresultatt")) {return "35";}
      else if (fieldname.equals("idexpsystem")) {return "35";}

    } else if (datatype.equals("needed")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "yes";}
      else if (fieldname.equals("codomain")) {return "yes";}
      else if (fieldname.equals("valuetype")) {return "yes";}
      else if (fieldname.equals("initialattribute")) {return "no";}
      else if (fieldname.equals("question")) {return "no";}
      else if (fieldname.equals("adddata")) {return "no";}
      else if (fieldname.equals("description")) {return "no";}
      else if (fieldname.equals("idexpcodomain")) {return "no";}
      else if (fieldname.equals("idexpcondition")) {return "no";}
      else if (fieldname.equals("idresultatt")) {return "no";}
      else if (fieldname.equals("idexpsystem")) {return "no";}

    }
    return "";
  }

  /**
   * The method <CODE>getBlobField</CODE> returns the field contens of blob fields.
   * @param fieldname the name of table field.
   * @return field contens.
   * @exception Exception
   * @since 1.0
  */
  public byte[] getBlobField(String fieldname) throws Exception {
    if (false) {}

    return null;
  }

  /**
   * The method <CODE>getFieldAsStringKey</CODE> returns the field contens as a string.
   * @param fieldname the name of table field.
   * @param getkey, get the key value.
   * @return field contens
   * @exception Exception
   * @since 1.0
  */
  public String getFieldAsStringKey(String fieldname , Boolean getkey) throws Exception {
    if (false) {}
    else if (fieldname.equals("uniqueid")) {return uniqueid == null?"":uniqueid;}
    else if (fieldname.equals("version")) {return Long.toString(version);}
    else if (fieldname.equals("created")) {return created == null?"":created.toString();}
    else if (fieldname.equals("creator")) {return creator == null?"":creator;}
    else if (fieldname.equals("changed")) {return changed == null?"":changed.toString();}
    else if (fieldname.equals("changedfrom")) {return changedfrom == null?"":changedfrom;}
    else if (fieldname.equals("inuse")) {return Integer.toString(inuse);}
    else if (fieldname.equals("client")) {return client == null?"":client;}
    else if (fieldname.equals("name")) {return name == null?"":name ;}
    else if (fieldname.equals("codomain")) {if(getkey.booleanValue()) {return codomain == null?"":codomain;} else {return codomain == null?"":getKeyWord(codomain);}}
    else if (fieldname.equals("valuetype")) {if(getkey.booleanValue()) {return valuetype == null?"":valuetype;} else {return valuetype == null?"":getKeyWord(valuetype);}}
    else if (fieldname.equals("initialattribute")) {if(getkey.booleanValue()) {return Integer.toString(initialattribute);} else {return initialattribute == 1?t4m.getMessage("msg0015", null):t4m.getMessage("msg0016", null);}}
    else if (fieldname.equals("question")) {return question == null?"":question ;}
    else if (fieldname.equals("adddata")) {return adddata == null?"":adddata ;}
    else if (fieldname.equals("description")) {return description == null?"":description ;}
    else if (fieldname.equals("idexpsystem")) {return getMyDbAccessReadExpSystem()==null?"":getMyDbAccessReadExpSystem().getUniqueid();}

    return "";
  }

  /**
   * The method <CODE>getDataSet</CODE> returns a data set from table.
   * @param addsqlorder order of sql result
   * @return dataSet
   * @exception Exception
   * @since 1.0
  */
  public ArrayList getDataSet(String addsqlorder) throws Exception {
    DatabaseDatasetLoader dbl = new DatabaseDatasetLoader();
    ArrayList dataSet = dbl.getDataSetRead(sysUser , dbtablename , fields4load , privfkey4load , publfkey4load , sqlAdd , addsqlorder , "expertanddecision");
    return dataSet;
  }

  /**
   * The method <CODE>getTranslationText</CODE> returns the translated text.
   * @param ndx the text index.
   * @return the translation.
   * @exception Exception
   * @since 1.0
  */
  public String getTranslationText(int ndx) throws Exception {
    if (false) {
    } else if (sysUser.getUserLanguageCode().equals("de")) {
      if (false) {}
      else if (ndx == 31) {return "Erg\u00e4nzung";}
      else if (ndx == 30) {return "Frage";}
      else if (ndx == 29) {return "Initialattribut";}
      else if (ndx == 28) {return "Wertetyp";}
      else if (ndx == 27) {return "Wertebereich";}
      else if (ndx == 2) {return "Beschreibung";}
      else if (ndx == 1) {return "Name";}

    } else if (sysUser.getUserLanguageCode().equals("en")) {
      if (false) {}
      else if (ndx == 31) {return "Addition";}
      else if (ndx == 30) {return "Question";}
      else if (ndx == 29) {return "Initial Attribute";}
      else if (ndx == 28) {return "Value Type";}
      else if (ndx == 27) {return "Codomain";}
      else if (ndx == 2) {return "Description";}
      else if (ndx == 1) {return "Name";}


    }
    return "no text available";
  }

  /**
   * The method <CODE>getKeyWord</CODE> gets a keyword from keyset.
   * @param uid uniqueid of keyword
   * @return keyword as text
  */
  public String getKeyWord(String uid) throws Exception {
    Tool4DbDatabase t4db = new Tool4DbDatabase(sysUser);
    return t4db.getKeyWordNameByUid(uid);
  }

  /**
   * The method <CODE>getKeyWordList</CODE> gets a list of all keywords from keyset.
   * @param keyset name of keyset
   * @param dontshowaskey key word uniqueids without showing separated by semicolon.
   * @return keyword list as ArrayList
  */
  public ArrayList getKeyWordList(String keyset, String dontshowaskey) throws Exception {
    Tool4DbDatabase t4db = new Tool4DbDatabase(sysUser);
    return t4db.getKeyWordNameList(keyset, dontshowaskey);
  }

}
