/**
 * ImportKeySet.java
 *
 * @version 12. Mai 2006, 15:18
 *
 * <P><B>title: Dialog Library</B></P>
 * <P>description: common classes for dialog applications.</P>
 * <P>copyright: (c) 2010 thomas kaffka, born at 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 */

package n4e.expertanddecision.saxparser;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

import java.io.*;

import n4e.core.saxparser.SaxParserCH;
import n4e.core.saxparser.SaxParserDTD;
import n4e.core.saxparser.SaxParserError;
import n4e.core.saxparser.SaxParserLexi;
import n4e.core.system.*;
import n4e.core.list.*;
import n4e.core.tool.*;

/**
 * The <CODE>ImportExpSystem</CODE> class imports a given keyset as xml-file.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version 12. Mai 2006, 15:18
 * @since 1.0
 */
public class ImportExpSystem {
    
    /**
     * internal error code of class.
     * @since 1.0
     */
    public int errCode = 0;
    /**
     * internal error message of class.
     * @since 1.0
     */
    public String errMess = "";
    /**
     * true, if success.
     * @since 1.0
     */
    public boolean success = false;
    
    private String xmlFile;
    private SaxParserCH contenthandler;
    private XMLReader parser = null;
    private List4SysUser sysUser;
    
    /**
     * The constructor <CODE>ImportExpSystem</CODE> creates a new instance(object) of the class.
     * @param sysUser the current user object.
     * @since 1.0
     */
    public ImportExpSystem(List4SysUser sysUser) {
        try {
            this.sysUser = sysUser;
            
            // build parser
            parser = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
            if (SystemService.getSaxParserDebug()) {
                System.out.println("parser created...");
            }
            
            // build error handler
            SaxParserError errorhandler = new SaxParserError();
            parser.setErrorHandler(errorhandler);
            if (SystemService.getSaxParserDebug()) {
                System.out.println("set ErrorHandler...");
            }
            
            // put parser to "validation parser"
            parser.setFeature("http://xml.org/sax/features/validation", true);
            if (SystemService.getSaxParserDebug()) {
                System.out.println("turn on 'validation parser'...");
            }
            
            // drop namespace-awareness
            parser.setFeature("http://xml.org/sax/features/namespaces", false);
            if (SystemService.getSaxParserDebug()) {
                System.out.println("turn off 'namespace awareness'...");
            }
            
            // activate lexical handler
            parser.setProperty("http://xml.org/sax/properties/lexical-handler", new SaxParserLexi());
            if (SystemService.getSaxParserDebug()) {
                System.out.println("set LexicalHandler...");
            }
            
            // activate dtd handler
            parser.setDTDHandler(new SaxParserDTD());
            if (SystemService.getSaxParserDebug()) {
                System.out.println("set DTDHandler...");
            }
            
        } catch (SAXException err) {
            errCode = -1000;
            errMess = Tool4Throwable.getCallStackOut(err);
            success = false;
            return;
        } catch (Exception err) {
            errCode = -2000;
            errMess = Tool4Throwable.getCallStackOut(err);
            success = false;
            return;
        }
        
        success = true;
        
        // message.
        if (SystemService.getSaxParserDebug()) {
            System.out.println("... no error right now :-)");
        }
    }
    
    /**
     * The method <CODE>parseDocument</CODE> reads an xml listing of keyset and put it into database.
     * @param pathfrom path to xml listing.
     * @param file name of file of xml listing.
     * @since 1.0
     */
    public void parseDocument(String pathfrom, String file) throws Exception {
        // transfer parameters
        success = true;
        this.xmlFile = pathfrom.trim() + file.trim();
        if (SystemService.getSaxParserDebug()) System.out.println("XML-File: " + xmlFile);
        
        File fileTest = new File(this.xmlFile);
        if (!fileTest.exists()) {
            errCode = -3000; // do not change errCode
            errMess = "ImportExpSystem.parseDocument: xml file (" + file + ") not found!";
            success = false;
            return;
        }
        
        // build content handler
        contenthandler = new ImportExpSystemCH(sysUser);
        parser.setContentHandler(contenthandler);
        
        // parse current document
        try {
        	
            parser.parse(xmlFile);
            
        } catch (SAXException err) {
            errCode = -5000;
            errMess = "ImportKeySet.parseDocument: parser with sax-error: "+err.getMessage()+".!";
            success = false;
            return;
        } catch (Exception err) {
            errCode = -4000;
            errMess = "ImportKeySet.parseDocument: parser with error: "+err.getMessage()+".!";
            success = false;
            return;
        }
        
        success = true;
    }
}

