/**
 * ImportKeySetCH.java
 *
 * @version 12. Mai 2006, 15:18
 *
 * <P><B>title: Dialog Library</B></P>
 * <P>description: common classes for dialog applications.</P>
 * <P>copyright: (c) 2010 thomas kaffka, born at 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
* @author thomas kaffka , mba , cologne , germany.
 */

package n4e.expertanddecision.saxparser;

import java.util.Enumeration;
import java.util.Hashtable;

import org.xml.sax.*;

import n4e.core.dbaccess.DbAccessKeySet;
import n4e.core.dbaccess.DbAccessKeyWord;
import n4e.core.dbaccess.DbAccessReadKeySet;
import n4e.core.dbaccess.DbAccessReadKeyWord;
import n4e.core.saxparser.SaxParserCH;
import n4e.core.system.*;
import n4e.core.list.*;
import n4e.core.tool.*;
import n4e.expertanddecision.dbaccess.DbAccessExpAttribute;
import n4e.expertanddecision.dbaccess.DbAccessExpCodomain;
import n4e.expertanddecision.dbaccess.DbAccessExpCondition;
import n4e.expertanddecision.dbaccess.DbAccessExpRule;
import n4e.expertanddecision.dbaccess.DbAccessExpSystem;
import n4e.expertanddecision.system.Globals;

/**
 * The <CODE>ImportExpSystemCH</CODE> class imports or exports a given keyset or all to xml
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version 12. Mai 2006, 15:18
 * @since 1.0
 */
public class ImportExpSystemCH extends SaxParserCH {
    
    private List4SysUser sysUser;

    private Hashtable<String,String> uid; 
    
    /**
     * The constructor <CODE>ImportExpSystemCH</CODE> creates a new instance(object) of the class.
     * @param sysUser the current user object.
     * @param idclientkey the uniqueid of the current client.
     * @since 1.0
     */
    public ImportExpSystemCH(List4SysUser sysUser) {
        if (SystemService.getSaxParserDebug()) {
            System.out.println("instantiation of ImportExpSystemCH()");
        }
        this.sysUser = sysUser;
        uid = new Hashtable<String,String>();
    }
    
    /**
     * The method <CODE>startDocument</CODE> is called if document starts.
     * @exception SAXException
     * @since 1.0
     */
    public void startDocument() throws SAXException {
        super.startDocument();
    }
    
    /**
     * The method <CODE>endDocument</CODE> is called if document ends.
     * @exception SAXException
     * @since 1.0
     */
    public void endDocument() throws SAXException {
        super.endDocument();
    }
    
    /**
     * The method <CODE>startElement</CODE> is called, if xml-token starts.
     * @param namespaceURI name space.
     * @param localName locale.
     * @param rawName name of token.
     * @param atts attributes of token.
     * @exception SAXException.
     * @since 1.0
     */
    public void startElement(String namespaceURI , String localName ,
            String rawName , Attributes atts) throws SAXException {
        super.startElement(namespaceURI , localName , rawName , atts);
        
       try {
            
            if (rawName.equals("ExpSystem")) {
                
            	DbAccessExpSystem exp = new DbAccessExpSystem(Globals.sysUser, null, "all", null, null);
            	exp.setName("Import: " + atts.getValue("name").substring(0, 22));
            	exp.setDescription(atts.getValue("description"));
            	exp.setType(Integer.parseInt(atts.getValue("type")));
            	exp.setResultnumber(Integer.parseInt(atts.getValue("resultnumber")));
            	exp.putToSink(null);
            	uid.put(atts.getValue("uniqueid"), exp.getUniqueid());
                
            } else if (rawName.equals("ExpAttribute")) {
                
            	DbAccessExpAttribute att = new DbAccessExpAttribute(Globals.sysUser, null, "all", "idexpsystem", null);
            	att.setName(atts.getValue("name"));
            	att.setCodomain(atts.getValue("codomain"));
            	att.setValuetype(atts.getValue("valuetype"));
            	att.setInitialattribute(Integer.parseInt(atts.getValue("initialattribute")));
            	att.setQuestion(atts.getValue("question"));
            	att.setAdddata(atts.getValue("adddata"));
            	att.setDescription(atts.getValue("description"));
            	String uniqueid = uid.get(atts.getValue("idexpsystem"));
            	att.setFieldAsString("idexpsystem", uniqueid);
            	att.putToSink(null);
            	uid.put(atts.getValue("uniqueid"), att.getUniqueid());

            } else if (rawName.equals("ExpCodomain")) {
                
            	DbAccessExpCodomain cod = new DbAccessExpCodomain(Globals.sysUser, null, "all", "idexpattribute", null);
            	cod.setText(atts.getValue("text"));
            	String uniqueid = uid.get(atts.getValue("idexpattribute"));
            	cod.setFieldAsString("idexpattribute", uniqueid);
            	cod.putToSink(null);
            	uid.put(atts.getValue("uniqueid"), cod.getUniqueid());

            } else if (rawName.equals("ExpRule")) {
                
            	DbAccessExpRule rul = new DbAccessExpRule(Globals.sysUser, null, "all", "idexpattribute", null);
            	rul.setRuleid(Integer.parseInt(atts.getValue("ruleid")));
            	rul.setEnabled(Integer.parseInt(atts.getValue("enabled")));
            	rul.setDescription(atts.getValue("description"));
            	rul.setPriorprob(Double.parseDouble(atts.getValue("priorprob")));
            	String uniqueid = uid.get(atts.getValue("idexpsystem"));
            	rul.setFieldAsString("idexpsystem", uniqueid);
            	rul.putToSink(null);
            	uid.put(atts.getValue("uniqueid"), rul.getUniqueid());

            } else if (rawName.equals("ExpCondition")) {
                
            	DbAccessExpCondition con = new DbAccessExpCondition(Globals.sysUser, null, "all", "idexpattribute;idexprule", null);
            	DbAccessExpAttribute att = new DbAccessExpAttribute(Globals.sysUser, uid.get(atts.getValue("idexpattribute")), "codomain", "idexpsystem", null);
            	con.setSort(Integer.parseInt(atts.getValue("sort")));
            	con.setCondition(atts.getValue("condition"));
            	con.setOperator(atts.getValue("operator"));
            	if (att.getCodomain().equals("-72c8c14d:13cbf89ea6b:-7ffc")) { // set
                	con.setOperand(uid.get(atts.getValue("operand")));
            	} else {
                	con.setOperand(atts.getValue("operand"));
            	}
            	String uniqueid = uid.get(atts.getValue("idexprule"));
            	con.setFieldAsString("idexprule", uniqueid);
            	uniqueid = uid.get(atts.getValue("idexpattribute"));
            	con.setFieldAsString("idexpattribute", uniqueid);
            	con.putToSink(null);
  
            }
            
            
        } catch (Exception err) {
			System.out.println(Tool4Throwable.getCallStackOut(err));
        }
    }
    
    /**
     * The method <CODE>endElement</CODE> is called, if xml-token ends.
     * @param namespaceURI name space.
     * @param localName locale.
     * @param rawName name of token.
     * @exception SAXException.
     * @since 1.0
     */
    public void endElement(String namespaceURI , String localName , String rawName) throws SAXException {
        super.endElement(namespaceURI , localName , rawName);
    }
    
}
