package n4e.expertanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import n4e.core.controls.*;
import n4e.core.database.*;
import n4e.core.screen.*;
import n4e.core.tool.*;
import n4e.expertanddecision.dbaccess.DbAccessExpAttribute;
import n4e.expertanddecision.dbaccess.DbAccessExpCodomain;
import n4e.expertanddecision.dbaccess.DbAccessReadExpAttribute;
import n4e.expertanddecision.system.ExpertAndDecision;
import n4e.expertanddecision.system.Globals;

/**
 * The <CODE>ExpAttributeListSite</CODE> class ...
 * <P><B>title: AuditAndReport</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2014 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 15.10.2014 14:50:42
 * @since 1.0
 */

public class ExpAttributeListSite extends DataSite {

    private AWTableModel modExpAttributeMod;
    private JLabel lblTableRows;

    private AWButton btnNew;

    private String index_uniqueid;
    private int pnl_width; // width of the calling frame.
    private int pnl_height; // height of the calling frame.

    /**
	 * The constructor <CODE>ExpAttributeListSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car data tabbed pain.
	 * @param pnl_height the height of the car data tabbed pain.
	 * @since 1.0
	 */
	public ExpAttributeListSite(ExpertAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_hight;
			label_width = Globals.label_width;
			control_hight = Globals.control_hight;
			control_width = Globals.control_width;
			this.pnl_width = pnl_width;
			this.pnl_height = pnl_height;

			pnlScreen = createPanel(pnl_width - 25, pnl_height - 45);
			createHead(pnlScreen, Globals.message.getMessage("msg0035"));

			// calculate the width of the table columns.
			int fac = 150/4; // action columns.
			int[] column_len = new int[4];
            column_len[0] = (int) (pnl_width * (250.0 / 640.0)) - fac;
            column_len[1] = (int) (pnl_width * (130.0 / 640.0)) - fac;
            column_len[2] = (int) (pnl_width * (100.0 / 640.0)) - fac;
            column_len[3] = (int) (pnl_width * (80.0 / 640.0)) - fac;
			Hashtable column_width = t4dialog.createColumnWidthTable(column_len);

			modExpAttributeMod = createTable("ExpAttribute", pnlScreen,
					new DbAccessReadExpAttribute(Globals.sysUser),
					"name,codomain,valuetype,initialattribute", 20, 40,
					pnl_width, 90, pnl_height, 140, this.BUTTON_USE_AND_DELETE,
					null, column_width, null, Globals.fontStandard12);

			modExpAttributeMod.getTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
	            public void valueChanged(ListSelectionEvent e) {
	            	try {
	                	int select = modExpAttributeMod.getSelectedRow();
	                	if (select > -1) {
	                		Globals.curr_attribute_uniqueid = modExpAttributeMod.getIndexKey();
 	                	}
	        		} catch (Exception ex) {
	        			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
	        		}
	                
	            }
	        });

			lblTableRows = new JLabel("");
			lblTableRows.setFont(Globals.fontStandard12);
			lblTableRows.setBounds(20, pnl_height - 90, 100, label_hight);
			pnlScreen.add(lblTableRows);

			btnNew = createButton(pnlScreen, pnl_height, 90, pnl_width, 100,
	        		DataSite.BUTTON_NEW, Globals.message.getMessage("msg0006"));
			btnNew.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnNew_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnNew.setEnabled(false);

			createCurrentData(null);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>calcDimension</CODE> calculates the dimension of the controls.
	 * @param width the width of the panel.
	 * @param height the height of the panel.
	 * @since 1.0
	 */
	public void calcDimension(int width, int height) {
		super.calcDimension(width, height);
		pnlScreen.setBounds(10, 10, width - 45, height - 65);
		btnNew.calcDimension(width, height);
		lblTableRows.setBounds(20, height - 90, 100, label_hight);
	}

	/**
	 * The method <CODE>btnNew_actionPerformed</CODE> action of the btnNew.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnNew_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		fireSiteChangeEvent_1(null);
	}

	/**
	 * The method <CODE>callUseFromTable</CODE> performs the use button functionality,
	 * if use button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callUseFromTable(String table_name) throws Exception {

		Globals.info_message.setText("");

    	int select = modExpAttributeMod.getSelectedRow();
    	if (select > -1) {
    		fireSiteChangeEvent_1(modExpAttributeMod.getIndexKey());
    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>callDeleteFromTable</CODE> performs the delete button functionality,
	 * if delete button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callDeleteFromTable(String table_name) throws Exception {

    	int select = modExpAttributeMod.getSelectedRow();
    	if (select > -1) {
    		
    		// delete codomain first.
    		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
    		ArrayList cods = dsl.getDataSet(Globals.sysUser, "ExpCodomain", 
    				null, null, null, "idexpattribute='" + modExpAttributeMod.getIndexKey() + "'", 
    				null, "expertanddecision");
    		int len = cods.size();
			for (int ndx=0;ndx<len;ndx++) {
		   		DbAccessExpCodomain cod = (DbAccessExpCodomain) cods.get(ndx) ;
	    		if (cod.errCode == 0) {
	    			if (!cod.deleteInSink(null)) {
	    				t4dialog.showException(cod.errCode + "\r\n" + cod.errMess);
	    				return;
	    			}
	    		} else {
    				t4dialog.showException(cod.errCode + "\r\n" + cod.errMess);
    				return;
	    		}
			}

			DbAccessExpAttribute tab = new DbAccessExpAttribute(Globals.sysUser,
    				modExpAttributeMod.getIndexKey(), null, null, null);

    		if (tab.errCode == 0) {
    			if (!tab.deleteInSink(null)) {
    				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    				return;
    			}
    			Globals.info_message.setText(Globals.message.getMessage("msg0029"));
				createCurrentData(null);
   		} else {
				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    		}

    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>enableButtons</CODE> enable the buttons.
	 * @exception Exception
	 * @since 1.0
	 */
	public void enableButtons(boolean enable) throws Exception {
		btnNew.setEnabled(enable);
	}

	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		index_uniqueid = uniqueid;

		modExpAttributeMod.resetKeyList();

		if (Globals.curr_expsystem_uniqueid_2 == null) {
			return;
		}

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "ExpAttribute",
				"name,codomain,valuetype,initialattribute", null, null, 
				"idexpsystem='" + Globals.curr_expsystem_uniqueid_2 + "'",
				"name ASC", "expertanddecision");

    	int len = tabs.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadExpAttribute tab = (DbAccessReadExpAttribute) tabs.get(ndx);
       		if (tab.errCode == 0) {
    			modExpAttributeMod.addKey(tab.getUniqueid());

    			Object[] row = new Object[6];
                row[0] = tab.getFieldAsString("name");
                row[1] = tab.getFieldAsString("codomain");
                row[2] = tab.getFieldAsString("valuetype");
                row[3] = tab.getFieldAsString("initialattribute");
                row[4] = "";
                row[5] = "";
    			modExpAttributeMod.addRow(row);
    		}
    	}
    	lblTableRows.setText(Globals.message.getMessage("msg0031") + modExpAttributeMod.getRowCount());
	}
}
