package n4e.expertanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import n4e.core.controls.*;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.screen.*;
import n4e.core.system.*;
import n4e.core.tool.*;
import n4e.expertanddecision.dbaccess.DbAccessExpCondition;
import n4e.expertanddecision.dbaccess.DbAccessExpRule;
import n4e.expertanddecision.dbaccess.DbAccessReadExpCodomain;
import n4e.expertanddecision.dbaccess.DbAccessReadExpCondition;
import n4e.expertanddecision.dbaccess.DbAccessReadExpRule;
import n4e.expertanddecision.system.ExpertAndDecision;
import n4e.expertanddecision.system.Globals;

/**
 * The <CODE>ExpRuleDataSite</CODE> class ...
 * <P><B>title: AuditAndReport</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2014 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 15.10.2014 11:45:56
 * @since 1.0
 */

public class ExpRuleDataSite extends DataSite {

    private JTextField tfRuleID; 
    private JTextArea taDescription; 
    private JCheckBox cbEnabled;
    private JLabel lblPriorProb;
    private JTextField tfPriorProb; 
    private AWTableModel modConditionMod; 
    private JLabel lblTableRows;

    private JButton btnNewCon; 
    private JButton btnRowUp;
    private JButton btnRowDown;
    private JButton btnNew;
    private JButton btnSave;
    private JButton btnReturn;

    private DatabaseDatasetLoader dsl;

    private String curr_uniqueid;
	private long rule_sort;

	/**
	 * The constructor <CODE>ExpRuleDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public ExpRuleDataSite(ExpertAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			dsl = new DatabaseDatasetLoader();

			label_hight = Globals.label_hight;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_hight;
			control_width = Globals.control_width;

			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(590, 390));

			createHead(pnlScreen, Globals.message.getMessage("msg0041"));

			DbAccessReadExpRule tab = new DbAccessReadExpRule(Globals.sysUser);

			if (!GlobalsDialog.production && Globals.debug) {
				tfRuleID = createTextField(pnlScreen, control_column, 100);
			}

	        createLabel(pnlScreen, tab.getDefData("enabled", "label"));
	        cbEnabled = createCheckBox(pnlScreen);

			createLabel(pnlScreen, tab.getDefData("description", "label"));
	        taDescription = createTextArea(pnlScreen, 380, 60);

	        label_line += 40;

	        lblPriorProb = createLabel(pnlScreen, tab.getDefData("priorprob", "label"));
	        lblPriorProb.setVisible(false);
	        tfPriorProb = createTextField(pnlScreen);
	        tfPriorProb.setVisible(false);

	        createSeparator(pnlScreen, 550);

	        label_line += 30;
			// calculate the width of the table columns.
			int fac = 150/4; // action columns.
			int[] column_len = new int[4];
            column_len[0] = (int) (pnl_width * (100.0 / 640.0)) - fac;
            column_len[1] = (int) (pnl_width * (200.0 / 640.0)) - fac;
            column_len[2] = (int) (pnl_width * (100.0 / 640.0)) - fac;
            column_len[3] = (int) (pnl_width * (160.0 / 640.0)) - fac;
			Hashtable column_width = t4dialog.createColumnWidthTable(column_len);

			modConditionMod = createTable("Condition", pnlScreen,
					new DbAccessReadExpCondition(Globals.sysUser),
					"condition," + Globals.message.getMessage("msg0040") + ",operator,operand", 20, label_line,
					570, 0, 160, 0, this.BUTTON_USE_AND_DELETE,
					null, column_width, null, Globals.fontStandard12);
	        
			lblTableRows = new JLabel("");
			lblTableRows.setFont(Globals.fontStandard12);
			lblTableRows.setBounds(20, 350, 100, label_hight);
			pnlScreen.add(lblTableRows);

			sclScreen = createScrollPane(pnlScreen, pnl_width - 30, pnl_height - 50);

			btnNewCon = createButton(pnlScreen, 350, 370, 
					DataSite.BUTTON_NEW, Globals.message.getMessage("msg0042"));
			btnNewCon.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnNewCon_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

	        btnRowUp = createButton(pnlScreen, 350, 410, 
	        		DataSite.BUTTON_UP, Globals.message.getMessage("msg0038"));
	        btnRowUp.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnRowUp_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
	        
	        btnRowDown = createButton(pnlScreen, 350, 450, 
	        		DataSite.BUTTON_DOWN, Globals.message.getMessage("msg0039"));
	        btnRowDown.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnRowDown_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnNew = createButton(pnlScreen, 350, 490, 
					DataSite.BUTTON_NEW, Globals.message.getMessage("msg0006"));
			btnNew.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnNew_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnSave = createButton(pnlScreen, 350, 530,
					DataSite.BUTTON_SAVE, Globals.message.getMessage("msg0008"));
			btnSave.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnSave_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnReturn = createButton(pnlScreen, 350, 570,
					DataSite.BUTTON_BACK, Globals.message.getMessage("msg0030"));
			btnReturn.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnReturn_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>calcDimension</CODE> calculates the dimension of the controls.
	 * @param width the width of the panel.
	 * @param height the height of the panel.
	 * @since 1.0
	 */
	public void calcDimension(int width, int height) {
//		sclScreen.setBounds(10, 10, width - 45, height - 70);
	}

	/**
	 * The method <CODE>btnRowUp_actionPerformed</CODE> action of the btnRowUp.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnRowUp_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		int ndx = modConditionMod.getSelectedRow();
		if (ndx == -1) {
			Globals.info_message.setText(Globals.message.getMessage("msg0021"));
			return;
		}
		if (ndx == 0) {
			return; // do nothing.
		}
		String uniqueid_from = modConditionMod.getKeyNumber(ndx);
		String uniqueid_to = modConditionMod.getKeyNumber(ndx - 1);

		DbAccessExpCondition field_from = new DbAccessExpCondition(Globals.sysUser, 
				uniqueid_from, "sort", null, null);
		DbAccessExpCondition field_to = new DbAccessExpCondition(Globals.sysUser, 
				uniqueid_to, "sort", null, null);
		long temp_num = field_from.getSort();
		field_from.setSort(field_to.getSort());
		field_to.setSort(temp_num);
		if (!field_from.putToSink(null)) {
			t4dialog.showException("from " + field_from.errCode + "\r\n" + field_from.errMess);
		}
		if (!field_to.putToSink(null)) {
			t4dialog.showException("to " + field_to.errCode + "\r\n" + field_to.errMess);
			
		}
		createCurrentData(curr_uniqueid);
		modConditionMod.setIndexKey(uniqueid_from);
	}
	
	/**
	 * The method <CODE>btnRowDown_actionPerformed</CODE> action of the btnRowDown.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnRowDown_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		int ndx = modConditionMod.getSelectedRow();
		if (ndx == -1) {
			Globals.info_message.setText(Globals.message.getMessage("msg0021"));
			return;
		}
		if (ndx == modConditionMod.getLastRow()) {
			return; // do nothing.
		}
		String uniqueid_from = modConditionMod.getKeyNumber(ndx);
		String uniqueid_to = modConditionMod.getKeyNumber(ndx + 1);
		DbAccessExpCondition field_from = new DbAccessExpCondition(Globals.sysUser, 
				uniqueid_from, "sort", null, null);
		DbAccessExpCondition field_to = new DbAccessExpCondition(Globals.sysUser, 
				uniqueid_to, "sort", null, null);
		long temp_num = field_from.getSort();
		field_from.setSort(field_to.getSort());
		field_to.setSort(temp_num);
		if (!field_from.putToSink(null)) {
			t4dialog.showException("from " + field_from.errCode + "\r\n" + field_from.errMess);
			
		}
		if (!field_to.putToSink(null)) {
			t4dialog.showException("to " + field_to.errCode + "\r\n" + field_to.errMess);
			
		}
		createCurrentData(curr_uniqueid);
		modConditionMod.setIndexKey(uniqueid_from);
	}

	/**
	 * The method <CODE>btnNewCon_actionPerformed</CODE> action of the btnNewCon.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnNewCon_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		sclScreen.setVisible(false);
		fireSiteChangeEvent_2(null);
	}

	/**
	 * The method <CODE>btnNew_actionPerformed</CODE> action of the btnNew.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnNew_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
        createNewData();
	}

	/**
	 * The method <CODE>btnSave_actionPerformed</CODE> action of the btnSave.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnSave_actionPerformed(ActionEvent e) throws Exception {

		DbAccessExpRule tab = new DbAccessExpRule(Globals.sysUser, curr_uniqueid,
				"all", null, null);

		if (tab.errCode == 0 || tab.errCode == tab.ERROR_0001 || tab.errCode == tab.ERROR_0002) {

			tab.errCode = 0;
			tab.setRuleid(rule_sort);
			if (!GlobalsDialog.production && Globals.debug) {
				tab.setRuleid(Integer.parseInt(tfRuleID.getText()));
			}
			tab.setFieldAsString("description", taDescription.getText());
			if (Globals.curr_expsystem_type == Globals.STOCHASTIC) {
				tab.setFieldAsString("priorprob", tfPriorProb.getText());
		    }
			tab.setFieldAsString("idexpsystem", Globals.curr_expsystem_uniqueid_2);
            if (cbEnabled.isSelected()) {
                tab.setEnabled(1);
            } else {
                tab.setEnabled(0);
            }

            if (tab.errCode != 0) {
				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess, false);
				return;
			} else if (!tab.putToSink(null)) {
				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess, false);
				return;
			}

			Globals.info_message.setText(Globals.message.getMessage("msg0022"));
			curr_uniqueid = tab.getUniqueid();
			Globals.curr_rule_uniqueid = curr_uniqueid;
			createCurrentData(curr_uniqueid);

		} else {
			t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
		}

	}

	/**
	 * The method <CODE>btnReturn_actionPerformed</CODE> action of the btnReturn.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnReturn_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
    	sclScreen.setVisible(false);
		fireSiteChangeEvent_1(curr_uniqueid);
	}

	/**
	 * The method <CODE>callUseFromTable</CODE> performs the use button functionality, 
	 * if use button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callUseFromTable(String table_name) throws Exception {
		
		Globals.info_message.setText("");
    	
    	int select = modConditionMod.getSelectedRow();
    	if (select > -1) {
    		sclScreen.setVisible(false);
    		fireSiteChangeEvent_2(modConditionMod.getIndexKey());
    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
    	
	}

	/**
	 * The method <CODE>callDeleteFromTable</CODE> performs the delete button functionality, 
	 * if delete button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callDeleteFromTable(String table_name) throws Exception {
		
    	int select = modConditionMod.getSelectedRow();
    	if (select > -1) {
    		
    		// read the author.
    		DbAccessExpCondition con = new DbAccessExpCondition(Globals.sysUser, 
    				modConditionMod.getIndexKey(), null, null, null);
    		
    		if (con.errCode == 0) {
    			if (!con.deleteInSink(null)) {
    				t4dialog.showException(con.errCode + "\r\n" + con.errMess);
    				return;
    			}
    			Globals.info_message.setText(Globals.message.getMessage("msg0029"));
				createCurrentData(curr_uniqueid);
    		} else {
				t4dialog.showException(con.errCode + "\r\n" + con.errMess);
    		}
    		
    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>createNewData</CODE> resets the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createNewData() throws Exception {
        sclScreen.setVisible(true);
        curr_uniqueid = null;
		rule_sort = Globals.t4base.getMaxNumberOfRules(Globals.curr_expsystem_uniqueid_2) + 1;
        if (!GlobalsDialog.production && Globals.debug) {
        	tfRuleID.setText(Long.toString(rule_sort));
        }
        cbEnabled.setSelected(true);
	    taDescription.setText("");
        if (Globals.curr_expsystem_type == Globals.STOCHASTIC) {
	        lblPriorProb.setVisible(true);
        	tfPriorProb.setVisible(true);
	    	tfPriorProb.setText(Globals.t4n.getNumber2String(0.0, 2, Globals.sysUser.getUserLocale()));
	    } else if (Globals.curr_expsystem_type == Globals.DETERMINISTIC) {
	        lblPriorProb.setVisible(false);
        	tfPriorProb.setVisible(false);
	    }
		modConditionMod.resetKeyList();
	    btnNewCon.setEnabled(false); 
	    btnRowUp.setEnabled(false);
	    btnRowDown.setEnabled(false);

	}

	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		sclScreen.setVisible(true);
		curr_uniqueid = uniqueid;
	    btnNewCon.setEnabled(true); 
	    btnRowUp.setEnabled(true);
	    btnRowDown.setEnabled(true);

		DbAccessReadExpRule tab = new DbAccessReadExpRule(Globals.sysUser,
				uniqueid, "all", null, null);
		if (tab.errCode != 0) {
			t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
			return;
		}

        if (!GlobalsDialog.production && Globals.debug) {
        	tfRuleID.setText(Long.toString(tab.getRuleid()));
        }

        if (tab.getEnabled() == 1) {
            cbEnabled.setSelected(true);
        } else {
            cbEnabled.setSelected(false);
        }
	    rule_sort = tab.getRuleid();

		taDescription.setText(tab.getDescription());
	    taDescription.setSelectionStart(0);
	    taDescription.setSelectionEnd(0);
	    
        if (Globals.curr_expsystem_type == Globals.STOCHASTIC) {
	        lblPriorProb.setVisible(true);
        	tfPriorProb.setVisible(true);
	    	tfPriorProb.setText(tab.getFieldAsString("priorprob"));
	    } else if (Globals.curr_expsystem_type == Globals.DETERMINISTIC) {
	        lblPriorProb.setVisible(false);
        	tfPriorProb.setVisible(false);
	    }

		// fetch conditions. 
		ArrayList cons = dsl.getDataSetRead(Globals.sysUser, "ExpCondition", 
				"condition,operator,operand", "idexpattribute(name,codomain)", null, 
				"idexprule='" + curr_uniqueid + "'", 
				"sort", "auditandreport");
    	
		modConditionMod.resetKeyList();
    	
    	int len = cons.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadExpCondition con = (DbAccessReadExpCondition) cons.get(ndx) ;
    		if (con.errCode == 0) {
    			modConditionMod.addKey(con.getUniqueid());
    			Object[] row = new Object[6]; 
    			row[0] = con.getFieldAsString("condition"); 
    			row[1] = con.getMyDbAccessReadExpAttribute().getName();
    			row[2] = con.getFieldAsString("operator");
    			String codomain = con.getMyDbAccessReadExpAttribute().getCodomain();
    			if (codomain.equals("-72c8c14d:13cbf89ea6b:-7ffc")) { // set.
    	    		DbAccessReadExpCodomain cod = new DbAccessReadExpCodomain(Globals.sysUser, con.getOperand(), "text", null, null);
    				row[3] = cod.getText();
    			} else {
        			row[3] = con.getOperand();
    			}
    			row[4] = "";
    			row[5] = "";
    			modConditionMod.addRow(row);
    		}
    	}
    	lblTableRows.setText(Globals.message.getMessage("msg0031") + modConditionMod.getRowCount());

	}
}
