package n4e.expertanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import n4e.core.controls.*;
import n4e.core.database.*;
import n4e.core.screen.*;
import n4e.core.tool.*;
import n4e.expertanddecision.dbaccess.DbAccessExpCondition;
import n4e.expertanddecision.dbaccess.DbAccessExpRule;
import n4e.expertanddecision.dbaccess.DbAccessReadExpRule;
import n4e.expertanddecision.system.ExpertAndDecision;
import n4e.expertanddecision.system.Globals;

/**
 * The <CODE>ExpRuleListSite</CODE> class ...
 * <P><B>title: AuditAndReport</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2014 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 15.10.2014 11:46:01
 * @since 1.0
 */

public class ExpRuleListSite extends DataSite {

    private AWTableModel modExpRuleMod;
    private JLabel lblTableRows;

    private AWButton btnNumbering; 
    private AWButton btnSetEnabled; 
    private AWButton btnRowUp;
    private AWButton btnRowDown;
    private AWButton btnNew;

    private String index_uniqueid;
    private int pnl_width; // width of the calling frame.
    private int pnl_height; // height of the calling frame.

    /**
	 * The constructor <CODE>ExpRuleListSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car data tabbed pain.
	 * @param pnl_height the height of the car data tabbed pain.
	 * @since 1.0
	 */
	public ExpRuleListSite(ExpertAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_hight;
			label_width = Globals.label_width;
			control_hight = Globals.control_hight;
			control_width = Globals.control_width;
			this.pnl_width = pnl_width;
			this.pnl_height = pnl_height;

			pnlScreen = createPanel(pnl_width - 25, pnl_height - 45);
			createHead(pnlScreen, Globals.message.getMessage("msg0034"));

			// calculate the width of the table columns.
			int fac = 150/2; // action columns.
			int[] column_len = new int[3];
            column_len[0] = (int) (pnl_width * (30.0 / 640.0)) - fac;
            column_len[1] = (int) (pnl_width * (330.0 / 640.0)) - fac;
            column_len[2] = (int) (pnl_width * (150.0 / 640.0)) - fac;
			Hashtable column_width = t4dialog.createColumnWidthTable(column_len);

			modExpRuleMod = createTable("ExpRule", pnlScreen,
					new DbAccessReadExpRule(Globals.sysUser),
					"ruleid,description,enabled", 20, 40,
					pnl_width, 90, pnl_height, 140, this.BUTTON_USE_AND_DELETE,
					null, column_width, null, Globals.fontStandard12);

			modExpRuleMod.getTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
	            public void valueChanged(ListSelectionEvent e) {
	            	try {
	                	int select = modExpRuleMod.getSelectedRow();
	                	if (select > -1) {
	                		Globals.curr_rule_uniqueid = modExpRuleMod.getIndexKey();
 	                	}
	        		} catch (Exception ex) {
	        			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
	        		}
	                
	            }
	        });

			lblTableRows = new JLabel("");
			lblTableRows.setFont(Globals.fontStandard12);
			lblTableRows.setBounds(20, pnl_height - 90, 100, label_hight);
			pnlScreen.add(lblTableRows);

			btnNumbering = createButton(pnlScreen, pnl_height, 90, pnl_width, 260, 
	        		DataSite.BUTTON_SORT, Globals.message.getMessage("msg0092"));
	        btnNumbering.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnNumbering_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
	        btnNumbering.setEnabled(false);

			btnSetEnabled = createButton(pnlScreen, pnl_height, 90, pnl_width, 220, 
	        		DataSite.BUTTON_CHECK, Globals.message.getMessage("msg0037"));
			btnSetEnabled.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnSetEnabled_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnSetEnabled.setEnabled(false);

	        btnRowUp = createButton(pnlScreen, pnl_height, 90, pnl_width, 180, 
	        		DataSite.BUTTON_UP, Globals.message.getMessage("msg0038"));
	        btnRowUp.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnRowUp_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
	        btnRowUp.setEnabled(false);
	        
	        btnRowDown = createButton(pnlScreen, pnl_height, 90, pnl_width, 140, 
	        		DataSite.BUTTON_DOWN, Globals.message.getMessage("msg0039"));
	        btnRowDown.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnRowDown_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
	        btnRowDown.setEnabled(false);

			btnNew = createButton(pnlScreen, pnl_height, 90, pnl_width, 100,
	        		DataSite.BUTTON_NEW, Globals.message.getMessage("msg0006"));
			btnNew.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnNew_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnNew.setEnabled(false);

			createCurrentData(null);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>calcDimension</CODE> calculates the dimension of the controls.
	 * @param width the width of the panel.
	 * @param height the height of the panel.
	 * @since 1.0
	 */
	public void calcDimension(int width, int height) {
		super.calcDimension(width, height);
		pnlScreen.setBounds(10, 10, width - 45, height - 65);
		btnSetEnabled.calcDimension(width, height);
		btnRowUp.calcDimension(width, height);
		btnRowDown.calcDimension(width, height);
		btnNew.calcDimension(width, height);
		lblTableRows.setBounds(20, height - 90, 100, label_hight);
	}

	/**
	 * The method <CODE>btnNumbering_actionPerformed</CODE> action of the btnNumbering.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnNumbering_actionPerformed(ActionEvent e) throws Exception {
		Globals.t4base.numberingAllRules(Globals.curr_expsystem_uniqueid_2);
		createCurrentData(index_uniqueid);
		Globals.info_message.setText(Globals.message.getMessage("msg0094"));
	}
	/**
	 * 
	 * The method <CODE>btnSetEnabled_actionPerformed</CODE> action of the btnSetEnabled.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnSetEnabled_actionPerformed(ActionEvent e) throws Exception {
		Globals.t4base.enableAllRules(Globals.curr_expsystem_uniqueid_2);
		createCurrentData(index_uniqueid);
		Globals.info_message.setText(Globals.message.getMessage("msg0093"));
	}

	/**
	 * The method <CODE>btnRowUp_actionPerformed</CODE> action of the btnRowUp.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnRowUp_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		int ndx = modExpRuleMod.getSelectedRow();
		if (ndx == -1) {
			Globals.info_message.setText(Globals.message.getMessage("msg0021"));
			return;
		}
		if (ndx == 0) {
			return; // do nothing.
		}
		String uniqueid_from = modExpRuleMod.getKeyNumber(ndx);
		String uniqueid_to = modExpRuleMod.getKeyNumber(ndx - 1);

		DbAccessExpRule field_from = new DbAccessExpRule(Globals.sysUser, 
				uniqueid_from, "ruleid", null, null);
		DbAccessExpRule field_to = new DbAccessExpRule(Globals.sysUser, 
				uniqueid_to, "ruleid", null, null);
		long temp_num = field_from.getRuleid();
		field_from.setRuleid(field_to.getRuleid());
		field_to.setRuleid(temp_num);
		if (!field_from.putToSink(null)) {
			t4dialog.showException("from " + field_from.errCode + "\r\n" + field_from.errMess);
		}
		if (!field_to.putToSink(null)) {
			t4dialog.showException("to " + field_to.errCode + "\r\n" + field_to.errMess);
			
		}
		createCurrentData(index_uniqueid);
		modExpRuleMod.setIndexKey(uniqueid_from);
	}
	
	/**
	 * The method <CODE>btnRowDown_actionPerformed</CODE> action of the btnRowDown.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnRowDown_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		int ndx = modExpRuleMod.getSelectedRow();
		if (ndx == -1) {
			Globals.info_message.setText(Globals.message.getMessage("msg0021"));
			return;
		}
		if (ndx == modExpRuleMod.getLastRow()) {
			return; // do nothing.
		}
		String uniqueid_from = modExpRuleMod.getKeyNumber(ndx);
		String uniqueid_to = modExpRuleMod.getKeyNumber(ndx + 1);
		DbAccessExpRule field_from = new DbAccessExpRule(Globals.sysUser, 
				uniqueid_from, "ruleid", null, null);
		DbAccessExpRule field_to = new DbAccessExpRule(Globals.sysUser, 
				uniqueid_to, "ruleid", null, null);
		long temp_num = field_from.getRuleid();
		field_from.setRuleid(field_to.getRuleid());
		field_to.setRuleid(temp_num);
		if (!field_from.putToSink(null)) {
			t4dialog.showException("from " + field_from.errCode + "\r\n" + field_from.errMess);
			
		}
		if (!field_to.putToSink(null)) {
			t4dialog.showException("to " + field_to.errCode + "\r\n" + field_to.errMess);
			
		}
		createCurrentData(index_uniqueid);
		modExpRuleMod.setIndexKey(uniqueid_from);
	}

	/**
	 * The method <CODE>btnNew_actionPerformed</CODE> action of the btnNew.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnNew_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		fireSiteChangeEvent_1(null);
	}

	/**
	 * The method <CODE>callUseFromTable</CODE> performs the use button functionality,
	 * if use button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callUseFromTable(String table_name) throws Exception {

		Globals.info_message.setText("");

    	int select = modExpRuleMod.getSelectedRow();
    	if (select > -1) {
    		fireSiteChangeEvent_1(modExpRuleMod.getIndexKey());
    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>callDeleteFromTable</CODE> performs the delete button functionality,
	 * if delete button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callDeleteFromTable(String table_name) throws Exception {

    	int select = modExpRuleMod.getSelectedRow();
    	if (select > -1) {
    		
    	    // delete conditions first. 
    		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
    		ArrayList cons = dsl.getDataSet(Globals.sysUser, "ExpCondition", 
    				null, null, null, "idexprule='" + modExpRuleMod.getIndexKey() + "'", 
    				null, "auditandreport");
    		int len = cons.size();
			for (int ndx=0;ndx<len;ndx++) {
	    		DbAccessExpCondition con = (DbAccessExpCondition) cons.get(ndx) ;
	    		if (con.errCode == 0) {
	    			if (!con.deleteInSink(null)) {
	    				t4dialog.showException(con.errCode + "\r\n" + con.errMess);
	    				return;
	    			}
	    		}
			}
    		
    		DbAccessExpRule tab = new DbAccessExpRule(Globals.sysUser,
    				modExpRuleMod.getIndexKey(), null, null, null);

    		if (tab.errCode == 0) {
    			if (!tab.deleteInSink(null)) {
    				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    				return;
    			}
    			Globals.info_message.setText(Globals.message.getMessage("msg0029"));
				createCurrentData(null);
    		} else {
				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    		}

    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>enableButtons</CODE> enable the buttons.
	 * @exception Exception
	 * @since 1.0
	 */
	public void enableButtons(boolean enable) throws Exception {
		btnNumbering.setEnabled(enable);
		btnSetEnabled.setEnabled(enable);
        btnRowUp.setEnabled(enable);
        btnRowDown.setEnabled(enable);
		btnNew.setEnabled(enable);
	}
	
	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		index_uniqueid = uniqueid;

		modExpRuleMod.resetKeyList();

		if (Globals.curr_expsystem_uniqueid_2 == null) {
			return;
		}
		
		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "ExpRule",
				"ruleid,enabled,description", null, null, "idexpsystem='" + Globals.curr_expsystem_uniqueid_2 + "'",
				"ruleid ASC", "expertanddecision");

    	int len = tabs.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadExpRule tab = (DbAccessReadExpRule) tabs.get(ndx);
       		if (tab.errCode == 0) {
    			modExpRuleMod.addKey(tab.getUniqueid());

     			Object[] row = new Object[5];
    			String desc = tab.getDescription();
    			if (desc != null && desc.length() > 50) {
    				desc = desc.substring(0, 50);
    			}
    			row[0] = tab.getRuleid(); 
                row[1] = desc;
                row[2] = tab.getFieldAsString("enabled");
                row[3] = "";
                row[4] = "";
    			modExpRuleMod.addRow(row);
    		}
    	}
    	lblTableRows.setText(Globals.message.getMessage("msg0031") + modExpRuleMod.getRowCount());
	}
}
