package n4e.expertanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import n4e.core.controls.*;
import n4e.core.database.*;
import n4e.core.screen.*;
import n4e.core.system.GlobalsDialog;
import n4e.core.tool.*;
import n4e.expertanddecision.dbaccess.DbAccessExpAttribute;
import n4e.expertanddecision.dbaccess.DbAccessExpCodomain;
import n4e.expertanddecision.dbaccess.DbAccessExpCondition;
import n4e.expertanddecision.dbaccess.DbAccessExpRule;
import n4e.expertanddecision.dbaccess.DbAccessExpSystem;
import n4e.expertanddecision.dbaccess.DbAccessReadExpSystem;
import n4e.expertanddecision.list.DiseaseElement;
import n4e.expertanddecision.list.RuleProbability;
import n4e.expertanddecision.list.SymptomElement;
import n4e.expertanddecision.saxparser.ImportExpSystem;
import n4e.expertanddecision.system.ExpertAndDecision;
import n4e.expertanddecision.system.Globals;

/**
 * The <CODE>ExpSystemListSite</CODE> class ...
 * <P><B>title: AuditAndReport</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2014 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 15.10.2014 11:11:41
 * @since 1.0
 */

public class ExpSystemListSite extends DataSite {

    private AWTableModel modExpSystemMod;
    private JLabel lblTableRows;

    private JButton btnImport;
    private AWButton btnNew;

    private String index_uniqueid;
    private int pnl_width; // width of the calling frame.
    private int pnl_height; // height of the calling frame.
    
    /**
	 * The constructor <CODE>ExpSystemListSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car data tabbed pain.
	 * @param pnl_height the height of the car data tabbed pain.
	 * @since 1.0
	 */
	public ExpSystemListSite(ExpertAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {
			
			label_hight = Globals.label_hight;
			label_width = Globals.label_width;
			control_hight = Globals.control_hight;
			control_width = Globals.control_width;
			this.pnl_width = pnl_width;
			this.pnl_height = pnl_height;

			pnlScreen = createPanel(pnl_width - 25, pnl_height - 45);
			createHead(pnlScreen, Globals.message.getMessage("msg0033"));

			// calculate the width of the table columns.
			int fac = 150/1; // action columns.
			int[] column_len = new int[1];
            column_len[0] = (int) (pnl_width * (570.0 / 640.0)) - fac;
			Hashtable column_width = t4dialog.createColumnWidthTable(column_len);

			modExpSystemMod = createTable("ExpSystem", pnlScreen,
					new DbAccessReadExpSystem(Globals.sysUser),
					"name", 20, 40,
					pnl_width, 70, pnl_height, 120, this.BUTTON_USE_AND_DELETE,
					null, column_width, null, Globals.fontStandard12);

			modExpSystemMod.getTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
	            public void valueChanged(ListSelectionEvent e) {
	            	try {
	                	int select = modExpSystemMod.getSelectedRow();
	                	if (select > -1) {
	                		Globals.curr_expsystem_uniqueid_2 = modExpSystemMod.getIndexKey();
	                		DbAccessReadExpSystem es = new DbAccessReadExpSystem(Globals.sysUser, modExpSystemMod.getIndexKey(), "type", null, null);
	                		Globals.curr_expsystem_type = es.getType();
	                		Globals.ead.rls.createCurrentData(null);
	                		Globals.ead.rls.enableButtons(true);
	                		Globals.ead.als.createCurrentData(null);
	                		Globals.ead.als.enableButtons(true);
	                		Globals.info_message.setText("");
 	                	}
	        		} catch (Exception ex) {
	        			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
	        		}
	                
	            }
	        });


			lblTableRows = new JLabel("");
			lblTableRows.setFont(Globals.fontStandard12);
			lblTableRows.setBounds(20, pnl_height - 70, 100, label_hight);
			pnlScreen.add(lblTableRows);

			btnImport = createButton(pnlScreen, pnl_height, 70, pnl_width, 120, 
					DataSite.BUTTON_IMPORT, Globals.message.getMessage("msg0069"));
			btnImport.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnImport_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnNew = createButton(pnlScreen, pnl_height, 70, pnl_width, 80,
	        		DataSite.BUTTON_NEW, Globals.message.getMessage("msg0006"));
			btnNew.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnNew_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			createCurrentData(null);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>calcDimension</CODE> calculates the dimension of the controls.
	 * @param width the width of the panel.
	 * @param height the height of the panel.
	 * @since 1.0
	 */
	public void calcDimension(int width, int height) {
		super.calcDimension(width, height);
		pnlScreen.setBounds(10, 10, width - 25, height - 45);
		btnNew.calcDimension(width, height);
		lblTableRows.setBounds(20, height - 70, 100, label_hight);
	}

	/**
	 * The method <CODE>btnImport_actionPerformed</CODE> action of the btnImport.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnImport_actionPerformed(ActionEvent e) throws Exception {
    	int val = t4dialog.showDialog(Globals.message.getMessage("msg0071"));
    	if (val == 0) {

            File genDir = new File(Globals.t4f.getFullPathToSystem("data"));
            String[] files = genDir.list();
            if (files != null && files.length > 0) {
                int len = files.length;
                for (int i = 0; i < len; i++) {
                    if (files[i] != null && files[i].length() > 0) {
                        if (files[i].endsWith(".xml") || files[i].endsWith(".XML")) {
                            if (!files[i].startsWith("KeySet_")) {
                        		ImportExpSystem imp = new ImportExpSystem(Globals.sysUser);
                        		imp.parseDocument(Globals.t4f.getFullPathToSystem("data"), files[i]);
                    	   		Globals.info_message.setText(Globals.message.getMessage("msg0074"));
                    			createCurrentData(null);
                        		break;
                            }
                        }
                    }
                }
            }   		
    	}
	}
	
	/**
	 * The method <CODE>btnNew_actionPerformed</CODE> action of the btnNew.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnNew_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		fireSiteChangeEvent_1(null);
	}

	/**
	 * The method <CODE>callUseFromTable</CODE> performs the use button functionality,
	 * if use button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callUseFromTable(String table_name) throws Exception {

		Globals.info_message.setText("");

    	int select = modExpSystemMod.getSelectedRow();
    	if (select > -1) {
    		fireSiteChangeEvent_1(modExpSystemMod.getIndexKey());
    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>callDeleteFromTable</CODE> performs the delete button functionality,
	 * if delete button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callDeleteFromTable(String table_name) throws Exception {

    	int select = modExpSystemMod.getSelectedRow();
    	if (select > -1) {
   
    		Globals.t4base.deleteAllAtts(modExpSystemMod.getIndexKey());
    		Globals.t4base.deleteAllRules(modExpSystemMod.getIndexKey());

    		// read the audit.
    		DbAccessExpSystem tab = new DbAccessExpSystem(Globals.sysUser,
    				modExpSystemMod.getIndexKey(), null, null, null);

    		if (tab.errCode == 0) {
    			if (!tab.deleteInSink(null)) {
    				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    				return;
    			}
    			Globals.info_message.setText(Globals.message.getMessage("msg0029"));
				createCurrentData(null);
				Globals.curr_expsystem_uniqueid_2 = null;
        		Globals.ead.rls.createCurrentData(null);
        		Globals.ead.als.createCurrentData(null);
        		Globals.ead.rls.enableButtons(false);
        		Globals.ead.als.enableButtons(false);
				
   		} else {
				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    		}

    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		index_uniqueid = uniqueid;

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "ExpSystem",
				"name", null, null, null,
				"name ASC", "expertanddecision");

		modExpSystemMod.resetKeyList();

    	int len = tabs.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadExpSystem tab = (DbAccessReadExpSystem) tabs.get(ndx);
       		if (tab.errCode == 0) {
    			modExpSystemMod.addKey(tab.getUniqueid());

    			Object[] row = new Object[3];
                row[0] = tab.getFieldAsString("name");
                row[1] = "";
                row[2] = "";
    			modExpSystemMod.addRow(row);
    		}
    	}
    	lblTableRows.setText(Globals.message.getMessage("msg0031") + modExpSystemMod.getRowCount());
    	modExpSystemMod.setIndexKey(index_uniqueid);
	}
}
