package n4e.expertanddecision.system;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import n4e.core.database.DatabaseManager;
import n4e.core.system.GNULicense;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.Help;
import n4e.core.system.Info;
import n4e.core.system.MainDialog;
import n4e.core.system.Options;
import n4e.core.system.SystemService;
import n4e.core.tool.Tool4File;
import n4e.core.tool.Tool4Throwable;
import n4e.expertanddecision.screen.ExpApplicationDataSite;
import n4e.expertanddecision.screen.ExpAttributeDataSite;
import n4e.expertanddecision.screen.ExpAttributeListSite;
import n4e.expertanddecision.screen.ExpCodomainDataSite;
import n4e.expertanddecision.screen.ExpConditionDataSite;
import n4e.expertanddecision.screen.ExpParameterDataSite;
import n4e.expertanddecision.screen.ExpRuleDataSite;
import n4e.expertanddecision.screen.ExpRuleListSite;
import n4e.expertanddecision.screen.ExpSystemDataSite;
import n4e.expertanddecision.screen.ExpSystemListSite;
import n4e.expertanddecision.screen.InfoBarDataSite;
import n4e.expertanddecision.screen.ProjectDataSite;
import n4e.expertanddecision.screen.ProjectListSite;
import n4e.expertanddecision.tool.Tool4MyDialog;

/**
 * The <CODE>ExpertAndDecision</CODE> class ...
 * <P><B>title: ExpertAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2015 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 26.04.2015 16:19:05
 * @since 1.0
 */

public class ExpertAndDecision extends MainDialog implements ChangeListener {

	private JTabbedPane tpExpertSystem;

	private JPanel pnlProject;
	private ProjectListSite pls;
	private ProjectDataSite pds;

	private JPanel pnlExpertSystem;
	private ExpSystemListSite els;
	private ExpSystemDataSite eds;

	private JTabbedPane tpExpertSystemData;
	
	private JPanel pnlExpApplicationDataSite;
	public ExpApplicationDataSite eads;

	private JPanel pnlExpParameterDataSite;
	public ExpParameterDataSite pads;

	private JTabbedPane tpRuleAndAttribute;
	
	private JPanel pnlRule;
	public ExpRuleListSite rls;
	private ExpRuleDataSite rds;
	private ExpConditionDataSite ecds;

	private JPanel pnlAttribute;
	public ExpAttributeListSite als;
	private ExpAttributeDataSite ads;
	private ExpCodomainDataSite cds;

	private JPanel pnlInfoBar;
	public InfoBarDataSite ibds;

	private DatabaseManager dbm;

	private Tool4MyDialog t4md;
	private Tool4File t4f;

	/**
	 * The constructor <CODE>ExpertAndDecision</CODE> creates a new instance(object) of the class.
	 * @param
	 * @exception
	 * @see package.class#member label
	 * @since 1.0
	 */
	public ExpertAndDecision(String program_name, ImageIcon intro_icon, String copyright_add, 
			String database_name) {

		super(program_name, "0.1", intro_icon, copyright_add, "2015", database_name, 
				Globals.screen_height, Globals.screen_width, true, true);

		try {
			
			Globals.doAfterInitials();
			t4md = new Tool4MyDialog();
			t4f = new Tool4File();
			Image icon = t4md.getImagePure("app.jpg");
			setIconImage(icon);
			
			setMinimumSize(new Dimension(640, 480));
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			if (screen.width + 20 >= Globals.screen_width && screen.height - 40 <= Globals.screen_height) {
				setExtendedState(Frame.MAXIMIZED_BOTH);
			} else if (Globals.screen_width == 0 && Globals.screen_height == 0) {
				Globals.screen_width = screen.width;
				Globals.screen_height = screen.height;
				setExtendedState(Frame.MAXIMIZED_BOTH);
			}
				
			windowInit();
			
			this.addComponentListener(new ComponentAdapter() {
				public void componentResized(ComponentEvent e) {
					Dimension screen_size = getSize();
					Globals.screen_width = screen_size.width;
					Globals.screen_height = screen_size.height;
					Globals.has_changed = true;
	            }
	        });

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
		
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @exception Exception.
	 * @since 1.0
	 */
	private void windowInit() throws Exception {
		
		try {

			if (GlobalsDialog.production) {
				setSystemOutput("n4esystem", "log");
			}
			
			Globals.ead = this;
			
			miOptionItem = getMenuItem(menu_extras, null, Globals.message.getMessage("msg0001") + "...", 
					Globals.message.getMessage("msg0001").charAt(0), null, null);  

			miHelpItem = getMenuItem(menu_extras, null, GlobalsDialog.message.getMessage("msg0008") + "...", 'i', null, null); 
			
			setIntroMessage("system");
			new SystemService(null);
			SystemService.setClientPackageName("expertanddecision");
			SystemService.setDefinitions(GlobalsDialog.language_code);
			if (!initialiseDatabase()) {
				System.exit(0);
			}

			// add the controls.
			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 80, screen.width, 20);
			c.setSize(new Dimension(screen.width, screen.height));
			c.setBackground(Color.LIGHT_GRAY);
			
			// calculate the screen geometry.
			int expPnlWidth = screen.width / 2 - 20;
			int expPnlHeight = (int) (screen.height * 0.700);

//			int expDataOffSetHeight = expPnlHeight + 14;
//			int expDataWidth = screen.width / 2;
//			int expDataHeight = (int) (screen.height * 0.618) - 172;
			int expDataOffSetHeight = expPnlHeight + 14;
			int expDataOffSetWidth = expPnlWidth + 20;
			int expDataWidth = screen.width / 2 - 20;
//			int expDataHeight = (int) (screen.height * 0.618) - 172;
			int expDataHeight = (int) (screen.height * 0.700);
			
			int infoOffSetHeight = screen.height - 150;
			int infoWidth = screen.width - 35;
			int infoHeight = 60;

			setIntroMessage("expert and decision");

			tpExpertSystem = new JTabbedPane();
			tpExpertSystem.setBounds(10, 10, expPnlWidth, expPnlHeight);
			tpExpertSystem.addChangeListener(this);
			c.add(tpExpertSystem);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpExpertSystem.addTab(Globals.message.getMessage("msg0048"), pnlProject);

			pls = new ProjectListSite(this, expPnlWidth, expPnlHeight);
			pnlProject.add(pls.pnlScreen);

			pds = new ProjectDataSite(this, expPnlWidth, expPnlHeight);
			pds.sclScreen.setVisible(false);
			pnlProject.add(pds.sclScreen);

			if (!GlobalsDialog.production) {
				pnlExpertSystem = new JPanel();
				pnlExpertSystem.setLayout(null);
				tpExpertSystem.addTab(Globals.message.getMessage("msg0033"), pnlExpertSystem);
	
				els = new ExpSystemListSite(this, expPnlWidth, expPnlHeight);
				pnlExpertSystem.add(els.pnlScreen);
	
				eds = new ExpSystemDataSite(this, expPnlWidth, expPnlHeight);
				eds.sclScreen.setVisible(false);
				pnlExpertSystem.add(eds.sclScreen);
			}

			setIntroMessage("expert systen data");
			tpExpertSystemData = new JTabbedPane();
//			tpExpertSystemData.setBounds(10, expDataOffSetHeight, expDataWidth, expDataHeight);
			tpExpertSystemData.setBounds(expDataOffSetWidth, 10, expDataWidth, expDataHeight);
			tpExpertSystemData.addChangeListener(this);
			c.add(tpExpertSystemData);

			pnlExpApplicationDataSite = new JPanel();	
			pnlExpApplicationDataSite.setBounds(10, 10, expDataWidth, expDataHeight);
			tpExpertSystemData.addTab(Globals.message.getMessage("msg0082"), pnlExpApplicationDataSite);
			
			eads = new ExpApplicationDataSite(this, expDataWidth, expDataHeight);
			pnlExpApplicationDataSite.add(eads.sclScreen);

			pnlExpParameterDataSite = new JPanel();	
			pnlExpParameterDataSite.setBounds(10, expDataOffSetHeight, expDataWidth, expDataHeight);
			tpExpertSystemData.addTab(Globals.message.getMessage("msg0064"), pnlExpParameterDataSite);

			pads = new ExpParameterDataSite(this, expDataWidth, expDataHeight);
			pnlExpParameterDataSite.add(pads.sclScreen);

			if (!GlobalsDialog.production) {
				setIntroMessage("rules and attributes");
				tpRuleAndAttribute = new JTabbedPane();
//				tpRuleAndAttribute.setBounds(10, expDataOffSetHeight, expDataWidth, expDataHeight);
				tpRuleAndAttribute.setBounds(expDataOffSetWidth, 10, expDataWidth, expDataHeight);
				tpRuleAndAttribute.setVisible(false);
				c.add(tpRuleAndAttribute);
	
				pnlRule = new JPanel();
				pnlRule.setLayout(null);
				tpRuleAndAttribute.addTab(Globals.message.getMessage("msg0034"), pnlRule);
	
				rls = new ExpRuleListSite(this, expDataWidth, expDataHeight);
				pnlRule.add(rls.pnlScreen);
	
				rds = new ExpRuleDataSite(this, expDataWidth, expDataHeight);
				rds.sclScreen.setVisible(false);
				pnlRule.add(rds.sclScreen);
	
				ecds = new ExpConditionDataSite(this, expDataWidth, expDataHeight);
				ecds.sclScreen.setVisible(false);
				pnlRule.add(ecds.sclScreen);
	
				pnlAttribute = new JPanel();
				pnlAttribute.setLayout(null);
				tpRuleAndAttribute.addTab(Globals.message.getMessage("msg0035"), pnlAttribute);
	
				als = new ExpAttributeListSite(this, expDataWidth, expDataHeight);
				pnlAttribute.add(als.pnlScreen);
	
				ads = new ExpAttributeDataSite(this, expDataWidth, expDataHeight);
				ads.sclScreen.setVisible(false);
				pnlAttribute.add(ads.sclScreen);
	
				cds = new ExpCodomainDataSite(this, expDataWidth, expDataHeight);
				cds.sclScreen.setVisible(false);
				pnlAttribute.add(cds.sclScreen);
			}

			setIntroMessage("infobar");
			pnlInfoBar = new JPanel();
			pnlInfoBar.setLayout(null);
			pnlInfoBar.setBounds(10, infoOffSetHeight, infoWidth, infoHeight);
			c.add(pnlInfoBar);

			ibds = new InfoBarDataSite(this, infoWidth, infoHeight);
			pnlInfoBar.add(ibds.sclScreen);
			Globals.info_message = ibds;

			// set the calling connections.
			pls.addSiteChangeListener_1(pds);
			pds.addSiteChangeListener_1(pls);
			if (!GlobalsDialog.production) {
				els.addSiteChangeListener_1(eds);
				eds.addSiteChangeListener_1(els);
				rls.addSiteChangeListener_1(rds);
				rds.addSiteChangeListener_1(rls);
				rds.addSiteChangeListener_2(ecds);
				ecds.addSiteChangeListener_2(rds);
				als.addSiteChangeListener_1(ads);
				ads.addSiteChangeListener_1(als);
				ads.addSiteChangeListener_2(cds);
				cds.addSiteChangeListener_2(ads);
			}
			
			if (GlobalsDialog.show_intro) {
				Thread.currentThread().sleep(5000);
				oIntro.dispose();
			}
			
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
		
		setVisible(true);
	}

	/**
	 * The method <CODE>initialiseDatabase</CODE> initialize the data base.
	 * @since 1.0
	 */
	private boolean initialiseDatabase() throws Exception {
		
		try {
	        dbm = new DatabaseManager(
	        		"./conf/", //SystemPathSet.getPath2ApplicationDataConf(), 
	                "./log/", //SystemPathSet.getPath2ApplicationDataLog(),
	                "./log/olddblog/", //SystemPathSet.getPath2ApplicationDataLogOldDb(),
	                false); //SystemService.getDatabaseConnectionSetDebug());
		} catch (Exception ex) {
			t4dialog.showErrorDialog(Globals.message.getMessage("msg0032"));
			return false;
		}
		return true;
	}

	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			
			if (e.getSource() == miOptionItem) {
				miOptionItem.setEnabled(false);
				Options oOptions = new Options(this, "expertanddecision");
				
			} else if (e.getSource() == miInfoItem) {
				miInfoItem.setEnabled(false);
				ArrayList data = new ArrayList();
				data.add("expert and decision");
				data.add("decide with expert knowledge.");
				ArrayList credits = new ArrayList();
		        credits.add("c r e d i t s . . .");
		        credits.add("BOOKS:");
		        credits.add("c. ullenboom;");
		        credits.add("   java ist auch eine insel;");
		        credits.add("   galileo computing; 2. auflage; 2003.");
		        credits.add("SOFTWARE:");
		        credits.add("j. dinkins;");
		        credits.add("   ExampleFileFilter.java;");
		        credits.add("   1.16, 04/07/26.");
		        credits.add("hyper sql;");
		        credits.add("   hsqldb.jar;");
		        credits.add("   2.3.2");
				data.add(credits);
				Info oInfo = new Info(this, data);
				oInfo.setVisible(true);
				
			} else if (e.getSource() == miHelpItem) {
				// get the help text.
				Tool4File t4f = new Tool4File();
				ArrayList content = null;
				if (GlobalsDialog.language_code.equals("de")) {
					content = t4f.getFileContent("help_de.htm");
				} else if (GlobalsDialog.language_code.equals("en")) {
					content = t4f.getFileContent("help_en.htm");
				}
				if (content != null) {
					String help_text = (String) content.get(0);
					miHelpItem.setEnabled(false);
					Help oHelp = new Help(this, help_text);
					oHelp.setVisible(true);
				}

			} else if (e.getSource() == miLicenseItem) {
				GNULicense oLicense = new GNULicense(this, "2015", "expert and decision, decide with expert knowledge.\r\n");
				oLicense.setVisible(true);
			}
			
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>closeEnvironmet</CODE> closes the resources.
	 * @since 1.0
	 */
	public void closeEnvironmet() {
		try {
			super.closeEnvironmet();
			Globals.saveGlobalsProperties();
			t4f.deleteOldLogs();
			dbm.releaseAllConnections();
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 1.0
	 */
	public static void main(String[] args) {
		Tool4MyDialog t4md_main = new Tool4MyDialog();
		ImageIcon intro_icon = null;
		try {
			new Globals();
			intro_icon = t4md_main.getImage("intro.jpg");
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new ExpertAndDecision("expert and decision", intro_icon, "...", "expertanddecision");
	}

	@Override
	public void stateChanged(ChangeEvent arg0) {
		try {
			if (arg0.getSource() == tpExpertSystem && pnlProject != null && pnlProject.isShowing()) {
				tpExpertSystemData.setVisible(true);
				tpRuleAndAttribute.setVisible(false);
			} else if (arg0.getSource() == tpExpertSystem && pnlExpertSystem != null && pnlExpertSystem.isShowing()) {
				tpRuleAndAttribute.setVisible(true);
				tpExpertSystemData.setVisible(false);
				Globals.curr_expsystem_uniqueid_2 = Globals.curr_expsystem_uniqueid_1;
				rls.createCurrentData(Globals.curr_expsystem_uniqueid_1);
				els.createCurrentData(Globals.curr_expsystem_uniqueid_1);
			}

			if (arg0.getSource() == tpExpertSystemData && pnlExpParameterDataSite != null && pnlExpParameterDataSite.isShowing()) {
				pads.createCurrentData(null);
			}
			
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

}

