package n4e.expertanddecision.tool;

import java.awt.Color;
import java.sql.ResultSet;
import java.util.ArrayList;

import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.database.DatabaseSQLResultSet;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.SystemService;
import n4e.expertanddecision.dbaccess.DbAccessExpAttribute;
import n4e.expertanddecision.dbaccess.DbAccessExpCodomain;
import n4e.expertanddecision.dbaccess.DbAccessExpCondition;
import n4e.expertanddecision.dbaccess.DbAccessExpRule;
import n4e.expertanddecision.dbaccess.DbAccessReadExpRule;
import n4e.expertanddecision.dbaccess.DbAccessResultAtt;
import n4e.expertanddecision.dbaccess.DbAccessResultValue;
import n4e.expertanddecision.system.Globals;

/**
 * The <CODE>Tool4Database</CODE> class ...
 * <P><B>title: AuditAndReport</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2014 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 14.10.2014 10:36:30
 * @since 1.0
 */

public class Tool4Database {

	private DatabaseDatasetLoader dsl;

	/**
	 * The constructor <CODE>Tool4Database</CODE> creates a new instance(object) of the class.
	 * @param
	 * @exception
	 * @see package.class#member label
	 * @since 1.0
	 */
	public Tool4Database() {
		dsl = new DatabaseDatasetLoader();
	}

    /**
     * The method <CODE>deleteAllRules</CODE> deletes the rules of an expert system.
     * @param uniqueid the uniqueid of the current expert system.
     * @exception Exception.
     * @since 1.0
     */
	public void deleteAllRules(String uniqueid) throws Exception {
		ArrayList rules = dsl.getDataSet(Globals.sysUser, "ExpRule", 
				null, null, null, "idexpsystem='" + uniqueid + "'", 
				null, "expertanddecision");
    	int len = rules.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessExpRule rule = (DbAccessExpRule) rules.get(ndx) ;
    		if (rule.errCode == 0) {
    			deleteAllConditions(rule.getUniqueid());
	   			if (!rule.deleteInSink(null)) {
	    			throw new Exception(rule.errCode + "\r\n" + rule.errMess);
	    		}
    		} else {
				throw new Exception(rule.errCode + "\r\n" + rule.errMess);
    		}
    	}
	}

    /**
     * The method <CODE>deleteAllConditions</CODE> deletes the conditions of a rule.
     * @param uniqueid the uniqueid of the current condition.
     * @exception Exception.
     * @since 1.0
     */
	public void deleteAllConditions(String uniqueid) throws Exception {
		ArrayList conds = dsl.getDataSet(Globals.sysUser, "ExpCondition", 
				null, null, null, "idexprule='" + uniqueid + "'", 
				null, "expertanddecision");
    	int len = conds.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessExpCondition cond = (DbAccessExpCondition) conds.get(ndx) ;
    		if (cond.errCode == 0) {
	   			if (!cond.deleteInSink(null)) {
	    			throw new Exception(cond.errCode + "\r\n" + cond.errMess);
	    		}
    		} else {
				throw new Exception(cond.errCode + "\r\n" + cond.errMess);
    		}
    	}
	}

    /**
     * The method <CODE>deleteAllAtts</CODE> deletes the attributes of an expert system.
     * @param uniqueid the uniqueid of the current expert system.
     * @exception Exception.
     * @since 1.0
     */
	public void deleteAllAtts(String uniqueid) throws Exception {
		ArrayList atts = dsl.getDataSet(Globals.sysUser, "ExpAttribute", 
				null, null, null, "idexpsystem='" + uniqueid + "'", 
				null, "expertanddecision");
    	int len = atts.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessExpAttribute att = (DbAccessExpAttribute) atts.get(ndx) ;
    		if (att.errCode == 0) {
    			deleteAllCodomains(att.getUniqueid());
	   			if (!att.deleteInSink(null)) {
	    			throw new Exception(att.errCode + "\r\n" + att.errMess);
	    		}
    		} else {
				throw new Exception(att.errCode + "\r\n" + att.errMess);
    		}
    	}
	}

    /**
     * The method <CODE>deleteAllCodomains</CODE> deletes the codomains of an attribute.
     * @param uniqueid the uniqueid of the current attribute.
     * @exception Exception.
     * @since 1.0
     */
	public void deleteAllCodomains(String uniqueid) throws Exception {
		ArrayList cods = dsl.getDataSet(Globals.sysUser, "ExpCodomain", 
				null, null, null, "idexpattribute='" + uniqueid + "'", 
				null, "expertanddecision");
    	int len = cods.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessExpCodomain cod = (DbAccessExpCodomain) cods.get(ndx) ;
    		if (cod.errCode == 0) {
	   			if (!cod.deleteInSink(null)) {
	    			throw new Exception(cod.errCode + "\r\n" + cod.errMess);
	    		}
    		} else {
				throw new Exception(cod.errCode + "\r\n" + cod.errMess);
    		}
    	}
	}

	/**
     * The method <CODE>enableAllRules</CODE> enable all rules.
     * @param expsystem_uniqueid the uniqueid of the expert system.
     * @exception Exception.
     * @since 1.0
     */
	public void enableAllRules(String expsystem_uniqueid) throws Exception {
		ArrayList rules = dsl.getDataSet(Globals.sysUser, "ExpRule", 
				"enabled", null, null, "enabled=0 AND idexpsystem='" 
				+ expsystem_uniqueid + "'", null, "expertanddecision");
		int len = rules.size();
		for (int ndx=0;ndx<len;ndx++) {
    		DbAccessExpRule rule = (DbAccessExpRule) rules.get(ndx) ;
    		if (rule.errCode == 0) {
    			rule.setEnabled(1);
    			if (!rule.putToSink(null)) {
    				throw new Exception (rule.errCode + "\r\n" + rule.errMess);
    			}
    		}
		}
	}

    /**
     * The method <CODE>getMaxNumberOfRules</CODE> calculates the maximum number of
     * the rules.
     * @return the maximum.
     * @exception Exception.
     * @since 1.0
     */
	public int getMaxNumberOfRules(String exp_uniqueid) throws Exception {
		
		DatabaseSQLResultSet dbRes = new DatabaseSQLResultSet(Globals.sysUser.getClientDbsystemforeclient(), null);
        dbRes.setDebug(SystemService.getDatabaseSQLResultSetDebug());
        
        String query = "SELECT MAX(ruleid) AS x FROM ExpRule WHERE idexpsystem='" + exp_uniqueid + "'";
        ResultSet resultset = dbRes.getResultSet(query);

        int max = 0;
        while (resultset.next()) {
        	max = resultset.getInt("x");
        }
        
        dbRes.closeResultSet();
        resultset.close();
        
		return max;
	}

	/**
     * The method <CODE>getMaxNumberOfConditions</CODE> calculates the maximum number of
     * the conditions of a rule.
     * @param uniqueid the uniqueid of the current rule.
     * @return the maximum.
     * @exception Exception.
     * @since 1.0
     */
	public int getMaxNumberOfConditions(String uniqueid) throws Exception {
		
		DatabaseSQLResultSet dbRes = new DatabaseSQLResultSet(Globals.sysUser.getClientDbsystemforeclient(), null);
        dbRes.setDebug(SystemService.getDatabaseSQLResultSetDebug());
        
        String query = "SELECT MAX(sort) AS x FROM ExpCondition WHERE idexprule='" + uniqueid + "'";
        ResultSet resultset = dbRes.getResultSet(query);

        int max = 0;
        while (resultset.next()) {
        	max = resultset.getInt("x");
        }
        
        dbRes.closeResultSet();
        resultset.close();
        
		return max;
	}

    /**
     * The method <CODE>isResultAttribute</CODE> tests, if result attributes for the project are available.
     * @param project_id the uniqueid of the project.
     * @return true, if available.
     * @exception Exception.
     * @since 1.0
     */
	public boolean isResultAttribute(String project_id) throws Exception {
		
		DatabaseSQLResultSet dbRes = new DatabaseSQLResultSet(Globals.sysUser.getClientDbsystemforeclient(), null);
        dbRes.setDebug(SystemService.getDatabaseSQLResultSetDebug());
        
        String query = "SELECT COUNT(*) AS x FROM ResultAtt WHERE idproject='" + project_id + "'";
        ResultSet resultset = dbRes.getResultSet(query);

        int count = 0;
        if (resultset.next()) {
        	count = resultset.getInt("x");
        }
        
        dbRes.closeResultSet();
        resultset.close();

        if (count > 0) {
        	return true;
        }
        
        return false;
        
	}

	/**
	 * The method <CODE>deleteAllAttributes</CODE> delete all attributes from result table.
	 * @param proj_id the uniqueid of the current project 
	 * @exception Exception
	 * @since 1.0
	 */
	public boolean deleteAllAttributes(String proj_id) throws Exception {
		
    	ArrayList tabs = dsl.getDataSet(Globals.sysUser, "ResultAtt",
				null, null, null, 
				"idproject='" + proj_id + "'",
				null, "expertanddecision");
    	
		int len = tabs.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessResultAtt tab = (DbAccessResultAtt) tabs.get(ndx);
       		if (tab.errCode == 0) {
       			
       			ArrayList tabs2 = dsl.getDataSet(Globals.sysUser, "ResultValue",
       					null, null, null, "idresultatt='" + tab.getUniqueid() + "'",
       					null, "expertanddecision");
       			
       			int len2 = tabs2.size();
       	    	for (int ndx2=0; ndx2<len2;ndx2++) {
       	    		DbAccessResultValue tab2 = (DbAccessResultValue) tabs2.get(ndx2);
       	       		if (tab.errCode == 0) {
       	       			if (!tab2.deleteInSink(null)) {
       	    				Globals.t4dialog.showException(tab2.errCode + "\r\n" + tab2.errMess);
       	    				return false;
       	       			}
       	       		} else {
   	    				Globals.t4dialog.showException(tab2.errCode + "\r\n" + tab2.errMess);
       	       			return false;
       	       		}
       	    	}
       			
       			if (!tab.deleteInSink(null)) {
    				Globals.t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    				return false;
       			}
       			
       		} else {
				Globals.t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
       			return false;
       		}
    	}
    	return true;
	}

	/**
	 * The method <CODE>numberingAllRules</CODE> renumbering all rules of the expert system.
	 * @param expsys_id the uniqueid of the current expert system. 
	 * @exception Exception
	 * @since 1.0
	 */
	public void numberingAllRules(String expsys_id) throws Exception {
		
		ArrayList tabs = dsl.getDataSet(Globals.sysUser, "ExpRule",
				"ruleid", null, null, "idexpsystem='" + expsys_id + "'",
				"ruleid ASC", "expertanddecision");

		int count = 1;
    	int len = tabs.size();
    	System.out.println(len);
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessExpRule tab = (DbAccessExpRule) tabs.get(ndx);
       		if (tab.errCode == 0) {
       			tab.setRuleid(count);
       			count++;
       			if (!tab.putToSink(null)) {
    				Globals.t4dialog.showException(tab.errCode + "\r\n" + tab.errMess, false);
    				return;
    			}
       		}
    	}
	}
}

