package n4e.expertanddecision.tool;

import java.util.ArrayList;

import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.tool.Tool4Dialog;
import n4e.core.tool.Tool4Throwable;
import n4e.expertanddecision.dbaccess.DbAccessReadExpAttribute;
import n4e.expertanddecision.dbaccess.DbAccessReadExpCodomain;
import n4e.expertanddecision.dbaccess.DbAccessReadExpCondition;
import n4e.expertanddecision.dbaccess.DbAccessReadExpRule;
import n4e.expertanddecision.dbaccess.DbAccessReadExpSystem;
import n4e.expertanddecision.system.Globals;

/**
 * The <CODE>Tool4Export</CODE> class ...
 * <P><B>title: ExpertAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2015 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 30.04.2015 14:26:18
 * @since 1.0
 */

public class Tool4Export {

	Tool4Dialog t4dialog;
	DatabaseDatasetLoader dsl;
	
	/**
	 * The constructor <CODE>Tool4Export</CODE> creates a new instance(object) of the class.
	 * @param
	 * @exception
	 * @see package.class#member label
	 * @since 1.0
	 */
	public Tool4Export() {
		t4dialog = new Tool4Dialog(Globals.ead);
		dsl = new DatabaseDatasetLoader();
	}

    /**
     * The method <CODE>getXMLListing</CODE> gives a pattern xml listing.
     * @param exp_uniqueid uniqueid of the expert system.
     * @param exp_name name of expert system.
     * @param exp_description description of the expert system.
     * @return xml listing.
     * @exception Exception
     * @since 1.0
     */
    public String createXMLContent(String exp_uniqueid) throws Exception {
    	
    	String listing = "";
    	
    	DbAccessReadExpSystem exp = new DbAccessReadExpSystem(Globals.sysUser, exp_uniqueid, "all", null, null);
    	if (exp.errCode != 0) {
			t4dialog.showException(exp.errCode + "\r\n" + exp.errMess);
			return null;
    	}
    	
    	listing += getXMLListing(exp_uniqueid, exp.getName(), exp.getDescription(), 
    			Integer.toString(exp.getType()), Long.toString(exp.getResultnumber()));
    	
    	// make attribute and codomain content.
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "ExpAttribute",
				"all", null, null, 
				"idexpsystem='" + exp_uniqueid + "'",
				"name ASC", "expertanddecision");

		String temp_listing = "";
    	int len = tabs.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadExpAttribute tab = (DbAccessReadExpAttribute) tabs.get(ndx);
       		if (tab.errCode == 0) {
       			temp_listing += getXMLAttribute(tab.getUniqueid(), tab.getName(), tab.getCodomain(), 
       	     		tab.getValuetype(), Integer.toString(tab.getInitialattribute()), tab.getQuestion(), 
       	    		tab.getAdddata(), tab.getDescription(), exp_uniqueid);

       	    	// make codomain content.
       			ArrayList temp_tabs = dsl.getDataSetRead(Globals.sysUser, "ExpCodomain",
       					"all", null, null, "idexpattribute='" + tab.getUniqueid() + "'",
       					"text ASC", "expertanddecision");
       	    	int len2 = temp_tabs.size();
       	    	for (int ndx2=0; ndx2<len2;ndx2++) {
       	    		DbAccessReadExpCodomain cod = (DbAccessReadExpCodomain) temp_tabs.get(ndx2);
       	       		if (cod.errCode == 0) {
       	       			temp_listing += getXMLCodomain(cod.getUniqueid(), cod.getText(), tab.getUniqueid());
       	       		}
       	    	}
       		}
    	}
   	
    	listing = Globals.t4s.replaceString(listing, "@attributelist@", temp_listing);
    	
    	// make rule and condition content.
		tabs = dsl.getDataSetRead(Globals.sysUser, "ExpRule",
				"all", null, null, 
				"idexpsystem='" + exp_uniqueid + "'",
				"ruleid ASC", "expertanddecision");

		temp_listing = "";
    	len = tabs.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadExpRule tab = (DbAccessReadExpRule) tabs.get(ndx);
       		if (tab.errCode == 0) {
       			temp_listing += getXMLRule(tab.getUniqueid(), Long.toString(tab.getRuleid()), 
       					Integer.toString(tab.getEnabled()), tab.getDescription(), 
       					Double.toString(tab.getPriorprob()), exp_uniqueid);

       	    	// make codomain content.
       			ArrayList temp_tabs = dsl.getDataSetRead(Globals.sysUser, "ExpCondition",
       					"all", "idexpattribute;idexprule", null, "idexprule='" + tab.getUniqueid() + "'",
       					"sort ASC", "expertanddecision");
       	    	int len2 = temp_tabs.size();
       	    	for (int ndx2=0; ndx2<len2;ndx2++) {
       	    		DbAccessReadExpCondition con = (DbAccessReadExpCondition) temp_tabs.get(ndx2);
       	       		if (con.errCode == 0) {
       	       			temp_listing += getXMLCondition(con.getUniqueid(), Long.toString(con.getSort()), 
       	       					con.getCondition(), con.getOperator(), con.getOperand(), 
       	       					con.getMyDbAccessReadExpAttribute().getUniqueid(),
       	       					tab.getUniqueid());
       	       		}
       	    	}
       		}
    	}
   	
    	listing = Globals.t4s.replaceString(listing, "@rulelist@", temp_listing);

    	return listing;
    }
    
    /**
     * The method <CODE>getXMLListing</CODE> gives a pattern xml listing.
     * @param exp_uniqueid uniqueid of the expert system.
     * @param exp_name name of expert system.
     * @param exp_description description of the expert system.
     * @return xml listing.
     * @exception Exception
     * @since 1.0
     */
    private String getXMLListing(String exp_uniqueid , String exp_name , 
    		String exp_description, String exp_type, String exp_resultnumber) throws Exception {
        
        if (exp_uniqueid == null) {
            throw new Exception("ExpSystem export without uniqueid.");
        }
        if (exp_name == null) {
            throw new Exception("ExpSystem export without name.");
        }
        
        String listing = "";
        listing += "<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>\r\n";
        listing += "<!-- export of expert system: " + exp_name + " -->\r\n";
        listing += "<!DOCTYPE ExpSystem [\r\n";
        listing += "  <!ELEMENT ExpSystem ANY>\r\n";
        listing += "  <!ATTLIST ExpSystem uniqueid CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpSystem name CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpSystem description CDATA #IMPLIED>\r\n";
        listing += "  <!ATTLIST ExpSystem type CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpSystem resultnumber CDATA #REQUIRED>\r\n";
        listing += "  <!ELEMENT ExpAttribute ANY>\r\n";
        listing += "  <!ATTLIST ExpAttribute uniqueid CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpAttribute name CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpAttribute codomain CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpAttribute valuetype CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpAttribute initialattribute CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpAttribute question CDATA #IMPLIED>\r\n";
        listing += "  <!ATTLIST ExpAttribute adddata CDATA #IMPLIED>\r\n";
        listing += "  <!ATTLIST ExpAttribute description CDATA #IMPLIED>\r\n";
        listing += "  <!ATTLIST ExpAttribute idexpsystem CDATA #REQUIRED>\r\n";
        listing += "  <!ELEMENT ExpCodomain ANY>\r\n";
        listing += "  <!ATTLIST ExpCodomain uniqueid CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCodomain text CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCodomain idexpattribute CDATA #REQUIRED>\r\n";
        listing += "  <!ELEMENT ExpRule ANY>\r\n";
        listing += "  <!ATTLIST ExpRule uniqueid CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpRule ruleid CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpRule enabled CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpRule description CDATA #IMPLIED>\r\n";
        listing += "  <!ATTLIST ExpRule priorprob CDATA #IMPLIED>\r\n";
        listing += "  <!ATTLIST ExpRule idexpsystem CDATA #REQUIRED>\r\n";
        listing += "  <!ELEMENT ExpCondition ANY>\r\n";
        listing += "  <!ATTLIST ExpCondition uniqueid CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCondition sort CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCondition condition CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCondition operator CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCondition operand CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCondition idexpattribute CDATA #REQUIRED>\r\n";
        listing += "  <!ATTLIST ExpCondition idexprule CDATA #REQUIRED>\r\n";
        listing += "]>\r\n";
        listing += "<ExpSystem uniqueid=\"" + exp_uniqueid + "\" name=\"" + exp_name 
        		+ "\" description=\"" + exp_description + "\" type=\"" + exp_type 
        		+ "\" resultnumber=\"" + exp_resultnumber + "\">\r\n"; 
        listing += "@attributelist@";
        listing += "@rulelist@";
        listing += "</ExpSystem>\r\n";
        return listing;
    }

    /**
     * The method <CODE>getXMLAttribute</CODE> gives a pattern row for an attribute in a xml listing.
     * @param uniqueid of KeyWord.
     * @return row of xml listing.
     * @exception Exception
     * @since 1.0
     */
    private String getXMLAttribute(String att_uniqueid, String att_name, String att_codomain, 
    		String att_valuetype, String att_initialattribute, String att_question, 
    		String att_adddata, String att_description, String att_idexpsystem) throws Exception {
        
        if (att_uniqueid == null) {
            throw new Exception("Attribute export without uniqueid.");
        }
        if (att_name == null) {
            throw new Exception("Attribute export without idclient.");
        }
        return "  <ExpAttribute uniqueid=\"" + att_uniqueid + "\" name=\"" + att_name 
                + "\" codomain=\"" + att_codomain + "\" valuetype=\"" + att_valuetype 
                + "\" initialattribute=\"" + att_initialattribute
                + "\" question=\"" + att_question + "\" adddata=\"" + att_adddata
                + "\" description=\"" + att_description + "\" idexpsystem=\"" + att_idexpsystem
                + "\"/>\r\n";
    }

    /**
     * The method <CODE>getXMLCodomain</CODE> gives a pattern row for a codomain in a xml listing.
     * @param uniqueid of KeyWord.
     * @return row of xml listing.
     * @exception Exception
     * @since 1.0
     */
    private String getXMLCodomain(String cod_uniqueid, String cod_text, String cod_idexpattribute) throws Exception {
        
        if (cod_uniqueid == null) {
            throw new Exception("Codomain export without uniqueid.");
        }
        if (cod_text == null) {
            throw new Exception("Codomain export without text.");
        }
        return "  <ExpCodomain uniqueid=\"" + cod_uniqueid + "\" text=\"" + cod_text 
                + "\" idexpattribute=\"" + cod_idexpattribute 
                + "\"/>\r\n";
    }
    
    /**
     * The method <CODE>getXMLRule</CODE> gives a pattern row for a rule in a xml listing.
     * @param uniqueid of KeyWord.
     * @return row of xml listing.
     * @exception Exception
     * @since 1.0
     */
    private String getXMLRule(String rul_uniqueid, String rul_ruleid, String rul_enabled,
    		String rul_description, String rul_priorprob, String rul_idexpsystem) throws Exception {
        
        if (rul_uniqueid == null) {
            throw new Exception("Rule export without uniqueid.");
        }
        return "  <ExpRule uniqueid=\"" + rul_uniqueid + "\" ruleid=\"" + rul_ruleid 
                + "\" enabled=\"" + rul_enabled + "\" description=\"" + rul_description   
                + "\" priorprob=\"" + rul_priorprob + "\" idexpsystem=\"" + rul_idexpsystem 
                + "\"/>\r\n";
    }
    
    /**
     * The method <CODE>getXMLCondition</CODE> gives a pattern row for a condition in a xml listing.
     * @param uniqueid of KeyWord.
     * @return row of xml listing.
     * @exception Exception
     * @since 1.0
     */
    private String getXMLCondition(String con_uniqueid, String con_sort, String con_condition,
    		String con_operator, String con_operand, String con_idexpattribute, String con_idexprule) throws Exception {
        
        if (con_uniqueid == null) {
            throw new Exception("Rule export without uniqueid.");
        }
        return "  <ExpCondition uniqueid=\"" + con_uniqueid + "\" sort=\"" + con_sort 
                + "\" condition=\"" +  con_condition  + "\" operator=\"" + con_operator 
                + "\" operand=\"" + con_operand  + "\" idexpattribute=\"" + con_idexpattribute 
                + "\" idexprule=\"" + con_idexprule  + "\"/>\r\n";
    }
    
    
}

