package de.hopfieldnet.system;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;

import de.hopfieldnet.control.Value;
import n4e.core.controls.AWTextField;
import n4e.core.controls.ExampleFileFilter;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.MainDialog;
import n4e.core.tool.Tool4Throwable;

/**
 * The <CODE>HopfieldNet</CODE> class for a hopfield net.
 * description: hopfield net.
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne , germany.
 * @version 02.03.2017 11:57:59
 * @since 0.1
 */

public class HopfieldNet extends MainDialog {

	private Value[][] pictureX; // left picture build with pixels.
	private Value[][] pictureY; // right picture build with pixels.

	private JLabel lblPatternSize;
	private JLabel lblCurrPattern;
	private AWTextField tfNoise;
	private JLabel lblEnergy;

	private JButton btnGreenX;
	private JButton btnYellowX;
	private JButton btnNoise;
	private JButton btnDeleteY;
	private JButton btnStepLeft;
	private JButton btnStepRight;

	private JButton btnStep;
	private JButton btnExecute;
	private JButton btnTraining;
	private JButton btnReset;
	private JButton btnPattern;
	private JButton btnOpen;
	private JButton btnSave;
	private JButton btnClose;
	
	private int picture_line; // row, where the pictures begin.
	private int picture_X_column; // column of the left picture.
	private int picture_Y_column; // column of the right picture.
	private int curr_pic; // counter of the current picture.

	private double[][] weights; // the weights of the neural network.
	private int[] layer; // the layer of the neural network.
	private ArrayList<int[]> pattern; // the list of the pattern. 
	private int L; // number of the pixel.
	private boolean get_picture; // label for picture transfer to the layer.

	/**
	 * The constructor <CODE>HopfieldNet</CODE> creates a new instance(object)
	 * of the class.
	 * @since 0.1
	 */
	public HopfieldNet() {
		super("hopfield net", "0.1", "2017", 480, 740);

		try {

			setResizable(false);
			Globals.hn = this;
			Globals.doAfterInitials();
			Image icon = Globals.t4md.getImagePure("app.jpg");
			setIconImage(icon);
			windowInit();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @exception Exception if error.
	 * @since 0.1
	 */
	private void windowInit() throws Exception {

		try {
			
			int controls_line = 370;

			// define the neural network.
			L = 100;
			weights = new double[L][L];
			layer = new int[L];
			
			picture_line = 50;
			picture_X_column = 50;
			picture_Y_column = 400;
			curr_pic = 0;
			get_picture = true;

			pattern = new ArrayList<int[]>();
			pictureX = new Value[10][10];
			pictureY = new Value[10][10];

			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 49, screen.width, 20);
			Container con = getContentPane();
			con.setLayout(null);
			con.setSize(new Dimension(screen.width, screen.height));
			con.setBackground(Color.LIGHT_GRAY);
			
			// create the window screen.
			createLabel(con, 10, controls_line, 120, 20, Globals.message.getMessage("msg0005"));
			lblPatternSize = createLabel(con, 140, controls_line, 40, 20, "0");

			createLabel(con, 10, controls_line + 30, 120, 20, Globals.message.getMessage("msg0015"));
			lblEnergy = createLabel(con, 140, controls_line + 30, 140, 20, "0");

			btnGreenX = createButton(con, picture_X_column, 20, 23, 23, Globals.message.getMessage("msg0002") + " X", false, Color.GREEN);
			btnYellowX = createButton(con, picture_X_column + 25, 20, 23, 23, Globals.message.getMessage("msg0003") + " X", false, Color.YELLOW);
			btnNoise = createButton(con, picture_X_column + 50, 20, 23, 23, Globals.message.getMessage("msg0013"), false, null);
			tfNoise = createTextField(con, picture_X_column + 75, 20, 40, 20, 0);
			btnDeleteY = createButton(con, picture_Y_column, 20, 23, 23, Globals.message.getMessage("msg0004"), false, null);

			btnStepLeft = createButton(con, picture_X_column + 50, 320, 50, 23, "<<", true, null);
			lblCurrPattern = createLabel(con, picture_X_column + 110, 320, 20, 20, "0");
			btnStepRight = createButton(con, picture_X_column + 130, 320, 50, 23, ">>", true, null);

			btnStep = createButton(con, 290, controls_line, 100, 23, Globals.message.getMessage("msg0006"), true, null);
			btnExecute = createButton(con, 400, controls_line, 100, 23, Globals.message.getMessage("msg0007"), true, null);
			btnTraining = createButton(con, 510, controls_line, 100, 23, Globals.message.getMessage("msg0008"), true, null);
			btnReset = createButton(con, 620, controls_line, 100, 23, Globals.message.getMessage("msg0009"), true, null);
			
			btnPattern = createButton(con, 290, controls_line + 30, 100, 23, Globals.message.getMessage("msg0012"), true, null);
			btnOpen = createButton(con, 400, controls_line + 30, 100, 23, Globals.message.getMessage("msg0010"), true, null);
			btnSave = createButton(con, 510, controls_line + 30, 100, 23, Globals.message.getMessage("msg0011"), true, null);
			btnClose = createButton(con, 620, controls_line + 30, 100, 23, Globals.message.getMessage("msg0001"), true, null);
			
			createControls();
			
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}

		setVisible(true);

	}

	/**
	 * The method <CODE>createLabel</CODE> creates a label.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @return a label.
	 * @since 0.1
	 */
	private JLabel createLabel(Container c, int x, int y, int width, int height, String text) {
        JLabel label = new JLabel(text);
		label.setBounds(x, y, width, height);
        c.add(label);
        return label;
	}
	
	/**
	 * The method <CODE>createTextField</CODE> creates a text field.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param value the value of the control.
	 * @return a text field.
	 * @since 0.1
	 */
	private AWTextField createTextField(Container c, int x, int y, int width, int height, int value) {
		AWTextField tf = new AWTextField();
        tf.setBounds(x, y, width, height);
        tf.setFont(Globals.fontStandard11);
        tf.setText(Integer.toString(value));
		c.add(tf);
		return tf;
	}

	/**
	 * The method <CODE>createButton</CODE> creates a button.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @param show true -> show the text.
	 * @return a button.
	 * @since 0.1
	 */
	private JButton createButton(Container c, int x, int y, int width, int height, String text, boolean show, Color color) {
		String curr_text = "";
		if (show) {
			curr_text = text;
		}
		JButton btn = new JButton(curr_text);
	    btn.setBounds(x, y, width, height);
	    btn.setFont(GlobalsDialog.fontStandard12);
	    btn.setToolTipText(text);
	    if (color != null) {
	    	btn.setBackground(color);
	    }
	    btn.addActionListener(this);
		c.add(btn);
		return btn;
	}
	
	
	/**
	 * The method <CODE>createControls</CODE> creates pictures pixel of the neural network.
	 * @since 0.1
	 */
	private void createControls() {
		
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pictureX[i][j] = new Value(c, picture_X_column + (25 * j), picture_line + (25 * i), 20, 20);
            	pictureX[i][j].setValue(0);
            }
        }
	        
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pictureY[i][j] = new Value(c, picture_Y_column + (25 * j), picture_line + (25 * i), 20, 20);
            	pictureY[i][j].setValue(0);
            }
        }
	}

	/**
	 * The method <CODE>colorPicture</CODE> colors the picture.
	 * @param pic the picture array.
	 * @param color_value the color value.
	 * @since 0.1
	 */
	private void colorPicture(Value[][] pic, int color_value) {
        for (int i=0;i<10;i++) {
            for (int j=0;j<10;j++) {
            	pic[i][j].setValue(color_value);
            }
        }
	}

	/**
	 * The method <CODE>deleteY</CODE> clears the y picture to white.
	 * @since 0.1
	 */
	private void deleteY() {
        for (int i=0;i<10;i++) {
	        for (int j=0;j<10;j++) {
	        	pictureY[i][j].setValue(0);
	        }
        }
	}

	/**
	 * The method <CODE>getPicturesFromPatters</CODE> get the X picture from the pattern list.
	 * @since 0.1
	 */
	private void getPicturesFromPatters() {
		int[] pic_val = pattern.get(curr_pic);
		int count = 0;
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				pictureX[i][j].setValue(pic_val[count]);
				count++;
			}
		}
		lblCurrPattern.setText(Integer.toString(curr_pic + 1));
	}

	/**
	 * The method <CODE>savePicturesAsPatters</CODE> save the X pictures as a pattern.
	 * @since 0.1
	 */
	private void savePicturesAsPatters() {
		int[] pic_val = new int[100];
		int count = 0;
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				pic_val[count] = pictureX[i][j].getValue();
				count++;
			}
		}
		pattern.add(pic_val);

		int len = pattern.size();
		lblPatternSize.setText(Integer.toString(len));
		curr_pic = len - 1;
		lblCurrPattern.setText(Integer.toString(curr_pic + 1));
	}	

	/**
	 * The method <CODE>reset</CODE> resets the neural network and the pictures.
	 * @since 0.1
	 */
	private void reset() {
		pattern = new ArrayList<int[]>();
		layer = new int[L];
		for (int i=0;i<L;i++) {
			for (int j=0;j<L;j++) {
				weights[i][j] = 0;
			}
		}

		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				pictureX[i][j].setValue(0);
				pictureY[i][j].setValue(0);
			}
		}
		
		lblPatternSize.setText("0");
		lblCurrPattern.setText("0");
		lblEnergy.setText("0");
	}
	
	/**
	 * The method <CODE>training</CODE> the training of the neural network.
	 * @since 0.1
	 */
	private void training() {
		int p = pattern.size();
		for (int i=0;i<L;i++) {
			for (int j=0;j<L;j++) {
				double sum = 0;
				for (int k=0;k<p;k++) {
					int[] pic_val = pattern.get(k);
					sum += pic_val[i] * pic_val[j];
				}
				if (i == j) {
					weights[i][j] = 0;
				} else {
					weights[i][j] = 1.0 / (double) L * sum;
				}
			}
		}
	}
	
	/**
	 * The method <CODE>getCurrPictureX</CODE> transfer the current picture to the layer.
	 * @since 0.1
	 */
	private void getCurrPictureX() {
		int count = 0;
		for (int i=0;i<10;i++) {
			for (int j=0;j<10;j++) {
				layer[count] = pictureX[i][j].getValue();
				count++;
			}
		}
	}
	
	/**
	 * The method <CODE>execute</CODE> execution of the neural network.
	 * @since 0.1
	 */
	private void execute() {
		getCurrPictureX();
		boolean changed = true;
		do {
			changed = step();
		} while (changed);
	}
	
	/**
	 * The method <CODE>step</CODE> one step of the execution of the neural network.
	 * @return true if layer changed.
	 * @since 0.1
	 */
	private boolean step() {
		boolean changed = false;
		for (int i=0;i<L;i++) {
			double sum = 0;
			for (int j=0;j<L;j++) {
				if (j != i) {
					sum += weights[i][j] * layer[j];
				}
			}
			if (sum >= 0) {
				if (layer[i] == -1) {
					changed = true;
				}
				layer[i] = 1;
			} else {
				if (layer[i] == 1) {
					changed = true;
				}
				layer[i] = -1;
			}
		}
		calcEnergy();
		return changed;
	}
	
	/**
	 * The method <CODE>calcEnergy</CODE> calculates the energy of the neural network.
	 * @since 0.1
	 */
	private void calcEnergy() {
		double sum = 0;
		for (int i=0;i<L;i++) {
			for (int j=0;j<L;j++) {
				sum += weights[i][j] * layer[i] * layer[j];
			}
		}
		double energy = -0.5 * sum;
		lblEnergy.setText(Double.toString(energy));
	}
	
	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 * @param e the action event.
	 * @since 0.1
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			if (e.getSource() == btnClose) {
				System.exit(0);
			} else if (e.getSource() == btnGreenX) {
				colorPicture(pictureX, 1);
			} else if (e.getSource() == btnYellowX) {
				colorPicture(pictureX, -1);
			} else if (e.getSource() == btnDeleteY) {
				deleteY();
			} else if (e.getSource() == btnStepLeft) {
				if (pattern.size() == 0) {
					return;
				}
				curr_pic -= 1;
				if (curr_pic < 0) {
					curr_pic = 0;
				}
				getPicturesFromPatters();
				get_picture = true;
				lblEnergy.setText("0");
			} else if (e.getSource() == btnStepRight) {
				if (pattern.size() == 0) {
					return;
				}
				curr_pic += 1;
				if (curr_pic > pattern.size() - 1) {
					curr_pic = pattern.size() - 1;
				}
				getPicturesFromPatters();
				get_picture = true;
				lblEnergy.setText("0");
			} else if (e.getSource() == btnPattern) {
				savePicturesAsPatters();
			} else if (e.getSource() == btnTraining) {
				training();
			} else if (e.getSource() == btnStep) {
				if (get_picture) {
					getCurrPictureX();
					get_picture = false;
				}
				step();
				int count = 0;
				for (int i=0;i<10;i++) {
					for (int j=0;j<10;j++) {
						pictureY[i][j].setValue(layer[count]);
						count++;
					}
				}
			} else if (e.getSource() == btnExecute) {
				execute();
				int count = 0;
				for (int i=0;i<10;i++) {
					for (int j=0;j<10;j++) {
						pictureY[i][j].setValue(layer[count]);
						count++;
					}
				}
			} else if (e.getSource() == btnReset) {
				reset();
			} else if (e.getSource() == btnNoise) {
				ArrayList<String> perclist = new ArrayList<String>();
				double percent = Integer.parseInt(tfNoise.getText());
				if (percent < 0 || percent > 100) {
	    			Globals.t4d.showErrorDialog(Globals.message.getMessage("msg0016"));
	    			return;
				}
				int pixel = (int) (100 * (percent/ 100));
				for (int ndx=0;ndx<pixel;ndx++) {
					int x = 0;
					int y = 0;
					String pair = null;
					do {
						x = (int) (Math.random() * 10);
						y = (int) (Math.random() * 10);
						pair = Integer.toString(x) + Integer.toString(y); 
					} while (perclist.contains(pair));
					perclist.add(pair);
					int val = pictureX[x][y].getValue();
					val *= -1;
					pictureX[x][y].setValue(val);
				}
			} else if (e.getSource() == btnOpen) {
				
		        JFileChooser chooser = null;
		        File curr_dir = new File(Globals.t4f.getFullPathToSystem("data"));
		        if (curr_dir.exists()) {
		            chooser = new JFileChooser(curr_dir);
		        } else {
		            chooser = new JFileChooser(".");
		        }

				String[] extension = new String[1];
				String message = Globals.message.getMessage("msg0014");
				extension[0] = "dat";

		        chooser.setFileFilter(new ExampleFileFilter(extension, message));
		        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				
		        int state = chooser.showOpenDialog(this);
		        if (state == JFileChooser.APPROVE_OPTION) {
		        	
					pattern = new ArrayList<int[]>();
		        	String path = chooser.getSelectedFile().getPath();
		        	String content = Globals.t4f.getContent(path);
		        	StringTokenizer st = new StringTokenizer(content, "\r\n");
		        	int count = Integer.parseInt(st.nextToken());
		        	while(count > 0) {
		        		int[] pic = new int[L];
	        			String line = st.nextToken();
	        			for (int j=0;j<L;j++) {
	        				String token = line.substring(j, j+1);
	        				if (token.equals("#")) {
	        					pic[j] = 1;
	        				} else if (token.equals("-")) {
	        					pic[j] = -1;
	        				}
	        			}
		        		pattern.add(pic);
		        		count--;
		        	}
		        	
					int len = pattern.size();
					lblPatternSize.setText(Integer.toString(len));
					curr_pic = 0;
					getPicturesFromPatters();
					get_picture = true;
					lblEnergy.setText("0");
		        }
			} else if (e.getSource() == btnSave) {
				
		        JFileChooser chooser = null;
		        File curr_dir = new File(Globals.t4f.getFullPathToSystem("data"));
		        if (curr_dir.exists()) {
		            chooser = new JFileChooser(curr_dir);
		        } else {
		            chooser = new JFileChooser(".");
		        }

				String[] extension = new String[1];
				String message = Globals.message.getMessage("msg0014");
				extension[0] = "dat";

		        chooser.setFileFilter(new ExampleFileFilter(extension, message));
				chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				
		        int state = chooser.showSaveDialog(this);
		        if (state == JFileChooser.APPROVE_OPTION) {
		        	String path = chooser.getSelectedFile().getPath();
		        	if (path.indexOf(".") == -1) {
		        		path += "." + extension[0];
		        	}
		        	int len = pattern.size();
		        	String content = Integer.toString(len) + "\r\n";
		        	for (int ndx=0;ndx<len;ndx++) {
		        		int[] pic = pattern.get(ndx);
	        			String line = "";
		        		for (int i=0;i<L;i++) {
	        				if (pic[i] == 1) {
	        					line += "#";
	        				} else if (pic[i] == -1) {
	        					line += "-";
	        				}
	        			}
	        			content += line + "\r\n";
	        		}
    	        	Globals.t4f.putTheFile(path, "", content, false);
	        	}
	        }
			
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}
	
	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 0.1
	 */
	public static void main(String[] args) {
		try {
			new Globals();
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new HopfieldNet();
	}

}
