package n4e.netanddecision.dbaccess;

/**
* <p>title: n4e application</p>
* <p>description: netanddecision</p>
* <p>copyright: copyright (c) 2006-2011 by thomas kaffka; mba; all rights reserved.</p>
* <p>company: n.n.</p>
* @author thomas kaffka, mba.
* @version Tue Mar 14 12:11:07 CET 2017
*/

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;


/**
 * The <CODE>DbAccessCheckKeyWord</CODE> class checks fields of KeyWord table in database.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version Tue Mar 14 12:11:07 CET 2017
 * @since 1.0
*/
public class DbAccessCheckKeyWord extends DbCheck {
  // technical fields
  public int errCode = 0;
  public String errMess = "";
  private boolean hasdata = false;
  private Tool4Number t4n = new Tool4Number();
  private Tool4DbMessages t4m = null;

  // database standard fields
  private String uniqueid = ""; // unique key
  private long version = 0; // version counter for database store without locking
  private java.util.Date created = null; // date of creation
  private String creator = ""; // user created
  private java.util.Date changed = null; // date of last change data
  private String changedfrom = ""; // user changed
  private int inuse = 0; // read but not updated
  private String client = ""; // foreign key to client table

  // individual database fields
  private String name = ""; // Schluesselname
  private String name2 = ""; // Schluesselname
  private String name3 = ""; // Schluesselname
  private String name4 = ""; // Schluesselname
  private String name5 = ""; // Schluesselname
  private int canchangename = 0; // Schluesselname aenderbar
  private String description = ""; // Beschreibung
  private String description2 = ""; // Beschreibung
  private String description3 = ""; // Beschreibung
  private String description4 = ""; // Beschreibung
  private String description5 = ""; // Beschreibung
  private String parameter1 = ""; // Parameter 1
  private String parameter2 = ""; // Parameter 2
  private String parameter3 = ""; // Parameter 3
  private String parameter4 = ""; // Parameter 4
  private String parameter5 = ""; // Parameter 5
  private int candelete = 0; // vom Admin loeschbar
  private DbAccessKeySet myDbAccessKeySet = new DbAccessKeySet() ; // KeySet


  // sets and gets of standard table fields
  public void setUniqueid(String input){uniqueid = input;}
  public String getUniqueid(){return uniqueid;}
  public void setVersion(long input){version = input;}
  public long getVersion(){return version;}
  public void setCreated(java.util.Date input){created = input;}
  public java.util.Date getCreated(){return created;}
  public void setCreator(String input){creator = input;}
  public String getCreator(){return creator;}
  public void setChanged(java.util.Date input){changed = input;}
  public java.util.Date getChanged(){return changed;}
  public void setChangedfrom(String input){changedfrom = input;}
  public String getChangedfrom(){return changedfrom;}
  public void setInuse(int input){inuse = input;}
  public int getInuse(){return inuse;}
  public void setClient(String input){client = input;}
  public String getClient(){return client;}

  // sets and gets of individual table fields
  public void setName(String input){name = input;}
  public String getName(){return name;}
  public void setName2(String input){name2 = input;}
  public String getName2(){return name2;}
  public void setName3(String input){name3 = input;}
  public String getName3(){return name3;}
  public void setName4(String input){name4 = input;}
  public String getName4(){return name4;}
  public void setName5(String input){name5 = input;}
  public String getName5(){return name5;}
  public void setCanchangename(int input){canchangename = input;}
  public int getCanchangename(){return canchangename;}
  public void setDescription(String input){description = input;}
  public String getDescription(){return description;}
  public void setDescription2(String input){description2 = input;}
  public String getDescription2(){return description2;}
  public void setDescription3(String input){description3 = input;}
  public String getDescription3(){return description3;}
  public void setDescription4(String input){description4 = input;}
  public String getDescription4(){return description4;}
  public void setDescription5(String input){description5 = input;}
  public String getDescription5(){return description5;}
  public void setParameter1(String input){parameter1 = input;}
  public String getParameter1(){return parameter1;}
  public void setParameter2(String input){parameter2 = input;}
  public String getParameter2(){return parameter2;}
  public void setParameter3(String input){parameter3 = input;}
  public String getParameter3(){return parameter3;}
  public void setParameter4(String input){parameter4 = input;}
  public String getParameter4(){return parameter4;}
  public void setParameter5(String input){parameter5 = input;}
  public String getParameter5(){return parameter5;}
  public void setCandelete(int input){candelete = input;}
  public int getCandelete(){return candelete;}
  public void setMyDbAccessKeySet(DbAccessKeySet input){myDbAccessKeySet = input;}
  public DbAccessKeySet getMyDbAccessKeySet(){return myDbAccessKeySet;}


  /**
   * The constructor <CODE>DbAccessCheckKeyWord</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessCheckKeyWord(Tool4DbMessages pt4m){
    t4m = pt4m;
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens of dataset
   * @param dbaccessclass Class DbAccessCustomer.
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessKeyWord dbaccessclass , String field_list){
    return checkData(dbaccessclass , false , field_list);
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens
   * @param dbaccessclass Class DbAccess...
   * @param first = true => check standard fields
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessKeyWord dbaccessclass , boolean first , String field_list){
    // fill standard fields
    setUniqueid(dbaccessclass.getUniqueid());
    setVersion(dbaccessclass.getVersion());
    setCreated(dbaccessclass.getCreated());
    setCreator(dbaccessclass.getCreator());
    setChanged(dbaccessclass.getChanged());
    setChangedfrom(dbaccessclass.getChangedfrom());
    setInuse(dbaccessclass.getInuse());
    setClient(dbaccessclass.getClient());
    hasdata = true;

    // fill individual fields
    setName(dbaccessclass.getName());
    setName2(dbaccessclass.getName2());
    setName3(dbaccessclass.getName3());
    setName4(dbaccessclass.getName4());
    setName5(dbaccessclass.getName5());
    setCanchangename(dbaccessclass.getCanchangename());
    setDescription(dbaccessclass.getDescription());
    setDescription2(dbaccessclass.getDescription2());
    setDescription3(dbaccessclass.getDescription3());
    setDescription4(dbaccessclass.getDescription4());
    setDescription5(dbaccessclass.getDescription5());
    setParameter1(dbaccessclass.getParameter1());
    setParameter2(dbaccessclass.getParameter2());
    setParameter3(dbaccessclass.getParameter3());
    setParameter4(dbaccessclass.getParameter4());
    setParameter5(dbaccessclass.getParameter5());
    setCandelete(dbaccessclass.getCandelete());
    setMyDbAccessKeySet(dbaccessclass.getMyDbAccessKeySet());


    try{
      // check standard fields
      if (!first) {
        if (getUniqueid() == null || (getUniqueid() != null && getUniqueid().length() == 0)) {
           errCode = ERROR_0009;
           errMess = t4m.getMessage("msg0009", null);
           return false;
        }
        if (getUniqueid() != null && getUniqueid().length() > 35) {
           errCode = ERROR_0010;
           errMess = t4m.getMessage("msg0010", null);
           return false;
        }
        if (getClient() == null || (getClient() != null && getClient().length() == 0)) {
           errCode = ERROR_0011;
           errMess = t4m.getMessage("msg0011", null);;
           return false;
        }
        if (getClient() != null && getClient().length() > 35) {
           errCode = ERROR_0012;
           errMess = t4m.getMessage("msg0012", null);
           return false;
        }
      }
      if (getCreator() != null && getCreator().length() > 35) {
        errCode = ERROR_0013;
        errMess = t4m.getMessage("msg0013", null);
        return false;
      }
      if (getChangedfrom() != null && getChangedfrom().length() > 35) {
        errCode = ERROR_0014;
        errMess = t4m.getMessage("msg0014", null);
        return false;
      }

      // check individual fields
      if ((first) && (getName() == null || (getName() != null && getName().length() == 0))) {
        errCode = ERROR_7_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("name" , "label"));
        return false;
      } else if ((field_list.indexOf(",name,")!=-1) && (getName() == null || (getName() != null && getName().length() == 0))) {
        errCode = ERROR_8_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("name" , "label"));
        return false;
      }
      if (getName() != null && (getName().length() > 32)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name" , "label") + "|32");
        return false;
      }
      if (getName2() != null && (getName2().length() > 32)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name2" , "label") + "|32");
        return false;
      }
      if (getName3() != null && (getName3().length() > 32)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name3" , "label") + "|32");
        return false;
      }
      if (getName4() != null && (getName4().length() > 32)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name4" , "label") + "|32");
        return false;
      }
      if (getName5() != null && (getName5().length() > 32)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name5" , "label") + "|32");
        return false;
      }
      if (getDescription() != null && (getDescription().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description" , "label") + "|100");
        return false;
      }
      if (getDescription2() != null && (getDescription2().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description2" , "label") + "|100");
        return false;
      }
      if (getDescription3() != null && (getDescription3().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description3" , "label") + "|100");
        return false;
      }
      if (getDescription4() != null && (getDescription4().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description4" , "label") + "|100");
        return false;
      }
      if (getDescription5() != null && (getDescription5().length() > 100)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("description5" , "label") + "|100");
        return false;
      }
      if (getParameter1() != null && (getParameter1().length() > 50)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("parameter1" , "label") + "|50");
        return false;
      }
      if (getParameter2() != null && (getParameter2().length() > 50)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("parameter2" , "label") + "|50");
        return false;
      }
      if (getParameter3() != null && (getParameter3().length() > 50)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("parameter3" , "label") + "|50");
        return false;
      }
      if (getParameter4() != null && (getParameter4().length() > 50)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("parameter4" , "label") + "|50");
        return false;
      }
      if (getParameter5() != null && (getParameter5().length() > 50)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("parameter5" , "label") + "|50");
        return false;
      }

    } catch (Exception e) {
      System.err.println(e.getMessage());
      errCode = ERROR_Ex;
      errMess = e.getMessage();
    }
    return true;
  }
}
