package n4e.netanddecision.dbaccess;

/**
* <p>title: n4e application</p>
* <p>description: netanddecision</p>
* <p>copyright: copyright (c) 2006-2011 by thomas kaffka; mba; all rights reserved.</p>
* <p>company: n.n.</p>
* @author thomas kaffka, mba.
* @version Tue Mar 14 12:11:07 CET 2017
*/

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;


/**
 * The <CODE>DbAccessCheckProject</CODE> class checks fields of Project table in database.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version Tue Mar 14 12:11:07 CET 2017
 * @since 1.0
*/
public class DbAccessCheckProject extends DbCheck {
  // technical fields
  public int errCode = 0;
  public String errMess = "";
  private boolean hasdata = false;
  private Tool4Number t4n = new Tool4Number();
  private Tool4DbMessages t4m = null;

  // database standard fields
  private String uniqueid = ""; // unique key
  private long version = 0; // version counter for database store without locking
  private java.util.Date created = null; // date of creation
  private String creator = ""; // user created
  private java.util.Date changed = null; // date of last change data
  private String changedfrom = ""; // user changed
  private int inuse = 0; // read but not updated
  private String client = ""; // foreign key to client table

  // individual database fields
  private String name = ""; // Name
  private java.util.Date projdate = null; // Datum
  private String remarks = ""; // Bemerkungen
  private long xlayer = 0; // X Schicht
  private long ylayer = 0; // Y Schicht
  private long hlayer = 0; // Hidden Schicht
  private double epsilon = 0; // Epsilon
  private long learnsteps = 0; // Lernschritte
  private int savelearn = 0; // Lernkurve speichern
  private ArrayList myDbAccessVarName = new ArrayList() ; // VarName
  private ArrayList myDbAccessWeight = new ArrayList() ; // Weight
  private ArrayList myDbAccessLearning = new ArrayList() ; // Learning


  // sets and gets of standard table fields
  public void setUniqueid(String input){uniqueid = input;}
  public String getUniqueid(){return uniqueid;}
  public void setVersion(long input){version = input;}
  public long getVersion(){return version;}
  public void setCreated(java.util.Date input){created = input;}
  public java.util.Date getCreated(){return created;}
  public void setCreator(String input){creator = input;}
  public String getCreator(){return creator;}
  public void setChanged(java.util.Date input){changed = input;}
  public java.util.Date getChanged(){return changed;}
  public void setChangedfrom(String input){changedfrom = input;}
  public String getChangedfrom(){return changedfrom;}
  public void setInuse(int input){inuse = input;}
  public int getInuse(){return inuse;}
  public void setClient(String input){client = input;}
  public String getClient(){return client;}

  // sets and gets of individual table fields
  public void setName(String input){name = input;}
  public String getName(){return name;}
  public void setProjdate(java.util.Date input){projdate = input;}
  public java.util.Date getProjdate(){return projdate;}
  public void setRemarks(String input){remarks = input;}
  public String getRemarks(){return remarks;}
  public void setXlayer(long input){xlayer = input;}
  public long getXlayer(){return xlayer;}
  public void setYlayer(long input){ylayer = input;}
  public long getYlayer(){return ylayer;}
  public void setHlayer(long input){hlayer = input;}
  public long getHlayer(){return hlayer;}
  public void setEpsilon(double input){epsilon = input;}
  public double getEpsilon(){return epsilon;}
  public void setLearnsteps(long input){learnsteps = input;}
  public long getLearnsteps(){return learnsteps;}
  public void setSavelearn(int input){savelearn = input;}
  public int getSavelearn(){return savelearn;}
  public void setMyDbAccessVarName(ArrayList input){myDbAccessVarName = input;}
  public ArrayList getMyDbAccessVarName(){return myDbAccessVarName;}
  public void setMyDbAccessWeight(ArrayList input){myDbAccessWeight = input;}
  public ArrayList getMyDbAccessWeight(){return myDbAccessWeight;}
  public void setMyDbAccessLearning(ArrayList input){myDbAccessLearning = input;}
  public ArrayList getMyDbAccessLearning(){return myDbAccessLearning;}


  /**
   * The constructor <CODE>DbAccessCheckProject</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessCheckProject(Tool4DbMessages pt4m){
    t4m = pt4m;
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens of dataset
   * @param dbaccessclass Class DbAccessCustomer.
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessProject dbaccessclass , String field_list){
    return checkData(dbaccessclass , false , field_list);
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens
   * @param dbaccessclass Class DbAccess...
   * @param first = true => check standard fields
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessProject dbaccessclass , boolean first , String field_list){
    // fill standard fields
    setUniqueid(dbaccessclass.getUniqueid());
    setVersion(dbaccessclass.getVersion());
    setCreated(dbaccessclass.getCreated());
    setCreator(dbaccessclass.getCreator());
    setChanged(dbaccessclass.getChanged());
    setChangedfrom(dbaccessclass.getChangedfrom());
    setInuse(dbaccessclass.getInuse());
    setClient(dbaccessclass.getClient());
    hasdata = true;

    // fill individual fields
    setName(dbaccessclass.getName());
    setProjdate(dbaccessclass.getProjdate());
    setRemarks(dbaccessclass.getRemarks());
    setXlayer(dbaccessclass.getXlayer());
    setYlayer(dbaccessclass.getYlayer());
    setHlayer(dbaccessclass.getHlayer());
    setEpsilon(dbaccessclass.getEpsilon());
    setLearnsteps(dbaccessclass.getLearnsteps());
    setSavelearn(dbaccessclass.getSavelearn());
    setMyDbAccessVarName(dbaccessclass.getMyDbAccessVarName());
    setMyDbAccessWeight(dbaccessclass.getMyDbAccessWeight());
    setMyDbAccessLearning(dbaccessclass.getMyDbAccessLearning());


    try{
      // check standard fields
      if (!first) {
        if (getUniqueid() == null || (getUniqueid() != null && getUniqueid().length() == 0)) {
           errCode = ERROR_0009;
           errMess = t4m.getMessage("msg0009", null);
           return false;
        }
        if (getUniqueid() != null && getUniqueid().length() > 35) {
           errCode = ERROR_0010;
           errMess = t4m.getMessage("msg0010", null);
           return false;
        }
        if (getClient() == null || (getClient() != null && getClient().length() == 0)) {
           errCode = ERROR_0011;
           errMess = t4m.getMessage("msg0011", null);;
           return false;
        }
        if (getClient() != null && getClient().length() > 35) {
           errCode = ERROR_0012;
           errMess = t4m.getMessage("msg0012", null);
           return false;
        }
      }
      if (getCreator() != null && getCreator().length() > 35) {
        errCode = ERROR_0013;
        errMess = t4m.getMessage("msg0013", null);
        return false;
      }
      if (getChangedfrom() != null && getChangedfrom().length() > 35) {
        errCode = ERROR_0014;
        errMess = t4m.getMessage("msg0014", null);
        return false;
      }

      // check individual fields
      if (getName() != null && (getName().length() > 50)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("name" , "label") + "|50");
        return false;
      }
      if (getRemarks() != null && (getRemarks().length() > 500)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("remarks" , "label") + "|500");
        return false;
      }

    } catch (Exception e) {
      System.err.println(e.getMessage());
      errCode = ERROR_Ex;
      errMess = e.getMessage();
    }
    return true;
  }
}
