package n4e.netanddecision.dbaccess;

/**
* <p>title: n4e application</p>
* <p>description: netanddecision</p>
* <p>copyright: copyright (c) 2006-2011 by thomas kaffka; mba; all rights reserved.</p>
* <p>company: n.n.</p>
* @author thomas kaffka, mba.
* @version Tue Mar 14 12:11:07 CET 2017
*/

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;
import java.util.zip.*;
import java.io.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;
import n4e.core.list.*;



/**
 * The <CODE>DbAccessReadProject</CODE> class reads fields of Project table in database.
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version Tue Mar 14 12:11:07 CET 2017
*/
public class DbAccessReadProject extends DbAccessRead {

  // individual database fields
  private String name = ""; // Name
  private java.util.Date projdate = null; // Datum
  private String remarks = ""; // Bemerkungen
  private long xlayer = 0; // X Schicht
  private long ylayer = 0; // Y Schicht
  private long hlayer = 0; // Hidden Schicht
  private double epsilon = 0; // Epsilon
  private long learnsteps = 0; // Lernschritte
  private int savelearn = 0; // Lernkurve speichern
  private ArrayList myDbAccessVarName = null ; // VarName
  private ArrayList myDbAccessWeight = null ; // Weight
  private ArrayList myDbAccessLearning = null ; // Learning


  // sets and gets of individual table fields
  public void setName(String input){name = input;}
  public String getName(){return name;}
  public void setProjdate(java.util.Date input){projdate = input;}
  public java.util.Date getProjdate(){return projdate;}
  public void setRemarks(String input){remarks = input;}
  public String getRemarks(){return remarks;}
  public void setXlayer(long input){xlayer = input;}
  public long getXlayer(){return xlayer;}
  public void setYlayer(long input){ylayer = input;}
  public long getYlayer(){return ylayer;}
  public void setHlayer(long input){hlayer = input;}
  public long getHlayer(){return hlayer;}
  public void setEpsilon(double input){epsilon = input;}
  public double getEpsilon(){return epsilon;}
  public void setLearnsteps(long input){learnsteps = input;}
  public long getLearnsteps(){return learnsteps;}
  public void setSavelearn(int input){savelearn = input;}
  public int getSavelearn(){return savelearn;}
  public void setMyDbAccessVarName(ArrayList input){myDbAccessVarName = input;}
  public ArrayList getMyDbAccessVarName(){return myDbAccessVarName;}
  public void setMyDbAccessWeight(ArrayList input){myDbAccessWeight = input;}
  public ArrayList getMyDbAccessWeight(){return myDbAccessWeight;}
  public void setMyDbAccessLearning(ArrayList input){myDbAccessLearning = input;}
  public ArrayList getMyDbAccessLearning(){return myDbAccessLearning;}


  /**
   * The constructor <CODE>DbAccessReadProject</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessReadProject(){
      setWorkFields();
  }

  /**
   * The constructor <CODE>DbAccessReadProject</CODE> initialized the object.
   * @param sysUser the system user.
   * @since 1.0
  */
  public DbAccessReadProject(List4SysUser sysUser){
    setSystemUser(sysUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }
  }

  /**
   * The constructor <CODE>DbAccessReadProject</CODE> initialized the object
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadProject(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , null , true);

  }

  /**
   * The constructor <CODE>DbAccessReadProject</CODE> initialized the object
   * @param conn database connection
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadProject(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , null , true);

  }

  /**
   * The constructor <CODE>DbAccessReadProject</CODE> initialized the object with additional sqlwhere
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadProject(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd , true);

  }

  /**
   * The constructor <CODE>DbAccessReadProject</CODE> initialized the object with additional sqlwhere
    this.conn 0 conn;
   * @param conn database connection
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception
   * @since 1.0
  */
  public DbAccessReadProject(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd , true);

  }

  /**
   * The method <CODE>initiate</CODE> initializes the object with additional sqlwhere.
   * @param pUser the system user object.
   * @param uniqueid the unique key.
   * @param fieldlist List of fields to load.
   * @param privfkey List of privfkeys to load.
   * @param publfkey List of publfkeys to load.
   * @param sqlAdd additional where clause.
   * @param read_with_client read with use of the client id.
   * @exception Exception
   * @since 1.0
  */
  public void initiate(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd , boolean read_with_client) throws Exception {

    this.read_with_client = read_with_client;
    setSystemUser(pUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    publicforeignkeytable.put("idvarname","VarName|idproject");
    publicforeignkeytable.put("idweight","Weight|idproject");
    publicforeignkeytable.put("idlearning","Learning|idproject");



    success = loadFromTable(uniqueid , fieldlist , privfkey , publfkey , sqlAdd );
  }

  /**
   * The method <CODE>setWorkFields</CODE> sets the work fiels to current values.
   * @since 1.0
  */
  public void setWorkFields() {
      dbtablename = "Project"; // name of own database table
      if (sysUser != null) {
          databasename = sysUser.getClientDbsystemforeclient(); // name of own database
      }
      tblnameofprivk = ","; // list of table names of private foreign keys
      tblnameofpublvk = ",VarName,Weight,Learning,"; // list of table names of public foreign keys
      keynameofpublvk = ",idproject,idproject,idproject,"; // list of key names of table names of public foreign keys
      tblnameofownerfield = ","; // list of names of table names of owner key
      indivTableFields = ",name,projdate,remarks,xlayer,ylayer,hlayer,epsilon,learnsteps,savelearn,"; // list of inidividual table fields
      privateforeignkey = ";"; // list of private foreign key fields
      publicforeignkey = ";idvarname;idweight;idlearning;"; // list of public foreign key fields
  }

  /**
   * The method <CODE>loadFromTable</CODE> gets tabledata from database.
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @return true if success
   * @exception Exception
   * @since 1.0
  */
  public boolean loadFromTable(String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    errCode = 0;
    errMess = "";

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    hasdata = false;
    // create connection to defined database system
    String dbsystem = databasename;
    DatabaseConnectionSet myDatabaseConnectionSet = DatabaseManager.getDatabaseConnectionSet(dbsystem);
    if (this.myconn) {
        this.conn = myDatabaseConnectionSet.conn();
    }
    DatabaseSQLResultSet resSet = new DatabaseSQLResultSet(dbsystem , conn);

    // handle fields to load
    String sqlFieldlist = "";
    String sqlPrivfkey = "";

    // build sql field list
    if (fieldlist != null && fieldlist.equals("all")) {
      sqlFieldlist = indivTableFields.substring(1);
    } else { // make individual fieldlist
      if (fieldlist != null) {
        sqlFieldlist = fieldlist;
      } else {
        sqlFieldlist = "";
      }
    }

    // build sql field list of private foreign keys
    if (privfkey != null && privfkey.equals("all")) {
      sqlPrivfkey = privateforeignkey.substring(1);
    } else { // make individual fieldlist
      if (privfkey != null) {
          int pos = privfkey.indexOf("(");
          if (pos >= 0) {
            StringTokenizer stok = new StringTokenizer(privfkey , ";"); // public foreign keys separated by ;
            while (stok.hasMoreTokens()) { // for each public foreign key
                String xfield = stok.nextToken();
                pos = xfield.indexOf("(");
                if (pos >= 0) {
                    sqlPrivfkey += "," + xfield.substring(0, pos);
                } else {
                    sqlPrivfkey += "," + xfield;
                }
            }
            sqlPrivfkey = sqlPrivfkey.substring(1);
          } else {
            sqlPrivfkey = t4st.replaceToken(privfkey , ";" , ",");
          }
      } else {
        sqlPrivfkey = "";
      }
    }

    StringTokenizer stok = null;

    String workingSqlFieldlist = stdTableFields + sqlFieldlist;
    if (workingSqlFieldlist.endsWith(",")) {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?sqlPrivfkey:"")+",";
    } else {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?","+sqlPrivfkey:"")+",";
    }


    int len = workingSqlFieldlist.length();
    workingSqlFieldlist = workingSqlFieldlist.substring(1,len-1);
    if (workingSqlFieldlist.endsWith(",")) {
      len = workingSqlFieldlist.length();
      workingSqlFieldlist = workingSqlFieldlist.substring(0,len-1);
    }

    resetFields();

    // grip into data sink
    if (uniqueid == null && sqlAdd == null) {
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
        errCode = ERROR_0001;
        errMess = t4m.getMessage("msg0001", null);
        return false;
    }

    String client_temp = client;
    if (!read_with_client) {
        client = null;
    }

    String where1 = ((client!=null&&client.length()>0)?("client='"+client+"'"):"");
    String where2 = ((uniqueid!=null&&uniqueid.length()>0)?("uniqueid='"+uniqueid+"'"):"");
    String where3 = ((sqlAdd!=null&&sqlAdd.length()>0)?sqlAdd:"");
    String where = "";
    if (where1.length()==0 && where2.length()==0 && where3.length()==0) { // 000
      where = "";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()==0) { // 100
      where = " WHERE (" + where1 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()==0) { // 010
      where = " WHERE (" + where2 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()==0) { // 110
      where = " WHERE (" + where1 + ") AND (" + where2 + ")";
    } else if (where1.length()==0 && where2.length()==0 && where3.length()>0) { // 001
      where = " WHERE (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()>0) { // 101
      where = " WHERE (" + where1 + ") AND (" + where3 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()>0) { // 011
      where = " WHERE (" + where2 + ") AND (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()>0) { // 111
      where = " WHERE (" + where1 + ") AND (" + where2 + ") AND (" + where3 + ")";
    }

    String sqlStatement = "SELECT "+workingSqlFieldlist+" FROM Project " + where;
    conn.connectionlog("DbAccessReadProject","sql statement", sqlStatement);
    ResultSet result = resSet.getResultSet(sqlStatement);
    if (result.next() && (result.isFirst() && result.isLast())) {
      hasdata = true;
      // get standard fields
      setUniqueid(result.getString(1));
      setVersion(result.getLong(2));
      long temp_timestamp = result.getLong(3);
      java.util.Date temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setCreated(temp_date);
      setCreator(result.getString(4));
      temp_timestamp = result.getLong(5);
      temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setChanged(temp_date);
      setChangedfrom(result.getString(6));
      setInuse(result.getInt(7));
      setClient(result.getString(8));

      //get individual fields
      int i = 9;
      workingSqlFieldlist = sqlFieldlist + (sqlPrivfkey.length()>0?","+sqlPrivfkey:"");
      stok = new StringTokenizer(workingSqlFieldlist ,",");
      while (stok.hasMoreTokens()) {
        String xfield = stok.nextToken();
        if (false) {}
          // dummy
          else if (xfield.equals("name")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setName(value);}
          else if (xfield.equals("projdate")) {java.util.Date value = result.getDate(i);setProjdate(value);}
          else if (xfield.equals("remarks")) {String value = result.getString(i)==null?"":t4st.replaceSQLWithChars(result.getString(i));setRemarks(value);}
          else if (xfield.equals("xlayer")) {long value = result.getLong(i);setXlayer(value);}
          else if (xfield.equals("ylayer")) {long value = result.getLong(i);setYlayer(value);}
          else if (xfield.equals("hlayer")) {long value = result.getLong(i);setHlayer(value);}
          else if (xfield.equals("epsilon")) {double value = result.getDouble(i);setEpsilon(value);}
          else if (xfield.equals("learnsteps")) {long value = result.getLong(i);setLearnsteps(value);}
          else if (xfield.equals("savelearn")) {int value = result.getInt(i);setSavelearn(value);}

        i++;
      }

      //get individual privateforeignkeys
      if (privfkey != null && privfkey.length()>0) {
        String pffieldlist = "";
        if (privfkey.equals("all")) { // load all private foreign key objects with field list 'all'
          privfkey = privateforeignkey.substring(1);
        }
        stok = new StringTokenizer(privfkey , ";"); // private foreign keys separated by ;
        while (stok.hasMoreTokens()) { // for each private foreign key
          String xfield = stok.nextToken();
          if (xfield == null || xfield.length() == 0) continue;
          if (xfield.indexOf("(") > -1) { // private foreign key with field list
            StringTokenizer stfield = new StringTokenizer(xfield ,"(");
            if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (pffieldlist.endsWith(")")) {
              pffieldlist = pffieldlist.substring(0, pffieldlist.length()-1);
            }
          } else {
            pffieldlist = "all";
          }
          if (false) {}

        }
      }

    } else { // no data found
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
       errCode = ERROR_0002;
       errMess = t4m.getMessage("msg0002", null);
       return false;
    }

    //get individual publicforeignkeys
    if (publfkey != null && publfkey.length()>0) {
      String pffieldlist = "";
      if (publfkey.equals("all")) { // load all public foreign key objects with field list 'all'
        publfkey = publicforeignkey.substring(1);
      }
      DatabaseDatasetLoader dsl = new DatabaseDatasetLoader(this.conn);
      stok = new StringTokenizer(publfkey ,";"); // public foreign keys separated by ;
      while (stok.hasMoreTokens()) { // for each public foreign key
        String xfield = stok.nextToken();
        if (xfield == null || xfield.length() == 0) continue;
        StringTokenizer stfield = null;
        if (xfield.indexOf("(") > -1) { // public foreign key with field list
          stfield = new StringTokenizer(xfield ,"(");
          if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
          if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
          if (pffieldlist.endsWith(")")) {
            pffieldlist = pffieldlist.substring(0 , pffieldlist.length()-1);
          }
        } else {
          pffieldlist = "all";
        }
        String orderby = null;
        if (pffieldlist.indexOf("|") > -1) { // order by feilds available.
            stfield = new StringTokenizer(pffieldlist ,"|");
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (stfield.hasMoreTokens()) orderby = stfield.nextToken();
        }
        String pfTableName = (String) publicforeignkeytable.get(xfield);
        String pfkeyfield = "";
        StringTokenizer staccess = new StringTokenizer(pfTableName ,"|");
        if (staccess.hasMoreTokens()) pfTableName = staccess.nextToken();
        if (staccess.hasMoreTokens()) pfkeyfield = staccess.nextToken();
        String sql = pfkeyfield + " = '" + result.getString("uniqueid")+"'";
        ArrayList keyList = dsl.getDataSetRead(sysUser , pfTableName , pffieldlist , null , null , sql , orderby , "netanddecision");
        if (false) {}
        else if (xfield.equals("idvarname")) {
          setMyDbAccessVarName(keyList);
        }
        else if (xfield.equals("idweight")) {
          setMyDbAccessWeight(keyList);
        }
        else if (xfield.equals("idlearning")) {
          setMyDbAccessLearning(keyList);
        }

      }
    }

    if (!read_with_client) {
        client = client_temp;
    }

    resSet.closeResultSet();
    resSet = null;
    if (this.myconn) {
        myDatabaseConnectionSet.connClose(conn);
    }

    return true;
  }

  /**
   * The method <CODE>resetFields</CODE> sets fields to initvalue.
   * @exception Exception
   * @since 1.0
  */
  private void resetFields() throws Exception {
    uniqueid = null;
    version = 0;
    created = null;
    creator = null;
    changed = null;
    changedfrom = null;
    inuse = 0;
    name = null;
    projdate = null;
    remarks = null;
    xlayer = 0;
    ylayer = 0;
    hlayer = 0;
    epsilon = 0;
    learnsteps = 0;
    savelearn = 0;
    myDbAccessVarName = new ArrayList();
    myDbAccessWeight = new ArrayList();
    myDbAccessLearning = new ArrayList();

  }

  /**
   * The method <CODE>getDefData</CODE> returns data concerning a table field.
   * @param fieldname the name of table field
   * @param datatype type of data to return
   * @return def data
   * @exception Exception
   * @since 1.0
  */
  public String getDefData(String fieldname , String datatype) throws Exception {
    if (datatype.equals("label")) {
      if (false) {}
      else if (fieldname.equals("name")) {return getTranslationText(24);}
      else if (fieldname.equals("projdate")) {return getTranslationText(25);}
      else if (fieldname.equals("remarks")) {return getTranslationText(26);}
      else if (fieldname.equals("xlayer")) {return getTranslationText(27);}
      else if (fieldname.equals("ylayer")) {return getTranslationText(28);}
      else if (fieldname.equals("hlayer")) {return getTranslationText(29);}
      else if (fieldname.equals("epsilon")) {return getTranslationText(30);}
      else if (fieldname.equals("learnsteps")) {return getTranslationText(31);}
      else if (fieldname.equals("savelearn")) {return getTranslationText(37);}

//       return dbtablename + fieldname;
    } else if (datatype.equals("type")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "char";}
      else if (fieldname.equals("projdate")) {return "date";}
      else if (fieldname.equals("remarks")) {return "char";}
      else if (fieldname.equals("xlayer")) {return "long";}
      else if (fieldname.equals("ylayer")) {return "long";}
      else if (fieldname.equals("hlayer")) {return "long";}
      else if (fieldname.equals("epsilon")) {return "decimal_hp";}
      else if (fieldname.equals("learnsteps")) {return "long";}
      else if (fieldname.equals("savelearn")) {return "flag";}
      else if (fieldname.equals("idvarname")) {return "publicforeignkey";}
      else if (fieldname.equals("idweight")) {return "publicforeignkey";}
      else if (fieldname.equals("idlearning")) {return "publicforeignkey";}

    } else if (datatype.equals("length")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "50";}
      else if (fieldname.equals("projdate")) {return "15";}
      else if (fieldname.equals("remarks")) {return "500";}
      else if (fieldname.equals("xlayer")) {return "20";}
      else if (fieldname.equals("ylayer")) {return "20";}
      else if (fieldname.equals("hlayer")) {return "20";}
      else if (fieldname.equals("epsilon")) {return "6";}
      else if (fieldname.equals("learnsteps")) {return "20";}
      else if (fieldname.equals("savelearn")) {return "1";}
      else if (fieldname.equals("idvarname")) {return "35";}
      else if (fieldname.equals("idweight")) {return "35";}
      else if (fieldname.equals("idlearning")) {return "35";}

    } else if (datatype.equals("needed")) {
      if (false) {}
      else if (fieldname.equals("name")) {return "no";}
      else if (fieldname.equals("projdate")) {return "no";}
      else if (fieldname.equals("remarks")) {return "no";}
      else if (fieldname.equals("xlayer")) {return "no";}
      else if (fieldname.equals("ylayer")) {return "no";}
      else if (fieldname.equals("hlayer")) {return "no";}
      else if (fieldname.equals("epsilon")) {return "no";}
      else if (fieldname.equals("learnsteps")) {return "no";}
      else if (fieldname.equals("savelearn")) {return "no";}
      else if (fieldname.equals("idvarname")) {return "no";}
      else if (fieldname.equals("idweight")) {return "no";}
      else if (fieldname.equals("idlearning")) {return "no";}

    }
    return "";
  }

  /**
   * The method <CODE>getBlobField</CODE> returns the field contens of blob fields.
   * @param fieldname the name of table field.
   * @return field contens.
   * @exception Exception
   * @since 1.0
  */
  public byte[] getBlobField(String fieldname) throws Exception {
    if (false) {}

    return null;
  }

  /**
   * The method <CODE>getFieldAsStringKey</CODE> returns the field contens as a string.
   * @param fieldname the name of table field.
   * @param getkey, get the key value.
   * @return field contens
   * @exception Exception
   * @since 1.0
  */
  public String getFieldAsStringKey(String fieldname , Boolean getkey) throws Exception {
    if (false) {}
    else if (fieldname.equals("uniqueid")) {return uniqueid == null?"":uniqueid;}
    else if (fieldname.equals("version")) {return Long.toString(version);}
    else if (fieldname.equals("created")) {return created == null?"":created.toString();}
    else if (fieldname.equals("creator")) {return creator == null?"":creator;}
    else if (fieldname.equals("changed")) {return changed == null?"":changed.toString();}
    else if (fieldname.equals("changedfrom")) {return changedfrom == null?"":changedfrom;}
    else if (fieldname.equals("inuse")) {return Integer.toString(inuse);}
    else if (fieldname.equals("client")) {return client == null?"":client;}
    else if (fieldname.equals("name")) {return name == null?"":name ;}
    else if (fieldname.equals("projdate")) {return projdate == null?"":t4d.getDate2String(projdate, SystemService.getDateDef());}
    else if (fieldname.equals("remarks")) {return remarks == null?"":remarks ;}
    else if (fieldname.equals("xlayer")) {return Long.toString(xlayer);}
    else if (fieldname.equals("ylayer")) {return Long.toString(ylayer);}
    else if (fieldname.equals("hlayer")) {return Long.toString(hlayer);}
    else if (fieldname.equals("epsilon")) {return t4n.getNumber2String(epsilon , 3, SystemService.getLocaleDef());}
    else if (fieldname.equals("learnsteps")) {return Long.toString(learnsteps);}
    else if (fieldname.equals("savelearn")) {if(getkey.booleanValue()) {return Integer.toString(savelearn);} else {return savelearn == 1?t4m.getMessage("msg0015", null):t4m.getMessage("msg0016", null);}}

    return "";
  }

  /**
   * The method <CODE>getDataSet</CODE> returns a data set from table.
   * @param addsqlorder order of sql result
   * @return dataSet
   * @exception Exception
   * @since 1.0
  */
  public ArrayList getDataSet(String addsqlorder) throws Exception {
    DatabaseDatasetLoader dbl = new DatabaseDatasetLoader();
    ArrayList dataSet = dbl.getDataSetRead(sysUser , dbtablename , fields4load , privfkey4load , publfkey4load , sqlAdd , addsqlorder , "netanddecision");
    return dataSet;
  }

  /**
   * The method <CODE>getTranslationText</CODE> returns the translated text.
   * @param ndx the text index.
   * @return the translation.
   * @exception Exception
   * @since 1.0
  */
  public String getTranslationText(int ndx) throws Exception {
    if (false) {
    } else if (sysUser.getUserLanguageCode().equals("de")) {
      if (false) {}
      else if (ndx == 37) {return "Lernkurve speichern";}
      else if (ndx == 31) {return "Lernschritte";}
      else if (ndx == 30) {return "Epsilon";}
      else if (ndx == 29) {return "Hidden Schicht";}
      else if (ndx == 28) {return "Y Schicht";}
      else if (ndx == 27) {return "X Schicht";}
      else if (ndx == 26) {return "Bemerkungen";}
      else if (ndx == 25) {return "Datum";}
      else if (ndx == 24) {return "Name";}

    } else if (sysUser.getUserLanguageCode().equals("en")) {
      if (false) {}
      else if (ndx == 37) {return "Save Learn Curve";}
      else if (ndx == 31) {return "Learn Steps";}
      else if (ndx == 30) {return "Epsilon";}
      else if (ndx == 29) {return "Hidden Layer";}
      else if (ndx == 28) {return "Y Layer";}
      else if (ndx == 27) {return "X Layer";}
      else if (ndx == 26) {return "Remarks";}
      else if (ndx == 25) {return "Date";}
      else if (ndx == 24) {return "Name";}


    }
    return "no text available";
  }


}
