/**
 * ImportKeySetCH.java
 *
 * @version 12. Mai 2006, 15:18
 *
 * <P><B>title: Dialog Library</B></P>
 * <P>description: common classes for dialog applications.</P>
 * <P>copyright: (c) 2010 thomas kaffka, born at 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
* @author thomas kaffka , mba , cologne , germany.
 */

package n4e.netanddecision.saxparser;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

import org.xml.sax.*;

import n4e.core.saxparser.SaxParserCH;
import n4e.core.system.*;
import n4e.core.list.*;
import n4e.core.tool.*;

/**
 * The <CODE>ImportExpSystemCH</CODE> class imports or exports a given keyset or all to xml
 * @author Dipl.-Kfm. Thomas Kaffka
 * @version 12. Mai 2006, 15:18
 * @since 1.0
 */
public class ImportExamplesCH extends SaxParserCH {
    
	private ArrayList<ArrayList<Number>> examples;
	private ArrayList<Number> line; 
	private ArrayList<Integer> dimension;
	private int count_value;
	private int number_x;
	private int number_y;
	
    /**
     * The constructor <CODE>ImportExpSystemCH</CODE> creates a new instance(object) of the class.
     * @param sysUser the current user object.
     * @param idclientkey the uniqueid of the current client.
     * @since 1.0
     */
    public ImportExamplesCH(ArrayList<ArrayList<Number>> examples, ArrayList<Integer> dimension) {
        if (SystemService.getSaxParserDebug()) {
            System.out.println("instantiation of ImportExpSystemCH()");
        }
        this.examples = examples;
        this.dimension = dimension;
        count_value = 0;
    }
    
    /**
     * The method <CODE>startDocument</CODE> is called if document starts.
     * @exception SAXException
     * @since 1.0
     */
    public void startDocument() throws SAXException {
        super.startDocument();
    }
    
    /**
     * The method <CODE>endDocument</CODE> is called if document ends.
     * @exception SAXException
     * @since 1.0
     */
    public void endDocument() throws SAXException {
        super.endDocument();
    }
    
    /**
     * The method <CODE>startElement</CODE> is called, if xml-token starts.
     * @param namespaceURI name space.
     * @param localName locale.
     * @param rawName name of token.
     * @param atts attributes of token.
     * @exception SAXException.
     * @since 1.0
     */
    public void startElement(String namespaceURI , String localName ,
            String rawName , Attributes atts) throws SAXException {
        super.startElement(namespaceURI , localName , rawName , atts);
        
       try {
            
            if (rawName.equals("NeuralNetwork")) {
                
            	number_x = Integer.parseInt(atts.getValue("numberx"));
            	number_y = Integer.parseInt(atts.getValue("numbery"));
            	dimension.add(number_x);
            	dimension.add(number_y);
            } else if (rawName.equals("ExampleX")) {
                if (count_value == 0) {
                	line = new ArrayList<Number>();
                }
                
                Number value = Double.parseDouble(atts.getValue("value"));
                line.add(value);
                count_value++;

            } else if (rawName.equals("ExampleY")) {
                
                Number value = Double.parseDouble(atts.getValue("value"));
                line.add(value);
                count_value++;
                
                if (count_value >= number_x + number_y) {
                	count_value = 0;
                	examples.add(line);
                }

            }
            
            
        } catch (Exception err) {
			System.out.println(Tool4Throwable.getCallStackOut(err));
        }
    }
    
    /**
     * The method <CODE>endElement</CODE> is called, if xml-token ends.
     * @param namespaceURI name space.
     * @param localName locale.
     * @param rawName name of token.
     * @exception SAXException.
     * @since 1.0
     */
    public void endElement(String namespaceURI , String localName , String rawName) throws SAXException {
        super.endElement(namespaceURI , localName , rawName);
    }
    
}
