package n4e.netanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import n4e.core.controls.*;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.screen.*;
import n4e.core.system.GlobalsDialog;
import n4e.core.tool.*;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.dbaccess.DbAccessReadVarName;
import n4e.netanddecision.saxparser.ImportExamples;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;
import n4e.netanddecision.tool.Tool4MyDialog;

/**
 * The <CODE>LayerNameDataSite</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2014 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 18.02.2017 14:29:36
 * @since 1.0
 */

public class ExampleDataSite extends DataSite {

    private JLabel lblCount;
    private AWTextField[] tfValue;
    private JLabel[] lblName;
    private AWCombo[] cbxValue;
    private JSeparator separator;

    private JButton btnDown;
    private JButton btnUp;
    private JButton btnChange;
    private JButton btnDelete;
    private JButton btnLoad;
    private JButton btnFile;

    private String project_uniqueid;
    
    private int var_number_old;
    private int panel_width;
    private int panel_height;
    private int example_ndx;
    private String xml_data;
    private int var_len;
    private int number_x_layer;
    private int number_y_layer;
    private String proj_name;

    /**
	 * The constructor <CODE>LayerNameDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public ExampleDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			panel_width = 600;
			panel_height = 340;
			
			var_number_old = 0;
			
			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));

			createHead(pnlScreen, Globals.message.getMessage("msg0036"));

        	createLabel(pnlScreen, Globals.message.getMessage("msg0051"));
        	lblCount = createLabelField(pnlScreen, control_column, 100, false);
			
			btnDown = createButton(pnlScreen, panel_height - 30, 370,
					DataSite.BUTTON_DOWN, Globals.message.getMessage("msg0053"));
			btnDown.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnDown_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnDown.setEnabled(false);

			btnUp = createButton(pnlScreen, panel_height - 30, 410,
					DataSite.BUTTON_UP, Globals.message.getMessage("msg0054"));
			btnUp.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnUp_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnUp.setEnabled(false);

			btnChange = createButton(pnlScreen, panel_height - 30, 450,
					DataSite.BUTTON_USE, Globals.message.getMessage("msg0082"));
			btnChange.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnChange_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnChange.setEnabled(false);

			btnDelete = createButton(pnlScreen, panel_height - 30, 490,
					DataSite.BUTTON_DELETE, Globals.message.getMessage("msg0081"));
			btnDelete.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnDelete_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnDelete.setEnabled(false);

			btnFile = createButton(pnlScreen, panel_height - 30, 530,
					DataSite.BUTTON_SAVE, Globals.message.getMessage("msg0056"));
			btnFile.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnFile_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnFile.setEnabled(false);

			btnLoad = createButton(pnlScreen, panel_height - 30, 570,
					DataSite.BUTTON_UPLOAD, Globals.message.getMessage("msg0055"));
			btnLoad.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnLoad_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnLoad.setEnabled(false);

			sclScreen = createScrollPane(pnlScreen, pnl_width - 20, pnl_height - 50);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>functionEditable</CODE> blocks the functions, if training.
	 * @param editable false -> block.
	 * @since 1.0
	 */
	public void functionEditable(boolean editable) {
	    btnDown.setEnabled(editable);
	    btnUp.setEnabled(editable);
	    btnChange.setEnabled(editable);
	    btnDelete.setEnabled(editable);
	    btnLoad.setEnabled(editable);
	    btnFile.setEnabled(editable);
	}

	/**
	 * The method <CODE>btnFile_actionPerformed</CODE> action of the btnFile.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnFile_actionPerformed(ActionEvent e) throws Exception {
		
		if (Globals.examples == null) {
			Globals.t4dialog.showErrorDialog(Globals.message.getMessage("msg0078"));
			return;
		}
		
		String[] extension = null;
		String message = null;
		extension = new String[1];
		extension[0] = "xml";
		message = Globals.message.getMessage("msg0052");

		String path = Globals.t4md.fileSelectionDialog(extension, message, Tool4MyDialog.SAVE);
		if (path != null) {
			if (!path.toLowerCase().endsWith(".xml")) {
				path += ".xml";
			}
			
			xml_data = Globals.t4xml.getExampleHeader(proj_name, number_x_layer, number_y_layer);

			int len = Globals.examples.size();
			for (int ndx=0;ndx<len;ndx++) {
				ArrayList<Number> line = Globals.examples.get(ndx);
				for (int ndx2=0;ndx2<number_x_layer;ndx2++) {
					Number value = line.get(ndx2);
					String xml_line = Globals.t4xml.getExampleXValue(Double.toString(value.doubleValue()));
					xml_data = Globals.t4s.preInsertString(xml_data, "@examplelist@", xml_line);
				}
				for (int ndx2=number_x_layer;ndx2<number_x_layer + number_y_layer;ndx2++) {
					Number value = line.get(ndx2);
					String xml_line = Globals.t4xml.getExampleYValue(Double.toString(value.doubleValue()));
					xml_data = Globals.t4s.preInsertString(xml_data, "@examplelist@", xml_line);
				}
			}
			xml_data = Globals.t4s.replaceString(xml_data, "@examplelist@\r\n", "");
			Globals.t4f.putTheFile(path, "", xml_data);
		}
	}
	
	/**
	 * The method <CODE>btnLoad_actionPerformed</CODE> action of the btnLoad.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnLoad_actionPerformed(ActionEvent e) throws Exception {
		String[] extension = null;
		String message = null;
		extension = new String[1];
		extension[0] = "xml";
		message = Globals.message.getMessage("msg0052");

		String path = Globals.t4md.fileSelectionDialog(extension, message, Tool4MyDialog.SELECT);
		if (path != null) {
			example_ndx = 0;
			Globals.examples = new ArrayList<ArrayList<Number>>();
			ArrayList<Integer> dimension = new ArrayList<Integer>();
			ImportExamples imp = new ImportExamples();
			imp.parseDocument(path, Globals.examples, dimension);
			if (imp.errCode != 0) {
				t4dialog.showException(imp.errCode + "\r\n" + imp.errMess);
			}
			
			DbAccessReadProject project = new DbAccessReadProject(Globals.sysUser, project_uniqueid, "xlayer,ylayer", null, null);
			int x_layer = dimension.get(0);
			int y_layer = dimension.get(1);
			if (x_layer != project.getXlayer() || y_layer != project.getYlayer()) {
				Globals.t4dialog.showErrorDialog(Globals.message.getMessage("msg0064"));
				Globals.examples = new ArrayList<ArrayList<Number>>();
				return;
			}
			
			if (Globals.examples.size() > 0) {
				setControlsWithValues();
			}
		}
	}
	
	/**
	 * The method <CODE>btnDown_actionPerformed</CODE> action of the btnDown.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnDown_actionPerformed(ActionEvent e) throws Exception {
		
		if (Globals.examples == null) {
			Globals.examples = new ArrayList<ArrayList<Number>>();
		}

		example_ndx++;
		
    	if (example_ndx == Globals.examples.size()) {
        	for (int ndx=0;ndx<var_len;ndx++) {
				if (tfValue[ndx] != null) {
					tfValue[ndx].setText("");
				}
				if (cbxValue[ndx] != null) {
					cbxValue[ndx].setIndexKey("1");
				}
        	}
        	
    	} else if (example_ndx > Globals.examples.size()) {
    		ArrayList<Number> line = new ArrayList<Number>();
			try {
	        	for (int ndx=0;ndx<var_len;ndx++) {
	        		Number value = null;
					if (tfValue[ndx] != null) {
						value = Double.parseDouble(Globals.t4s.replaceString(tfValue[ndx].getText(), ",", "."));
						tfValue[ndx].setText("");
					}
					if (cbxValue[ndx] != null) {
						value = Integer.parseInt(cbxValue[ndx].getIndexKey());
						cbxValue[ndx].setIndexKey("1");
					}
					if (value != null) {
						line.add(value);
					}
	        	}
	        	
			} catch (Exception ex) {
				example_ndx--;
				return;
			}
			
			Globals.examples.add(line);
			
    	} else {
    		
    		setControlsWithValues();
    		
    	}

    	lblCount.setText(Integer.toString(example_ndx+1) 
    			+ " | " + Integer.toString(Globals.examples.size()));
	}
	
	/**
	 * The method <CODE>btnUp_actionPerformed</CODE> action of the btnUp.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnUp_actionPerformed(ActionEvent e) throws Exception {
		
		if (Globals.examples == null || Globals.examples.size() == 0) {
			Globals.t4dialog.showErrorDialog(Globals.message.getMessage("msg0078"));
			return;
		}

		example_ndx--;
		if (example_ndx < 0) {
			example_ndx = 0;
		}
    	lblCount.setText(Integer.toString(example_ndx+1) + " | " + Integer.toString(Globals.examples.size()));
    	setControlsWithValues();
	}

	/**
	 * The method <CODE>btnChange_actionPerformed</CODE> action of the btnChange.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnChange_actionPerformed(ActionEvent e) throws Exception {
		
		if (Globals.examples == null || Globals.examples.size() == 0) {
			return;
		}
		
		if (example_ndx == Globals.examples.size()) {
			return;
		}

		ArrayList<Number> line = new ArrayList<Number>();
		try {
        	for (int ndx=0;ndx<var_len;ndx++) {
        		Number value = null;
				if (tfValue[ndx] != null) {
					value = Double.parseDouble(Globals.t4s.replaceString(tfValue[ndx].getText(), ",", "."));
					tfValue[ndx].setText("");
				}
				if (cbxValue[ndx] != null) {
					value = Integer.parseInt(cbxValue[ndx].getIndexKey());
					cbxValue[ndx].setIndexKey("1");
				}
				if (value != null) {
					line.add(value);
				}
        	}
        	
		} catch (Exception ex) {
			return;
		}
		
		Globals.examples.remove(example_ndx);
		Globals.examples.add(example_ndx, line);
    	setControlsWithValues();
	}
	
	/**
	 * The method <CODE>showDeleteDialog</CODE> shows a window with the delete question.
	 * @return the answer status.
	 * @since 1.0
	 */
	public int showDeleteDialog() {
		JOptionPane warning = new JOptionPane(GlobalsDialog.message.getMessage("msg0003"), 
				JOptionPane.WARNING_MESSAGE , JOptionPane.YES_NO_OPTION);
		JDialog dialog = warning.createDialog(null, GlobalsDialog.message.getMessage("msg0004"));
		dialog.setVisible(true);
		Object selectedValue = warning.getValue();
		int value = Integer.parseInt(selectedValue.toString());
		System.out.println(value);
		return value;
	}

	/**
	 * The method <CODE>btnDelete_actionPerformed</CODE> action of the btnDelete.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnDelete_actionPerformed(ActionEvent e) throws Exception {
		
		if (Globals.examples == null || Globals.examples.size() == 0) {
			return;
		}
		
		if (example_ndx == Globals.examples.size()) {
			return;
		}
		
		if (showDeleteDialog() == 1) {
			return;
		}
		
		Globals.examples.remove(example_ndx);
		example_ndx--;
		
		if (example_ndx < 0) {
			example_ndx = 0;
		}
		
		if (Globals.examples.size() == 0) {
			
			Globals.examples = null;
			
        	for (int ndx=0;ndx<var_len;ndx++) {
				if (tfValue[ndx] != null) {
					tfValue[ndx].setText("");
				}
				if (cbxValue[ndx] != null) {
					cbxValue[ndx].setIndexKey("1");
				}
        	}
        	
		} else { 
	    	setControlsWithValues();
		}

		if (Globals.examples == null) {
			lblCount.setText("1 | 0");
		} else {
			lblCount.setText(Integer.toString(example_ndx+1) 
	    			+ " | " + Integer.toString(Globals.examples.size()));
		}
	}
	
	/**
	 * The method <CODE>setControlsWithValues</CODE> set the control data with values.
	 * @exception Exception
	 * @since 1.0
	 */
	public void setControlsWithValues() throws Exception {
    	ArrayList<Number> line = Globals.examples.get(example_ndx);
    	
    	for (int ndx=0;ndx<var_len;ndx++) {
    		Number value = line.get(ndx);
			if (tfValue[ndx] != null) {
				tfValue[ndx].setText(Globals.t4n.getNumber2String(value.doubleValue(), 15, GlobalsDialog.locale_language));
			}
			if (cbxValue[ndx] != null) {
				cbxValue[ndx].setIndexKey(Integer.toString(value.intValue()));
			}
    	}
	}
	
	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		sclScreen.setVisible(true);
		project_uniqueid = uniqueid;
		example_ndx = 0;
    	lblCount.setText("1 | 0");
		label_line = 50;
		
		DbAccessReadProject proj = new DbAccessReadProject(Globals.sysUser, project_uniqueid, "name,xlayer,ylayer", null, null);
		if (proj.errCode == 0) {
			number_x_layer = (int) proj.getXlayer();
			number_y_layer = (int) proj.getYlayer();
			proj_name = proj.getName();
		} else {
			t4dialog.showException(proj.errCode + "\r\n" + proj.errMess);
			return;
		}
		
		btnDown.setEnabled(true);
		btnUp.setEnabled(true);
		btnChange.setEnabled(true);
		btnDelete.setEnabled(true);
		btnFile.setEnabled(true);
		btnLoad.setEnabled(true);
		
		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "VarName",
				"name,vartype,layer", null, null, "idproject='" + project_uniqueid + "'",
				"layer,name ASC", "netanddecision");
		
		var_len = tabs.size();

		// remove old controls.
		if (var_number_old != 0) {
			for (int ndx=0;ndx<var_number_old;ndx++) {
				pnlScreen.remove(lblName[ndx]);
				if (separator != null) {
					pnlScreen.remove(separator);
				}
				if (tfValue[ndx] != null) {
					pnlScreen.remove(tfValue[ndx]);
				}
				if (cbxValue[ndx] != null) {
					pnlScreen.remove(cbxValue[ndx]);
				}
			}
		}
		var_number_old = var_len;

		// lengthen the window.
		if (var_len > 11) {
			int more_vars = var_len - 10;
			int curr_height = panel_height + (more_vars * 20) + 20;
			pnlScreen.setPreferredSize(new Dimension(panel_width, curr_height));
			btnDown.setBounds(370, curr_height - 30, 30, 23);
			btnUp.setBounds(410, curr_height - 30, 30, 23);
			btnChange.setBounds(450, curr_height - 30, 30, 23);
			btnDelete.setBounds(490, curr_height - 30, 30, 23);
			btnFile.setBounds(530, curr_height - 30, 30, 23);
			btnLoad.setBounds(570, curr_height - 30, 30, 23);
		} else {
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));
			btnDown.setBounds(370, panel_height - 30, 30, 23);
			btnUp.setBounds(410, panel_height - 30, 30, 23);
			btnChange.setBounds(450, panel_height - 30, 30, 23);
			btnDelete.setBounds(490, panel_height - 30, 30, 23);
			btnFile.setBounds(530, panel_height - 30, 30, 23);
			btnLoad.setBounds(570, panel_height - 30, 30, 23);
		}
		
		// get the data and show it.
		lblName = new JLabel[var_len];
		tfValue = new AWTextField[var_len];
		cbxValue = new AWCombo[var_len];

		boolean separator_done = false;
    	for (int ndx=0;ndx<var_len;ndx++) {
			
        	DbAccessReadVarName vn = (DbAccessReadVarName) tabs.get(ndx);
        	
        	if (vn.getLayer() == Globals.Y_LAYER && !separator_done) {
        		separator = createSeparator(pnlScreen, 600);
    	        separator_done = true;
        	}
        	
        	lblName[ndx] = createLabel(pnlScreen, vn.getName());
            
            if (vn.getVartype().equals("288ed0e9:15a56a92806:-7fff")) { // continuous.
            	tfValue[ndx] = createTextField(pnlScreen);
            } else if (vn.getVartype().equals("288ed0e9:15a56a92806:-7ffe")) { // discrete.
                cbxValue[ndx] = createCombo(pnlScreen, control_column, 200);
                Globals.t4md.fillComboValue(cbxValue[ndx], vn.getUniqueid(), null);
            }

		}
    	
    	pnlScreen.repaint();
	}

}
