package n4e.netanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import n4e.core.controls.*;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.screen.*;
import n4e.core.system.GlobalsDialog;
import n4e.core.tool.*;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.dbaccess.DbAccessReadVarName;
import n4e.netanddecision.saxparser.ImportExamples;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;
import n4e.netanddecision.tool.Tool4MyDialog;

/**
 * The <CODE>LayerNameDataSite</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 18.02.2017 14:29:36
 * @since 1.0
 */

public class ExecutionDataSite extends DataSite {

    private AWTextField[] tfValue;
    private JLabel[] lblName;
    private AWCombo[] cbxValue;
    private JSeparator separator;

    private JButton btnCalc;

    private DbAccessReadProject proj;
    
    private String project_uniqueid;
    
    private int var_number_old;
    private int panel_width;
    private int panel_height;
    private int var_len;

    /**
	 * The constructor <CODE>LayerNameDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public ExecutionDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			panel_width = 600;
			panel_height = 340;
			
			var_number_old = 0;
			
			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));

			createHead(pnlScreen, Globals.message.getMessage("msg0039"));

			btnCalc = createButton(pnlScreen, panel_height - 30, 570,
					DataSite.BUTTON_USE, Globals.message.getMessage("msg0068"));
			btnCalc.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnCalc_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
			btnCalc.setEnabled(false);

			sclScreen = createScrollPane(pnlScreen, pnl_width - 50, pnl_height - 50);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>functionEditable</CODE> blocks the functions, if training.
	 * @param editable false -> block.
	 * @since 1.0
	 */
	public void functionEditable(boolean editable) {
		btnCalc.setEnabled(editable);
	}

	/**
	 * The method <CODE>btnCalc_actionPerformed</CODE> action of the btncalc.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnCalc_actionPerformed(ActionEvent e) throws Exception {
		
		DbAccessReadProject proj = new DbAccessReadProject(Globals.sysUser, project_uniqueid, "xlayer,ylayer", null, null);
		if (proj.errCode != 0) {
			t4dialog.showException(proj.errCode + "\r\n" + proj.errMess);
			return;
		}
		
		int x_layer = (int) proj.getXlayer();
		int y_layer = (int) proj.getYlayer();
		double[] x = new double[x_layer];
		for (int ndx=0;ndx<x_layer;ndx++) {
			if (tfValue[ndx] != null && tfValue[ndx].getText().length() > 0) {
				x[ndx] = Globals.t4n.getString2Double(tfValue[ndx].getText(), GlobalsDialog.locale_language);
			}
			if (cbxValue[ndx] != null) {
				x[ndx] = Double.parseDouble(cbxValue[ndx].getIndexKey());
			}
		}
		
		double[] result = Globals.t4nn.executeFunction(x);
		
		if (result == null) {
			Globals.t4dialog.showErrorDialog(Globals.message.getMessage("msg0083"));
			return;
		}
		
		int count = 0;
		for (int ndx=x_layer;ndx<x_layer+y_layer;ndx++) {
			if (tfValue[ndx] != null) {
				tfValue[ndx].setText(Globals.t4n.getNumber2String(result[count], 15, GlobalsDialog.locale_language));
			}
			if (cbxValue[ndx] != null) {
				cbxValue[ndx].setIndexKey(Long.toString(Math.round(result[count])));
			}
			count++;
		}
	}
	
	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		sclScreen.setVisible(true);
		project_uniqueid = uniqueid;
		label_line = 20;
		
		btnCalc.setEnabled(true);

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "VarName",
				"name,vartype,layer", null, null, "idproject='" + project_uniqueid + "'",
				"layer,name ASC", "netanddecision");
		
		var_len = tabs.size();

		// remove old controls.
		if (var_number_old != 0) {
			for (int ndx=0;ndx<var_number_old;ndx++) {
				pnlScreen.remove(lblName[ndx]);
				if (separator != null) {
					pnlScreen.remove(separator);
				}
				if (tfValue[ndx] != null) {
					pnlScreen.remove(tfValue[ndx]);
				}
				if (cbxValue[ndx] != null) {
					pnlScreen.remove(cbxValue[ndx]);
				}
			}
		}
		var_number_old = var_len;

		// lengthen the window.
		if (var_len > 11) {
			int more_vars = var_len - 10;
			int curr_height = panel_height + (more_vars * 20) + 20;
			pnlScreen.setPreferredSize(new Dimension(panel_width, curr_height));
			btnCalc.setBounds(570, curr_height - 30, 30, 23);
		} else {
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));
			btnCalc.setBounds(570, panel_height - 30, 30, 23);
		}
		
		// get the data and show it.
		lblName = new JLabel[var_len];
		tfValue = new AWTextField[var_len];
		cbxValue = new AWCombo[var_len];

		boolean separator_done = false;
    	for (int ndx=0;ndx<var_len;ndx++) {
			
        	DbAccessReadVarName vn = (DbAccessReadVarName) tabs.get(ndx);
        	
        	if (vn.getLayer() == Globals.Y_LAYER && !separator_done) {
        		separator = createSeparator(pnlScreen, 600);
    	        separator_done = true;
        	}
        	
        	lblName[ndx] = createLabel(pnlScreen, vn.getName());
            
            if (vn.getVartype().equals("288ed0e9:15a56a92806:-7fff")) { // continuous.
            	tfValue[ndx] = createTextField(pnlScreen);
            	if (separator_done) {
            		tfValue[ndx].setEditable(false);
            	}
            } else if (vn.getVartype().equals("288ed0e9:15a56a92806:-7ffe")) { // discrete.
                cbxValue[ndx] = createCombo(pnlScreen, control_column, 200);
                Globals.t4md.fillComboValue(cbxValue[ndx], vn.getUniqueid(), null);
            	if (separator_done) {
            		cbxValue[ndx].setEditable(false);
            	}
            }

		}
    	
    	pnlScreen.repaint();
	}

}
