package n4e.netanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.*;

import javax.swing.*;

import n4e.core.controls.AWTextField;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.list.ImageElement;
import n4e.core.list.ImageLine;
import n4e.core.list.ImagePoint;
import n4e.core.screen.*;
import n4e.core.tool.*;
import n4e.core.toolmath.Tool4ExpotentialFunction;
import n4e.core.toolmath.Tool4LinearRegression;
import n4e.core.toolmath.Tool4PowerFunction;
import n4e.core.toolmath.Tool4RootFunction;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;

/**
 * The <CODE>GraphicDataSite</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 22.02.2017 15:05:36
 * @since 1.0
 */

public class GraphicDataSite extends DataSite {

    private JLabel lblCoordSystem; // the coordinate system graphic.

    private JLabel lblWidth;
    private AWTextField tfAmount;

    private JButton btnPlus;
    private JButton btnMinus;

    private ArrayList<ImageElement> graphic_data;
    
    private int graphic_height; // the height of the graphic.
    private int graphic_width; // the width of the graphic.
    private double d;

    /**
	 * The constructor <CODE>GraphicDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public GraphicDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			int panel_width = 600;
			int panel_height = 340;
			d = 4000;
			
			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));

		    sclScreen = createScrollPane(pnlScreen, pnl_width - 20, pnl_height - 50);

			graphic_height = 300;
			graphic_width = 595;
			lblCoordSystem = new JLabel("");
			lblCoordSystem.setBounds(10, 10, graphic_width, graphic_height);
	        pnlScreen.add(lblCoordSystem);

	        label_line = panel_height - 20;
	        
	        createLabelAt(pnlScreen, Globals.message.getMessage("msg0076"), 280);
	        lblWidth = createLabelAt(pnlScreen, "", 370);
	        lblWidth.setText(Double.toString(d));
	        
	        tfAmount = createTextField(pnlScreen, 450, 70);
	        tfAmount.setText("100");
	        
			btnPlus = createButton(pnlScreen, panel_height - 20, 530,
					DataSite.BUTTON_PLUS, Globals.message.getMessage("msg0074"));
			btnPlus.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnPlus_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnMinus = createButton(pnlScreen, panel_height - 20, 570,
					DataSite.BUTTON_MINUS, Globals.message.getMessage("msg0075"));
			btnMinus.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnMinus_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

	        
	        resetGraphicData();
	        
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>btnPlus_actionPerformed</CODE> action of the btnPlus.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnPlus_actionPerformed(ActionEvent e) throws Exception {
		int amount = Integer.parseInt(tfAmount.getText());
		d += amount;
        lblWidth.setText(Double.toString(d));
	}
	
	/**
	 * The method <CODE>btnMinus_actionPerformed</CODE> action of the btnMinus.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnMinus_actionPerformed(ActionEvent e) throws Exception {
		int amount = Integer.parseInt(tfAmount.getText());
		d -= amount;
        lblWidth.setText(Double.toString(d));
	}
	
	private ArrayList<Double> transform(double x, double y, double z, double cx, double cy) {
		double rho   = 30;
		double theta = (double) 0.3;
		double phi   = (double) 0.8;
		double s1  = (double)Math.sin(theta);
		double s2  = (double)Math.sin(phi);
		double c1  = (double)Math.cos(theta);
		double c2  = (double)Math.cos(phi);
		double xe = -x * s1 + y * c1;
		double ye = -x * c1 * c2 - y * s1 * c2 + z * s2;
		double ze = -x * s2 * c1 - y * s2 * s1 - z * c2 + rho;
		double sx = d * xe / ze + cx;
		double sy = cy - d * ye / ze;
		ArrayList<Double> result = new ArrayList<Double>();
		result.add(sx);
		result.add(sy);
		return result;
	}

	public ArrayList<ImageElement> getGraphicData() {
		return graphic_data;
	}
	
	/**
	 * The method <CODE>resetGraphicData</CODE> reset the graphic data.
	 * @since 1.0
	 */
	public void resetGraphicData() throws Exception {
        paintTheCoordSystem3D();
	}

	/**
	 * The method <CODE>setLearningCurve</CODE> sets the learn curve.
	 * @param gd the graphic data.
	 * @since 1.0
	 */
	public void setGraphic(ArrayList<ImageElement> gd) throws Exception {
		graphic_data = gd;
	}
	
	/**
	 * The method <CODE>paintGraphic</CODE> paints the graphic.
	 * @param gd the graphic data.
	 * @since 1.0
	 */
	public boolean paintGraphic() throws Exception {
		
		DbAccessReadProject proj = new DbAccessReadProject(Globals.sysUser, Globals.project_uniqueid, "all", null, null);
		
		if (proj.getXlayer() == 1 && proj.getYlayer() == 1) {
			paintTheCoordSystem2D(Globals.message.getMessage("msg0041"), true);
			return true;
		} else if (proj.getXlayer() == 2 && proj.getYlayer() == 1) {
			paintTheCoordSystem3D();
			return true;
		}
		
		paintTheCoordSystem2D(Globals.message.getMessage("msg0072"), false);

		return false;
	}

	/**
     * The method <CODE>paintTheCoordSystem2D</CODE> paint the 2D graphics.
     * @param graphic_data the list with the graphic elements.
     * @param t4lr the Tool4LinearRegression.
	 * @exception Exception
     * @since 1.0
     */
	private void paintTheCoordSystem2D(String head_line, boolean draw_it) throws Exception {
		
		if (Globals.examples == null) {
			return;
		}
		
		Tool4Image2 t4i = new Tool4Image2(null, Tool4Image.COORDSYS_CARTESIAN, 
        		Tool4Image.COORDSYS_DIM2, graphic_width, graphic_height, true);
		
		graphic_data = new ArrayList<ImageElement>();
		
		// prepare the graphic data.
		double old_x = 0;
		double old_y = 0;
		boolean first = true;
		int len = Globals.examples.size();
		for (int ndx=0;ndx<len;ndx++) {
	        ArrayList<Number> line = Globals.examples.get(ndx);
			double x = line.get(0).doubleValue();
			double y = line.get(1).doubleValue();
			ImagePoint point = new ImagePoint(x, y, 0, 1, false);
			point.setColor(Globals.color_graphic_paint_point);
            graphic_data.add(point);
	        double[] x_net = new double[1];
	        x_net[0] = x;
	        double[] y_net = Globals.t4nn.executeFunction(x_net);
	        if (first) {
	        	old_x = x;
	        	old_y = y;
	        	first = false;
	        } else {
	        	ImageLine image_line = new ImageLine(old_x, old_y, 0, x, y_net[0], 0, false);
	            graphic_data.add(image_line);
	        	old_x = x;
	        	old_y = y_net[0];
	        }
		}
		
        // define the grid.
	    t4i.setXaxisName("X");
        t4i.setYaxisName("Y");

        // define the graphic.
        t4i.setBackgroundColor(Globals.color_graphic_background);
        t4i.doInitialize();
        t4i.setLines(null);
        t4i.setGraphicBorder(10);
        t4i.setGridWidth(40);
        
        t4i.setPaintColor(Globals.color_graphic_coordinate);
        t4i.setKoordSystem2D(true);
        t4i.setAxisNames();
        
        t4i.setPaintColor(Globals.color_graphic_border);
        t4i.drawBorder(); 

        // draw points and lines.
        if (draw_it) {
	        t4i.setPaintColor(Globals.color_graphic_coordinate);
	        t4i.setLines(graphic_data);
	        t4i.calcFactors(Tool4Image.COORDSYS_XNOTY);
	        t4i.setAxisMaxValues();
        }

        t4i.setPaintColor(Globals.color_graphic_grid);
        t4i.drawGridLines();
        t4i.setPaintColor(Globals.color_graphic_coordinate);
        if (draw_it) {
        	t4i.drawGridValues(Globals.fontStandard11, 0, 3, Globals.sysUser.getUserLocale());
        }

        String graphic_name = head_line;
        t4i.setGraphicName(graphic_name);
        t4i.drawGraphicName(graphic_width / 2 - (graphic_name.length() * 7) / 2, 16);
        
        t4i.setPaintColor(Globals.color_graphic_paint_line);
        t4i.drawAllElements2D(false, true);
        t4i.deleteAddons();
        
		lblCoordSystem.setIcon(new ImageIcon(t4i.getImage()));
    }

	public void paintTheCoordSystem3D() {
		
		// define graphics.
		BufferedImage image = new BufferedImage(graphic_width, graphic_height, BufferedImage.TYPE_3BYTE_BGR);
		Graphics graph = image.getGraphics();
        graph.setColor(Globals.color_graphic_background);
        graph.fillRect(0, 0, graphic_width, graphic_height);

        // set graphic title.
    	graph.setColor(Globals.color_graphic_coordinate);
    	graph.setFont(new Font("Monospaced", Font.BOLD, 16));
        graph.drawString(Globals.message.getMessage("msg0041"), 280, 30);

        // draw border.
    	graph.setColor(Globals.color_graphic_coordinate);
    	graph.drawLine(0, 0, 0, graphic_height);
    	graph.drawLine(0, 0, graphic_width, 0);
    	graph.drawLine(graphic_width-1, 0, graphic_width-1, graphic_width-1);
    	graph.drawLine(0, graphic_height-1, graphic_width, graphic_height-1);
        
    	if (Globals.examples == null) {
    		lblCoordSystem.setIcon(new ImageIcon(image));
    		return;
    	}
    	
    	double dimension = Math.sqrt(Globals.examples.size());
    	if (dimension != Math.floor(dimension)) {
    		lblCoordSystem.setIcon(new ImageIcon(image));
    		return;
    	}
    	
		int POINT_LEN = (int) Math.sqrt(Globals.examples.size());

		double sx_old = 0;
		double sy_old = 0;
		boolean begin_points;

		double[][] point_old = new double[POINT_LEN][2];
		double[][] point_new = new double[POINT_LEN][2];
		double center_XA = 100;
		double center_XB = 350;
		double center_Y  = 150;

		// paint example.
    	graph.setColor(Color.BLUE);
		int exam_ndx = 0;
		double sx = 0;
		double sy = 0;
		for (int x=0;x<POINT_LEN;x++) {
			begin_points = false;
			for (int y=0;y<POINT_LEN;y++) {
				sx_old = sx;
		        sy_old = sy;
		        ArrayList<Number> line = Globals.examples.get(exam_ndx);
		        ArrayList<Double> result = transform(line.get(0).doubleValue(), line.get(1).doubleValue(), line.get(2).doubleValue(), center_XA, center_Y);
		        sx = result.get(0);
		        sy = result.get(1);
		        if (x == 0) {
		        	point_old[y][0] = sx;
		        	point_old[y][1] = sy;
		        }
		        if (!begin_points) {
		        	begin_points = true;
		        	sx_old = sx;
		        	sy_old = sy;
		        } else {
		        	graph.drawLine((int) Math.round(sx_old), (int) Math.round(sy_old), (int) Math.round(sx), (int) Math.round(sy));
		        }
		        point_new[y][0] = sx;
		        point_new[y][1] = sy;
		        exam_ndx++;
			}
			
			// paint quarder.
			for (int i=0;i<POINT_LEN;i++) {
	        	graph.drawLine((int) Math.round(point_old[i][0]), (int) Math.round(point_old[i][1]), (int) Math.round(point_new[i][0]), (int) Math.round(point_new[i][1]));
		        point_old[i][0] = point_new[i][0];
		        point_old[i][1] = point_new[i][1];
			}
		}

        // paint neural network.
    	graph.setColor(Color.GREEN);
		exam_ndx = 0;
		sx = 0;
		sy = 0;
		for (int x=0;x<POINT_LEN;x++) {
			begin_points = false;
			for (int y=0;y<POINT_LEN;y++) {
				sx_old = sx;
		        sy_old = sy;
		        ArrayList<Number> line = Globals.examples.get(exam_ndx);
		        double[] x_net = new double[2];
		        x_net[0] = line.get(0).doubleValue();
		        x_net[1] = line.get(1).doubleValue();
		        double[] y_net = Globals.t4nn.executeFunction(x_net);
		        ArrayList<Double> result = transform(x_net[0], x_net[1], y_net[0], center_XB, center_Y);
		        sx = result.get(0);
		        sy = result.get(1);
		        if (x == 0) {
		        	point_old[y][0] = sx;
		        	point_old[y][1] = sy;
		        }
		        if (!begin_points) {
		        	begin_points = true;
		        	sx_old = sx;
		        	sy_old = sy;
		        } else {
		        	graph.drawLine((int) Math.round(sx_old), (int) Math.round(sy_old), (int) Math.round(sx), (int) Math.round(sy));
		        }
		        point_new[y][0] = sx;
		        point_new[y][1] = sy;
		        exam_ndx++;
			}
			
			// paint quarder.
			for (int i=0;i<POINT_LEN;i++) {
	        	graph.drawLine((int) Math.round(point_old[i][0]), (int) Math.round(point_old[i][1]), (int) Math.round(point_new[i][0]), (int) Math.round(point_new[i][1]));
		        point_old[i][0] = point_new[i][0];
		        point_old[i][1] = point_new[i][1];
			}
		}

		lblCoordSystem.setIcon(new ImageIcon(image));
		
	}

}
