package n4e.netanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import n4e.core.controls.*;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.screen.*;
import n4e.core.system.*;
import n4e.core.tool.*;
import n4e.netanddecision.dbaccess.DbAccessProject;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.dbaccess.DbAccessReadVarName;
import n4e.netanddecision.dbaccess.DbAccessVarName;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;

/**
 * The <CODE>LayerNameDataSite</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2014 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 18.02.2017 14:29:36
 * @since 1.0
 */

public class LayerNameDataSite extends DataSite implements ActionListener {

    private AWTextField[] tfName;
    private JLabel[] lblName;
    private AWCombo[] cbxVarType;
    private JButton[] btnContinuous;

    private JButton btnSave;
    private JButton btnReturn;

    private String project_uniqueid;
    
    private int var_number;
    private int var_number_old;
    private int layer_number;
    private int panel_width;
    private int panel_height;
    private boolean ignore_action;
    private String[] varname_uid;

    /**
	 * The constructor <CODE>LayerNameDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public LayerNameDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			panel_width = 600;
			panel_height = 340;
			
			var_number_old = 0;
			ignore_action = true;
			
			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));

			createHead(pnlScreen, Globals.message.getMessage("msg0045"));

			btnSave = createButton(pnlScreen, panel_height - 30, 530,
					DataSite.BUTTON_SAVE, Globals.message.getMessage("msg0008"));
			btnSave.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnSave_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnReturn = createButton(pnlScreen, panel_height - 30, 570,
					DataSite.BUTTON_BACK, Globals.message.getMessage("msg0030"));
			btnReturn.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnReturn_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

		    sclScreen = createScrollPane(pnlScreen, pnl_width - 20, pnl_height - 50);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>functionEditable</CODE> blocks the functions, if training.
	 * @param editable false -> block.
	 * @since 1.0
	 */
	public void functionEditable(boolean editable) {
		btnSave.setEnabled(editable);
		btnReturn.setEnabled(editable);
	}

	/**
	 * The method <CODE>setVarNumber</CODE> sets the variable number.
	 * @param number the variable number.
	 * @since 1.0
	 */
	public void setVarNumber(int number, int layer) {
		var_number = number;
		layer_number = layer;
	}
	
	/**
	 * The method <CODE>btnSave_actionPerformed</CODE> action of the btnSave.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnSave_actionPerformed(ActionEvent e) throws Exception {

		enableAllButtons(true);
		
		int len = varname_uid.length;
		for (int ndx=0;ndx<var_number;ndx++) {
			
			DbAccessVarName vn = null;
			if (ndx < len) {
				vn = new DbAccessVarName(Globals.sysUser, varname_uid[ndx], "all", null, null);
			} else {
				vn = new DbAccessVarName(Globals.sysUser, null, "all", null, null);
			}
			
			if (vn.errCode == 0 || vn.errCode == vn.ERROR_0001 || vn.errCode == vn.ERROR_0002) {
				
				vn.setName(tfName[ndx].getText());
				vn.setVartype(cbxVarType[ndx].getIndexKey());
				vn.setLayer(layer_number);
				vn.setFieldAsString("idproject", project_uniqueid);
				
				if (!vn.putToSink(null)) {
					t4dialog.showException(vn.errCode + "\r\n" + vn.errMess, false);
					return;
				}

			} else {
				t4dialog.showException(vn.errCode + "\r\n" + vn.errMess);
				return;
			}
		}
		
		Globals.info_message.setText(Globals.message.getMessage("msg0022"));
		Globals.t4md.initializeExampleExecution(project_uniqueid);
		createCurrentData(project_uniqueid);
		
	}

	/**
	 * The method <CODE>btnReturn_actionPerformed</CODE> action of the btnReturn.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnReturn_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
    	sclScreen.setVisible(false);
		fireSiteChangeEvent_2(project_uniqueid);
	}

	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		sclScreen.setVisible(true);
		project_uniqueid = uniqueid;
		
		// remove old controls.
		if (var_number_old != 0) {
			label_line = 20;
			for (int ndx=0;ndx<var_number_old;ndx++) {
				pnlScreen.remove(lblName[ndx]);
				pnlScreen.remove(tfName[ndx]);
				pnlScreen.remove(cbxVarType[ndx]);
				pnlScreen.remove(btnContinuous[ndx]);
			}
		}
		var_number_old = var_number;
		
		// lengthen the window.
		if (var_number > 11) {
			int more_vars = var_number - 11;
			int curr_height = panel_height + (more_vars * 20) + 20;
			pnlScreen.setPreferredSize(new Dimension(panel_width, curr_height));
			btnSave.setBounds(530, curr_height - 30, 30, 23);
			btnReturn.setBounds(570, curr_height - 30, 30, 23);
		} else {
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));
			btnSave.setBounds(530, panel_height - 30, 30, 23);
			btnReturn.setBounds(570, panel_height - 30, 30, 23);
		}
		
		// get the data and show it.
		lblName = new JLabel[var_number];
		tfName = new AWTextField[var_number];
		cbxVarType = new AWCombo[var_number];
		btnContinuous = new JButton[var_number];
		varname_uid = new String[var_number];

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "VarName",
				"name,vartype", null, null, "idproject='" + project_uniqueid + "' AND layer=" + Integer.toString(layer_number),
				"name ASC", "netanddecision");
		
		int len = tabs.size();
		int ndx_varname = 0;
		for (int ndx=0;ndx<var_number;ndx++) {
            lblName[ndx] = createLabel(pnlScreen, Globals.message.getMessage("msg0046"));
            tfName[ndx] = createTextField(pnlScreen);

            cbxVarType[ndx] = createCombo(pnlScreen, control_column + 210, 100);
            cbxVarType[ndx].addActionListener(this);
    		ignore_action = true;
            Globals.t4md.fillCombo(cbxVarType[ndx], "288ed0e9:15a56a92806:-8000", false, false, "288ed0e9:15a56a92806:-7fff", true);
            
    		btnContinuous[ndx] = new JButton(Globals.message.getMessage("msg0049"));
    		btnContinuous[ndx].setBounds(control_column + 320, label_line, 80, 20);
    		btnContinuous[ndx].addActionListener(this);
    		btnContinuous[ndx].setVisible(false);
    		pnlScreen.add(btnContinuous[ndx]);

    		if (ndx_varname < len) {
            	DbAccessReadVarName vn = (DbAccessReadVarName) tabs.get(ndx);
            	tfName[ndx].setText(vn.getName());
            	varname_uid[ndx] = vn.getUniqueid();
            	cbxVarType[ndx].setIndexKey(vn.getVartype());
            	if (vn.getVartype().equals("288ed0e9:15a56a92806:-7ffe")) { // discrete.
            		btnContinuous[ndx].setVisible(true);
            	}
            	ndx_varname++;
            }
    		ignore_action = false;
		}
	}

	/**
	 * The method <CODE>enableAllButtons</CODE> enables all buttons for enter.
	 * @param enable true -> button enabled.
	 * @exception Exception
	 * @since 1.0
	 */
	private void enableAllButtons(boolean enable) {
		for (int ndx=0;ndx<var_number;ndx++) {
    		btnContinuous[ndx].setEnabled(enable);
		}
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		try {

			for (int ndx=0;ndx<var_number;ndx++) {
				if (e.getSource() == cbxVarType[ndx]) {
					if (ignore_action) {
						return;
					}
					if (cbxVarType[ndx].getIndexKey().equals("288ed0e9:15a56a92806:-7ffe")) { // discrete.
			    		btnContinuous[ndx].setVisible(true);
			    		enableAllButtons(false);
					} else if (cbxVarType[ndx].getIndexKey().equals("288ed0e9:15a56a92806:-7fff")) { // continuous.
			    		btnContinuous[ndx].setVisible(false);
					}
				}

				if (e.getSource() == btnContinuous[ndx]) {
					sclScreen.setVisible(false);
					Globals.nad.vvds.setVarName(tfName[ndx].getText(), project_uniqueid);
					fireSiteChangeEvent_1(varname_uid[ndx]);
				}
			}

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

}
