package n4e.netanddecision.screen;

import java.awt.*;
import java.util.*;

import javax.swing.*;

import n4e.core.list.ImageLine;
import n4e.core.screen.*;
import n4e.core.tool.*;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;

/**
 * The <CODE>LearningDataSite</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 21.02.2017 10:37:36
 * @since 1.0
 */

public class LearningDataSite extends DataSite {

    private JLabel lblCoordSystem; // the coordinate system graphic.

    private ArrayList<ImageLine> graphic_data;
    
    private int graphic_height; // the height of the graphic.
    private int graphic_width; // the width of the graphic.

    /**
	 * The constructor <CODE>LearningDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public LearningDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			int panel_width = 600;
			int panel_height = 340;
			
			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));

		    sclScreen = createScrollPane(pnlScreen, pnl_width - 20, pnl_height - 50);

			graphic_height = 320;
			graphic_width = 595;
			lblCoordSystem = new JLabel("");
			lblCoordSystem.setBounds(10, 10, graphic_width, graphic_height);
	        pnlScreen.add(lblCoordSystem);

	        resetGraphicData();
	        
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	public ArrayList<ImageLine> getGraphicData() {
		return graphic_data;
	}
	
	/**
	 * The method <CODE>resetGraphicData</CODE> reset the graphic data.
	 * @since 1.0
	 */
	public void resetGraphicData() throws Exception {
		graphic_data = new ArrayList<ImageLine>();
        paintTheCoordSystem();
	}

	/**
	 * The method <CODE>setGraphicData</CODE> adds a line to the graphic data.
	 * @param x1 the coordinates of the line.
	 * @param x2 the coordinates of the line.
	 * @param y1 the coordinates of the line.
	 * @param y2 the coordinates of the line.
	 * @since 1.0
	 */
	public void setGraphicData(double x1, double y1, double x2, double y2) throws Exception {
        graphic_data.add(new ImageLine(x1, y1, 0, x2, y2, 0, false));
        paintTheCoordSystem();
	}
	
	/**
	 * The method <CODE>setGraphicData</CODE> adds a line to the graphic data.
	 * @param x1 the coordinates of the line.
	 * @param x2 the coordinates of the line.
	 * @param y1 the coordinates of the line.
	 * @param y2 the coordinates of the line.
	 * @since 1.0
	 */
	public void setLearningCurve(ArrayList<ImageLine> gd) throws Exception {
		graphic_data = gd;
		paintTheCoordSystem();
	}
	
	/**
     * The method <CODE>paintTheCoordSystem</CODE> paint the graphics.
     * @param graphic_data the list with the graphic elements.
     * @param t4lr the Tool4LinearRegression.
	 * @exception Exception
     * @since 1.0
     */
	private void paintTheCoordSystem() throws Exception {
		
		Tool4Image2 t4i = new Tool4Image2(null, Tool4Image.COORDSYS_CARTESIAN, 
        		Tool4Image.COORDSYS_DIM2, graphic_width, graphic_height, true);
		
        // define the grid.
	    t4i.setXaxisName("X");
        t4i.setYaxisName("Y");

        // define the graphic.
        t4i.setBackgroundColor(Globals.color_graphic_background);
        t4i.doInitialize();
        t4i.setLines(null);
        t4i.setGraphicBorder(10);
        t4i.setGridWidth(40);
        
        t4i.setPaintColor(Globals.color_graphic_coordinate);
        t4i.setKoordSystem2D(true);
        t4i.setAxisNames();
        
        t4i.setPaintColor(Globals.color_graphic_border);
        t4i.drawBorder(); 

        // draw points and lines.
        t4i.setPaintColor(Globals.color_graphic_coordinate);
        t4i.setLines(graphic_data);
        t4i.calcFactors(Tool4Image.COORDSYS_XNOTY);
        t4i.setAxisMaxValues();

        t4i.setPaintColor(Globals.color_graphic_grid);
        t4i.drawGridLines();
        t4i.setPaintColor(Globals.color_graphic_coordinate);
        t4i.drawGridValues(Globals.fontStandard11, 0, 3, Globals.sysUser.getUserLocale());

        String graphic_name = Globals.message.getMessage("msg0040");
        t4i.setGraphicName(graphic_name);
        t4i.drawGraphicName(graphic_width / 2 - (graphic_name.length() * 7) / 2, 16);
        
        t4i.setPaintColor(Globals.color_graphic_paint_line);
        t4i.drawAllElements2D(false, true);
        t4i.deleteAddons();
        
		lblCoordSystem.setIcon(new ImageIcon(t4i.getImage()));
    }

}
