package n4e.netanddecision.screen;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JLabel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import n4e.core.controls.AWButton;
import n4e.core.controls.AWTableModel;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.screen.DataSite;
import n4e.core.system.GlobalsDialog;
import n4e.core.tool.Tool4Throwable;
import n4e.netanddecision.dbaccess.DbAccessProject;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;

/**
 * The <CODE>ProjectListSite</CODE> class ...
 * <P><B>title: ExpertAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2015 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 28.04.2015 09:51:26
 * @since 1.0
 */

public class ProjectListSite extends DataSite {

    private AWTableModel modProjectMod;
    private JLabel lblTableRows;
    private AWButton btnNew;
    
    private int event_count;
    private boolean editable;

    /**
	 * The constructor <CODE>ProjectListSite</CODE> creates a new instance(object) of the class.
	 * @param
	 * @exception
	 * @see package.class#member label
	 * @since 1.0
	 */
	public ProjectListSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {
			
			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;
			editable = true;

			pnlScreen = createPanel(pnl_width - 25, pnl_height - 45);
			createHead(pnlScreen, Globals.message.getMessage("msg0033"));

			// calculate the width of the table columns.
			int fac = 150/2; // action columns.
			int[] column_len = new int[2];
            column_len[0] = (int) (pnl_width * (170.0 / 640.0)) - fac;
            column_len[1] = (int) (pnl_width * (400.0 / 640.0)) - fac;
			Hashtable column_width = t4dialog.createColumnWidthTable(column_len);

			modProjectMod = createTable("Project", pnlScreen,
					new DbAccessReadProject(Globals.sysUser),
					"projdate,name", 20, 40,
					pnl_width, 70, pnl_height, 120, this.BUTTON_USE_AND_DELETE,
					null, column_width, null, Globals.fontStandard12);

			modProjectMod.getTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
	            public void valueChanged(ListSelectionEvent e) {
	            	try {
	                	int select = modProjectMod.getSelectedRow();
	                	if (select > -1 && modProjectMod.getIndexKey() != null) {
	                		if (!editable) {
	                			return;
	                		}
	                		Globals.project_uniqueid = modProjectMod.getIndexKey();
	                		DbAccessReadProject proj = new DbAccessReadProject(Globals.sysUser, 
	                				modProjectMod.getIndexKey(), "name", null, null);
                			Globals.nad.setTitle(GlobalsDialog.program_name + " [" + proj.getName() + "]");
                			
                			if (event_count == 0) {
                				Globals.t4md.initializeNeuralNetwork(proj.getUniqueid());
	                			event_count++;
                			} else {
                				event_count = 0;
                			}
 	                	}
	        		} catch (Exception ex) {
	        			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
	        		}
	                
	            }
	        });


			lblTableRows = new JLabel("");
			lblTableRows.setFont(Globals.fontStandard12);
			lblTableRows.setBounds(20, pnl_height - 70, 100, label_hight);
			pnlScreen.add(lblTableRows);

			btnNew = createButton(pnlScreen, pnl_height, 70, pnl_width, 80,
	        		DataSite.BUTTON_NEW, Globals.message.getMessage("msg0006"));
			btnNew.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnNew_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			createCurrentData(null);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>functionEditable</CODE> blocks the functions, if training.
	 * @param editable false -> block.
	 * @since 1.0
	 */
	public void functionEditable(boolean editable) {
		this.editable = editable;
		btnNew.setEnabled(editable);
		the_table.setEnabled(editable);
	}
	
	/**
	 * The method <CODE>btnNew_actionPerformed</CODE> action of the btnNew.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnNew_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		fireSiteChangeEvent_1(null);
	}

	/**
	 * The method <CODE>callUseFromTable</CODE> performs the use button functionality,
	 * if use button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callUseFromTable(String table_name) throws Exception {

		Globals.info_message.setText("");

		if (!editable) {
			return;
		}

		int select = modProjectMod.getSelectedRow();
    	if (select > -1) {
    		fireSiteChangeEvent_1(modProjectMod.getIndexKey());
    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>callDeleteFromTable</CODE> performs the delete button functionality,
	 * if delete button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callDeleteFromTable(String table_name) throws Exception {

		if (!editable) {
			return;
		}

		int select = modProjectMod.getSelectedRow();
    	if (select > -1) {
    		
    		// delete the concerning data.
    		if (!Globals.t4base.deleteAllVarNames(modProjectMod.getIndexKey())) {
    			return;
    		}
    		if (!Globals.t4base.deleteWeights(modProjectMod.getIndexKey())) {
    			return;
    		}
    		if (!Globals.t4base.deleteLearningCurve(modProjectMod.getIndexKey())) {
    			return;
    		}
    		
    		// read the audit.
    		DbAccessProject tab = new DbAccessProject(Globals.sysUser,
    				modProjectMod.getIndexKey(), null, null, null);

    		if (tab.errCode == 0) {
    			if (!tab.deleteInSink(null)) {
    				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    				return;
    			}
    			Globals.info_message.setText(Globals.message.getMessage("msg0029"));
				createCurrentData(null);
    		} else {
				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
    		}

    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList tabs = dsl.getDataSetRead(Globals.sysUser, "Project",
				"projdate,name", null, null, null,
				"projdate DESC,name ASC", "expertanddecision");

		modProjectMod.resetKeyList();

    	int len = tabs.size();
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadProject tab = (DbAccessReadProject) tabs.get(ndx);
       		if (tab.errCode == 0) {
    			modProjectMod.addKey(tab.getUniqueid());

    			Object[] row = new Object[4];
                row[0] = tab.getFieldAsString("projdate");
                row[1] = tab.getFieldAsString("name");
                row[2] = "";
                row[3] = "";
    			modProjectMod.addRow(row);
    		}
    	}
    	lblTableRows.setText(Globals.message.getMessage("msg0031") + modProjectMod.getRowCount());
		modProjectMod.setIndexKey(uniqueid);
	}

}

