package n4e.netanddecision.screen;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import n4e.core.screen.*;
import n4e.core.tool.*;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;

/**
 * The <CODE>TrainingDataSite</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 20.02.2017 15:51:36
 * @since 1.0
 */

public class TrainingDataSite extends DataSite implements Runnable {

    private JLabel lblLearnStep;
    private JLabel lblMaxError;
    private JLabel lblMinError;
    private JLabel lblAverageError;
    private JLabel lblStandarddeviation;
    private JCheckBox cbSlowDown;

    private JButton btnTrainingStart;
    private JButton btnTrainingStop;
    
    private DbAccessReadProject proj;
    
    private int learn_step;
    private boolean stop;
    private boolean slow_down;
    private int graphic_learn_step;

	/**
	 * The constructor <CODE>TrainingDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public TrainingDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			int panel_width = 600;
			int panel_height = 340;
			stop = true;
			
			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));

			createHead(pnlScreen, Globals.message.getMessage("msg0038"));

            createLabel(pnlScreen, Globals.message.getMessage("msg0057"));
			lblLearnStep = createLabelField(pnlScreen, control_column, 200, false);

            createLabel(pnlScreen, Globals.message.getMessage("msg0058"));
			lblMaxError = createLabelField(pnlScreen, control_column, 200, false);

            createLabel(pnlScreen, Globals.message.getMessage("msg0059"));
			lblMinError = createLabelField(pnlScreen, control_column, 200, false);

            createLabel(pnlScreen, Globals.message.getMessage("msg0060"));
			lblAverageError = createLabelField(pnlScreen, control_column, 200, false);

            createLabel(pnlScreen, Globals.message.getMessage("msg0061"));
			lblStandarddeviation = createLabelField(pnlScreen, control_column, 200, false);

	        createLabel(pnlScreen, Globals.message.getMessage("msg0073"));
	        cbSlowDown = createCheckBox(pnlScreen);
	        slow_down = false;
	        cbSlowDown.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						cbSlowDown_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

	        btnTrainingStart = createButton(pnlScreen, panel_height - 30, 530,
					DataSite.BUTTON_START, Globals.message.getMessage("msg0062"));
			btnTrainingStart.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnTrainingStart_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnTrainingStop = createButton(pnlScreen, panel_height - 30, 570,
					DataSite.BUTTON_STOP, Globals.message.getMessage("msg0063"));
			btnTrainingStop.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnTrainingStop_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

		    sclScreen = createScrollPane(pnlScreen, pnl_width - 50, pnl_height - 50);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>cbSlowDown_actionPerformed</CODE> action of the cbSlowDown.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void cbSlowDown_actionPerformed(ActionEvent e) throws Exception {
		if (cbSlowDown.isSelected()) {
			slow_down = true;
		} else {
			slow_down = false;
		}
	}
	
	/**
	 * The method <CODE>btnTrainingStart_actionPerformed</CODE> action of the btnTrainingStart.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnTrainingStart_actionPerformed(ActionEvent e) throws Exception {
		
		if (Globals.examples == null) {
			Globals.t4dialog.showErrorDialog(Globals.message.getMessage("msg0065"));
			return;
		}
		
		learn_step = 0;
		graphic_learn_step = 0;
		stop = false;
		
		proj = new DbAccessReadProject(Globals.sysUser, Globals.project_uniqueid, "all", null, null);

		if (Globals.t4base.weightsAvailable(Globals.project_uniqueid)) {
			
			Globals.t4nn.setExamples(Globals.examples);
			Globals.t4nn.setLearnStep(proj.getLearnsteps());
			learn_step = (int) proj.getLearnsteps();
			graphic_learn_step = Globals.t4base.getMaxLearnStep(Globals.project_uniqueid) - 1;
			
		} else { 

			Globals.nad.lds.resetGraphicData();
			Globals.t4nn = new Tool4NeuralNetwork(Globals.examples, proj.getXlayer(), proj.getYlayer(), proj.getHlayer(), proj.getEpsilon(), true);
			
		}
		
		if (Globals.t4nn.errCode == -1) {
			Globals.t4dialog.showErrorDialog(Globals.message.getMessage("msg0064"));
			return;
		}

		Globals.t4md.blockAllFunctions(false);
		
		Globals.info_message.setText(Globals.message.getMessage("msg0066"));

		new Thread(this).start();

	}

	/**
	 * The method <CODE>btnTrainingStop_actionPerformed</CODE> action of the btnTrainingStop.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnTrainingStop_actionPerformed(ActionEvent e) throws Exception {
		if (stop) {
			Globals.t4dialog.showInfoDialog(Globals.message.getMessage("msg0069"));
			return;
		}

		Globals.t4md.blockAllFunctions(true);
		
		stop = true;
		if (proj != null) {
			Globals.t4base.saveWeights(proj.getUniqueid());
			Globals.t4base.saveLearningSteps(proj.getUniqueid());
			if (proj.getSavelearn() == 1) {
				Globals.t4base.saveLearningCurve(proj.getUniqueid());
			}
		}
		Globals.info_message.setText(Globals.message.getMessage("msg0067"));
	}
	
	/**
	 * The method <CODE>getLearnSteps</CODE> get the learn steps.
	 * @return learn steps.
	 * @since 1.0
	 */
	public int getLearnSteps() {
		return learn_step;
	}
	
	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
	    lblLearnStep.setText(Integer.toString(learn_step));
	    lblMaxError.setText(Globals.t4n.getNumber2String(Globals.t4nn.getMaxError(), 7, Globals.sysUser.getUserLocale()));
	    lblMinError.setText(Globals.t4n.getNumber2String(Globals.t4nn.getMinError(), 7, Globals.sysUser.getUserLocale()));
	    lblAverageError.setText(Globals.t4n.getNumber2String(Globals.t4nn.getAverageError(), 7, Globals.sysUser.getUserLocale()));
	    lblStandarddeviation.setText(Globals.t4n.getNumber2String(Globals.t4nn.getStdDeviation(), 7, Globals.sysUser.getUserLocale()));
	}

	@Override
	public void run() {
		double max = 0;
		boolean first = true;
		boolean graphic_go = true;
		while (true) {
			try {
				
				if (slow_down) {
			        Thread.sleep(50);
				}
				
				Globals.t4nn.learning();
				learn_step++;
				
				if (learn_step % Globals.delay_training == 0 || slow_down) {
					createCurrentData(null);
				}
				
				if (learn_step % Globals.delay_graphics == 0 || slow_down) {
					graphic_learn_step++;
					
					if (graphic_go) {
						graphic_go = Globals.nad.grds.paintGraphic();
					}
					
					if (first) {
						max = Globals.t4nn.getMaxError();
						first = false;
					} else {
						Globals.nad.lds.setGraphicData(graphic_learn_step - 1, max, graphic_learn_step, Globals.t4nn.getMaxError());
						max = Globals.t4nn.getMaxError();
					}
					
				}

				if (learn_step % Globals.delay_weights == 0 || slow_down) {
					Globals.nad.wds.setControlsWithValues();
				}

				if (stop) {
					break;
				}
				
			} catch (Exception ex) {
				t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
				break;
			}
		}
	}
}
