package n4e.netanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import n4e.core.controls.*;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.screen.*;
import n4e.core.system.*;
import n4e.core.tool.*;
import n4e.netanddecision.dbaccess.DbAccessReadVarValue;
import n4e.netanddecision.dbaccess.DbAccessVarValue;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;

/**
 * The <CODE>VariableValueDataSite</CODE> class ...
 * <P><B>title: NatAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 19.02.2017 16:41:56
 * @since 1.0
 */

public class VariableValueDataSite extends DataSite {

	private JLabel lblVarName;
    private AWTextField tfValue; 

    private AWTableModel modValueMod; 
    private JLabel lblTableRows;

    private JButton btnRowUp;
    private JButton btnRowDown;
    private JButton btnSave;
    private JButton btnReturn;

    private DatabaseDatasetLoader dsl;

    private String variable_uniqueid;
    private String project_uniqueid;

	/**
	 * The constructor <CODE>VariableValueDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public VariableValueDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			dsl = new DatabaseDatasetLoader();

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(600, 340));

			createHead(pnlScreen, Globals.message.getMessage("msg0050"));

			DbAccessReadVarValue tab = new DbAccessReadVarValue(Globals.sysUser);

	        createLabel(pnlScreen, Globals.message.getMessage("msg0046"));
	        lblVarName = createLabelField(pnlScreen, control_column, control_width, false);

	        createLabel(pnlScreen, tab.getDefData("value", "label"));
	        tfValue = createTextField(pnlScreen);

	        createSeparator(pnlScreen, 550);

	        label_line += 30;
			// calculate the width of the table columns.
			int fac = 150; // action columns.
			int[] column_len = new int[1];
            column_len[0] = (int) (pnl_width * (600.0 / 640.0)) - fac;
			Hashtable column_width = t4dialog.createColumnWidthTable(column_len);

			modValueMod = createTable("Condition", pnlScreen,
					new DbAccessReadVarValue(Globals.sysUser),
					"value", 20, label_line,
					570, 0, 160, 0, this.BUTTON_DELETE,
					null, column_width, null, Globals.fontStandard12);
	        
			lblTableRows = new JLabel("");
			lblTableRows.setFont(Globals.fontStandard12);
			lblTableRows.setBounds(20, 300, 100, label_hight);
			pnlScreen.add(lblTableRows);

			sclScreen = createScrollPane(pnlScreen, pnl_width - 30, pnl_height - 50);

	        btnRowUp = createButton(pnlScreen, 300, 450, 
	        		DataSite.BUTTON_UP, Globals.message.getMessage("msg0079"));
	        btnRowUp.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnRowUp_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
	        btnRowUp.setEnabled(false);
	        
	        btnRowDown = createButton(pnlScreen, 300, 490, 
	        		DataSite.BUTTON_DOWN, Globals.message.getMessage("msg0080"));
	        btnRowDown.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnRowDown_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});
	        btnRowDown.setEnabled(false);

	        btnSave = createButton(pnlScreen, 300, 530,
					DataSite.BUTTON_SAVE, Globals.message.getMessage("msg0008"));
			btnSave.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnSave_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

			btnReturn = createButton(pnlScreen, 300, 570,
					DataSite.BUTTON_BACK, Globals.message.getMessage("msg0030"));
			btnReturn.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					try {
						btnReturn_actionPerformed(ev);
					} catch (Exception ex) {
						t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
					}
				}
			});

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>setVarName</CODE> sets the variable name.
	 * @param name the variable name.
	 * @since 1.0
	 */
	public void setVarName(String name, String proj_uid) {
		lblVarName.setText(name);
		project_uniqueid = proj_uid;
	}
	
	/**
	 * The method <CODE>btnRowUp_actionPerformed</CODE> action of the btnRowUp.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnRowUp_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		int ndx = modValueMod.getSelectedRow();
		if (ndx == -1) {
			Globals.info_message.setText(Globals.message.getMessage("msg0021"));
			return;
		}
		if (ndx == 0) {
			return; // do nothing.
		}
		String uniqueid_from = modValueMod.getKeyNumber(ndx);
		String uniqueid_to = modValueMod.getKeyNumber(ndx - 1);

		DbAccessVarValue value_from = new DbAccessVarValue(Globals.sysUser, 
				uniqueid_from, "sort", null, null);
		DbAccessVarValue value_to = new DbAccessVarValue(Globals.sysUser, 
				uniqueid_to, "sort", null, null);
		long temp_num = value_from.getSort();
		value_from.setSort(value_to.getSort());
		value_to.setSort(temp_num);
		if (!value_from.putToSink(null)) {
			t4dialog.showException("from " + value_from.errCode + "\r\n" + value_from.errMess);
		}
		if (!value_to.putToSink(null)) {
			t4dialog.showException("to " + value_to.errCode + "\r\n" + value_to.errMess);
			
		}
		createCurrentData(variable_uniqueid);
		modValueMod.setIndexKey(uniqueid_from);
	}
	
	/**
	 * The method <CODE>btnRowDown_actionPerformed</CODE> action of the btnRowDown.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnRowDown_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
		int ndx = modValueMod.getSelectedRow();
		if (ndx == -1) {
			Globals.info_message.setText(Globals.message.getMessage("msg0021"));
			return;
		}
		if (ndx == modValueMod.getLastRow()) {
			return; // do nothing.
		}
		String uniqueid_from = modValueMod.getKeyNumber(ndx);
		String uniqueid_to = modValueMod.getKeyNumber(ndx + 1);
		DbAccessVarValue value_from = new DbAccessVarValue(Globals.sysUser, 
				uniqueid_from, "sort", null, null);
		DbAccessVarValue value_to = new DbAccessVarValue(Globals.sysUser, 
				uniqueid_to, "sort", null, null);
		long temp_num = value_from.getSort();
		value_from.setSort(value_to.getSort());
		value_to.setSort(temp_num);
		if (!value_from.putToSink(null)) {
			t4dialog.showException("from " + value_from.errCode + "\r\n" + value_from.errMess);
			
		}
		if (!value_to.putToSink(null)) {
			t4dialog.showException("to " + value_to.errCode + "\r\n" + value_to.errMess);
			
		}
		createCurrentData(variable_uniqueid);
		modValueMod.setIndexKey(uniqueid_from);
	}

	/**
	 * The method <CODE>btnSave_actionPerformed</CODE> action of the btnSave.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnSave_actionPerformed(ActionEvent e) throws Exception {

		DbAccessVarValue tab = new DbAccessVarValue(Globals.sysUser, null,
				"all", null, null);

		if (tab.errCode == 0 || tab.errCode == tab.ERROR_0001 || tab.errCode == tab.ERROR_0002) {

			long sort = Globals.t4base.getMaxNumberOfValues(variable_uniqueid) + 1;
			
			tab.errCode = 0;
			tab.setSort(sort);
			tab.setValue(tfValue.getText());
			tab.setFieldAsString("idvarname", variable_uniqueid);

            if (!tab.putToSink(null)) {
				t4dialog.showException(tab.errCode + "\r\n" + tab.errMess, false);
				return;
			}

			Globals.info_message.setText(Globals.message.getMessage("msg0022"));
			createCurrentData(variable_uniqueid);

		} else {
			t4dialog.showException(tab.errCode + "\r\n" + tab.errMess);
		}

	}

	/**
	 * The method <CODE>btnReturn_actionPerformed</CODE> action of the btnReturn.
	 * @param e the ActionEvent.
	 * @exception Exception
	 * @since 1.0
	 */
	private void btnReturn_actionPerformed(ActionEvent e) throws Exception {
		Globals.info_message.setText("");
    	sclScreen.setVisible(false);
		fireSiteChangeEvent_1(project_uniqueid);
	}

	/**
	 * The method <CODE>callDeleteFromTable</CODE> performs the delete button functionality, 
	 * if delete button in table is pressed.
	 * @param table_name the name of the belonging table.
	 * @exception Exception
	 * @since 1.0
	 */
	public void callDeleteFromTable(String table_name) throws Exception {
		
    	int select = modValueMod.getSelectedRow();
    	if (select > -1) {
    		
    		// read the author.
    		DbAccessVarValue val = new DbAccessVarValue(Globals.sysUser, 
    				modValueMod.getIndexKey(), null, null, null);
    		
    		if (val.errCode == 0) {
    			if (!val.deleteInSink(null)) {
    				t4dialog.showException(val.errCode + "\r\n" + val.errMess);
    				return;
    			}
    			Globals.info_message.setText(Globals.message.getMessage("msg0029"));
				createCurrentData(variable_uniqueid);
    		} else {
				t4dialog.showException(val.errCode + "\r\n" + val.errMess);
    		}
    		
    	} else {
    		Globals.info_message.setText(Globals.message.getMessage("msg0021"));
    	}
	}

	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		sclScreen.setVisible(true);
		variable_uniqueid = uniqueid;
		tfValue.setText("");
		
		// fetch values. 
		ArrayList values = dsl.getDataSetRead(Globals.sysUser, "VarValue", 
				"value", null, null, 
				"idvarname='" + variable_uniqueid + "'", 
				"sort ASC", "netanddecision");
    	
		modValueMod.resetKeyList();
    	
    	int len = values.size();
    	
    	if (len > 0) {
	        btnRowUp.setEnabled(true);
	        btnRowDown.setEnabled(true);
    	}
    	
    	for (int ndx=0; ndx<len;ndx++) {
    		DbAccessReadVarValue val = (DbAccessReadVarValue) values.get(ndx) ;
    		if (val.errCode == 0) {
    			modValueMod.addKey(val.getUniqueid());
    			Object[] row = new Object[2]; 
    			row[0] = val.getValue(); 
    			row[1] = "";
    			modValueMod.addRow(row);
    		}
    	}
    	lblTableRows.setText(Globals.message.getMessage("msg0031") + modValueMod.getRowCount());

	}
}
