package n4e.netanddecision.screen;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import n4e.core.controls.*;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.screen.*;
import n4e.core.tool.*;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.dbaccess.DbAccessReadVarName;
import n4e.netanddecision.saxparser.ImportExamples;
import n4e.netanddecision.system.Globals;
import n4e.netanddecision.system.NetAndDecision;
import n4e.netanddecision.tool.Tool4MyDialog;

/**
 * The <CODE>WeightDataSite</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 22.02.2017 11:48:36
 * @since 1.0
 */

public class WeightDataSite extends DataSite {

    private JLabel[][] lblW1;
    private JLabel[][] lblW2;

    private String project_uniqueid;
    
    private int panel_width;
    private int panel_height;

    private int w1_i;
    private int w1_j;
	private int w2_i;
	private int w2_j; 

    private int w1_i_old;
    private int w1_j_old;
	private int w2_i_old;
	private int w2_j_old; 

	/**
	 * The constructor <CODE>WeightDataSite</CODE> creates a new instance(object) of the class.
	 * @param frame the main window.
	 * @param pnl_width the width of the car site.
	 * @param pnl_height the height of the car site.
	 * @since 1.0
	 */
	public WeightDataSite(NetAndDecision frame, int pnl_width, int pnl_height) {
		super(frame, Globals.fontStandard12);

		try {

			label_hight = Globals.label_height;
			label_width = Globals.label_width;
			label_line_count = 24;
			control_hight = Globals.control_height;
			control_width = Globals.control_width;

			panel_width = 600;
			panel_height = 340;
			
			pnlScreen = new JPanel();
			pnlScreen.setBackground(Color.WHITE);
			pnlScreen.setLayout(null);
			pnlScreen.setPreferredSize(new Dimension(panel_width, panel_height));

			createHead(pnlScreen, Globals.message.getMessage("msg0035"));

			sclScreen = createScrollPane(pnlScreen, pnl_width - 20, pnl_height - 50);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>setControlsWithValues</CODE> set the control data with values.
	 * @exception Exception
	 * @since 1.0
	 */
	public void setControlsWithValues() throws Exception {
		for (int i=0;i<w1_i;i++) {
			for (int j=0;j<w1_j;j++) {
				lblW1[i][j].setText(Globals.t4n.getNumber2String(Globals.t4nn.getW1()[i][j], 3, Globals.sysUser.getUserLocale()));
			}
		}
		for (int i=0;i<w2_i;i++) {
			for (int j=0;j<w2_j;j++) {
				lblW2[i][j].setText(Globals.t4n.getNumber2String(Globals.t4nn.getW2()[i][j], 3, Globals.sysUser.getUserLocale()));
			}
		}
	}
	
	/**
	 * The method <CODE>createCurrentData</CODE> get data for the controls.
	 * @exception Exception
	 * @since 1.0
	 */
	public void createCurrentData(String uniqueid) throws Exception {
		super.createCurrentData();

		sclScreen.setVisible(true);
		project_uniqueid = uniqueid;
		label_line = 20;
		
		// remove old controls.
		if (lblW1 != null) {
			for (int i=0;i<w1_i_old;i++) {
				for (int j=0;j<w1_j_old;j++) {
					pnlScreen.remove(lblW1[i][j]);
				}
			}
			for (int i=0;i<w2_i_old;i++) {
				for (int j=0;j<w2_j_old;j++) {
					pnlScreen.remove(lblW2[i][j]);
				}
			}
		}

		// get neural net parameter.
		DbAccessReadProject proj = new DbAccessReadProject(Globals.sysUser, project_uniqueid, "all", null, null);

		w1_i = (int) proj.getXlayer() + 1;
	    w1_j = (int) proj.getHlayer() + 1;
		lblW1 = new JLabel[w1_i][w1_j];
		w1_i_old = w1_i;
		w1_j_old = w1_j;

		w2_i = (int) proj.getHlayer() + 1;
		w2_j = (int) proj.getYlayer(); 
		lblW2 = new JLabel[w2_i][w2_j];
		w2_i_old = w2_i;
		w2_j_old = w2_j;

		int width_offset = 20;
		int height_offset = 50;

		// lengthen the window.
		int curr_width = panel_width;
		int curr_height = panel_height;
		if (w1_i + w2_i > 8) {
			int more_vars = w1_j + w2_j - 8;
			curr_height = panel_height + (more_vars * 60) + height_offset;
		} 
		if (Math.max(w1_j,  w2_j) > 10) {
			int more_vars = Math.max(w1_j,  w2_j) - 10;
			curr_width = panel_width + (more_vars * 70);
		} 
		pnlScreen.setPreferredSize(new Dimension(curr_width, curr_height));

		for (int i=0;i<w1_i;i++) {
			for (int j=0;j<w1_j;j++) {
				lblW1[i][j] = new JLabel("0");
				lblW1[i][j].setBounds(width_offset + j * 60, height_offset + i * 30, 50, 20);
				lblW1[i][j].setBackground(Color.LIGHT_GRAY);
				lblW1[i][j].setOpaque(true);
				lblW1[i][j].setHorizontalAlignment(JLabel.RIGHT);
				pnlScreen.add(lblW1[i][j]);
			}
		}
		
		height_offset = w1_i * 30 + 60;
		
		for (int i=0;i<w2_i;i++) {
			for (int j=0;j<w2_j;j++) {
				lblW2[i][j] = new JLabel("0");
				lblW2[i][j].setBounds(width_offset + j * 60, height_offset + i * 30, 50, 20);
				lblW2[i][j].setBackground(Color.LIGHT_GRAY);
				lblW2[i][j].setOpaque(true);
				lblW2[i][j].setHorizontalAlignment(JLabel.RIGHT);
				pnlScreen.add(lblW2[i][j]);
			}
		}

		pnlScreen.repaint();
	}

}
