package n4e.netanddecision.system;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import n4e.core.database.DatabaseManager;
import n4e.core.system.GNULicense;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.Help;
import n4e.core.system.Info;
import n4e.core.system.MainDialog;
import n4e.core.system.Options;
import n4e.core.system.SystemService;
import n4e.core.tool.Tool4File;
import n4e.core.tool.Tool4Throwable;
import n4e.netanddecision.screen.ExampleDataSite;
import n4e.netanddecision.screen.ExecutionDataSite;
import n4e.netanddecision.screen.GraphicDataSite;
import n4e.netanddecision.screen.GeneratorDataSite;
import n4e.netanddecision.screen.InfoBarDataSite;
import n4e.netanddecision.screen.LayerNameDataSite;
import n4e.netanddecision.screen.LearningDataSite;
import n4e.netanddecision.screen.ProjectDataSite;
import n4e.netanddecision.screen.ProjectListSite;
import n4e.netanddecision.screen.TrainingDataSite;
import n4e.netanddecision.screen.VariableValueDataSite;
import n4e.netanddecision.screen.WeightDataSite;
import n4e.netanddecision.tool.Tool4MyDialog;

/**
 * The <CODE>NetAndDecision</CODE> class ...
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2015 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , cologne , germany.
 * @version 26.04.2015 16:19:05
 * @since 1.0
 */

public class NetAndDecision extends MainDialog {

	private JTabbedPane tpNet;

	private JPanel pnlProject;
	public ProjectListSite pls;
	public ProjectDataSite pds;
	public LayerNameDataSite lnds;
	public VariableValueDataSite vvds;
	public WeightDataSite wds;

	private JTabbedPane tpData;
	public ExampleDataSite eds;
	public GeneratorDataSite gds;
	
	private JTabbedPane tpTraining;
	public TrainingDataSite tds;
	public ExecutionDataSite exds;

	private JTabbedPane tpGraphics;
	public LearningDataSite lds;
	public GraphicDataSite grds;

	private JPanel pnlInfoBar;
	public InfoBarDataSite ibds;

	private DatabaseManager dbm;

	private Tool4MyDialog t4md;
	private Tool4File t4f;

	/**
	 * The constructor <CODE>NetAndDecision</CODE> creates a new instance(object) of the class.
	 * @param
	 * @exception
	 * @see package.class#member label
	 * @since 1.0
	 */
	public NetAndDecision(String program_name, ImageIcon intro_icon, String copyright_add, 
			String database_name) {

		super(program_name, "0.1", intro_icon, copyright_add, "2017", database_name, 
				Globals.window_height, Globals.window_width, true, true);

		try {
			
			Globals.doAfterInitials();
			t4md = new Tool4MyDialog();
			t4f = new Tool4File();
			Image icon = t4md.getImagePure("app.jpg");
			setIconImage(icon);
			
			setMinimumSize(new Dimension(640, 480));
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			if (Globals.window_width == 0 && Globals.window_height == 0) {
				Globals.window_width = screen.width;
				Globals.window_height = screen.height;
				setExtendedState(Frame.MAXIMIZED_BOTH);
			}
				
			windowInit();
			
			this.addComponentListener(new ComponentAdapter() {
				public void componentResized(ComponentEvent e) {
					Dimension screen_size = getSize();
					Globals.window_width = screen_size.width;
					Globals.window_height = screen_size.height;
					Globals.has_changed = true;
	            }
	        });

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
		
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @exception Exception.
	 * @since 1.0
	 */
	private void windowInit() throws Exception {
		
		try {

			Globals.nad = this;
			
			miOptionItem = getMenuItem(menu_extras, null, Globals.message.getMessage("msg0001") + "...", 
					Globals.message.getMessage("msg0001").charAt(0), null, null);  

			miHelpItem = getMenuItem(menu_extras, null, GlobalsDialog.message.getMessage("msg0008") + "...", 'i', null, null); 
			
			setIntroMessage("system");
			new SystemService(null);
			SystemService.setClientPackageName("netanddecision");
			SystemService.setDefinitions(GlobalsDialog.language_code);
			if (!initialiseDatabase()) {
				System.exit(0);
			}

			// add the controls.
			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 80, screen.width, 20);
			c.setSize(new Dimension(screen.width, screen.height));
			c.setBackground(Color.LIGHT_GRAY);
			
			// calculate the screen geometry.
			int expPnlWidth = screen.width / 2 - 10;
			int expPnlHeight = (int) (screen.height * 0.4);

			int expDataOffSetHeight = expPnlHeight + 14;
			int expDataWidth = screen.width / 2 - 10;
			int expDataHeight = (int) (screen.height * 0.4);
			
			int expTrainingOffSetWidth = expPnlWidth + 20;
			int expTrainingWidth = screen.width / 2 - 35;
			int expTrainingHeight = (int) (screen.height * 0.4);

			int expGraphicsOffSetWidth = expPnlWidth + 20;
			int expGraphicsOffSetHeight = expPnlHeight + 14;
			int expGraphicsWidth = screen.width / 2 - 35;
			int expGraphicsHeight = (int) (screen.height * 0.4);

			int infoOffSetHeight = screen.height - 150;
			int infoWidth = screen.width - 35;
			int infoHeight = 60;

			setIntroMessage("net and decision");

			tpNet = new JTabbedPane();
			tpNet.setBounds(10, 10, expPnlWidth, expPnlHeight);
			c.add(tpNet);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpNet.addTab(Globals.message.getMessage("msg0033"), pnlProject);

			pls = new ProjectListSite(this, expPnlWidth, expPnlHeight);
			pnlProject.add(pls.pnlScreen);

			pds = new ProjectDataSite(this, expPnlWidth, expPnlHeight);
			pds.sclScreen.setVisible(false);
			pnlProject.add(pds.sclScreen);

			lnds = new LayerNameDataSite(this, expPnlWidth, expPnlHeight);
			lnds.sclScreen.setVisible(false);
			pnlProject.add(lnds.sclScreen);

			vvds = new VariableValueDataSite(this, expPnlWidth, expPnlHeight);
			vvds.sclScreen.setVisible(false);
			pnlProject.add(vvds.sclScreen);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpNet.addTab(Globals.message.getMessage("msg0035"), pnlProject);

			wds = new WeightDataSite(this, expPnlWidth, expPnlHeight);
			pnlProject.add(wds.sclScreen);

			tpData = new JTabbedPane();
			tpData.setBounds(10, expDataOffSetHeight, expDataWidth, expDataHeight);
			c.add(tpData);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpData.addTab(Globals.message.getMessage("msg0036"), pnlProject);

			eds = new ExampleDataSite(this, expDataWidth, expDataHeight);
			pnlProject.add(eds.sclScreen);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpData.addTab(Globals.message.getMessage("msg0037"), pnlProject);

			gds = new GeneratorDataSite(this, expDataWidth, expDataHeight);
			pnlProject.add(gds.sclScreen);

			tpTraining = new JTabbedPane();
			tpTraining.setBounds(expTrainingOffSetWidth, 10, expTrainingWidth, expTrainingHeight);
			c.add(tpTraining);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpTraining.addTab(Globals.message.getMessage("msg0038"), pnlProject);

			tds = new TrainingDataSite(this, expDataWidth, expDataHeight);
			pnlProject.add(tds.sclScreen);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpTraining.addTab(Globals.message.getMessage("msg0039"), pnlProject);

			exds = new ExecutionDataSite(this, expDataWidth, expDataHeight);
			pnlProject.add(exds.sclScreen);

			tpGraphics = new JTabbedPane();
			tpGraphics.setBounds(expGraphicsOffSetWidth, expGraphicsOffSetHeight, expGraphicsWidth, expGraphicsHeight);
			c.add(tpGraphics);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpGraphics.addTab(Globals.message.getMessage("msg0040"), pnlProject);

			lds = new LearningDataSite(this, expGraphicsWidth,  expGraphicsHeight);
			pnlProject.add(lds.sclScreen);

			pnlProject = new JPanel();
			pnlProject.setLayout(null);
			tpGraphics.addTab(Globals.message.getMessage("msg0041"), pnlProject);

			grds = new GraphicDataSite(this, expGraphicsWidth,  expGraphicsHeight);
			pnlProject.add(grds.sclScreen);

			setIntroMessage("infobar");
			pnlInfoBar = new JPanel();
			pnlInfoBar.setLayout(null);
			pnlInfoBar.setBounds(10, infoOffSetHeight, infoWidth, infoHeight);
			c.add(pnlInfoBar);

			ibds = new InfoBarDataSite(this, infoWidth, infoHeight);
			pnlInfoBar.add(ibds.sclScreen);
			Globals.info_message = ibds;

			// set the calling connections.
			pls.addSiteChangeListener_1(pds);
			pds.addSiteChangeListener_1(pls);
			pds.addSiteChangeListener_2(lnds);
			lnds.addSiteChangeListener_2(pds);
			vvds.addSiteChangeListener_1(lnds);
			lnds.addSiteChangeListener_1(vvds);
			
			if (GlobalsDialog.show_intro) {
				Thread.currentThread().sleep(5000);
				oIntro.dispose();
			}
			
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
		
		setVisible(true);
	}

	/**
	 * The method <CODE>initialiseDatabase</CODE> initialize the data base.
	 * @since 1.0
	 */
	private boolean initialiseDatabase() throws Exception {
		
		try {
	        dbm = new DatabaseManager(
	        		"./conf/", //SystemPathSet.getPath2ApplicationDataConf(), 
	                "./log/", //SystemPathSet.getPath2ApplicationDataLog(),
	                "./log/olddblog/", //SystemPathSet.getPath2ApplicationDataLogOldDb(),
	                false); //SystemService.getDatabaseConnectionSetDebug());
		} catch (Exception ex) {
			t4dialog.showErrorDialog(Globals.message.getMessage("msg0032"));
			return false;
		}
		return true;
	}

	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			
			if (e.getSource() == miOptionItem) {
				miOptionItem.setEnabled(false);
				Options oOptions = new Options(this, "netanddecision");
				
			} else if (e.getSource() == miInfoItem) {
				miInfoItem.setEnabled(false);
				ArrayList data = new ArrayList();
				data.add("net and decision");
				data.add("decide with use of a neural network.");
				ArrayList credits = new ArrayList();
		        credits.add("c r e d i t s . . .");
		        credits.add("BOOKS:");
		        credits.add("c. ullenboom;");
		        credits.add("   java ist auch eine insel;");
		        credits.add("   galileo computing; 2. auflage; 2003.");
		        credits.add("SOFTWARE:");
		        credits.add("j. dinkins;");
		        credits.add("   ExampleFileFilter.java;");
		        credits.add("   1.16, 04/07/26.");
		        credits.add("hyper sql;");
		        credits.add("   hsqldb.jar;");
		        credits.add("   2.3.2");
				data.add(credits);
				Info oInfo = new Info(this, data);
				oInfo.setVisible(true);
				
			} else if (e.getSource() == miHelpItem) {
				// get the help text.
				Tool4File t4f = new Tool4File();
				ArrayList content = null;
				if (GlobalsDialog.language_code.equals("de")) {
					content = t4f.getFileContent("help_de.htm");
				} else if (GlobalsDialog.language_code.equals("en")) {
					content = t4f.getFileContent("help_en.htm");
				}
				if (content != null) {
					String help_text = (String) content.get(0);
					miHelpItem.setEnabled(false);
					Help oHelp = new Help(this, help_text);
					oHelp.setVisible(true);
				}

			} else if (e.getSource() == miLicenseItem) {
				GNULicense oLicense = new GNULicense(this, "2017", "net and decision, decide with use of a neural network.\r\n");
				oLicense.setVisible(true);
			}
			
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>closeEnvironmet</CODE> closes the resources.
	 * @since 1.0
	 */
	public void closeEnvironmet() {
		try {
			super.closeEnvironmet();
			Globals.saveGlobalsProperties();
			t4f.deleteOldLogs();
			dbm.releaseAllConnections();
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 1.0
	 */
	public static void main(String[] args) {
		Tool4MyDialog t4md_main = new Tool4MyDialog();
		ImageIcon intro_icon = null;
		try {
			new Globals();
			intro_icon = t4md_main.getImage("intro.jpg");
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new NetAndDecision("net and decision", intro_icon, "...", "netanddecision");
	}

}

