package n4e.netanddecision.tool;

import java.sql.ResultSet;
import java.util.ArrayList;

import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.database.DatabaseSQLResultSet;
import n4e.core.list.ImageLine;
import n4e.core.system.SystemService;
import n4e.core.tool.Tool4NeuralNetwork;
import n4e.netanddecision.dbaccess.DbAccessLearning;
import n4e.netanddecision.dbaccess.DbAccessProject;
import n4e.netanddecision.dbaccess.DbAccessReadProject;
import n4e.netanddecision.dbaccess.DbAccessReadWeight;
import n4e.netanddecision.dbaccess.DbAccessVarName;
import n4e.netanddecision.dbaccess.DbAccessVarValue;
import n4e.netanddecision.dbaccess.DbAccessWeight;
import n4e.netanddecision.system.Globals;

/**
 * The <CODE>Tool4Database</CODE> class provides functions for database processing.
 * <P><B>title: NetAndDecision</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2017 thomas kaffka, born at 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 19.02.2017 17:36:18
 * @since 1.0
 */
public class Tool4Database {

	DatabaseDatasetLoader dsl;
	
	/**
	 * The constructor <CODE>Tool4Database</CODE> creates a new instance(object) of the class.
	 * @since 1.0
	 */
	public Tool4Database() {
		dsl = new DatabaseDatasetLoader();
	}
	
	/**
	 * The method <CODE>deleteAllVarNames</CODE> deletes all variable names and their values.
	 * @param proj_uid the uniqueid of the project.
	 * @exception Exception
	 * @since 1.0
	 */
	public boolean deleteAllVarNames(String proj_uid) throws Exception {
		
		ArrayList tabs = dsl.getDataSet(Globals.sysUser, "VarName",
				null, null, null, "idproject='" + proj_uid + "'",
				null, "netanddecision");

		int len = tabs.size();
		for (int ndx=0;ndx<len;ndx++) {
			DbAccessVarName vn = (DbAccessVarName) tabs.get(ndx);
			if (!deleteAllVarValues(vn.getUniqueid())) {
				return false;
			}
			if (vn.errCode != 0) {
				Globals.t4dialog.showException(vn.errCode + "\r\n" + vn.errMess);
				return false;
			}
			if (!vn.deleteInSink(null)) {
				Globals.t4dialog.showException(vn.errCode + "\r\n" + vn.errMess);
				return false;
			}
		}
		
		return true;
	}
	
	/**
	 * The method <CODE>deleteAllVarValues</CODE> deletes all variable values of a variable name.
	 * @param var_uid the uniqueid of the variable name.
	 * @exception Exception
	 * @since 1.0
	 */
	public boolean deleteAllVarValues(String var_uid) throws Exception {
		
		ArrayList tabs = dsl.getDataSet(Globals.sysUser, "VarValue",
				null, null, null, "idvarname='" + var_uid + "'",
				null, "netanddecision");

		int len = tabs.size();
		for (int ndx=0;ndx<len;ndx++) {
			DbAccessVarValue vv = (DbAccessVarValue) tabs.get(ndx);
			if (vv.errCode != 0) {
				Globals.t4dialog.showException(vv.errCode + "\r\n" + vv.errMess);
				return false;
			}
			if (!vv.deleteInSink(null)) {
				Globals.t4dialog.showException(vv.errCode + "\r\n" + vv.errMess);
				return false;
			}
		}
		
		return true;
	}

    /**
     * The method <CODE>getMaxNumberOfValues</CODE> calculates the maximum number of
     * the variable sort values.
     * @return the maximum.
     * @exception Exception.
     * @since 1.0
     */
	public int getMaxNumberOfValues(String var_uniqueid) throws Exception {
		
		DatabaseSQLResultSet dbRes = new DatabaseSQLResultSet(Globals.sysUser.getClientDbsystemforeclient(), null);
        dbRes.setDebug(SystemService.getDatabaseSQLResultSetDebug());
        
        String query = "SELECT MAX(sort) AS x FROM VarValue WHERE idvarname='" + var_uniqueid + "'";
        ResultSet resultset = dbRes.getResultSet(query);

        int max = 0;
        while (resultset.next()) {
        	max = resultset.getInt("x");
        }
        
        dbRes.closeResultSet();
        resultset.close();
        
		return max;
	}

    /**
     * The method <CODE>getMaxLearnStep</CODE> calculates the maximum number of
     * the learn steps.
     * @param proj_uniqueid the uniqueid of the current project.
     * @return the maximum.
     * @exception Exception.
     * @since 1.0
     */
	public int getMaxLearnStep(String proj_uniqueid) throws Exception {
		
		DatabaseSQLResultSet dbRes = new DatabaseSQLResultSet(Globals.sysUser.getClientDbsystemforeclient(), null);
        dbRes.setDebug(SystemService.getDatabaseSQLResultSetDebug());
        
        String query = "SELECT MAX(xvalue) AS x FROM Learning WHERE idproject='" + proj_uniqueid + "'";
        ResultSet resultset = dbRes.getResultSet(query);

        int max = 0;
        while (resultset.next()) {
        	max = resultset.getInt("x");
        }
        
        dbRes.closeResultSet();
        resultset.close();
        
		return max;
	}

	/**
     * The method <CODE>deleteWeights</CODE> delete the weights and the factors of the neural network.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public boolean deleteWeights(String proj_uniqueid) throws Exception {
		
		ArrayList tabs = dsl.getDataSet(Globals.sysUser, "Weight",
				null, null, null, "idproject='" + proj_uniqueid + "'",
				null, "expertanddecision");

		int len = tabs.size();
		for (int ndx=0;ndx<len;ndx++) {
			DbAccessWeight weight = (DbAccessWeight) tabs.get(ndx);
			if (weight.errCode != 0) {
				Globals.t4dialog.showException(weight.errCode + "\r\n" + weight.errMess);
				return false;
			}
			if (!weight.deleteInSink(null)) {
				Globals.t4dialog.showException(weight.errCode + "\r\n" + weight.errMess);
				return false;
			}
		}
		return true;
	}
	
    /**
     * The method <CODE>saveWeights</CODE> save the weights of the neural network.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public void saveWeights(String proj_uniqueid) throws Exception {
		
		// delete all weights and factors first.
		if (!deleteWeights(proj_uniqueid)) {
			return;
		}
		
		// save the w1 weights.
		double[][] w = Globals.t4nn.getW1();
		int leni = w.length;
		int lenj = w[0].length;

		int sort = 0;
		for (int i=0;i<leni;i++) {
			for (int j=0;j<lenj;j++) {
				sort++;
				DbAccessWeight weight = new DbAccessWeight(Globals.sysUser, null, "all", null, null);
				weight.setSort(sort);
				weight.setMatrix(1); // weights 1
				weight.setValue(w[i][j]);
				weight.setFieldAsString("idproject", proj_uniqueid);
				if (!weight.putToSink(null)) {
					Globals.t4dialog.showException(weight.errCode + "\r\n" + weight.errMess);
					return;
				}
			}
		}
        
		// save the w2 weights.
		w = Globals.t4nn.getW2();
		leni = w.length;
		lenj = w[0].length;
		
		sort = 0;
		for (int i=0;i<leni;i++) {
			for (int j=0;j<lenj;j++) {
				sort++;
				DbAccessWeight weight = new DbAccessWeight(Globals.sysUser, null, "all", null, null);
				weight.setSort(sort);
				weight.setMatrix(2); // weights 2
				weight.setValue(w[i][j]);
				weight.setFieldAsString("idproject", proj_uniqueid);
				if (!weight.putToSink(null)) {
					Globals.t4dialog.showException(weight.errCode + "\r\n" + weight.errMess);
					return;
				}
			}
		}

		// save the factors.
		double[] fac = Globals.t4nn.getFactor();
		leni = fac.length;
		
		sort = 0;
		for (int i=0;i<leni;i++) {
			sort++;
			DbAccessWeight weight = new DbAccessWeight(Globals.sysUser, null, "all", null, null);
			weight.setSort(sort);
			weight.setMatrix(3); // factors.
			weight.setValue(fac[i]);
			weight.setFieldAsString("idproject", proj_uniqueid);
			if (!weight.putToSink(null)) {
				Globals.t4dialog.showException(weight.errCode + "\r\n" + weight.errMess);
				return;
			}
		}

	}

    /**
     * The method <CODE>saveWeights</CODE> save the weights of the neural network.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public void restoreNeuralNetwork(String proj_uniqueid) throws Exception {
		
		DbAccessReadProject proj = new DbAccessReadProject(Globals.sysUser, proj_uniqueid, "all", null, null);

		Globals.t4nn = new Tool4NeuralNetwork(Globals.examples, proj.getXlayer(), proj.getYlayer(), proj.getHlayer(), proj.getEpsilon(), true);
		
		ArrayList tabs = dsl.getDataSet(Globals.sysUser, "Weight",
				"value", null, null, "idproject='" + proj_uniqueid + "' AND matrix=3",
				"sort", "expertanddecision");

		int len = tabs.size();
		double[] factor = new double[len];
		for (int ndx=0;ndx<len;ndx++) {
			DbAccessWeight weight = (DbAccessWeight) tabs.get(ndx);
			factor[ndx] = weight.getValue(); 
		}
		Globals.t4nn.setValues((int) proj.getHlayer(), (int) proj.getXlayer(), (int) proj.getYlayer(), factor, proj.getEpsilon());
		
		tabs = dsl.getDataSet(Globals.sysUser, "Weight",
				"value", null, null, "idproject='" + proj_uniqueid + "' AND matrix=1",
				"sort", "expertanddecision");

		if (tabs.size() == 0) {
			return;
		}
		
		int ndx = 0;
		int w1_i = (int) proj.getXlayer() + 1;
		int w1_j = (int) proj.getHlayer() + 1;
		double[][] w1 = new double[w1_i][w1_j];
		for (int i=0;i<w1_i;i++) {
			for (int j=0;j<w1_j;j++) {
				DbAccessWeight weight = (DbAccessWeight) tabs.get(ndx);
				w1[i][j] = weight.getValue();
				ndx++;
			}
		}
		Globals.t4nn.setW1(w1);

		tabs = dsl.getDataSet(Globals.sysUser, "Weight",
				"value", null, null, "idproject='" + proj_uniqueid + "' AND matrix=2",
				"sort", "expertanddecision");

		ndx = 0;
		int w2_i = (int) proj.getHlayer() + 1;
		int w2_j = (int) proj.getYlayer(); 
		double[][] w2 = new double[w2_i][w2_j]; 
		for (int i=0;i<w2_i;i++) {
			for (int j=0;j<w2_j;j++) {
				DbAccessWeight weight = (DbAccessWeight) tabs.get(ndx);
				w2[i][j] = weight.getValue();
				ndx++;
			}
		}
		Globals.t4nn.setW2(w2);
	}

    /**
     * The method <CODE>weightsAvailable</CODE> detects, if waights are available.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public boolean weightsAvailable(String proj_uniqueid) throws Exception {
		
		DatabaseSQLResultSet dbRes = new DatabaseSQLResultSet(Globals.sysUser.getClientDbsystemforeclient(), null);
        dbRes.setDebug(SystemService.getDatabaseSQLResultSetDebug());
        
        String query = "SELECT COUNT(*) AS x FROM Weight WHERE idproject='" + proj_uniqueid + "'";
        ResultSet resultset = dbRes.getResultSet(query);

        int max = 0;
        while (resultset.next()) {
        	max = resultset.getInt("x");
        }
        
        dbRes.closeResultSet();
        resultset.close();

		return max>0;
	}
	
    /**
     * The method <CODE>deleteLearningCurve</CODE> delete the learning curve of the neural network.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public boolean deleteLearningCurve(String proj_uniqueid) throws Exception {
		
		ArrayList tabs = dsl.getDataSet(Globals.sysUser, "Learning",
				null, null, null, "idproject='" + proj_uniqueid + "'",
				null, "netanddecision");

		int len = tabs.size();
		for (int ndx=0;ndx<len;ndx++) {
			DbAccessLearning learn = (DbAccessLearning) tabs.get(ndx);
			if (learn.errCode != 0) {
				Globals.t4dialog.showException(learn.errCode + "\r\n" + learn.errMess);
				return false;
			}
			if (!learn.deleteInSink(null)) {
				Globals.t4dialog.showException(learn.errCode + "\r\n" + learn.errMess);
				return false;
			}
		}
		return true;
	}
	
    /**
     * The method <CODE>saveLearningSteps</CODE> save the learning steps of the neural network.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public void saveLearningSteps(String proj_uniqueid) throws Exception {
		
		int learn_steps = Globals.nad.tds.getLearnSteps();
		
		DbAccessProject proj = new DbAccessProject(Globals.sysUser, proj_uniqueid, "learnsteps", null, null);
		if (proj.errCode == 0) {
			proj.setLearnsteps(learn_steps);
			if (!proj.putToSink(null)) {
				Globals.t4dialog.showException(proj.errCode + "\r\n" + proj.errMess);
				return;
			}
		} else {
			Globals.t4dialog.showException(proj.errCode + "\r\n" + proj.errMess);
			return;
		}
	}
	
    /**
     * The method <CODE>saveLearningCurve</CODE> save the learning curve of the neural network.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public void saveLearningCurve(String proj_uniqueid) throws Exception {
		
		// delete all learn elements first.
		if (!deleteLearningCurve(proj_uniqueid)) {
			return;
		}

		// save the learn curve.
		ArrayList<ImageLine> graphic_data = Globals.nad.lds.getGraphicData();
		
		int len = graphic_data.size();
		for (int ndx=0;ndx<len;ndx++) {
			ImageLine line = graphic_data.get(ndx);
			DbAccessLearning learn = new DbAccessLearning(Globals.sysUser, null, "all", null, null);
			learn.setSort(ndx+1);
			learn.setXvalue((int) line.getX1());
			learn.setYvalue(line.getY1());
			learn.setFieldAsString("idproject", proj_uniqueid);
			if (!learn.putToSink(null)) {
				Globals.t4dialog.showException(learn.errCode + "\r\n" + learn.errMess);
				return;
			}
		}
	}

    /**
     * The method <CODE>getLearningCurve</CODE> gets the learning curve of the neural network.
     * @param the uniqueid of the related project.
     * @exception Exception.
     * @since 1.0
     */
	public void getLearningCurve(String proj_uniqueid) throws Exception {
		
		ArrayList<ImageLine> graphic_data = new ArrayList<ImageLine>();
		
		// delete all learn elements first.
		ArrayList tabs = dsl.getDataSet(Globals.sysUser, "Learning",
				"xvalue,yvalue", null, null, "idproject='" + proj_uniqueid + "'",
				"sort", "netanddecision");

		boolean first = true;
		double old_x = 0;
		double old_y = 0;
		int len = tabs.size();
		for (int ndx=0;ndx<len;ndx++) {
			DbAccessLearning learn = (DbAccessLearning) tabs.get(ndx);
			if (learn.errCode != 0) {
				Globals.t4dialog.showException(learn.errCode + "\r\n" + learn.errMess);
				return;
			}
			
			ImageLine line = null;
			if (first) {
				old_x = learn.getXvalue();
				old_y = learn.getYvalue();
				first = false;
			} else {
				line = new ImageLine(old_x, old_y, 0, learn.getXvalue(), learn.getYvalue(), 0, false);
				old_x = learn.getXvalue();
				old_y = learn.getYvalue();
				graphic_data.add(line);
			}
		}
		
		Globals.nad.lds.setLearningCurve(graphic_data);
		
	}
}
