package n4e.netanddecision.tool;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;

import n4e.core.controls.AWCombo;
import n4e.core.controls.ExampleFileFilter;
import n4e.core.database.DatabaseDatasetLoader;
import n4e.core.system.GlobalsDialog;
import n4e.core.tool.Tool4Throwable;
import n4e.netanddecision.dbaccess.DbAccessReadKeyWord;
import n4e.netanddecision.dbaccess.DbAccessReadVarValue;
import n4e.netanddecision.system.Globals;

/**
 * The <CODE>Tool4MyDialog</CODE> class provides common dialog functions.
 * <P><B>title: DunningAndControlling</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2011 thomas kaffka, born at 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , mba , cologne , germany.
 * @version 30.06.2011 18:41:18
 * @since 1.0
 */

public class Tool4MyDialog {

	public static final int SAVE = 0;
	public static final int SELECT = 1;

	/**
	 * The constructor <CODE>Tool4MyDialog</CODE> creates a new instance(object) of the class.
	 * @since 1.0
	 */
	public Tool4MyDialog() {
	}

	/**
	 * The method <CODE>getImage</CODE> gets an image from the picture folder.
	 * @param picname name of the picture to get.
	 * @return image object.
	 * @exception Exception
	 * @since 1.0
	 */
	public ImageIcon getImage(String picname) {
		try {
			Image im = Toolkit.getDefaultToolkit().getImage(
					getClass().getResource("resources/" + picname));
			return new ImageIcon(im);
		} catch (Exception ex) {
			System.out.println("getImage " + Tool4Throwable.getCallStackOut(ex));
		}
		return null;
	}

	/**
	 * The method <CODE>getImage</CODE> gets an image from the picture folder.
	 * @param picname name of the picture to get.
	 * @return image object.
	 * @exception Exception
	 * @since 1.0
	 */
	public Image getImagePure(String picname) throws Exception {
		try {
			Toolkit tk = Toolkit.getDefaultToolkit();
			Image im = tk.getImage(getClass().getResource("resources/" + picname)); 
			while (!tk.prepareImage(im, -1, -1, null)) {
				try {
					Thread.sleep(200);
				} catch (Exception ex) {
					
				}
			}
			return im;
		} catch (Exception ex) {
			System.out.println("getImagePure " + Tool4Throwable.getCallStackOut(ex));
		}
		return null;
	}

	/**
	 * The method <CODE>createColumnWidthTable</CODE> creates a hash table with 
	 * the column width. A -1 indicates: no width.
	 * @param column_width the width values separated by comma.
	 * @return the table with the column widths.
	 * @exception Exception
	 * @since 1.0
	 */
	public Hashtable createColumnWidthTable(String column_width) throws Exception {
		
		Hashtable column_width_table = new Hashtable();
		
		StringTokenizer st = new StringTokenizer(column_width, ",");
		int column_count = 0;
		while(st.hasMoreTokens()) {
			String s_width = st.nextToken();
			Integer i_width = new Integer(s_width);
			if (i_width > 0) {
				column_width_table.put(new Integer(column_count), i_width);
			}
			column_count++;
		}
		
		return column_width_table;
	}

	/**
	 * The method <CODE>fillCombo</CODE> fills the combo box with data.
	 * @param box the combo box object.
	 * @param uniqueidks the uniqueid of the key set.
	 * @param get_description adds the description of the key in the box row.
	 * @param set_empty_row first line is empty.
	 * @param set_uniqueid set the index to the current row.
	 * @exception Exception.
	 * @since 1.0
	 */
	public void fillCombo(AWCombo box, String uniqueidks, boolean get_description, 
			boolean set_empty_row, String set_uniqueid, boolean sort_name) throws Exception {

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();

    	int index = (Integer) GlobalsDialog.language_index.get(GlobalsDialog.language_code);
    	
		String sort_field = null;
		if (sort_name) {
			if (index == 0) {
				sort_field = "name";
			} else if (index == 1) {
				sort_field = "name2";
			} else if (index == 2) {
				sort_field = "name3";
			}
		} else {
			sort_field = "parameter1";
		}
		
		ArrayList keywords = dsl.getDataSetRead(Globals.sysUser, "KeyWord", "name,name2,name3,description,description2,description3", 
				null, null, "idkeyset='" + uniqueidks + "'", sort_field, "netanddecision");

		box.resetKeyList();
		
    	int len = keywords.size();
    	if (len > 0 && set_empty_row) {
			box.addItem(" ");
			box.addKey("n4eempty");
    	}
    	
    	for (int ndx=0; ndx<len; ndx++) {
    		DbAccessReadKeyWord keyword = (DbAccessReadKeyWord) keywords.get(ndx) ;
    		if (keyword.errCode == 0) {
    			if (index == 0) {
    				box.addItem(keyword.getName() + (get_description?" " + keyword.getDescription():""));
    			} else if (index == 1) {
    				box.addItem(keyword.getName2() + (get_description?" " + keyword.getDescription2():""));
    			} else if (index == 2) {
    				box.addItem(keyword.getName3() + (get_description?" " + keyword.getDescription3():""));
    			}
				box.addKey(keyword.getUniqueid());
    		}
    	}

    	if (set_uniqueid != null) {
    		box.setIndexKey(set_uniqueid);
    	} else {
    		if (len > 0) {
    			box.setSelectedIndex(0);
    		}
    	}
    	
	}

	/**
	 * The method <CODE>fillComboValue</CODE> fills the combo box with value data of the variable.
	 * @param box the combo box object.
	 * @param uniqueidks the uniqueid of the key set.
	 * @param get_description adds the description of the key in the box row.
	 * @param set_empty_row first line is empty.
	 * @param set_uniqueid set the index to the current row.
	 * @exception Exception.
	 * @since 1.0
	 */
	public void fillComboValue(AWCombo box, String uniqueid_var, String set_uniqueid) throws Exception {

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();

    	int index = (Integer) GlobalsDialog.language_index.get(GlobalsDialog.language_code);
		
		ArrayList values = dsl.getDataSetRead(Globals.sysUser, "VarValue", 
				"value", null, null, "idvarname='" + uniqueid_var + "'", 
				"sort", "netanddecision");

		box.resetKeyList();
		
    	int len = values.size();
    	
    	for (int ndx=0; ndx<len; ndx++) {
    		DbAccessReadVarValue value = (DbAccessReadVarValue) values.get(ndx) ;
    		if (value.errCode == 0) {
				box.addItem(value.getValue());
				box.addKey(Integer.toString(ndx + 1));
    		}
    	}

    	if (set_uniqueid != null) {
    		box.setIndexKey(set_uniqueid);
    	} else {
    		if (len > 0) {
    			box.setSelectedIndex(0);
    		}
    	}
    	
	}

	/**
	 * The method <CODE>fillCombo</CODE> fills the combo box with data.
	 * @param box the combo box object.
	 * @param uniqueidks the uniqueid of the key set.
	 * @param get_description adds the description of the key in the box row.
	 * @param set_empty_row first line is empty.
	 * @param set_uniqueid set the index to the current row.
	 * @param allow list of the allowed operands (i.e.: ,<,>,=,).
	 * @exception Exception.
	 * @since 1.0
	 */
	public void fillComboExt(AWCombo box, String uniqueidks, boolean get_description, 
			boolean set_empty_row, String set_uniqueid, String allow) throws Exception {

		DatabaseDatasetLoader dsl = new DatabaseDatasetLoader();
		ArrayList keywords = dsl.getDataSetRead(Globals.sysUser, "KeyWord", "name,name2,description,description2", 
				null, null, "idkeyset='" + uniqueidks + "'", "name", "core");
    	
		box.resetKeyList();
		
    	int len = keywords.size();
    	if (len > 0 && set_empty_row) {
			box.addItem(" ");
			box.addKey("n4eempty");
    	}
    	
    	int index = (Integer) GlobalsDialog.language_index.get(GlobalsDialog.language_code);

    	for (int ndx=0; ndx<len; ndx++) {
    		n4e.core.dbaccess.DbAccessReadKeyWord keyword = (n4e.core.dbaccess.DbAccessReadKeyWord) keywords.get(ndx) ;
    		if (keyword.errCode == 0) {
    			if (allow != null) {
    				if (allow.indexOf(keyword.getName()) == -1) {
    					continue;
    				}
    			}
    			if (index == 0) {
    				box.addItem(keyword.getName() + (get_description?" " + keyword.getDescription():""));
    			} else if (index == 1) {
    				box.addItem(keyword.getName2() + (get_description?" " + keyword.getDescription2():""));
    			}
				box.addKey(keyword.getUniqueid());
    		}
    	}

    	if (set_uniqueid != null) {
    		box.setIndexKey(set_uniqueid);
    	} else {
    		if (len > 0) {
    			box.setSelectedIndex(0);
    		}
    	}
	}

	/**
	 * The method <CODE>fileSelectionDialog</CODE> shows the file selection dialog. 
	 * @param extension the extension of the files.
	 * @param message the message shown.
	 * @param type the dialog type.
	 * @return the path.
	 * @since 1.0
	 */
	public String fileSelectionDialog(String[] extension, String message, int type) {
		
        JFileChooser chooser = new JFileChooser(".");

        chooser.setFileFilter(new ExampleFileFilter(extension, message));

        String file_path = null;
        int state = 0;
        if (type == SELECT) {
            state = chooser.showOpenDialog(Globals.nad);
        } else if (type == SAVE) {
            state = chooser.showSaveDialog(Globals.nad);
        }
        if (state == JFileChooser.APPROVE_OPTION) {
        	file_path = chooser.getSelectedFile().getPath();
        }
		return file_path;
	}

	/**
	 * The method <CODE>initializeNeuralNetwork</CODE> initializes the new neural network. 
	 * @param proj_uid the uniqueid of the current project.
	 * @exception Exception.
	 * @since 1.0
	 */
	public void initializeNeuralNetwork(String proj_uid) throws Exception {
		Globals.examples = null;
		initializeExampleExecution(proj_uid);
		Globals.t4base.restoreNeuralNetwork(proj_uid);
		Globals.t4base.getLearningCurve(proj_uid);
		Globals.nad.wds.createCurrentData(proj_uid);
		Globals.nad.wds.setControlsWithValues();
		Globals.nad.grds.paintGraphic();
		Globals.nad.gds.functionEditable(true);
		Globals.nad.repaint();
	}

	/**
	 * The method <CODE>initializeExampleExecution</CODE> initializes the example and the execution site. 
	 * @param proj_uid the uniqueid of the current project.
	 * @exception Exception.
	 * @since 1.0
	 */
	public void initializeExampleExecution(String proj_uid) throws Exception {
		Globals.nad.eds.createCurrentData(proj_uid); // ExampleDataSite
		Globals.nad.exds.createCurrentData(proj_uid); // ExecutionDataSite
	}

	/**
	 * The method <CODE>blockAllFunctions</CODE> blocks all functions, if training. 
	 * @param editable false -> block.
	 * @exception Exception.
	 * @since 1.0
	 */
	public void blockAllFunctions(boolean editable) throws Exception {
		Globals.nad.pls.functionEditable(editable);
		Globals.nad.pds.functionEditable(editable);
		Globals.nad.exds.functionEditable(editable);
		Globals.nad.eds.functionEditable(editable);
		Globals.nad.gds.functionEditable(editable);
		Globals.nad.lnds.functionEditable(editable);
	}
}

