package de.neuralnetwork.system;

import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;

import de.neuralnetwork.saxparser.ImportExamples;
import de.neuralnetwork.saxparser.ImportExecution;
import de.neuralnetwork.tool.Tool4Throwable;
import n4e.core.tool.Tool4NeuralNetwork;

/**
 * The <CODE>NeuralNetwork</CODE> class provides a Neural Network with the Backpropagation Technology.
 * <P><B>title: NeuralNetwork</B></P>
 * <P>description: Neural Network with the Backpropagation Technology</P>
 * <P>copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , cologne , germany.
 * @version 09.05.2017 13:47:05
 * @since 1.0
 */

public class NeuralNetwork {

    private BufferedReader input;
    private ArrayList<ArrayList<Number>> examples;
    private int x_layer;
    private int hidden;
    private int y_layer;
    private double epsilon;
    private int learn_step;
    private Tool4NeuralNetwork t4nn;
	
	/**
	 * The constructor <CODE>NeuralNetwork</CODE> creates a new instance(object) of the class.
	 * @since 1.0
	 */
	public NeuralNetwork() {
		
	    try {

	    	input = new BufferedReader(new InputStreamReader(System.in));
		
	    	System.out.println("Neural Network");

	    	while (!getCommand().equals("ende"));
		
	    } catch (final Exception e) {
        	System.out.println(Tool4Throwable.getCallStackOut(e));
	    }
	}

    /**
     * The method <CODE>getCommand</CODE> gets the current command from the user.
     * @exception Exception if error
     * @since 1.0
     */
	private String getCommand() throws Exception {
    	
    	System.out.print("Kommando: ");
		String in = input.readLine();
		
		if (in.equals("ende")) {
			// do nothing.
			
		} else if (in.equals("hilfe")) {
			outHelp();
			
		} else if (in.startsWith("h")) { // define hidden layer.
			hidden = Integer.parseInt(in.substring(2)); // bigness.
			
		} else if (in.startsWith("e")) { // define epsilon.
			epsilon = Double.parseDouble(in.substring(2)); // epsilon.
			
		} else if (in.startsWith("l")) { // load examples.
			in = in.substring(2); // get the file name.
			String path = getFullPathToSystem("data") + in;
			examples = new ArrayList<ArrayList<Number>>();
			ArrayList<Integer> dimension = new ArrayList<Integer>();
			ImportExamples imp = new ImportExamples();
			imp.parseDocument(path, examples, dimension);
			if (imp.errCode != 0) {
				System.out.println(imp.errCode + "\r\n" + imp.errMess);
			}
			x_layer = dimension.get(0);
			y_layer = dimension.get(1);
			System.out.println("X-Schicht: " + x_layer);
			System.out.println("Y-Schicht: " + y_layer);
			System.out.println("Anzahl: " + examples.size());
			
		} else if (in.equals("t")) { // training.
			if (hidden == 0) {
				System.out.println("Bitte Mchtigkeit der Hidden-Schicht definieren!");
				return in;
			}
			if (epsilon == 0.0) {
				System.out.println("Bitte Wert von Epsilon definieren!");
				return in;

			}
			if (examples == null || examples.size() == 0) {
				System.out.println("Bitte Beispiele laden!");
				return in;
			}

			learn_step = 0;
			t4nn = new Tool4NeuralNetwork(examples, x_layer, y_layer, hidden, epsilon, false);
			boolean go = true;
			
			while (go) {
				learn_step++;
				t4nn.learning();
				if (learn_step % 10000 == 0) {
					System.out.print(".");
				}
				if (learn_step % 100000 == 0) {
					Toolkit.getDefaultToolkit().beep();
			    	System.out.print(" " + learn_step + " (" + t4nn.getMaxError() + ") Kommando <stop>: ");
					in = input.readLine().toLowerCase();
					if (in.equals("stop")) {
						go = false;
					}
				}
			}

		} else if (in.startsWith("a")) { // execute the neural network.
			if (learn_step == 0) {
				System.out.println("Bitte trainieren!");
				return in;
			}

			double[] x = new double[x_layer];

			if (in.length() > 1) {

				in = in.substring(2); // get the file name.
				String path = getFullPathToSystem("data") + in;
				ArrayList<ArrayList<Number>> execution = new ArrayList<ArrayList<Number>>();
				ArrayList<Integer> dimension = new ArrayList<Integer>();
				ImportExecution imp = new ImportExecution();
				imp.parseDocument(path, execution, dimension);
				if (imp.errCode != 0) {
					System.out.println(imp.errCode + "\r\n" + imp.errMess);
					return "";
				}
				int test_x_layer = dimension.get(0);
				int test_y_layer = dimension.get(1);
				
				if (test_x_layer != x_layer || test_y_layer != y_layer) {
					System.out.println("Daten passen nicht zu den Beispielen!");
					return "";
				}

				ArrayList<Number> line = execution.get(0);
				for (int ndx=0;ndx<x_layer;ndx++) {
					x[ndx] = (double) line.get(ndx);
				}

			} else {
			
				for (int ndx=0;ndx<x_layer;ndx++) {
			    	System.out.print("X-Wert (" + (ndx+1) + "): ");
					in = input.readLine().toLowerCase();
					x[ndx] = Double.parseDouble(in);
				}
			
			}
			
			double[] result = t4nn.executeFunction(x);
			
			for (int ndx=0;ndx<y_layer;ndx++) {
		    	System.out.println("Y-Wert (" + (ndx+1) + "): " + Double.toString(result[ndx]));
			}
			
		} else {
			outHelp();
		}
		
		return in;
	}
	
    /**
     * The method <CODE>outHelp</CODE> prints all commands.
     * @since 1.0
     */
	private void outHelp() {
    	System.out.println("h (Hidden-Schicht definieren) <Mchtigkeit>");
    	System.out.println("e (psilon definieren) <Wert>");
    	System.out.println("l (aden der Beispiele) <Datei.xml>");
    	System.out.println("t (raining)");
    	System.out.println("a (usfhren des Neuronalen Netzes) <Datei.xml>");
    	System.out.println("hilfe");
    	System.out.println("ende");
    	System.out.println();
	}
	
    /**
     * The method <CODE>getFullPathToSystem</CODE> gets the full path to the current folder.
     * @param folder the folder of the file system.
     * @return full path.
     * @exception Exception if error
     * @since 1.0
     */
    public String getFullPathToSystem(String folder) throws Exception {
        File cfile = new File("");
        String fullpath = cfile.getAbsolutePath();
        fullpath += File.separator + folder + File.separator;
        return fullpath;
    }

    /**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 1.0
	 */
	public static void main(String[] args) {
		new NeuralNetwork();
	}

}
