package de.neuralnetwork.tool;

import java.text.*;
import java.util.*;
import java.io.*;

/**
 * The <CODE>Tool4Throwable</CODE> the class creates a log string for exception log with stack trace.
 * @author Thomas Kaffka
 * @version 8. Nov 2005, 15:18
 * @since 1.0
 */
public class Tool4Throwable {

    private static final String lineend = "\r\n";

    /**
     * The constructor <CODE>Tool4Throwable</CODE> creates a new instance(object) of the class.
     * @since 1.0
     */
    public Tool4Throwable() {}

    /**
     * get field: current time formated as timestamp.
     * @since 1.0
     */
    private static synchronized String getCurrTime(){
        return new SimpleDateFormat("MM/dd/yyyy HH:mm:ss").format(new Date());
    }

    /**
     * The method <CODE>getCallStackOut</CODE> creates the log string with stack trace of an exception.
     * @param error the throwable/exception.
     * @return stack trace as string and n4e formatted.
     * @since 1.0
     */
    public static synchronized String getCallStackOut(Throwable error) {

        String log = "*****************************************" + lineend;
        log += "< " + getCurrTime() + " >" + " call stack out..." + lineend;
        log += "message: " + error.getMessage() + lineend;
        log += "localized message: " + error.getLocalizedMessage() + lineend;
        StackTraceElement[] ste = error.getStackTrace();
        int len = ste.length;
        for (int i = 0; i < len; i++) {
            StackTraceElement stelement = ste[i];
            String className = stelement.getClassName();
            String methodName = stelement.getMethodName();
            String lineNumber = Integer.toString(stelement.getLineNumber());
            log += "... class: " + className + " ... method: " + methodName + " ... (" + lineNumber + ")." + lineend;
        }

        log += "*****************************************" + lineend + lineend;

        return log;
    }

}

