package de.password.system;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;

import n4e.core.controls.AWTextField;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.MainDialog;
import n4e.core.tool.Tool4Throwable;

/**
 * The <CODE>Password</CODE> class for password editing.
 * description: password editing.
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne , germany.
 * @version 10.05.2017 14:26:59
 * @since 0.1
 */

public class Password extends MainDialog {

	private int FIELD_COUNT = 5;
	
    private JLabel lblGraphics;
	private AWTextField tfPerson;
	private JLabel lblPatternCount;

	private JButton btnXMLExample;
	private JButton btnXMLExecution;
	private JButton btnEnter;
    private JButton btnReset;
	private JButton btnClose;

	private BufferedImage image;
	private Graphics graph;
    private int graphic_height; // the height of the graphic.
    private int graphic_width; // the width of the graphic.
    private int mouse_x;
	private int mouse_y;
	private int pattern_count;
	private int step;
	
	private ArrayList<ArrayList<Number>> pattern_list;
	private ArrayList<Number> person_list;

    /**
	 * The constructor <CODE>Password</CODE> creates a new instance(object)
	 * of the class.
	 * @since 0.1
	 */
	public Password() {
		super("password", "0.1", "2017", 500, 410);

		try {

			setResizable(false);
			Globals.pw = this;
			Globals.doAfterInitials();
			Image icon = Globals.t4md.getImagePure("app.jpg");
			setIconImage(icon);
			windowInit();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @exception Exception if error.
	 * @since 0.1
	 */
	private void windowInit() throws Exception {

		try {

			int controls_line = 350;
			pattern_list = new ArrayList<ArrayList<Number>>();
			person_list = new ArrayList<Number>();
			pattern_count = 0;

			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 49, screen.width, 20);
			Container con = getContentPane();
			con.setLayout(null);
			con.setSize(new Dimension(screen.width, screen.height));
			con.setBackground(Color.LIGHT_GRAY);
			
			graphic_height = 380;
			graphic_width = 380;
			lblGraphics = new JLabel("");
			lblGraphics.setBounds(10, 10, graphic_width, graphic_height);
			lblGraphics.setBackground(Color.WHITE);
			lblGraphics.setOpaque(true);
	        con.add(lblGraphics);

			image = new BufferedImage(graphic_width, graphic_height, BufferedImage.TYPE_3BYTE_BGR);
			graph = image.getGraphics();

	        step = graphic_height / FIELD_COUNT;
	        makeGraphFields();
	        
	        lblGraphics.setIcon(new ImageIcon(image));
			lblGraphics.addMouseMotionListener(new java.awt.event.MouseMotionListener() {

				@Override
				public void mouseDragged(MouseEvent e) {
					mouse_x = e.getX();
					mouse_y = e.getY();
					graph.drawLine(mouse_x-2, mouse_y-4, mouse_x+2, mouse_y-4);
					graph.drawLine(mouse_x-3, mouse_y-3, mouse_x+3, mouse_y-3);
					graph.fillRect(mouse_x-4, mouse_y-2, 8, 5);
					graph.drawLine(mouse_x-3, mouse_y+3, mouse_x+3, mouse_y+3);
					graph.drawLine(mouse_x-2, mouse_y+4, mouse_x+2, mouse_y+4);
					repaint();
				}

				@Override
				public void mouseMoved(MouseEvent e) {
				}
				
			});

			createLabel(con, 10, controls_line + 45, 120, 20, Globals.message.getMessage("msg0003"));
			tfPerson = createTextField(con, 60, controls_line + 45, 40, 20, 0);

			lblPatternCount = new JLabel("");
			lblPatternCount.setBounds(120, controls_line + 45, 40, 20);
			lblPatternCount.setText("0");
	        con.add(lblPatternCount);

	        btnEnter = createButton(con, 180, controls_line + 45, 100, 23, Globals.message.getMessage("msg0002"), true);
			btnReset = createButton(con, 290, controls_line + 45, 100, 23, Globals.message.getMessage("msg0004"), true);
			btnXMLExample = createButton(con, 70, controls_line + 70, 100, 23, Globals.message.getMessage("msg0005"), true);
			btnXMLExecution = createButton(con, 180, controls_line + 70, 100, 23, Globals.message.getMessage("msg0006"), true);
			btnClose = createButton(con, 290, controls_line + 70, 100, 23, Globals.message.getMessage("msg0001"), true);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}

		setVisible(true);

	}

	/**
	 * The method <CODE>createLabel</CODE> creates a label.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @return a label.
	 * @since 0.1
	 */
	private JLabel createLabel(Container c, int x, int y, int width, int height, String text) {
        JLabel label = new JLabel(text);
		label.setBounds(x, y, width, height);
        c.add(label);
        return label;
	}

	/**
	 * The method <CODE>createTextField</CODE> creates a text field.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param value the value of the control.
	 * @return a text field.
	 * @since 0.1
	 */
	private AWTextField createTextField(Container c, int x, int y, int width, int height, int value) {
		AWTextField tf = new AWTextField();
        tf.setBounds(x, y, width, height);
        tf.setFont(Globals.fontStandard11);
        tf.setText(Integer.toString(value));
		c.add(tf);
		return tf;
	}

	/**
	 * The method <CODE>createButton</CODE> creates a button.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @param show true: show the text.
	 * @return a button.
	 * @since 0.1
	 */
	private JButton createButton(Container c, int x, int y, int width, int height, String text, boolean show) {
		String curr_text = "";
		if (show) {
			curr_text = text;
		}
		JButton btn = new JButton(curr_text);
	    btn.setBounds(x, y, width, height);
	    btn.setFont(GlobalsDialog.fontStandard12);
	    btn.setToolTipText(text);
	    btn.addActionListener(this);
		c.add(btn);
		return btn;
	}
	
	/**
	 * The method <CODE>makeGraphFields</CODE> paints the fields.
	 * @since 0.1
	 */
	private void makeGraphFields() {
        graph.setColor(Color.WHITE);
        graph.fillRect(0, 0, graphic_width, graphic_height);
        graph.setColor(Color.GRAY);
        for (int ndx=0;ndx<graphic_height;ndx+=step) {
			graph.drawLine(ndx, 0, ndx, graphic_height);
        }
        for (int ndx=0;ndx<graphic_height;ndx+=step) {
			graph.drawLine(0, ndx, graphic_width, ndx);
        }
        graph.setColor(Color.BLACK);
	}
	
	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 * @since 0.1
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			if (e.getSource() == btnReset) {
		        makeGraphFields();
		        repaint();
			} else if (e.getSource() == btnClose) {
				System.exit(0);
			} else if (e.getSource() == btnEnter) {
				ArrayList<Number> line = new ArrayList<Number>();
		        for (int i=0;i<FIELD_COUNT;i++) {
			        for (int j=0;j<FIELD_COUNT;j++) {
			        	double black = countBlack(i, j);
			        	double percent = black / (step * step);
			        	int color = (int) (255 * percent);
			        	if (color == 0) {
			        		color = 255;
			        	}
			        	makeColor(i, j, color);
			        	line.add(color);
			        }
		        }
		        repaint();
		        pattern_list.add(line);
		        int person = Integer.parseInt(tfPerson.getText());
		        person_list.add(person * 10);
		        pattern_count++;
		        lblPatternCount.setText(Integer.toString(pattern_count));
		        
			} else if (e.getSource() == btnXMLExample) {
				saveExample();
			} else if (e.getSource() == btnXMLExecution) {
				saveExecution();
			}
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}
	
	/**
	 * The method <CODE>countBlack</CODE> counts the black pixel in a field.
	 * @param i the i coordinate.
	 * @param j the j coordinate.
	 * @return the number of black pixel.
	 * @since 0.1
	 */
	private int countBlack(int i, int j) {
		int count = 0;
        for (int x=i*step;x<i*step+step;x++) {
	        for (int y=j*step;y<j*step+step;y++) {
	        	int value = image.getRGB(x, y);
	        	int r = value & 0xFF;
	        	int g = value & 0xFF00 >> 8;
	        	int b = value & 0xFF0000 >> 16;
			    if (r == 0 && g == 0 && b == 0) { // black.
			    	count++;
			    }
	        }
        }
		return count;
	}
	
	/**
	 * The method <CODE>makeColor</CODE> colors a field.
	 * @param i the i coordinate.
	 * @param j the j coordinate.
	 * @param color the color.
	 * @since 0.1
	 */
	private void makeColor(int i, int j, int color) {
		Color c = new Color(color, color, color);
        graph.setColor(c);
        graph.fillRect(i*step, j*step, step, step);
	}

	/**
	 * The method <CODE>saveExapmle</CODE> save the output as example.
	 * @since 0.1
	 */
	private void saveExample() throws Exception {
		
		int len = pattern_list.size();

		// merge the examples.
		Random rand = new Random();
		for (int ndx=0;ndx<len*2;ndx++) {
			int i = rand.nextInt(len);
			int j = rand.nextInt(len);
			ArrayList<Number> line_i = pattern_list.get(i);
			ArrayList<Number> line_j = pattern_list.get(j);
			pattern_list.set(i, line_j);
			pattern_list.set(j, line_i);
			int value_i = (int) person_list.get(i);
			int value_j = (int) person_list.get(j);
			person_list.set(i, value_j);
			person_list.set(j, value_i);
		}

		// export the examples.
		String listing = Globals.t4xml.getExampleHeader(Globals.message.getMessage("msg0007"), FIELD_COUNT * FIELD_COUNT, 1);
		for (int ndx=0;ndx<len;ndx++) {
			ArrayList<Number> line = pattern_list.get(ndx);
			int line_len = line.size();
			for (int i=0;i<line_len;i++) {
				int color = (int) line.get(i);
				double dcolor = adjustValue(color);
				String xml_line = Globals.t4xml.getExampleXValue(Double.toString(dcolor));
				listing = Globals.t4s.preInsertString(listing, "@examplelist@", xml_line);
			}
			String xml_line = Globals.t4xml.getExampleYValue(Integer.toString((int)person_list.get(ndx)));
			listing = Globals.t4s.preInsertString(listing, "@examplelist@", xml_line);
		}
		listing = Globals.t4s.replaceString(listing, "@examplelist@\r\n", "");
		String path = Globals.t4f.getFullPathToSystem("data");
		Globals.t4f.putTheFile(path, "PasswordExamples.xml", listing);
	}

	/**
	 * The method <CODE>saveExecution</CODE> save the output as execution.
	 * @since 0.1
	 */
	private void saveExecution() throws Exception {
		String listing = Globals.t4xml.getExecutionHeader(Globals.message.getMessage("msg0007"), FIELD_COUNT * FIELD_COUNT, 1);
		int len = pattern_list.size();
		for (int ndx=0;ndx<len;ndx++) {
			ArrayList<Number> line = pattern_list.get(ndx);
			int line_len = line.size();
			for (int i=0;i<line_len;i++) {
				int color = (int) line.get(i); 
				double dcolor = adjustValue(color);
				String xml_line = Globals.t4xml.getExecutionXValue(Double.toString(dcolor));
				listing = Globals.t4s.preInsertString(listing, "@executionlist@", xml_line);
			}
		}
		listing = Globals.t4s.replaceString(listing, "@executionlist@\r\n", "");
		String path = Globals.t4f.getFullPathToSystem("data");
		Globals.t4f.putTheFile(path, "PasswordExecution.xml", listing);

		pattern_list = new ArrayList<ArrayList<Number>>();
		pattern_count = 0;
	}
	
	/**
	 * The method <CODE>adjustValue</CODE> adjust the values of black and white.
	 * @param value the value of the field.
	 * @since 0.1
	 */
	private double adjustValue(double value) {
		if (value == 255.0) {
			value = 30.0;
		} else {
			value = 10.0;
		}
		return value;
	}

	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 0.1
	 */
	public static void main(String[] args) {
		try {
			new Globals();
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new Password();
	}

}
