package de.perzeptron.control;

import de.perzeptron.system.Globals;

/**
 * The <CODE>Element</CODE> class for elements of the perzeptron.
 * description: Perzeptron
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne, germany.
 * @version 10.02.2017 09:17:59
 * @since 0.1
 */

public class Element {

	double[] w; // weights.
	
	/**
	 * The constructor <CODE>Element</CODE> creates a new instance(object) of the class.
	 * @since 0.1
	 */
	public Element() {
		w = new double[Globals.LINES];
		for (int ndx=0;ndx<Globals.LINES;ndx++) {
			w[ndx] = 1.0;
		}
	}

	/**
	 * The method <CODE>theta</CODE> computes the signal of the element.
	 * @param value the value.
	 * @since 0.1
	 */
	private int theta(double value) {
		if (value > 0.0001) {
			return 1;
		} else {
			return 0;
		}
	}

	/**
	 * The method <CODE>reaction</CODE> computes the reaction of the element.
	 * @param x the patterns.
	 * @since 0.1
	 */
	public int reaction(int[] x) {
        double y = 0;
        for (int ndx=0;ndx<Globals.LINES;ndx++) {
        	y += w[ndx] * x[ndx];
        }
        return theta(y);
	}
	
	/**
	 * The method <CODE>e</CODE> computes a component of the learn function.
	 * @param x the patterns.
	 * @since 0.1
	 */
    private double e(int[] x) {
    	double dividend = 0;
    	double divisor = 0;
    	for (int ndx=0;ndx<Globals.LINES;ndx++) {
    		dividend += w[ndx] * x[ndx];
    		divisor += Math.pow(x[ndx], 2);
    	}
    	return dividend / divisor;
    }
    
	/**
	 * The method <CODE>learn</CODE> the learn function.
	 * @param x the patterns.
	 * @param C the class (category).
	 * @since 0.1
	 */
    public void learn(int[] x, int C) {
        int y = reaction(x);
        if (y != C) {
	        for (int ndx=0;ndx<Globals.LINES;ndx++) {
	        	w[ndx] += e(x) * (C - y) * x[ndx];
	        }
        }
    }
}
