package de.perzeptron.system;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import de.perzeptron.control.Element;
import n4e.core.controls.AWTextArea;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.MainDialog;
import n4e.core.tool.Tool4Throwable;

/**
 * The <CODE>Perzeptron</CODE> class for perzeptron processing.
 * description: Perzeptron.
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne , germany.
 * @version 08.02.2017 12:19:59
 * @since 0.1
 */

public class Perzeptron extends MainDialog {

	private static final String LINE_END = "\r\n";
	public static final int LEARN_END = 10; // number of learn steps.
	public static final int N = 4; // number of elements.
	public static final int WP = 10; // number of world patterns.
	public static final int NP = 3; // number of new patterns.
	
	private AWTextArea taData;
	private JButton btnClose;

	private int[][] x =     {{1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0},  // Category 1
			                 {1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0},  // Category 1
			                 {1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0},  // Category 1
			             
			                 {0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0},  // Category 2
			                 {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},  // Category 2
			                 {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0},  // Category 2
			                 {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0},  // Category 2
			             
			                 {0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},  // Category 3
			                 {0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0},  // Category 3
			             
			                 {0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1}}; // Category 4
	
	private int[][] new_x = {{1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0}, 
			                 {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0}, 
			                 {0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1}};
	
	private int[] category_element;
	private Element[] element;
	
	/**
	 * The constructor <CODE>Perzeptron</CODE> creates a new instance(object)
	 * of the class.
	 * @since 0.1
	 */
	public Perzeptron() {
		super("perzeptron", "0.1", "2017", 480, 640);

		try {

			setResizable(false);
			Globals.p = this;
			Globals.doAfterInitials();
			Image icon = Globals.t4md.getImagePure("app.jpg");
			setIconImage(icon);
			windowInit();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @since 0.1
	 */
	private void windowInit() {

		try {

			element = new Element[N];
			
			category_element = new int[WP];
	        category_element[0] = 1;
	        category_element[1] = 1;
	        category_element[2] = 1;
	        category_element[3] = 2;
	        category_element[4] = 2;
	        category_element[5] = 2;
	        category_element[6] = 2;
	        category_element[7] = 3;
	        category_element[8] = 3;
	        category_element[9] = 4;
	        
			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 49, screen.width, 20);
			Container con = getContentPane();
			con.setLayout(null);
			con.setSize(new Dimension(screen.width, screen.height));
			con.setBackground(Color.LIGHT_GRAY);
			
	        taData = new AWTextArea(300, 5);
	        taData.setFont(Globals.fontResult11);
	        taData.setLineWrap(true);
	        taData.setWrapStyleWord(true);
	        taData.setEditable(false);
	        JScrollPane textScrollPane = new JScrollPane(taData);
	        textScrollPane.setViewportView(taData);
	        textScrollPane.setBounds(10, 10, 610, 380);
	        con.add(textScrollPane);

			btnClose = createButton(con, 520, 400, 100, 23, Globals.message.getMessage("msg0001"), true);

	        createOutput();
	        
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}

		setVisible(true);

	}

	/**
	 * The method <CODE>createButton</CODE> creates a button.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @param show true -> show the text.
	 * @return a button.
	 * @since 0.1
	 */
	private JButton createButton(Container c, int x, int y, int width, int height, String text, boolean show) {
		String curr_text = "";
		if (show) {
			curr_text = text;
		}
		JButton btn = new JButton(curr_text);
	    btn.setBounds(x, y, width, height);
	    btn.setFont(GlobalsDialog.fontStandard12);
	    btn.setToolTipText(text);
	    btn.addActionListener(this);
		c.add(btn);
		return btn;
	}
	
	/**
	 * The method <CODE>training</CODE> training of the perzeptron.
	 * @since 0.1
	 */
	private void training() {
		
		for (int ndx1=0;ndx1<N;ndx1++) {
			for (int ndx2=0;ndx2<WP;ndx2++) {
				int category = 0;
				if (ndx1 == 0 && category_element[ndx2] == 1) {
					category = 1;
				} else if (ndx1 == 1 && category_element[ndx2] == 2) {
					category = 1;
				} else if (ndx1 == 2 && category_element[ndx2] == 3) {
					category = 1;
				} else if (ndx1 == 3 && category_element[ndx2] == 4) {
					category = 1;
				}
				element[ndx1].learn(x[ndx2], category);
			}
		}

	}
	
	/**
	 * The method <CODE>learnStepResult</CODE> prints the results of the learn steps.
	 * @param step the learn step.
	 * @since 0.1
	 */
	private String learnStepResult(int step) {
		String output = Globals.message.getMessage("msg0005") + " " + step + LINE_END;
		output += Globals.message.getMessage("msg0004") + "   1 2 3 4" + LINE_END;
		output += "-------------------" + LINE_END;
		
		for (int i=0;i<WP;i++) {
			output += Globals.message.getMessage("msg0003") + " " + i + "| ";
			for (int j=0;j<N;j++) {
				output += (int) element[j].reaction(x[i]) + " ";
			}
			output += LINE_END;
		}
		
		return output;
	}
	
	/**
	 * The method <CODE>generalizationResult</CODE> prints the results of the generalization.
	 * @since 0.1
	 */
	private String generalizationResult() {
		String output = "";
		for (int i=0;i<NP;i++) {
			output += Globals.message.getMessage("msg0003") + " " + i + "| ";
			for (int j=0;j<N;j++) {
				output += (int) element[j].reaction(new_x[i]) + " ";
			}
			output += LINE_END;
		}
		return output;
	}

	/**
	 * The method <CODE>createOutput</CODE> creates the output.
	 * @since 0.1
	 */
	private void createOutput() {
		String output = "";
		
		// make head line.
		output += Globals.message.getMessage("msg0002") + LINE_END;
		int len = Globals.message.getMessage("msg0002").length();
		for (int ndx=0;ndx<len;ndx++) {
			output += "-";
		}
		output += LINE_END;
		output += LINE_END;

		// output patterns.
		for (int i=0;i<WP;i++) {
			output += Globals.message.getMessage("msg0003") + " " + i + ": ";
			for (int j=0;j<Globals.LINES;j++) {
				output += Integer.toString(x[i][j]);
			}
			output += " " + Globals.message.getMessage("msg0004") + " " + Integer.toString(category_element[i]);
			output += LINE_END;
		}
		
		output += LINE_END;

		// output learn steps.
		for (int ndx1=1;ndx1<=LEARN_END;ndx1++) {
			for (int ndx=0;ndx<N;ndx++) {
				element[ndx] = new Element();
			}
        	for (int ndx2=0;ndx2<ndx1;ndx2++) {
        		training();
        	}
        	output += learnStepResult(ndx1);
			output += LINE_END;
        }
		
		// output generalization
		output += Globals.message.getMessage("msg0006") + LINE_END;
		output += "-------------------" + LINE_END;
		for (int i=0;i<NP;i++) {
			output += Globals.message.getMessage("msg0003") + " " + i + ": ";
			for (int j=0;j<Globals.LINES;j++) {
				output += Integer.toString(new_x[i][j]);
			}
			output += LINE_END;
		}
		output += LINE_END;

		output += generalizationResult();
		output += LINE_END;
		
		taData.setText(output);
	}
	
	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 * @since 0.1
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			if (e.getSource() == btnClose) {
				System.exit(0);
			}
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}
	
	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 0.1
	 */
	public static void main(String[] args) {
		try {
			new Globals();
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new Perzeptron();
	}

}
