package n4e.rockthrow.system;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import n4e.core.controls.AWTextArea;
import n4e.core.controls.AWTextField;
import n4e.core.system.GlobalsDialog;
import n4e.core.system.MainDialog;
import n4e.core.tool.Tool4Throwable;
import n4e.rockthrow.list.Element;

/**
 * The <CODE>RockThrow</CODE> class for simulating a thrown rock.
 * description: simulating a thrown rock.
 * copyright: (c) 2017 thomas kaffka, born 11/08/1959 in dsseldorf - germany , all rights reserved.
 * @author thomas kaffka, cologne , germany.
 * @version 20.02.2017 14:07:59
 * @since 0.1
 */

public class RockThrow extends MainDialog {

	private AWTextArea taData;
    private AWTextField tfVelocity;
    private AWTextField tfAngle;
    private AWTextField tfCount;
	private JButton btnCalc;
	private JButton btnGenerate;
	private JButton btnSave;
	private JButton btnClose;
	
	private int example_len;
	private ArrayList<Element> examples;
	
	/**
	 * The constructor <CODE>RockThrow</CODE> creates a new instance(object)
	 * of the class.
	 * @since 0.1
	 */
	public RockThrow() {
		super("rock throw", "0.1", "2017", 480, 640);

		try {

			setResizable(false);
			Globals.rt = this;
			Globals.doAfterInitials();
			Image icon = Globals.t4md.getImagePure("app.jpg");
			setIconImage(icon);
			windowInit();

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}

	/**
	 * The method <CODE>windowInit</CODE> initializes the main window.
	 * @exception Exception.
	 * @since 0.1
	 */
	private void windowInit() throws Exception {

		try {

			Dimension screen = getSize();
			pnlStatusBar.setBounds(0, screen.height - 49, screen.width, 20);
			Container con = getContentPane();
			con.setLayout(null);
			con.setSize(new Dimension(screen.width, screen.height));
			con.setBackground(Color.LIGHT_GRAY);
			
	        taData = new AWTextArea(300, 5);
	        taData.setFont(Globals.fontResult11);
	        taData.setLineWrap(true);
	        taData.setWrapStyleWord(true);
	        taData.setEditable(false);
	        JScrollPane textScrollPane = new JScrollPane(taData);
	        textScrollPane.setViewportView(taData);
	        textScrollPane.setBounds(10, 10, 610, 350);
	        con.add(textScrollPane);

	        JLabel label = new JLabel(Globals.message.getMessage("msg0005"));
	        label.setBounds(10, 370, 100, 20);
	        c.add(label);

			tfVelocity = new AWTextField();
	        tfVelocity.setBounds(110, 370, 100, 20);
			c.add(tfVelocity);

	        label = new JLabel(Globals.message.getMessage("msg0006"));
	        label.setBounds(220, 370, 100, 20);
	        c.add(label);

			tfAngle = new AWTextField();
	        tfAngle.setBounds(280, 370, 100, 20);
			c.add(tfAngle);

			label = new JLabel(Globals.message.getMessage("msg0002"));
	        label.setBounds(10, 400, 100, 20);
	        c.add(label);
	        
			tfCount = new AWTextField();
	        tfCount.setBounds(110, 400, 100, 20);
	        tfCount.setText("300");
			c.add(tfCount);

            btnCalc = createButton(con, 400, 370, 100, 23, Globals.message.getMessage("msg0009"), true);
            btnGenerate = createButton(con, 280, 400, 100, 23, Globals.message.getMessage("msg0003"), true);
            btnSave = createButton(con, 400, 400, 100, 23, Globals.message.getMessage("msg0004"), true);
            btnClose = createButton(con, 520, 400, 100, 23, Globals.message.getMessage("msg0001"), true);

		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}

		setVisible(true);

	}

	/**
	 * The method <CODE>createButton</CODE> creates a button.
	 * @param c the container.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param width the width of the control.
	 * @param height the height of the control.
	 * @param text the text of the control.
	 * @param show true -> show the text.
	 * @return a button.
	 * @since 0.1
	 */
	private JButton createButton(Container c, int x, int y, int width, int height, String text, boolean show) {
		String curr_text = "";
		if (show) {
			curr_text = text;
		}
		JButton btn = new JButton(curr_text);
	    btn.setBounds(x, y, width, height);
	    btn.setFont(GlobalsDialog.fontStandard12);
	    btn.setToolTipText(text);
	    btn.addActionListener(this);
		c.add(btn);
		return btn;
	}
	
	/**
	 * The method <CODE>createOutput</CODE> creates the output.
	 * @since 0.1
	 */
	private void generateExamples() {
		
		examples = new ArrayList<Element>();
		
		String output = "";
		example_len = Integer.parseInt(tfCount.getText());
		
		Random rand = new Random();
		
		for (int ndx=0;ndx<example_len;ndx++) {
			double velocity = (rand.nextDouble() * 100.0 * (70.0 / 100.0)) + 30.0;
			double angle = ((rand.nextDouble() * 90.0 * (50.0 / 100.0)) + 30.0);
			double width = (velocity * velocity * Math.sin(2.0 * angle * (3.14 / 180.0))) / 9.8;
			double time = (velocity * 2.0 * Math.sin(angle * (3.14 / 180.0))) / 9.8;
			Element ele = new Element(velocity, angle, width, time);
			examples.add(ele);
			output += Double.toString(velocity) + " " + Double.toString(angle)+ " " + Double.toString(width)+ " " + Double.toString(time) + "\r\n";
		}
		
		taData.setText(output);
	}
	
	/**
	 * The method <CODE>calculate</CODE> calculates the output.
	 * @since 0.1
	 */
	private void calculate() {
		String output = "";
		
		double velocity = Double.parseDouble(tfVelocity.getText());
		double angle = Double.parseDouble(tfAngle.getText());
		double width = (velocity * velocity * Math.sin(2.0 * angle * (3.14 / 180.0))) / 9.8;
		double time = (velocity * 2.0 * Math.sin(angle * (3.14 / 180.0))) / 9.8;

		output += Globals.message.getMessage("msg0005") + " " + Double.toString(velocity) + " m/sec\r\n" 
				+ Globals.message.getMessage("msg0006") + " " + Double.toString(angle)+ " \r\n\r\n" 
				+ Globals.message.getMessage("msg0007") + " " + Double.toString(width)+ " m\r\n" 
				+ Globals.message.getMessage("msg0008") + " " + Double.toString(time) + " sec\r\n";
		taData.setText(output);
	}
	
	/**
	 * The method <CODE>save</CODE> save the output.
	 * @since 0.1
	 */
	private void save() throws Exception {
		String listing = Globals.t4xml.getExampleHeader(Globals.message.getMessage("msg0010"), 2, 2);
		int len = examples.size();
		for (int ndx=0;ndx<len;ndx++) {
			Element ele = examples.get(ndx);
			String line = Globals.t4xml.getExampleXValue(Double.toString(ele.getVelocity()));
			listing = Globals.t4s.preInsertString(listing, "@examplelist@", line);
			line = Globals.t4xml.getExampleXValue(Double.toString(ele.getAngle()));
			listing = Globals.t4s.preInsertString(listing, "@examplelist@", line);
			line = Globals.t4xml.getExampleYValue(Double.toString(ele.getWidth()));
			listing = Globals.t4s.preInsertString(listing, "@examplelist@", line);
			line = Globals.t4xml.getExampleYValue(Double.toString(ele.getTime()));
			listing = Globals.t4s.preInsertString(listing, "@examplelist@", line);
		}
		listing = Globals.t4s.replaceString(listing, "@examplelist@\r\n", "");
		String path = Globals.t4f.getFullPathToSystem("data");
		Globals.t4f.putTheFile(path, Globals.message.getMessage("msg0010") + ".xml", listing);
	}
	
	/**
	 * The method <CODE>actionPerformed</CODE> computes the button clicks.
	 * @since 0.1
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		
		try {
			if (e.getSource() == btnClose) {
				System.exit(0);
			} else if (e.getSource() == btnCalc) {
				calculate();
			} else if (e.getSource() == btnGenerate) {
				generateExamples();
			} else if (e.getSource() == btnSave) {
				save();
			}
		} catch (Exception ex) {
			t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
		}
	}
	
	/**
	 * The method <CODE>main</CODE> starts the program.
	 * @param args the command line arguments.
	 * @since 0.1
	 */
	public static void main(String[] args) {
		try {
			new Globals();
		} catch (Exception ex) {
			System.out.println(Tool4Throwable.getCallStackOut(ex));
		}
		new RockThrow();
	}

}
