package n4e.core.dbaccess;

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;


/**
 * The <CODE>DbAccessCheckExpCondition</CODE> class checks fields of ExpCondition table in database.
 * @author Thomas Kaffka
 * @version Thu Sep 17 11:40:14 CEST 2015
 * @since 1.0
*/
public class DbAccessCheckExpCondition extends DbCheck {
  // technical fields
  public int errCode = 0;
  public String errMess = "";
  private boolean hasdata = false;
  private Tool4Number t4n = new Tool4Number();
  private Tool4DbMessages t4m = null;

  // database standard fields
  private String uniqueid = ""; // unique key
  private long version = 0; // version counter for database store without locking
  private java.util.Date created = null; // date of creation
  private String creator = ""; // user created
  private java.util.Date changed = null; // date of last change data
  private String changedfrom = ""; // user changed
  private int inuse = 0; // read but not updated
  private String client = ""; // foreign key to client table

  // individual database fields
  private long sort = 0; // Sortierfeld
  private String condition = ""; // Bedingung
  private String operator = ""; // Operator
  private String operand = ""; // Operand
  private DbAccessExpRule myDbAccessExpRule = new DbAccessExpRule() ; // ExpRule
  private DbAccessExpAttribute myDbAccessExpAttribute = new DbAccessExpAttribute() ; // ExpAttribute


  // sets and gets of standard table fields
  public void setUniqueid(String input){uniqueid = input;}
  public String getUniqueid(){return uniqueid;}
  public void setVersion(long input){version = input;}
  public long getVersion(){return version;}
  public void setCreated(java.util.Date input){created = input;}
  public java.util.Date getCreated(){return created;}
  public void setCreator(String input){creator = input;}
  public String getCreator(){return creator;}
  public void setChanged(java.util.Date input){changed = input;}
  public java.util.Date getChanged(){return changed;}
  public void setChangedfrom(String input){changedfrom = input;}
  public String getChangedfrom(){return changedfrom;}
  public void setInuse(int input){inuse = input;}
  public int getInuse(){return inuse;}
  public void setClient(String input){client = input;}
  public String getClient(){return client;}

  // sets and gets of individual table fields
  public void setSort(long input){sort = input;}
  public long getSort(){return sort;}
  public void setCondition(String input){condition = input;}
  public String getCondition(){return condition;}
  public void setOperator(String input){operator = input;}
  public String getOperator(){return operator;}
  public void setOperand(String input){operand = input;}
  public String getOperand(){return operand;}
  public void setMyDbAccessExpRule(DbAccessExpRule input){myDbAccessExpRule = input;}
  public DbAccessExpRule getMyDbAccessExpRule(){return myDbAccessExpRule;}
  public void setMyDbAccessExpAttribute(DbAccessExpAttribute input){myDbAccessExpAttribute = input;}
  public DbAccessExpAttribute getMyDbAccessExpAttribute(){return myDbAccessExpAttribute;}


  /**
   * The constructor <CODE>DbAccessCheckExpCondition</CODE> initialized the object.
   * @param pt4m tool for messages
   * @since 1.0
  */
  public DbAccessCheckExpCondition(Tool4DbMessages pt4m){
    t4m = pt4m;
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens of dataset
   * @param dbaccessclass Class DbAccessCustomer.
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessExpCondition dbaccessclass , String field_list){
    return checkData(dbaccessclass , false , field_list);
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens
   * @param dbaccessclass Class DbAccess...
   * @param first = true: check standard fields
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessExpCondition dbaccessclass , boolean first , String field_list){
    // fill standard fields
    setUniqueid(dbaccessclass.getUniqueid());
    setVersion(dbaccessclass.getVersion());
    setCreated(dbaccessclass.getCreated());
    setCreator(dbaccessclass.getCreator());
    setChanged(dbaccessclass.getChanged());
    setChangedfrom(dbaccessclass.getChangedfrom());
    setInuse(dbaccessclass.getInuse());
    setClient(dbaccessclass.getClient());
    hasdata = true;

    // fill individual fields
    setSort(dbaccessclass.getSort());
    setCondition(dbaccessclass.getCondition());
    setOperator(dbaccessclass.getOperator());
    setOperand(dbaccessclass.getOperand());
    setMyDbAccessExpRule(dbaccessclass.getMyDbAccessExpRule());
    setMyDbAccessExpAttribute(dbaccessclass.getMyDbAccessExpAttribute());


    try{
      // check standard fields
      if (!first) {
        if (getUniqueid() == null || (getUniqueid() != null && getUniqueid().length() == 0)) {
           errCode = ERROR_0009;
           errMess = t4m.getMessage("msg0009", null);
           return false;
        }
        if (getUniqueid() != null && getUniqueid().length() > 35) {
           errCode = ERROR_0010;
           errMess = t4m.getMessage("msg0010", null);
           return false;
        }
        if (getClient() == null || (getClient() != null && getClient().length() == 0)) {
           errCode = ERROR_0011;
           errMess = t4m.getMessage("msg0011", null);;
           return false;
        }
        if (getClient() != null && getClient().length() > 35) {
           errCode = ERROR_0012;
           errMess = t4m.getMessage("msg0012", null);
           return false;
        }
      }
      if (getCreator() != null && getCreator().length() > 35) {
        errCode = ERROR_0013;
        errMess = t4m.getMessage("msg0013", null);
        return false;
      }
      if (getChangedfrom() != null && getChangedfrom().length() > 35) {
        errCode = ERROR_0014;
        errMess = t4m.getMessage("msg0014", null);
        return false;
      }

      // check individual fields
      if ((first) && (getCondition() == null || (getCondition() != null && getCondition().length() == 0))) {
        errCode = ERROR_12_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("condition" , "label"));
        return false;
      } else if ((field_list.indexOf(",condition,")!=-1) && (getCondition() == null || (getCondition() != null && getCondition().length() == 0) || (getCondition() != null && getCondition().equals("n4eempty")))) {
        errCode = ERROR_13_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("condition" , "label"));
        return false;
      }
      if (getCondition() != null && (getCondition().length() > 35)) {
        errCode = ERROR_14_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("condition" , "label") + "|35");
        return false;
      }
      if ((first) && (getOperator() == null || (getOperator() != null && getOperator().length() == 0))) {
        errCode = ERROR_12_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("operator" , "label"));
        return false;
      } else if ((field_list.indexOf(",operator,")!=-1) && (getOperator() == null || (getOperator() != null && getOperator().length() == 0) || (getOperator() != null && getOperator().equals("n4eempty")))) {
        errCode = ERROR_13_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("operator" , "label"));
        return false;
      }
      if (getOperator() != null && (getOperator().length() > 35)) {
        errCode = ERROR_14_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("operator" , "label") + "|35");
        return false;
      }
      if ((first) && (getOperand() == null || (getOperand() != null && getOperand().length() == 0))) {
        errCode = ERROR_7_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("operand" , "label"));
        return false;
      } else if ((field_list.indexOf(",operand,")!=-1) && (getOperand() == null || (getOperand() != null && getOperand().length() == 0))) {
        errCode = ERROR_8_0116;
        errMess = t4m.getMessage("msg0116", dbaccessclass.getDefData("operand" , "label"));
        return false;
      }
      if (getOperand() != null && (getOperand().length() > 35)) {
        errCode = ERROR_9_0117_0118;
        errMess = t4m.getMessage("msg0117", dbaccessclass.getDefData("operand" , "label") + "|35");
        return false;
      }

    } catch (Exception e) {
      System.err.println(e.getMessage());
      errCode = ERROR_Ex;
      errMess = e.getMessage();
    }
    return true;
  }
}
