package n4e.core.dbaccess;

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;


/**
 * The <CODE>DbAccessCheckResultAtt</CODE> class checks fields of ResultAtt table in database.
 * @author Thomas Kaffka
 * @version Fri May 01 16:26:19 CEST 2015
 * @since 1.0
*/
public class DbAccessCheckResultAtt extends DbCheck {
  // technical fields
  public int errCode = 0;
  public String errMess = "";
  private boolean hasdata = false;
  private Tool4Number t4n = new Tool4Number();
  private Tool4DbMessages t4m = null;

  // database standard fields
  private String uniqueid = ""; // unique key
  private long version = 0; // version counter for database store without locking
  private java.util.Date created = null; // date of creation
  private String creator = ""; // user created
  private java.util.Date changed = null; // date of last change data
  private String changedfrom = ""; // user changed
  private int inuse = 0; // read but not updated
  private String client = ""; // foreign key to client table

  // individual database fields
  private ArrayList myDbAccessResultValue = new ArrayList() ; // ResultValue
  private DbAccessExpAttribute myDbAccessExpAttribute = new DbAccessExpAttribute() ; // ExpAttribute
  private DbAccessProject myDbAccessProject = new DbAccessProject() ; // Project


  // sets and gets of standard table fields
  public void setUniqueid(String input){uniqueid = input;}
  public String getUniqueid(){return uniqueid;}
  public void setVersion(long input){version = input;}
  public long getVersion(){return version;}
  public void setCreated(java.util.Date input){created = input;}
  public java.util.Date getCreated(){return created;}
  public void setCreator(String input){creator = input;}
  public String getCreator(){return creator;}
  public void setChanged(java.util.Date input){changed = input;}
  public java.util.Date getChanged(){return changed;}
  public void setChangedfrom(String input){changedfrom = input;}
  public String getChangedfrom(){return changedfrom;}
  public void setInuse(int input){inuse = input;}
  public int getInuse(){return inuse;}
  public void setClient(String input){client = input;}
  public String getClient(){return client;}

  // sets and gets of individual table fields
  public void setMyDbAccessResultValue(ArrayList input){myDbAccessResultValue = input;}
  public ArrayList getMyDbAccessResultValue(){return myDbAccessResultValue;}
  public void setMyDbAccessExpAttribute(DbAccessExpAttribute input){myDbAccessExpAttribute = input;}
  public DbAccessExpAttribute getMyDbAccessExpAttribute(){return myDbAccessExpAttribute;}
  public void setMyDbAccessProject(DbAccessProject input){myDbAccessProject = input;}
  public DbAccessProject getMyDbAccessProject(){return myDbAccessProject;}


  /**
   * The constructor <CODE>DbAccessCheckResultAtt</CODE> initialized the object.
   * @param pt4m the tool 4 message
   * @since 1.0
  */
  public DbAccessCheckResultAtt(Tool4DbMessages pt4m){
    t4m = pt4m;
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens of dataset
   * @param dbaccessclass Class DbAccessCustomer.
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessResultAtt dbaccessclass , String field_list){
    return checkData(dbaccessclass , false , field_list);
  }

  /**
   * The method <CODE>checkData</CODE> test the admissibility of field contens
   * @param dbaccessclass Class DbAccess...
   * @param first = true: check standard fields
   * @param field_list the list of fields to be store.
   * @return true if no falts
   * @since 1.0
  */
  public boolean checkData(DbAccessResultAtt dbaccessclass , boolean first , String field_list){
    // fill standard fields
    setUniqueid(dbaccessclass.getUniqueid());
    setVersion(dbaccessclass.getVersion());
    setCreated(dbaccessclass.getCreated());
    setCreator(dbaccessclass.getCreator());
    setChanged(dbaccessclass.getChanged());
    setChangedfrom(dbaccessclass.getChangedfrom());
    setInuse(dbaccessclass.getInuse());
    setClient(dbaccessclass.getClient());
    hasdata = true;

    // fill individual fields
    setMyDbAccessResultValue(dbaccessclass.getMyDbAccessResultValue());
    setMyDbAccessExpAttribute(dbaccessclass.getMyDbAccessExpAttribute());
    setMyDbAccessProject(dbaccessclass.getMyDbAccessProject());


    try{
      // check standard fields
      if (!first) {
        if (getUniqueid() == null || (getUniqueid() != null && getUniqueid().length() == 0)) {
           errCode = ERROR_0009;
           errMess = t4m.getMessage("msg0009", null);
           return false;
        }
        if (getUniqueid() != null && getUniqueid().length() > 35) {
           errCode = ERROR_0010;
           errMess = t4m.getMessage("msg0010", null);
           return false;
        }
        if (getClient() == null || (getClient() != null && getClient().length() == 0)) {
           errCode = ERROR_0011;
           errMess = t4m.getMessage("msg0011", null);;
           return false;
        }
        if (getClient() != null && getClient().length() > 35) {
           errCode = ERROR_0012;
           errMess = t4m.getMessage("msg0012", null);
           return false;
        }
      }
      if (getCreator() != null && getCreator().length() > 35) {
        errCode = ERROR_0013;
        errMess = t4m.getMessage("msg0013", null);
        return false;
      }
      if (getChangedfrom() != null && getChangedfrom().length() > 35) {
        errCode = ERROR_0014;
        errMess = t4m.getMessage("msg0014", null);
        return false;
      }

      // check individual fields

    } catch (Exception e) {
      System.err.println(e.getMessage());
      errCode = ERROR_Ex;
      errMess = e.getMessage();
    }
    return true;
  }
}
