package n4e.core.dbaccess;

import java.sql.*;
import java.text.*;
import java.util.*;
import java.math.*;
import java.util.zip.*;
import java.io.*;

import n4e.core.database.*;
import n4e.core.dbaccess.*;
import n4e.core.system.*;
import n4e.core.tool.*;
import n4e.core.list.*;



/**
 * The <CODE>DbAccessReadResultAtt</CODE> class reads fields of ResultAtt table in database.
 * @author Thomas Kaffka
 * @version Fri May 01 16:26:19 CEST 2015
*/
public class DbAccessReadResultAtt extends DbAccessRead {

  // individual database fields
  private ArrayList myDbAccessResultValue = null ; // ResultValue
  private DbAccessReadExpAttribute myDbAccessReadExpAttribute = null ; // ExpAttribute
  private DbAccessReadProject myDbAccessReadProject = null ; // Project


  // sets and gets of individual table fields
  public void setMyDbAccessResultValue(ArrayList input){myDbAccessResultValue = input;}
  public ArrayList getMyDbAccessResultValue(){return myDbAccessResultValue;}
  public void setMyDbAccessReadExpAttribute(DbAccessReadExpAttribute input){myDbAccessReadExpAttribute = input;}
  public DbAccessReadExpAttribute getMyDbAccessReadExpAttribute(){return myDbAccessReadExpAttribute;}
  public void setMyDbAccessReadProject(DbAccessReadProject input){myDbAccessReadProject = input;}
  public DbAccessReadProject getMyDbAccessReadProject(){return myDbAccessReadProject;}


  /**
   * The constructor <CODE>DbAccessReadResultAtt</CODE> initialized the object.
   * @since 1.0
  */
  public DbAccessReadResultAtt(){
      setWorkFields();
  }

  /**
   * The constructor <CODE>DbAccessReadResultAtt</CODE> initialized the object.
   * @param sysUser the system user.
   * @since 1.0
  */
  public DbAccessReadResultAtt(List4SysUser sysUser){
    setSystemUser(sysUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }
  }

  /**
   * The constructor <CODE>DbAccessReadResultAtt</CODE> initialized the object
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception if error
   * @since 1.0
  */
  public DbAccessReadResultAtt(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , null , true);

  }

  /**
   * The constructor <CODE>DbAccessReadResultAtt</CODE> initialized the object
   * @param conn database connection
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @exception Exception if error
   * @since 1.0
  */
  public DbAccessReadResultAtt(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey ) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , null , true);

  }

  /**
   * The constructor <CODE>DbAccessReadResultAtt</CODE> initialized the object with additional sqlwhere
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception if error
   * @since 1.0
  */
  public DbAccessReadResultAtt(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd , true);

  }

  /**
   * The constructor <CODE>DbAccessReadResultAtt</CODE> initialized the object with additional sqlwhere
    this.conn 0 conn;
   * @param conn database connection
   * @param pUser the system user object
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @exception Exception if error
   * @since 1.0
  */
  public DbAccessReadResultAtt(DatabaseConnection conn , List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    if (conn != null) {
        this.conn = conn;
        this.myconn = false;
    }
    initiate(pUser , uniqueid , fieldlist , privfkey , publfkey , sqlAdd , true);

  }

  /**
   * The method <CODE>initiate</CODE> initializes the object with additional sqlwhere.
   * @param pUser the system user object.
   * @param uniqueid the unique key.
   * @param fieldlist List of fields to load.
   * @param privfkey List of privfkeys to load.
   * @param publfkey List of publfkeys to load.
   * @param sqlAdd additional where clause.
   * @param read_with_client read with use of the client id.
   * @exception Exception if error
   * @since 1.0
  */
  public void initiate(List4SysUser pUser , String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd , boolean read_with_client) throws Exception {

    this.read_with_client = read_with_client;
    setSystemUser(pUser);
    if (sysUser.getUserLanguageCode().equals("de")) {
        t4m = new Tool4DbMessages_de(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("en")) {
        t4m = new Tool4DbMessages_en(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("fr")) {
        t4m = new Tool4DbMessages_fr(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("es")) {
        t4m = new Tool4DbMessages_es(dbtablename);
    } else if (sysUser.getUserLanguageCode().equals("gr")) {
        t4m = new Tool4DbMessages_gr(dbtablename);
    } else {
        t4m = new Tool4DbMessages_en(dbtablename);
    }

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    publicforeignkeytable.put("idresultvalue","ResultValue|idresultatt");

    privateforeignkeytable.put("idexpattribute","ExpAttribute|null");
    privateforeignkeytable.put("idproject","Project|null");


    success = loadFromTable(uniqueid , fieldlist , privfkey , publfkey , sqlAdd );
  }

  /**
   * The method <CODE>setWorkFields</CODE> sets the work fiels to current values.
   * @since 1.0
  */
  public void setWorkFields() {
      dbtablename = "ResultAtt"; // name of own database table
      if (sysUser != null) {
          databasename = sysUser.getClientDbsystemforeclient(); // name of own database
      }
      tblnameofprivk = ",ExpAttribute,Project,"; // list of table names of private foreign keys
      tblnameofpublvk = ",ResultValue,"; // list of table names of public foreign keys
      keynameofpublvk = ",idresultatt,"; // list of key names of table names of public foreign keys
      tblnameofownerfield = ","; // list of names of table names of owner key
      indivTableFields = ","; // list of inidividual table fields
      privateforeignkey = ";idexpattribute;idproject;"; // list of private foreign key fields
      publicforeignkey = ";idresultvalue;"; // list of public foreign key fields
  }

  /**
   * The method <CODE>loadFromTable</CODE> gets tabledata from database.
   * @param uniqueid the unique key
   * @param fieldlist List of fields to load
   * @param privfkey List of privfkeys to load
   * @param publfkey List of publfkeys to load
   * @param sqlAdd additional where clause
   * @return true if success
   * @exception Exception if error
   * @since 1.0
  */
  public boolean loadFromTable(String uniqueid , String fieldlist , String privfkey , String publfkey , String sqlAdd ) throws Exception {

    errCode = 0;
    errMess = "";

    this.fields4load = fieldlist;
    this.privfkey4load = privfkey;
    this.publfkey4load = publfkey;
    this.sqlAdd = sqlAdd;
    hasdata = false;
    // create connection to defined database system
    String dbsystem = databasename;
    DatabaseConnectionSet myDatabaseConnectionSet = DatabaseManager.getDatabaseConnectionSet(dbsystem);
    if (this.myconn) {
        this.conn = myDatabaseConnectionSet.conn();
    }
    DatabaseSQLResultSet resSet = new DatabaseSQLResultSet(dbsystem , conn);

    // handle fields to load
    String sqlFieldlist = "";
    String sqlPrivfkey = "";

    // build sql field list
    if (fieldlist != null && fieldlist.equals("all")) {
      sqlFieldlist = indivTableFields.substring(1);
    } else { // make individual fieldlist
      if (fieldlist != null) {
        sqlFieldlist = fieldlist;
      } else {
        sqlFieldlist = "";
      }
    }

    // build sql field list of private foreign keys
    if (privfkey != null && privfkey.equals("all")) {
      sqlPrivfkey = privateforeignkey.substring(1);
    } else { // make individual fieldlist
      if (privfkey != null) {
          int pos = privfkey.indexOf("(");
          if (pos >= 0) {
            StringTokenizer stok = new StringTokenizer(privfkey , ";"); // public foreign keys separated by ;
            while (stok.hasMoreTokens()) { // for each public foreign key
                String xfield = stok.nextToken();
                pos = xfield.indexOf("(");
                if (pos >= 0) {
                    sqlPrivfkey += "," + xfield.substring(0, pos);
                } else {
                    sqlPrivfkey += "," + xfield;
                }
            }
            sqlPrivfkey = sqlPrivfkey.substring(1);
          } else {
            sqlPrivfkey = t4st.replaceToken(privfkey , ";" , ",");
          }
      } else {
        sqlPrivfkey = "";
      }
    }

    StringTokenizer stok = null;

    String workingSqlFieldlist = stdTableFields + sqlFieldlist;
    if (workingSqlFieldlist.endsWith(",")) {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?sqlPrivfkey:"")+",";
    } else {
      workingSqlFieldlist += (sqlPrivfkey.length()>0?","+sqlPrivfkey:"")+",";
    }


    int len = workingSqlFieldlist.length();
    workingSqlFieldlist = workingSqlFieldlist.substring(1,len-1);
    if (workingSqlFieldlist.endsWith(",")) {
      len = workingSqlFieldlist.length();
      workingSqlFieldlist = workingSqlFieldlist.substring(0,len-1);
    }

    resetFields();

    // grip into data sink
    if (uniqueid == null && sqlAdd == null) {
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
        errCode = ERROR_0001;
        errMess = t4m.getMessage("msg0001", null);
        return false;
    }

    String client_temp = client;
    if (!read_with_client) {
        client = null;
    }

    String where1 = ((client!=null&&client.length()>0)?("client='"+client+"'"):"");
    String where2 = ((uniqueid!=null&&uniqueid.length()>0)?("uniqueid='"+uniqueid+"'"):"");
    String where3 = ((sqlAdd!=null&&sqlAdd.length()>0)?sqlAdd:"");
    String where = "";
    if (where1.length()==0 && where2.length()==0 && where3.length()==0) { // 000
      where = "";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()==0) { // 100
      where = " WHERE (" + where1 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()==0) { // 010
      where = " WHERE (" + where2 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()==0) { // 110
      where = " WHERE (" + where1 + ") AND (" + where2 + ")";
    } else if (where1.length()==0 && where2.length()==0 && where3.length()>0) { // 001
      where = " WHERE (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()==0 && where3.length()>0) { // 101
      where = " WHERE (" + where1 + ") AND (" + where3 + ")";
    } else if (where1.length()==0 && where2.length()>0 && where3.length()>0) { // 011
      where = " WHERE (" + where2 + ") AND (" + where3 + ")";
    } else if (where1.length()>0 && where2.length()>0 && where3.length()>0) { // 111
      where = " WHERE (" + where1 + ") AND (" + where2 + ") AND (" + where3 + ")";
    }

    String sqlStatement = "SELECT "+workingSqlFieldlist+" FROM ResultAtt " + where;
    conn.connectionlog("DbAccessReadResultAtt","sql statement", sqlStatement);
    ResultSet result = resSet.getResultSet(sqlStatement);
    if (result.next() && (result.isFirst() && result.isLast())) {
      hasdata = true;
      // get standard fields
      setUniqueid(result.getString(1));
      setVersion(result.getLong(2));
      long temp_timestamp = result.getLong(3);
      java.util.Date temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setCreated(temp_date);
      setCreator(result.getString(4));
      temp_timestamp = result.getLong(5);
      temp_date = new java.util.Date();
      temp_date.setTime(temp_timestamp);
      setChanged(temp_date);
      setChangedfrom(result.getString(6));
      setInuse(result.getInt(7));
      setClient(result.getString(8));

      //get individual fields
      int i = 9;
      workingSqlFieldlist = sqlFieldlist + (sqlPrivfkey.length()>0?","+sqlPrivfkey:"");
      stok = new StringTokenizer(workingSqlFieldlist ,",");
      while (stok.hasMoreTokens()) {
        String xfield = stok.nextToken();
        if (false) {}
          // dummy
          else if (xfield.equals("idexpattribute")) {}
          else if (xfield.equals("idproject")) {}

        i++;
      }

      //get individual privateforeignkeys
      if (privfkey != null && privfkey.length()>0) {
        String pffieldlist = "";
        if (privfkey.equals("all")) { // load all private foreign key objects with field list 'all'
          privfkey = privateforeignkey.substring(1);
        }
        stok = new StringTokenizer(privfkey , ";"); // private foreign keys separated by ;
        while (stok.hasMoreTokens()) { // for each private foreign key
          String xfield = stok.nextToken();
          if (xfield == null || xfield.length() == 0) continue;
          if (xfield.indexOf("(") > -1) { // private foreign key with field list
            StringTokenizer stfield = new StringTokenizer(xfield ,"(");
            if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (pffieldlist.endsWith(")")) {
              pffieldlist = pffieldlist.substring(0, pffieldlist.length()-1);
            }
          } else {
            pffieldlist = "all";
          }
          if (false) {}
          else if (xfield.equals("idexpattribute")) {
            DbAccessReadExpAttribute tempIdexpattribute = new DbAccessReadExpAttribute(conn , sysUser , result.getString("idexpattribute") , pffieldlist , null , null);
            if (tempIdexpattribute.errCode == -17) {
              setMyDbAccessReadExpAttribute(null);
            } else if (tempIdexpattribute.errCode == 0) {
              setMyDbAccessReadExpAttribute(tempIdexpattribute);
            } else if (tempIdexpattribute.errCode != ERROR_0001) {
              errCode = tempIdexpattribute.errCode;
              errMess = tempIdexpattribute.errMess;
            }
          }
          else if (xfield.equals("idproject")) {
            DbAccessReadProject tempIdproject = new DbAccessReadProject(conn , sysUser , result.getString("idproject") , pffieldlist , null , null);
            if (tempIdproject.errCode == -17) {
              setMyDbAccessReadProject(null);
            } else if (tempIdproject.errCode == 0) {
              setMyDbAccessReadProject(tempIdproject);
            } else if (tempIdproject.errCode != ERROR_0001) {
              errCode = tempIdproject.errCode;
              errMess = tempIdproject.errMess;
            }
          }

        }
      }

    } else { // no data found
        if (this.myconn) {
            myDatabaseConnectionSet.connClose(conn);
        }
       errCode = ERROR_0002;
       errMess = t4m.getMessage("msg0002", null);
       return false;
    }

    //get individual publicforeignkeys
    if (publfkey != null && publfkey.length()>0) {
      String pffieldlist = "";
      if (publfkey.equals("all")) { // load all public foreign key objects with field list 'all'
        publfkey = publicforeignkey.substring(1);
      }
      DatabaseDatasetLoader dsl = new DatabaseDatasetLoader(this.conn);
      stok = new StringTokenizer(publfkey ,";"); // public foreign keys separated by ;
      while (stok.hasMoreTokens()) { // for each public foreign key
        String xfield = stok.nextToken();
        if (xfield == null || xfield.length() == 0) continue;
        StringTokenizer stfield = null;
        if (xfield.indexOf("(") > -1) { // public foreign key with field list
          stfield = new StringTokenizer(xfield ,"(");
          if (stfield.hasMoreTokens()) xfield = stfield.nextToken();
          if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
          if (pffieldlist.endsWith(")")) {
            pffieldlist = pffieldlist.substring(0 , pffieldlist.length()-1);
          }
        } else {
          pffieldlist = "all";
        }
        String orderby = null;
        if (pffieldlist.indexOf("|") > -1) { // order by feilds available.
            stfield = new StringTokenizer(pffieldlist ,"|");
            if (stfield.hasMoreTokens()) pffieldlist = stfield.nextToken();
            if (stfield.hasMoreTokens()) orderby = stfield.nextToken();
        }
        String pfTableName = (String) publicforeignkeytable.get(xfield);
        String pfkeyfield = "";
        StringTokenizer staccess = new StringTokenizer(pfTableName ,"|");
        if (staccess.hasMoreTokens()) pfTableName = staccess.nextToken();
        if (staccess.hasMoreTokens()) pfkeyfield = staccess.nextToken();
        String sql = pfkeyfield + " = '" + result.getString("uniqueid")+"'";
        ArrayList keyList = dsl.getDataSetRead(sysUser , pfTableName , pffieldlist , null , null , sql , orderby , "expertanddecision");
        if (false) {}
        else if (xfield.equals("idresultvalue")) {
          setMyDbAccessResultValue(keyList);
        }

      }
    }

    if (!read_with_client) {
        client = client_temp;
    }

    resSet.closeResultSet();
    resSet = null;
    if (this.myconn) {
        myDatabaseConnectionSet.connClose(conn);
    }

    return true;
  }

  /**
   * The method <CODE>resetFields</CODE> sets fields to initvalue.
   * @exception Exception if error
   * @since 1.0
  */
  private void resetFields() throws Exception {
    uniqueid = null;
    version = 0;
    created = null;
    creator = null;
    changed = null;
    changedfrom = null;
    inuse = 0;
    myDbAccessResultValue = new ArrayList();
    myDbAccessReadExpAttribute = null;
    myDbAccessReadProject = null;

  }

  /**
   * The method <CODE>getDefData</CODE> returns data concerning a table field.
   * @param fieldname the name of table field
   * @param datatype type of data to return
   * @return def data
   * @exception Exception if error
   * @since 1.0
  */
  public String getDefData(String fieldname , String datatype) throws Exception {
    if (datatype.equals("label")) {
      if (false) {}

//       return dbtablename + fieldname;
    } else if (datatype.equals("type")) {
      if (false) {}
      else if (fieldname.equals("idresultvalue")) {return "publicforeignkey";}
      else if (fieldname.equals("idexpattribute")) {return "privateforeignkey";}
      else if (fieldname.equals("idproject")) {return "privateforeignkey";}

    } else if (datatype.equals("length")) {
      if (false) {}
      else if (fieldname.equals("idresultvalue")) {return "35";}
      else if (fieldname.equals("idexpattribute")) {return "35";}
      else if (fieldname.equals("idproject")) {return "35";}

    } else if (datatype.equals("needed")) {
      if (false) {}
      else if (fieldname.equals("idresultvalue")) {return "no";}
      else if (fieldname.equals("idexpattribute")) {return "no";}
      else if (fieldname.equals("idproject")) {return "no";}

    }
    return "";
  }

  /**
   * The method <CODE>getBlobField</CODE> returns the field contens of blob fields.
   * @param fieldname the name of table field.
   * @return field contens.
   * @exception Exception if error
   * @since 1.0
  */
  public byte[] getBlobField(String fieldname) throws Exception {
    if (false) {}

    return null;
  }

  /**
   * The method <CODE>getFieldAsStringKey</CODE> returns the field contens as a string.
   * @param fieldname the name of table field.
   * @param getkey, get the key value.
   * @return field contens
   * @exception Exception if error
   * @since 1.0
  */
  public String getFieldAsStringKey(String fieldname , Boolean getkey) throws Exception {
    if (false) {}
    else if (fieldname.equals("uniqueid")) {return uniqueid == null?"":uniqueid;}
    else if (fieldname.equals("version")) {return Long.toString(version);}
    else if (fieldname.equals("created")) {return created == null?"":created.toString();}
    else if (fieldname.equals("creator")) {return creator == null?"":creator;}
    else if (fieldname.equals("changed")) {return changed == null?"":changed.toString();}
    else if (fieldname.equals("changedfrom")) {return changedfrom == null?"":changedfrom;}
    else if (fieldname.equals("inuse")) {return Integer.toString(inuse);}
    else if (fieldname.equals("client")) {return client == null?"":client;}
    else if (fieldname.equals("idexpattribute")) {return getMyDbAccessReadExpAttribute()==null?"":getMyDbAccessReadExpAttribute().getUniqueid();}
    else if (fieldname.equals("idproject")) {return getMyDbAccessReadProject()==null?"":getMyDbAccessReadProject().getUniqueid();}

    return "";
  }

  /**
   * The method <CODE>getDataSet</CODE> returns a data set from table.
   * @param addsqlorder order of sql result
   * @return dataSet
   * @exception Exception if error
   * @since 1.0
  */
  public ArrayList getDataSet(String addsqlorder) throws Exception {
    DatabaseDatasetLoader dbl = new DatabaseDatasetLoader();
    ArrayList dataSet = dbl.getDataSetRead(sysUser , dbtablename , fields4load , privfkey4load , publfkey4load , sqlAdd , addsqlorder , "expertanddecision");
    return dataSet;
  }

  /**
   * The method <CODE>getTranslationText</CODE> returns the translated text.
   * @param ndx the text index.
   * @return the translation.
   * @exception Exception if error
   * @since 1.0
  */
  public String getTranslationText(int ndx) throws Exception {
    if (false) {
    } else if (sysUser.getUserLanguageCode().equals("de")) {
      if (false) {}

    } else if (sysUser.getUserLanguageCode().equals("en")) {
      if (false) {}


    }
    return "no text available";
  }


}
