package n4e.core.system;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JFrame;

import n4e.core.controls.AWTextAreaPopUp;
import n4e.core.list.List4SysUser;
import n4e.core.tool.Tool4Dialog;
import n4e.core.tool.Tool4String;
import n4e.core.tool.Tool4Throwable;
import n4e.core.dbaccess.DbAccessReadExpAttribute;

/**
 * The <CODE>AttHelp</CODE> class provides the documentation for an attribute.
 * <P><B>title: SystemCoreExpLib</B></P>
 * <P>description: </P>
 * <P>copyright: (c) 2013 thomas kaffka, born at 11/08/1959 in dsseldorf - germany , all rights reserved.</P>
 * @author thomas kaffka , cologne , germany.
 * @version 21.02.2013 15:26:12
 * @since 1.0
 */

public class AttHelp extends JDialog implements ActionListener {

    private AWTextAreaPopUp taDocumentation; // the attribut's documentation.

    private JButton btnClose; // button: close window.
    
    private List4SysUser sysUser;

    private Tool4Dialog t4dialog; // the network tool.
    private Tool4String t4s;
    
    /**
	 * The constructor <CODE>AttHelp</CODE> creates a new instance(object) of the class.
	 * @param exp the calling frame.
	 * @param att_uid the uniqueid of the attribute.
	 * @param headline the head line.
	 * @param sysUser the system user.
	 * @since 1.0
	 */
	public AttHelp(JFrame exp, String att_uid, String headline, List4SysUser sysUser) {
        super(exp, headline);

        try {
        	
	        t4dialog = new Tool4Dialog(null);
	        t4s = new Tool4String();
	        this.sysUser = sysUser;
        	
	        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	        this.setBackground(Color.lightGray);
	        this.setSize(new Dimension(407, 395));
	
	        Font fontStandard = new Font("Dialog", Font.PLAIN, 12);
	        setFont(fontStandard);
	
	        Container c = getContentPane();
	        c.setLayout(null);
	        c.setSize(new Dimension(400, 370));
	
	        Font fontStandard16 = new Font("Dialog", Font.PLAIN, 16);
	        taDocumentation = new AWTextAreaPopUp(120, 300, AWTextAreaPopUp.PASTE_UNDO);
	        taDocumentation.setBounds(10, 10, 370, 300);
	        taDocumentation.setFont(fontStandard16);
	        taDocumentation.setLineWrap(true);
	        taDocumentation.setWrapStyleWord(true);
	        taDocumentation.setEditable(false);
	        JScrollPane scrollPane = new JScrollPane(taDocumentation);
	        scrollPane.setViewportView(taDocumentation);
	        scrollPane.setBounds(10,10,370,300);
	        c.add(scrollPane);

    		DbAccessReadExpAttribute att = new DbAccessReadExpAttribute(sysUser, 
    				att_uid, "description", null, null);
    		if (att.errCode != 0) {
    			t4dialog.showException(att.errCode + "\r\n" + att.errMess);
    			return;
    		}
    		taDocumentation.setText(t4s.replaceSQLWithChars(att.getDescription()));
	        
            btnClose = new JButton("close");
            btnClose.setBounds(310, 325, 72, 20);
            btnClose.setFont(new Font("Dialog", Font.PLAIN, 12));
            btnClose.addActionListener(this);
            c.add(btnClose);

	        center();
	        setVisible(true);

        } catch (Exception ex) {
        	t4dialog.showException(Tool4Throwable.getCallStackOut(ex));
        }
	}

    /**
     * The method <CODE>center</CODE> centers the main window.
     * @since 1.0
     */
    private void center() {
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((dim.width - getWidth()) / 2, (dim.height - getHeight()) / 2);
    }

    /**
     * The method <CODE>actionPerformed</CODE> processes the button clicks.
     * @param e the event.
     * @since 1.0
     */
	public void actionPerformed(ActionEvent e) {
        if (e.getSource() == btnClose) {
//        	exp.eads.btnHelp.setEnabled(true);
            dispose();
        }
	}
}

